package com.daqing.financial.hrms;

import com.daqing.framework.domain.crms.ext.CustomerCompanyVO;
import com.daqing.framework.domain.hrms.DeptEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.domain.hrms.request.DeptAddRequest;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Author: gongsj.
 * @Description: 部门管理controllerApi定义
 * @Date:Created in 2020/09/07 11:33.
 * @Modified By:
 */
@Api(value = "部门管理相关操作", tags = "提供部门信息增删改查等相关方法")
public interface DeptControllerApi {

    /**
     * 新增一条部门信息
     */
    @ApiOperation(value = "新增一条部门信息", notes = "新增一条部门信息")
    ResponseResult save(DeptAddRequest deptAddRequest);

    /**
     * 部门信息树形结构展示
     */
    @ApiOperation(value = "部门信息树形结构展示", notes = "部门信息树形结构展示")
    ResponseResult tree();

    /**
     * 拖拉拽重新排序，批量修改部门信息
     */
    @ApiOperation(value = "拖拉拽重新排序，批量修改部门信息", notes = "拖拉拽重新排序，批量修改部门信息")
    ResponseResult updateSort(List<DeptEntity> depts);

    /**
     * 根据id查询部门详细信息
     */
    @ApiOperation(value = "根据id查询部门详细信息", notes = "根据id查询部门详细信息")
    ResponseResult info(Long id);

    /**
     * 根据id修改部门信息
     */
    @ApiOperation(value = "根据id修改部门信息", notes = "根据id修改部门信息", response = CustomerCompanyVO.class)
    ResponseResult update(DeptEntity dept);

    /**
     * 根据id数组批量删除部门信息
     */
    @ApiOperation(value = "根据id数组批量删除部门信息", notes = "根据id数组批量删除部门信息")
    ResponseResult delete(Long[] ids);

    /**
     * 查询所有部门及部门下的员工信息
     */
    @ApiOperation(value = "查询所有部门及部门下的员工信息(加条件限制,即担保专员、ABC角不能为同一用户)", notes = "查询所有部门及部门下的员工信息")
    ResponseResult trees(String account);

    @ApiOperation(value = "查询所有部门及部门下的员工信息(不加条件限制)", notes = "查询所有部门及部门下的员工信息")
    ResponseResult empTrees(String account);

    /**
     * 查询该部门下所有子部门及部门下的的员工
     */
    @ApiOperation(value = "查询该部门下所有子部门及部门下的的员工", notes = "查询该部门下所有子部门及部门下的的员工")
    ResponseResult children(Long id);

    /**
     * 获取所有部门id和名称(用户团队效率的部门筛选)
     */
    @ApiOperation(value = "获取所有部门id和名称(用户团队效率的部门筛选)", notes = "获取所有部门id和名称(用户团队效率的部门筛选)")
    ResponseResult listDepartment();

    /**
     * 根据部门id查询该部门下所有员工
     */
    @ApiOperation(value = "根据部门id查询该部门下所有员工",notes = "根据部门id查询该部门下所有员工",response = EmployeeVO.class)
    ResponseResult listEmployeeByDeptId(Long deptId);
}
