package com.daqing.financial.hrms;

import com.daqing.framework.domain.crms.request.EmployeeDisableRequest;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeInfoVO;
import com.daqing.framework.domain.hrms.ext.EmployeeListVO;
import com.daqing.framework.domain.hrms.request.EmployeeRequest;
import com.daqing.framework.domain.hrms.response.RolePermissionResponse;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @auther River
 * @date 2020/9/9 17:41
 */
@Api(value = "员工管理相关操作", tags = "提供员工信息增删改查等相关方法")
public interface EmployeeControllerApi {

    @ApiOperation(value = "根据id查询员工的基本信息和部门信息", notes = "根据id查询员工的基本信息和部门信息")
    ResponseResult getEmployeeAndDeptById(Long[] ids);

    @ApiOperation(value = "根据id查询员工姓名", notes = "根据id查询员工姓名")
    ResponseResult getEmployeeById(Long id);

    @ApiOperation(value = "分页加条件查询员工信息", notes = "分页加条件查询员工信息",response = EmployeeListVO.class)
    ResponseResult list(Integer page, Integer size, EmployeeRequest employeeRequest);

    @ApiOperation(value = "根据id删除员工信息", notes = "根据id删除员工信息")
    ResponseResult delete(List<Long> ids);

    @ApiOperation(value = "启用",notes = "启用")
    ResponseResult start(Long id);

    @ApiOperation(value = "禁用",notes = "禁用")
    ResponseResult stop(EmployeeDisableRequest employeeDisableRequest);

    @ApiOperation(value = "禁用详情",notes = "禁用详情")
    ResponseResult forbidInfo(Long id);

    @ApiOperation(value = "重置密码",notes = "重置密码")
    ResponseResult reset(Long id);

    @ApiOperation(value = "详细信息",notes = "相信信息",response = EmployeeInfoVO.class)
    ResponseResult info(Long id);

    @ApiOperation(value = "角色对应的权限",notes = "角色对应的权限",response = RolePermissionResponse.class)
    ResponseResult tree(List<Long> ids);

    @ApiOperation(value = "更新",notes = "更新")
    ResponseResult update(EmployeeInfoVO employeeInfoVO);

    @ApiOperation(value = "新增",notes = "新增")
    ResponseResult save(EmployeeInfoVO employeeInfoVO);

    @ApiOperation(value = "导出",notes = "导出")
    ResponseResult excelExport(HttpServletResponse response);

    @ApiOperation(value = "导出模板",notes = "导出模板")
    ResponseResult excelTemplate(HttpServletResponse response);

    @ApiOperation(value = "导入",notes = "导入")
    ResponseResult excelImport(MultipartFile file);

    @ApiOperation(value = "查询所有员工id和姓名",notes = "查询所有员工id和姓名")
    ResponseResult listEmployeeName();

    @ApiOperation(value = "给用户赋予一个角色",notes = "给用户赋予一个角色")
    ResponseResult saveUserRole(Long userId,Long roleId);

    @ApiOperation(value = "根据用户id获取对应的员工名称", notes = "根据用户id获取对应的员工名称")
    ResponseResult listEmpNameByUserId(Integer[] userIds);
}
