package com.daqing.financial.crms.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.daqing.financial.crms.model.request.CrmsCompanyPersonalReq;
import com.daqing.financial.crms.service.ICrmsCompanyPersonalService;
import com.daqing.financial.crms.service.ICrmsCustomerRelatedService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.crms.CrmsCompanyPersonal;
import com.daqing.framework.domain.crms.CrmsCustomerRelated;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 关联人/关联企业 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-13
 */
@Api(tags = {"关联人/关联企业"})
@RestController
@RequestMapping("/crms-company-personal")
public class CrmsCompanyPersonalController {
    @Resource
    private ICrmsCompanyPersonalService crmsCompanyPersonalService;

    @Resource
    private ICrmsCustomerRelatedService crmsCustomerRelatedService;

    /**
     * 个人/企业关联人列表
     */
    @Log(detail = "个人/企业关联人列表",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @GetMapping("/companyPersonalList")
    @ApiOperation(value = "个人/企业关联人列表")
    public ResponseResult companyPersonalList(@RequestParam("page") Integer page, @RequestParam("size") Integer size
            , @RequestParam(value="customerId",required = false) Integer customerId) {
        PageUtils data = crmsCompanyPersonalService.queryPage(page, size, customerId);
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 新增个人/企业关联人
     */
    @Log(detail = "新增个人/企业关联人",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.INSERT)
    @PostMapping("/insertCompanyPersonal")
    @ApiOperation(value = "新增个人/企业关联人")
    public ResponseResult insertCompanyPersonal(@RequestBody @Valid CrmsCompanyPersonalReq personal) {
        CrmsCompanyPersonal crmsCompanyPersonal = new CrmsCompanyPersonal();
        if(personal.getRegisteredTime() !=null){
            SimpleDateFormat sdf =new SimpleDateFormat("yyyy-MM-dd");
            Date parse = null;
            try {
                parse = sdf.parse(personal.getRegisteredTime());
            } catch (ParseException e) {
                e.printStackTrace();
            }
            BeanUtils.copyProperties(personal,crmsCompanyPersonal);
            crmsCompanyPersonal.setRegisteredTime(parse);
        }else{
            BeanUtils.copyProperties(personal,crmsCompanyPersonal);
        }

        boolean result = crmsCompanyPersonalService.save(crmsCompanyPersonal);
        CrmsCustomerRelated crmsCustomerRelated = new CrmsCustomerRelated();
        crmsCustomerRelated.setRelatedId(crmsCompanyPersonal.getId().intValue());//关联人Id
        crmsCustomerRelated.setCustomerId(crmsCompanyPersonal.getRelatedCompanyId());//企业id

        if(crmsCustomerRelated.getRelatedId()!=null && crmsCustomerRelated.getCustomerId()!=null){
            boolean bindRelated = crmsCustomerRelatedService.save(crmsCustomerRelated);
        }
        //获取返回列表
        CrmsCompanyPersonal companyPersonal = crmsCompanyPersonalService.selectById(crmsCompanyPersonal.getId());

        List<CrmsCompanyPersonal> companyPersonalList = new ArrayList();
        companyPersonalList.add(companyPersonal);
        return result ? ResponseResult.SUCCESS(companyPersonalList) : ResponseResult.FAIL();
    }

    /**
     * 编辑个人/企业关联人
     * @param personal
     * @return
     */
    @Log(detail = "编辑个人/企业关联人",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.UPDATE)
    @PostMapping("/updateCompanyPersonal")
    @ApiOperation(value = "编辑个人/企业关联人")
    public ResponseResult updateCompanyPersonal(@RequestBody CrmsCompanyPersonalReq personal){
        CrmsCompanyPersonal crmsCompanyPersonal = new CrmsCompanyPersonal();
        if(personal.getRegisteredTime() !=null){
            SimpleDateFormat sdf =new SimpleDateFormat("yyyy-MM-dd");
            Date parse = null;
            try {
                parse = sdf.parse(personal.getRegisteredTime());
            } catch (ParseException e) {
                e.printStackTrace();
            }
            BeanUtils.copyProperties(personal,crmsCompanyPersonal);
            crmsCompanyPersonal.setRegisteredTime(parse);
        }else{
            BeanUtils.copyProperties(personal,crmsCompanyPersonal);
        }
        boolean result = crmsCompanyPersonalService.updateById(crmsCompanyPersonal);
        return result ? ResponseResult.SUCCESS(crmsCompanyPersonal) : ResponseResult.FAIL();
    }

    /**
     * 删除个人/企业关联人
     */
    @Log(detail = "删除个人/企业关联人",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.DELETE)
    @PostMapping("/deleteCompanyPersonal")
    @ApiOperation(value = "删除个人/企业关联人")
    public ResponseResult deleteCompanyPersonal(@RequestBody Long[] ids) {
        boolean result = crmsCompanyPersonalService.removeByIds(Arrays.asList(ids));
        //同时删除关联关系
        boolean relate = crmsCustomerRelatedService.deleteByRelateIds(Arrays.asList(ids));
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 编辑是否存在关联人
     * @param
     * @return
     */
    @PostMapping("/updateIsExistRelated")
    @ApiOperation(value = "编辑是否存在关联人")
    public ResponseResult updateIsExistRelated(@RequestParam("id")Integer id,@RequestParam("isExistRelated")Integer isExistRelated){
        boolean result = crmsCompanyPersonalService.updateIsExistRelated(id,isExistRelated);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 查询个人/企业关联人
     */
    @Log(detail = "查询个人/企业关联人",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @GetMapping("/selectCompanyPersonal")
    @ApiOperation(value = "查询个人/企业关联人")
    public ResponseResult selectCompanyPersonal(@RequestParam("id")Integer id) {
        CrmsCompanyPersonal crmsCompanyPersonal = crmsCompanyPersonalService.getOne(new QueryWrapper<CrmsCompanyPersonal>().eq("id",id));

        return ResponseResult.SUCCESS(crmsCompanyPersonal);
    }
}
