package com.daqing.financial.crms.controller;

import com.daqing.financial.crms.model.request.CompanyAppletRequest;
import com.daqing.financial.crms.model.request.CompanyCustomerRequest;
import com.daqing.financial.crms.model.request.PersonalAppletRequest;
import com.daqing.financial.crms.model.response.ApplicationRecordResponse;
import com.daqing.financial.crms.model.response.ApplyDetailResponse;
import com.daqing.financial.crms.service.CustomerAppletService;
import com.daqing.framework.domain.crms.ext.CustomerPersonalTOI;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 小程序客户模块数据录入(认证)
 *
 * @auther River
 * @date 2021/1/28 14:08
 */
@RestController
@RequestMapping("/customer/applet")
@Api(value = "客户数据录入(认证)", tags = "小程序客户数据录入(认证)")
public class CustomerAppletController {

    @Autowired
    private CustomerAppletService customerAppletService;

    /**
     * 个人类型客户数据录入(认证)
     */
    @PostMapping("/save/personal")
    @ApiOperation(value = "个人类型客户数据录入(认证)")
    public ResponseResult savePersonal(@RequestBody @Valid PersonalAppletRequest personalAppletRequest) {

        Boolean result = customerAppletService.savePersonal(personalAppletRequest);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 企业类型客户数据录入(认证)
     */
    @PostMapping("/save/company")
    @ApiOperation(value = "企业类型客户数据录入(认证)")
    public ResponseResult saveCompany(@RequestBody @Valid CompanyAppletRequest companyAppletRequest) {

        Boolean result = customerAppletService.saveCompany(companyAppletRequest);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 获取当前客户认证(录入)状态
     */
    @GetMapping("/get/status")
    @ApiOperation(value = "获取当前客户认证(录入)状态")
    public ResponseResult getCustomerStatus() {

        return ResponseResult.SUCCESS(customerAppletService.getCustomerStatus());
    }

    /**
     * 获取已认证(录入)的客户信息
     */
    @GetMapping("/get/customer")
    @ApiOperation(value = "获取已认证(录入)的客户信息")
    public ResponseResult getCustomer(@RequestParam("type") Integer type) {

        return ResponseResult.SUCCESS(customerAppletService.getCustomer(type));
    }

    /**
     * 更新个人类型客户信息
     */
    @ApiOperation(value = "更新个人类型的客户信息")
    @PostMapping("/update/personal")
    public ResponseResult updatePersonal(@RequestBody @Valid PersonalAppletRequest personalAppletRequest) {

        Boolean result = customerAppletService.updatePersonal(personalAppletRequest);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 更新企业类型的客户信息
     */
    @PostMapping("/update/company")
    @ApiOperation(value = "更新企业类型的客户信息")
    public ResponseResult updateCompany(@RequestBody @Valid CompanyAppletRequest companyAppletRequest) {

        Boolean result = customerAppletService.updateCompany(companyAppletRequest);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 申请记录
     */
    @GetMapping("/get/record")
    @ApiOperation(value = "获取申请记录", response = ApplicationRecordResponse.class)
    public ResponseResult getApplicationRecord(@RequestParam("page") Integer page,
                                               @RequestParam("size") Integer size, Integer type, Integer status) {

        return ResponseResult.SUCCESS(customerAppletService.getApplicationRecord(page, size, type, status));
    }

    /**
     * 获取申请额度
     */
    @GetMapping("/get/apply/mount")
    @ApiOperation(value = "获取已申请额度")
    public ResponseResult getApplyMount() {

        return ResponseResult.SUCCESS(customerAppletService.getApplyMount());
    }

    /**
     * 获取申请记录详情
     */
    @GetMapping("/get/apply/detail")
    @ApiOperation(value = "获取申请记录详情", response = ApplyDetailResponse.class)
    public ResponseResult getApplyDetail(@RequestParam("id") Integer id) {

        return ResponseResult.SUCCESS(customerAppletService.getApplyDetail(id));
    }
}
