package com.daqing.financial.crms.controller;

import com.daqing.financial.crms.CustomerControllerApi;
import com.daqing.financial.crms.model.request.CompanyCustomerRequest;
import com.daqing.financial.crms.service.CustomerService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.crms.CustomerEntity;
import com.daqing.framework.domain.crms.ext.CustomerPersonalTOI;
import com.daqing.framework.domain.crms.ext.CustomerPersonalTOU;
import com.daqing.framework.domain.crms.request.CustomerRequest;
import com.daqing.framework.domain.crms.request.DgApplyAmountInfoRequest;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.*;


/**
 * 记录客户基本信息
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-08 09:57:32
 */
@RestController
@Api(tags = {"记录客户基本信息"})
@RequestMapping("crms/customer")
public class CustomerController implements CustomerControllerApi {

    @Autowired
    private CustomerService customerService;

    /**
     * 列表
     */
    /*@GetMapping("/hrms_list")*/
    //@RequiresPermissions("hrms:dept:list")
   /* public ResponseResult hrmsList() {
        Map<String, Object> params = new HashMap<>();
        return hrmsFeignService.list(params);
    }*/

    /**
     * 查询客户列表(所有)、根据创建时间筛选、根据客户类型筛选、根据客户编号或者名称搜索
     */
    @Log(detail = "查询客户列表",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @GetMapping("/queryList")
    //@PreAuthorize("hasPermission('/crms/customer/queryList','222')")
    public ResponseResult queryList(@RequestParam(value = "page", required = false) Integer page, @RequestParam(value = "size", required = false) Integer size, CustomerRequest customerRequest) {

        return new ResponseResult<PageUtils>().SUCCESS(customerService.queryList(page, size, customerRequest));
    }

    /**
     * 根据客户id查询客户的详细信息
     */
    @Log(detail = "查询客户详细",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @GetMapping("/queryDetails")
    public ResponseResult queryDetails(@RequestParam("id") Long id){

        return customerService.queryCustomerById(id);
    }

    /**
     * 根据客户id删除所选客户
     */
    @Log(detail = "删除客户",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.DELETE)
    @DeleteMapping("/deleteCustomer")
    public ResponseResult deleteCustomer(@RequestBody Long[] ids){
        List<String> list = customerService.updateCustomerById(ids);
        if (list == null){
            return new ResponseResult(CommonCode.INVALID_PARAM);
        }else if (Objects.equals(list.get(0),PromptSuccess.DELETE_SUCCESS)){
            return ResponseResult.SUCCESS();
        }else if (Objects.equals(list.get(0),PromptSuccess.DELETE_FAIL)){
            return new ResponseResult(CommonCode.DELETE_FAIL);
        }
        else {
            return new ResponseResult<List<String>>().SUCCESS_DATA(list);
        }
    }

    /**
     * 保存个人类型客户信息
     */
    @Log(detail = "保存个人类型客户信息",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.INSERT)
    @PostMapping("/savePersonal")
    public ResponseResult savePersonal(@RequestBody @Valid CustomerPersonalTOI customerPersonalTOI) {
        boolean result = customerService.saveCustomerPersonal(customerPersonalTOI);
        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.SAVE_FAIL);
    }

    /**
     * 保存企业类型客户信息
     */
    @Log(detail = "保存企业类型客户信息",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.INSERT)
    @PostMapping("/saveCompany")
    public ResponseResult saveCompany(@RequestBody @Valid CompanyCustomerRequest companyCustomerRequest) {
        boolean result = customerService.insertCompany(companyCustomerRequest);
        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.SAVE_FAIL);
    }

    /**
     * 更新个人类型客户信息
     */
    @Log(detail = "更新个人类型客户信息",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.UPDATE)
    @PostMapping("/updatePersonal")
    public ResponseResult updatePersonal(@RequestBody @Valid CustomerPersonalTOU customerPersonalTOU) {
        boolean result = customerService.updateCustomerPersonal(customerPersonalTOU);
        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }

    /**
     * 更新企业类型客户信息
     */
    @ApiOperation(value = "更新企业类型客户信息", notes = "更新企业类型客户信息")
    @Log(detail = "更新企业类型客户信息",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.UPDATE)
    @PostMapping("/updateCompany")
    public ResponseResult updateCompany(@RequestBody @Valid CompanyCustomerRequest companyCustomerRequest) {
        boolean result = customerService.updateCompanyNew(companyCustomerRequest);
        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }

    /**
     * 导出数据到excel
     */
    @Log(detail = "导出数据",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    public void excelExportCustomer(HttpServletResponse response){

        customerService.excelExportCustomer(response);
    }

    /**
     * 导出excel模板
     */
    @Log(detail = "导出excel模板",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @GetMapping("/excelTemplate")
    public void excelTemplate(HttpServletResponse response){

        customerService.excelTemplate(response);
    }

    /**
     * 导入excel数据
     */
    @Log(detail = "导入excel数据",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.INSERT)
    @PostMapping("/excelImport")
    public ResponseResult excelImportCustomer(MultipartFile file){

        return customerService.excelImportCustomer(file);
    }

    /**
     * 根据客户编号/客户名称获取企业信息
     */
    @Log(detail = "获取企业信息",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @ApiOperation(value = "根据客户编号/客户名称获取企业信息")
    @PostMapping("/queryCustomerInfoById")
    public ResponseResult queryCustomerInfoById(@RequestBody DgApplyAmountInfoRequest applyAmountInfo){

        return customerService.queryCustomerInfoById(applyAmountInfo);
    }

    /**
     * 查询所有客户编号/客户名称
     */
    @GetMapping("/queryCompanyCodeAndName")
    public ResponseResult queryCompanyCodeAndName(int type){

        List<CustomerEntity>list = customerService.queryCompanyCodeAndName(type);
        return  ResponseResult.SUCCESS(list);
    }
}
