package com.daqing.financial.crms.controller;

import com.daqing.financial.crms.CustomerWorkbenchControllerApi;
import com.daqing.financial.crms.model.request.CompanyCustomerRequest;
import com.daqing.financial.crms.service.CustomerWorkbenchService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.crms.ext.CustomerPersonalTOI;
import com.daqing.framework.domain.crms.ext.CustomerPersonalTOU;
import com.daqing.framework.domain.crms.request.CustomerWorkbenchRequest;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * 工作台客户资源系统
 *
 * @auther River
 * @date 2020/10/19 10:05
 */
@Api(tags = {"工作台客户资源系统"})
@RestController
@RequestMapping("/crms/workbench")
public class CustomerWorkbenchController implements CustomerWorkbenchControllerApi {

    @Autowired
    private CustomerWorkbenchService customerWorkbenchService;

    /**
     * 列表
     */
    @Log(detail = "工作台客户资源列表",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @Override
    @GetMapping("/list")
    public ResponseResult List(@RequestParam("page") Integer page, @RequestParam("size") Integer size, CustomerWorkbenchRequest customerWorkbenchRequest) {

        PageUtils list = customerWorkbenchService.list(page, size, customerWorkbenchRequest);

        return new ResponseResult<PageUtils>().SUCCESS(list);
    }

    /**
     * 详情
     */
    @Log(detail = "工作台客户资源详情",level = 4,operationUnit = OperationUnit.COMPANY,operationType = OperationType.SELECT)
    @GetMapping("/info")
    @Override
    public ResponseResult info(@RequestParam("id") Long id) {

        return customerWorkbenchService.info(id);
    }

    /**
     * 删除
     */
    @Log(detail = "删除工作台客户",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.DELETE)
    @Override
    @PostMapping("/delete")
    public ResponseResult delete(@RequestBody Long[] ids) {
        List<String> list = customerWorkbenchService.delete(ids);
        if (list == null){
            return new ResponseResult(CommonCode.INVALID_PARAM);
        }else if (Objects.equals(list.get(0), PromptSuccess.DELETE_SUCCESS)){
            return ResponseResult.SUCCESS();
        }else if (Objects.equals(list.get(0),PromptSuccess.DELETE_FAIL)){
            return new ResponseResult(CommonCode.DELETE_FAIL);
        }
        else {
            return new ResponseResult<List<String>>().SUCCESS_DATA(list);
        }
    }

    /**
     * 新增个人
     */
    @Log(detail = "新增个人",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.INSERT)
    @PostMapping("/savePersonal") // ..................................
    @Override
    public ResponseResult savePersonal(@RequestBody CustomerPersonalTOI customerPersonalTOI) {

        boolean result = customerWorkbenchService.savePersonal(customerPersonalTOI);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.SAVE_FAIL);
    }

    /**
     * 新增企业
     */
    /*@PostMapping("/saveCompany")
    @Override
    public ResponseResult saveCompany(@RequestBody CustomerCompanyTOI customerCompanyTOI) {

        boolean result = customerWorkbenchService.saveCompany(customerCompanyTOI);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.SAVE_FAIL);
    }*/

    /**
     * 修改个人
     */
    @Log(detail = "修改个人",level = 3,operationUnit = OperationUnit.COMPANY,operationType = OperationType.UPDATE)
    @PostMapping("/updatePersonal") // ..............................
    @Override
    public ResponseResult updatePersonal(@RequestBody CustomerPersonalTOU customerPersonalTOU) {

        Boolean result = customerWorkbenchService.updatePersonal(customerPersonalTOU);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }

    /**
     * 修改企业
     */
    /*@PostMapping("/updateCompany")
    @Override
    public ResponseResult updateCompany(@RequestBody CustomerCompanyTOU customerCompanyTOU) {

        Boolean result = customerWorkbenchService.updateCompany(customerCompanyTOU);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }*/

    /**
     * 通过审核
     */
    @GetMapping("/pass")
    @Override
    public ResponseResult pass(@RequestParam("id") Long id, @RequestParam("manager") Long manager) {

        Boolean result = customerWorkbenchService.pass(id, manager);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 不通过审核
     */
    @GetMapping("/noPass")
    @Override
    public ResponseResult noPass(@RequestParam("id") Long id, @RequestParam("cause") String cause) {

        Boolean result = customerWorkbenchService.noPass(id, cause);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 获取当前客户部门经理的部门下所有客户经理
     */
    @GetMapping("/manager")
    @Override
    public ResponseResult getCustomerManager() {

        return new ResponseResult<List<EmployeeEntity>>().SUCCESS(customerWorkbenchService.getCustomerManager());
    }

    /**
     * 新增企业(新)
     */
    @ApiOperation(value = "新增企业(新)") // .............................
    @PostMapping("/insertCompany")
    public ResponseResult insertCompany(@RequestBody CompanyCustomerRequest companyCustomerReq) {

        boolean result = customerWorkbenchService.insertCompany(companyCustomerReq);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.SAVE_FAIL);
    }

    /**
     * 修改企业(新)
     */
    @ApiOperation(value = "修改企业(新)")// ......................................
    @PostMapping("/updateCompanyNew")
    public ResponseResult updateCompanyNew(@RequestBody CompanyCustomerRequest companyCustomerReq) {

        Boolean result = customerWorkbenchService.updateCompanyNew(companyCustomerReq);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }
}
