package com.daqing.financial.crms.model.request;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 企业类型客户信息新增
 * @auther
 * @date 2020/10/9 10:32
 */
@Data
@ToString
public class CompanyCustomerRequest implements Serializable {

    @TableId(value = "id", type = IdType.INPUT)
    private Long id;

    @ApiModelProperty(value = "客户类型")
    private Integer type;

    @ApiModelProperty(value = "客户经理人id")
    private Long manager;

    @NotNull(message = "客户名称不能为空")
    @ApiModelProperty(value = "客户名称")
    private String name;

    @NotNull(message = "社会统一代码不能为空")
    @ApiModelProperty(value = "社会统一代码")
    private String socialUnifiedCode;

    @NotNull(message = "联系电话不能为空")
    @Pattern(regexp = "^[0-9]*$",message = "联系电话格式错误")
    @ApiModelProperty(value = "联系电话")
    private String phone;

    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @ApiModelProperty(value = "注册时间")
    private Date registerTime;

    @Pattern(regexp = "^[0-9]+\\.{0,1}[0-9]{0,2}$",message = "注册资金格式有误")
    @NotNull(message = "注册资金不能为空")
    @ApiModelProperty(value = "注册资金")
    private String registeredCapital;

    @Pattern(regexp = "^[0-9]*$",message = "员工人数格式错误")
    @NotNull(message = "员工人数不能为空")
    @ApiModelProperty(value = "员工人数")
    private String empNum;

    @ApiModelProperty(value = "注册地址")
    private String registerAddr;

    @NotNull(message = "联系人不能为空")
    @ApiModelProperty(value = "联系人")
    private String linkman;

    @ApiModelProperty(value = "联系人电话")
    private String linkPhone;

    @ApiModelProperty(value = "经营地址")
    private String businessAddr;

    @NotNull(message = "业务来源不能为空")
    @ApiModelProperty(value = "业务来源")
    private String businessSource;

    @ApiModelProperty(value = "经营范围")
    private String businessScope;

    @ApiModelProperty(value = "法人-姓名")
    private String legalName;

    @ApiModelProperty(value = "法人-性别 1、男，0、女")
    private Integer legalGender;

    @ApiModelProperty(value = "法人-身份证号")
    private String legalIdNumber;

    @ApiModelProperty(value = "法人-户口所在地")
    private String legalHukouAddr;

    @ApiModelProperty(value = "法人-联系电话")
    private String legalPhone;

    @ApiModelProperty(value = "法人-家庭住址")
    private String legalHomeAddr;

    @ApiModelProperty(value = "是否存在关联人 0->否；1->是")
    private Integer isExistRelated;

    @ApiModelProperty(value = "关联人id")
    private List<Integer> relatedId;
}
