package com.daqing.financial.crms.model.request;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;

/**
 * 小程序客户模块个人类型数据录入(认证)
 *
 * @auther River
 * @date 2021/1/29 11:04
 */
@Data
@ToString
public class PersonalAppletRequest implements Serializable {

    @ApiModelProperty("客户表id")
    private Long companyId;

    @ApiModelProperty(value = "客户编号")
    private String code;

    @NotNull(message = "客户名称不能为空")
    @ApiModelProperty(value = "客户名称")
    private String name;

    @NotNull(message = "联系地址不能为空")
    @ApiModelProperty(value = "联系地址")
    private String addr;

    @Pattern(regexp = "^[0-9]*$",message = "联系电话格式错误")
    @NotNull(message = "联系电话不能为空")
    @ApiModelProperty(value = "联系电话")
    private String phone;

    @NotNull(message = "身份证号不能为空")
    @ApiModelProperty(value = "身份证号")
    private String idCard;

    @NotNull(message = "年龄不能为空")
    @ApiModelProperty(value = "年龄")
    private Integer age;

    @Min(value = 0,message = "性别格式有误")
    @Max(value = 1,message = "性别格式有误")
    @NotNull(message = "性别不能为空")
    @ApiModelProperty(value = "性别,1、男，0、女")
    private Integer gender;

    @Min(value = 0,message = "婚姻状况格式有误")
    @Max(value = 3,message = "婚姻状况格式有误")
    @NotNull(message = "婚姻状况不能为空")
    @ApiModelProperty(value = "婚姻状况,0:未婚,1:已婚,2:离异,3:再婚")
    private Integer maritalStatus;

    @Min(value = 0,message = "学历格式有误")
    @Max(value = 4,message = "学历格式有误")
    @NotNull(message = "学历不能为空")
    @ApiModelProperty(value = "学历,0:本科,1:大专,2:高职,3:中专,4:其他")
    private Integer education;

    @NotNull(message = "工作单位不能为空")
    @ApiModelProperty(value = "工作单位")
    private String employer;

    @NotNull(message = "职务不能为空")
    @ApiModelProperty(value = "职务")
    private String position;

    @NotNull(message = "工作年限不能为空")
    @ApiModelProperty(value = "工作年限")
    private Integer workingYears;

    @NotNull(message = "社保账号不能为空")
    @ApiModelProperty(value = "社保账号")
    private String socialSecurityNum;

    @NotNull(message = "居住情况不能为空")
    @ApiModelProperty(value = "居住情况")
    private String livingSituation;

    @NotNull(message = "户籍地址不能为空")
    @ApiModelProperty(value = "户籍地址")
    private String residenceAddr;

    @NotNull(message = "业务来源不能为空")
    @ApiModelProperty(value = "业务来源")
    private String businessSource;

    @NotNull(message = "紧急联系人不能为空")
    @ApiModelProperty(value = "紧急联系人")
    private String emergencyLinkman;

    @NotNull(message = "紧急联系人关系不能为空")
    @ApiModelProperty(value = "紧急联系人关系")
    private String emergencyLinkmanRelationship;

    @Pattern(regexp = "^[0-9]*$",message = "紧急联系人电话格式错误")
    @NotNull(message = "紧急联系人电话不能为空")
    @ApiModelProperty(value = "紧急联系人电话")
    private String emergencyLinkmanPhone;
}
