package com.daqing.financial.crms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.crms.dao.CompanyCustomerDao;
import com.daqing.financial.crms.dao.CustomerAppletDao;
import com.daqing.financial.crms.dao.CustomerDao;
import com.daqing.financial.crms.dao.PersonalCustomerDao;
import com.daqing.financial.crms.mapper.CrmsCustomerRelatedMapper;
import com.daqing.financial.crms.model.request.CompanyAppletRequest;
import com.daqing.financial.crms.model.request.PersonalAppletRequest;
import com.daqing.financial.crms.model.response.ApplicationRecordResponse;
import com.daqing.financial.crms.model.response.ApplyDetailResponse;
import com.daqing.financial.crms.service.CustomerAppletService;
import com.daqing.financial.crms.service.ICrmsCustomerRelatedService;
import com.daqing.framework.domain.crms.CompanyCustomerEntity;
import com.daqing.framework.domain.crms.CrmsCustomerRelated;
import com.daqing.framework.domain.crms.CustomerEntity;
import com.daqing.framework.domain.crms.PersonalCustomerEntity;
import com.daqing.framework.domain.crms.ext.ApplicationRecordPO;
import com.daqing.framework.domain.crms.ext.ApplyDetailPO;
import com.daqing.framework.domain.crms.ext.CrmsConstant;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;

/**
 * 客户小程序认证(录入)
 *
 * @auther River
 * @date 2021/1/27 15:53
 */
@Service
public class CustomerAppletServiceImpl extends ServiceImpl<CustomerAppletDao, CustomerEntity> implements CustomerAppletService {


    @Autowired
    private CustomerDao customerDao;

    @Autowired
    private PersonalCustomerDao personalCustomerDao;

    @Autowired
    private CompanyCustomerDao companyCustomerDao;

    @Autowired
    private ICrmsCustomerRelatedService crmsCustomerRelatedService;

    @Resource
    private CrmsCustomerRelatedMapper crmsCustomerRelatedMapper;

    /**
     * 录入(认证)个人类型客户数据
     */
    @Transactional
    @Override
    public Boolean savePersonal(PersonalAppletRequest personalAppletRequest) {
        if (this.getBaseMapper().getCustomerStatus(Integer.parseInt(this.getUserId()), 0) > 0) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_EXIST);
        }
        if (personalAppletRequest.getName() != null && personalAppletRequest.getName().length() != 0) {
            // 客户姓名判重
            List<String> names = customerDao.listCustomerName(0); // 个人类型客户名称
            for (String name : names) {
                if (personalAppletRequest.getName().equals(name)) {
                    ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        PersonalCustomerEntity personalCustomerEntity = new PersonalCustomerEntity();
        BeanUtils.copyProperties(personalAppletRequest, customerEntity);
        BeanUtils.copyProperties(personalAppletRequest, personalCustomerEntity);
        // 设置客户编号
        String code = customerDao.getCodeByType(CrmsConstant.CustomerType.PERSONAL_CUSTOMER.getType());
        if (code == null || code.length() == 0) {
            code = PromptSuccess.PERSONAL_START_CODE;
        }
        int codeNumber = Integer.parseInt(code.substring(code.length() - 4));
        customerEntity.setCode(PromptSuccess.PERSONAL_CODE + String.format("%04d", (codeNumber + 1)));
        customerEntity.setCreateTime(new Date());
        customerEntity.setMotifyTime(new Date());
        customerEntity.setType(CrmsConstant.CustomerType.PERSONAL_CUSTOMER.getType());
        boolean customer = customerDao.saveCustomer(customerEntity);
        // 将自增的客户id返回给工作台
        personalCustomerEntity.setCustomerId(customerEntity.getId());
        boolean personal = personalCustomerDao.savePersonalCustomer(personalCustomerEntity);
        Boolean user = this.getBaseMapper().saveCustomerIdAndUserId(customerEntity.getId().intValue(), Integer.parseInt(this.getUserId()), 0);
        return customer && personal && user;
    }

    /**
     * 录入(认证)企业类型的客户数据
     */
    @Transactional
    @Override
    public Boolean saveCompany(CompanyAppletRequest companyAppletRequest) {
        if (this.getBaseMapper().getCustomerStatus(Integer.parseInt(this.getUserId()), 1) > 0) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_EXIST);
        }
        // 客户名称判重
        if (companyAppletRequest.getName() != null && companyAppletRequest.getName().length() != 0) {
            List<String> names = customerDao.listCustomerName(1); // 企业类型客户名称
            for (String name : names) {
                if (companyAppletRequest.getName().equals(name)) {
                    ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        CompanyCustomerEntity companyCustomerEntity = new CompanyCustomerEntity();
        BeanUtils.copyProperties(companyAppletRequest, customerEntity);
        BeanUtils.copyProperties(companyAppletRequest, companyCustomerEntity);
        // 设置客户编号
        String code = customerDao.getCodeByType(CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType());
        if (code == null || code.length() == 0) {
            code = PromptSuccess.COMPANY_START_CODE;
        }
        int codeNumber = Integer.parseInt(code.substring(code.length() - 4));
        customerEntity.setCode(PromptSuccess.COMPANY_CODE + String.format("%04d", (codeNumber + 1)));
        customerEntity.setMotifyTime(new Date());
        customerEntity.setCreateTime(new Date());
        customerEntity.setType(CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType());
        boolean customer = customerDao.saveCustomer(customerEntity);
        companyCustomerEntity.setCustomerId(customerEntity.getId());
        int company = companyCustomerDao.insert(companyCustomerEntity);
        // 绑定客户信息和客户的用户信息
        Boolean user = this.getBaseMapper().saveCustomerIdAndUserId(customerEntity.getId().intValue(), Integer.parseInt(this.getUserId()), 1);
        if (companyAppletRequest.getIsExistRelated() == 1) {//有关联人的情况下才绑定关联关系
            //绑定关联关系
            List<CrmsCustomerRelated> relatedList = new ArrayList<>();
            List<Integer> intList = companyAppletRequest.getRelatedId();

            for (int i = 0; i < intList.size(); i++) {
                CrmsCustomerRelated crmsCustomerRelated = new CrmsCustomerRelated();
                Integer integer = intList.get(i);
                crmsCustomerRelated.setCustomerId(companyCustomerEntity.getId().intValue());//企业id
                crmsCustomerRelated.setRelatedId(integer);//关联人/企业id
                relatedList.add(crmsCustomerRelated);
            }
            crmsCustomerRelatedService.insertCustomerRelated(relatedList);
        }
        return customer && company > 0 && user;
    }

    /**
     * 获取当前客户的认证状态
     *
     * @return
     */
    @Transactional
    @Override
    public Map getCustomerStatus() {
        Map<String, Boolean> map = new HashMap<>();
        Integer personalCount = this.getBaseMapper().getCustomerStatus(Integer.parseInt(this.getUserId()), 0);
        Integer companyCount = this.getBaseMapper().getCustomerStatus(Integer.parseInt(this.getUserId()), 1);
        if (personalCount > 0) {
            map.put("personal", true);
        } else {
            map.put("personal", false);
        }
        if (companyCount > 0) {
            map.put("company", true);
        } else {
            map.put("company", false);
        }
        return map;
    }

    /**
     * 查看认证的详情
     */
    @Override
    public Object getCustomer(Integer type) {
        String userId = this.getUserId();
        Integer customerId = this.getBaseMapper().getCustomerId(Integer.parseInt(userId), type);
        if (customerId == null) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_IS_NULL);
        }
        // 员工基本信息
        CustomerEntity customer = customerDao.queryCustomerById((long) customerId);
        // 判断该客户的类型
        if (type == 0) {
            PersonalAppletRequest personalAppletRequest = new PersonalAppletRequest();
            PersonalCustomerEntity personalCustomerEntity = personalCustomerDao.queryPersonalCustomerById((long) customerId);
            BeanUtils.copyProperties(customer, personalAppletRequest);
            BeanUtils.copyProperties(personalCustomerEntity, personalAppletRequest);
            personalAppletRequest.setCompanyId(personalCustomerEntity.getId());
            return personalAppletRequest;
        }
        if (type == 1) {
            CompanyAppletRequest companyAppletRequest = new CompanyAppletRequest();
            CompanyCustomerEntity companyCustomerEntity = companyCustomerDao.selectOne(new QueryWrapper<CompanyCustomerEntity>()
                    .eq("customer_id", (long) customerId));
            BeanUtils.copyProperties(customer, companyAppletRequest);
            BeanUtils.copyProperties(companyCustomerEntity, companyAppletRequest);
            /*if (companyAppletRequest.getIsExistRelated() == 1) {
                Integer companyId = this.getBaseMapper().getCompanyId(customerId);
                if (companyId != null) {
                    companyAppletRequest.setCustomerId(companyId);
                }
            }*/
            companyAppletRequest.setCustomerId(this.getBaseMapper().getCompanyId(customerId));
            return companyAppletRequest;
        }
        return null;
    }

    /**
     * 修改个人类型的认证信息
     */
    @Transactional
    @Override
    public Boolean updatePersonal(PersonalAppletRequest personalAppletRequest) {
        Integer customerId = this.getBaseMapper().getCustomerId(Integer.parseInt(this.getUserId()), 0);
        if (customerId == null) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_IS_NULL);
        }
        // 客户名称判重,注意判断是否与原来相同
        if (personalAppletRequest.getName() != null && personalAppletRequest.getName().length() != 0) {
            String customerName = customerDao.getNameByCustomerId((long) customerId);
            if (!personalAppletRequest.getName().equals(customerName)) {
                List<String> names = customerDao.listCustomerName(0);
                for (String name : names) {
                    if (personalAppletRequest.getName().equals(name)) {
                        ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                    }
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        PersonalCustomerEntity personalCustomerEntity = new PersonalCustomerEntity();
        BeanUtils.copyProperties(personalAppletRequest, customerEntity);
        BeanUtils.copyProperties(personalAppletRequest, personalCustomerEntity);
        customerEntity.setId((long) customerId);
        customerEntity.setMotifyTime(new Date());
        boolean customer = this.getBaseMapper().updateCustomer(customerEntity);
        personalCustomerEntity.setCustomerId(customerEntity.getId());
        boolean personal = personalCustomerDao.updatePersonalCustomer(personalCustomerEntity);
        return customer && personal;
    }

    /**
     * 修改企业类型的认证信息
     */
    @Transactional
    @Override
    public Boolean updateCompany(CompanyAppletRequest companyAppletRequest) {
        Integer customerId = this.getBaseMapper().getCustomerId(Integer.parseInt(this.getUserId()), 1);
        if (customerId == null) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_IS_NULL);
        }
        // 客户名称判重,注意判断是否与原来相同
        if (companyAppletRequest.getName() != null && companyAppletRequest.getName().length() != 0) {
            String customerName = customerDao.getNameByCustomerId((long) customerId);
            if (!companyAppletRequest.getName().equals(customerName)) {
                List<String> names = customerDao.listCustomerName(1);
                for (String name : names) {
                    if (companyAppletRequest.getName().equals(name)) {
                        ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                    }
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        CompanyCustomerEntity companyCustomerEntity = new CompanyCustomerEntity();
        BeanUtils.copyProperties(companyAppletRequest, customerEntity);
        BeanUtils.copyProperties(companyAppletRequest, companyCustomerEntity);
        customerEntity.setId((long) customerId);
        customerEntity.setMotifyTime(new Date());
        boolean customer = this.getBaseMapper().updateCustomer(customerEntity);
        companyCustomerEntity.setCustomerId(customerEntity.getId());
        int company = companyCustomerDao.update(companyCustomerEntity, new QueryWrapper<CompanyCustomerEntity>()
                .eq("customer_id", companyCustomerEntity.getCustomerId()));
        // 添加关联人
        Long companyId = this.getBaseMapper().getCompanyIdByCustomerId((long)customerId);
        if (null != companyId) {
            companyCustomerEntity.setId(companyId);
            //根据企业id删除关联关系
            crmsCustomerRelatedMapper.delete(new QueryWrapper<CrmsCustomerRelated>().eq("customer_id", companyCustomerEntity.getId()));

            //绑定关联关系
            List<CrmsCustomerRelated> relatedList = new ArrayList<>();
            List<Integer> intList = companyAppletRequest.getRelatedId();

            for (int i = 0; i < intList.size(); i++) {
                CrmsCustomerRelated crmsCustomerRelated = new CrmsCustomerRelated();
                Integer integer = intList.get(i);
                crmsCustomerRelated.setCustomerId(companyCustomerEntity.getId().intValue());//企业id
                crmsCustomerRelated.setRelatedId(integer);//关联人/企业id
                relatedList.add(crmsCustomerRelated);
            }
            if (relatedList.size() > 0) {
                crmsCustomerRelatedService.insertCustomerRelated(relatedList);
            }
        }
        return customer && company > 0;
    }

    /**
     * 申请记录
     */
    @Transactional
    @Override
    public PageUtils getApplicationRecord(Integer page, Integer size, Integer type, Integer status) {
        if (page == null || size == null || page < 1 || size < 1) {
            page = 1;
            size = 10;
        }
        List<ApplicationRecordResponse> applicationRecordResponseList = new ArrayList<>();
        IPage<ApplicationRecordResponse> applicationRecordResponseIPage = new Page<>();
        // 获取客户的登录信息/暂时没有个人类型申请信息,所以个人类型返回空而企业类型返回所有
        if (type == null || (type != 0 && type != 1) || type == 1) {
            Integer customerId = this.getBaseMapper().getCustomerId(Integer.parseInt(this.getUserId()), 1);
            if (customerId == null) {
                ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_IS_NULL);
            }
            Integer companyId = this.getBaseMapper().getCompanyId(customerId);
            // 查询申请记录信息
            IPage<ApplicationRecordPO> iPage = this.getBaseMapper().getApplicationRecord(new Page(page, size), companyId, status);
            List<ApplicationRecordPO> applicationRecordPOList = iPage.getRecords();
            for (ApplicationRecordPO applicationRecordPO : applicationRecordPOList) {
                ApplicationRecordResponse applicationRecordResponse = new ApplicationRecordResponse();
                //status:0->已完成；1->已受理；2->审核中；3->已拒绝；4->已驳回；5->已撤销；
                // 状态为已受理、审核中、拒绝、驳回、已撤销情况
                if (applicationRecordPO.getAuditStatus() != null && (applicationRecordPO.getAuditStatus() == 1
                        || applicationRecordPO.getAuditStatus() == 2 || applicationRecordPO.getAuditStatus() == 3
                        || applicationRecordPO.getAuditStatus() == 4 || applicationRecordPO.getAuditStatus() == 5)) {
                    BeanUtils.copyProperties(applicationRecordPO, applicationRecordResponse);
                    applicationRecordResponse.setStatus(applicationRecordPO.getAuditStatus());
                    applicationRecordResponse.setType(1);
                }

                // 业务状态为已否决的情况,如果同时存在会覆盖以上情况
                if (applicationRecordPO.getBusinessStatus() != null && applicationRecordPO.getBusinessStatus() == 3) {
                    BeanUtils.copyProperties(applicationRecordPO, applicationRecordResponse);
                    applicationRecordResponse.setStatus(3);//状态设置为已拒绝
                    applicationRecordResponse.setType(1);
                }
                // 状态为已完成的情况,如果同时存在已完成会覆盖以上情况
                if (applicationRecordPO.getBusinessStatus() != null && applicationRecordPO.getBusinessStatus() == 2) {
                    BeanUtils.copyProperties(applicationRecordPO, applicationRecordResponse);
                    applicationRecordResponse.setStatus(0);//状态设置为已完成
                    applicationRecordResponse.setType(1);
                }
                applicationRecordResponseList.add(applicationRecordResponse);
            }
            // 设置分页数据
            BeanUtils.copyProperties(iPage, applicationRecordResponseIPage);
            applicationRecordResponseIPage.setRecords(applicationRecordResponseList);
        }
        return new PageUtils(applicationRecordResponseIPage);
    }

    /**
     * 获取申请额度
     */
    @Override
    public Map getApplyMount() {
        // 这个是否填小数没有影响,主要要是看赋予或计算的值是否包含小数
        BigDecimal company = new BigDecimal(0.00);
        Map<String, BigDecimal> map = new HashMap<>();
        map.put("company", company);
        // 个人类型暂时没有
        map.put("personal", new BigDecimal(0.00));
        // 是否与业务正在申请中
        map.put("underway", new BigDecimal(0));
        Integer customerId = this.getBaseMapper().getCustomerId(Integer.parseInt(this.getUserId()), 1);
        // 如果查询不到认证信息,直接返回0.00
        if (customerId == null) {
            map.put("company", company);
        } else {
            Integer companyId = this.getBaseMapper().getCompanyId(customerId);
            // 正在进行贷款数量
            Integer count= this.getBaseMapper().businessUnderwayCount(companyId);
            if (count > 0) {
                map.put("underway", new BigDecimal(1));
            }
            // 查询申请记录信息
            List<ApplicationRecordPO> applicationRecordPOList = this.getBaseMapper().getApplyMount(companyId);
            if (applicationRecordPOList.size() < 1) {
                return map;
            }
            for (ApplicationRecordPO applicationRecordPO : applicationRecordPOList) {
                company = company.add(applicationRecordPO.getApplyAmount() == null ? new BigDecimal(0.00) : applicationRecordPO.getApplyAmount());
            }
            map.put("company", company);
        }
        return map;
    }

    /**
     * 获取申请记录详情
     */
    @Override
    public ApplyDetailResponse getApplyDetail(Integer id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        ApplyDetailResponse applyDetailResponse = new ApplyDetailResponse();
        ApplyDetailPO applyDetail = this.getBaseMapper().getApplyDetail(id);
        if (applyDetail == null) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_STATUS_IS_NULL);
        }
        // 判断当前申请信息的状态
        if (applyDetail.getBusinessStatus() != null && applyDetail.getBusinessStatus() == 2) {
            // 已完成
            applyDetailResponse.setStatus(0);
            return applyDetailResponse;
        }
        if (applyDetail.getAuditStatus() == null) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_STATUS_IS_NULL);
        }

        //status:0->已完成；1->已受理；2->审核中；3->已拒绝；4->已驳回；5->已撤销；
        // 状态为已受理、审核中、拒绝、驳回、已撤销情况
        if (applyDetail.getAuditStatus() != null && (applyDetail.getAuditStatus() == 1
                || applyDetail.getAuditStatus() == 2 || applyDetail.getAuditStatus() == 3
                || applyDetail.getAuditStatus() == 4 || applyDetail.getAuditStatus() == 5)) {
            applyDetailResponse.setStatus(applyDetail.getAuditStatus());
        }

        // 业务状态为已否决的情况,如果同时存在会覆盖以上情况
        if (applyDetail.getBusinessStatus() != null && applyDetail.getBusinessStatus() == 3) {
            applyDetailResponse.setStatus(3);//状态设置为已拒绝
        }
        // 状态为已完成的情况,如果同时存在已完成会覆盖以上情况
        if (applyDetail.getBusinessStatus() != null && applyDetail.getBusinessStatus() == 2) {
            applyDetailResponse.setStatus(0);//状态设置为已完成
        }

//        switch (applyDetail.getAuditStatus()) {
//            case 1: // 已受理
//                applyDetailResponse.setStatus(applyDetail.getAuditStatus());
//                break;
//            case 2: // 审核中
//                applyDetailResponse.setStatus(applyDetail.getAuditStatus());
//                break;
//            case 3: // 已拒绝
//                applyDetailResponse.setStatus(applyDetail.getAuditStatus());
//                applyDetailResponse.setRemark(applyDetail.getRemark());
//                break;
//            case 4: // 已驳回
//                applyDetailResponse.setStatus(applyDetail.getAuditStatus());
//                applyDetailResponse.setRemark(applyDetail.getRemark());
//                break;
//            case 5: // 已撤销
//                applyDetailResponse.setStatus(applyDetail.getAuditStatus());
//                break;
//            case 6: // 草稿,正常不会出现这个状态,避免报错设置为已受理
//                applyDetailResponse.setStatus(1);
//                break;
//            default:
//                ExceptionCast.cast(CrmsCode.CUSTOMER_APPLET_STATUS_IS_NULL);
//        }

        //根据业务id查询该业务中最后一个驳回的人的审核意见
        String auditOpinion = this.getBaseMapper().getAuditOpinion(applyDetail.getBusinessId());
        if(auditOpinion != null && auditOpinion != "")
        applyDetailResponse.setRemark(auditOpinion);
        return applyDetailResponse;
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        // String userId = "63";
        return userId;
    }

}
