package com.daqing.financial.crms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.crms.dao.CompanyCustomerDao;
import com.daqing.financial.crms.dao.CustomerDao;
import com.daqing.financial.crms.dao.PersonalCustomerDao;
import com.daqing.financial.crms.feign.HrmsFeignService;
import com.daqing.financial.crms.mapper.CrmsCustomerRelatedMapper;
import com.daqing.financial.crms.model.request.CompanyCustomerRequest;
import com.daqing.financial.crms.service.CustomerService;
import com.daqing.financial.crms.service.ICrmsCustomerRelatedService;
import com.daqing.framework.domain.crms.CompanyCustomerEntity;
import com.daqing.framework.domain.crms.CrmsCustomerRelated;
import com.daqing.framework.domain.crms.CustomerEntity;
import com.daqing.framework.domain.crms.PersonalCustomerEntity;
import com.daqing.framework.domain.crms.ext.*;
import com.daqing.framework.domain.crms.request.CustomerRequest;
import com.daqing.framework.domain.crms.request.DgApplyAmountInfoRequest;
import com.daqing.framework.domain.crms.response.CompanyCustomerResponse;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.crms.response.CustomerEntityResponse;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeTO;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.*;


@Service("customerService")
public class CustomerServiceImpl extends ServiceImpl<CustomerDao, CustomerEntity> implements CustomerService {

    @Autowired
    private CustomerDao customerDao;

    @Autowired
    private PersonalCustomerDao personalCustomerDao;

    @Autowired
    private CompanyCustomerDao companyCustomerDao;

    @Autowired
    private HrmsFeignService hrmsFeignService;
    @Resource
    private ICrmsCustomerRelatedService crmsCustomerRelatedService;

    @Resource
    private CrmsCustomerRelatedMapper crmsCustomerRelatedMapper;

    /**
     * 查询客户列表(所有)、根据创建时间筛选、根据客户类型筛选、根据客户编号或者名称搜索
     *
     * @param page            当前页
     * @param size            每页条数
     * @param customerRequest 请求参数
     * @return 所有客户以及关联信息并完成分页的集合
     */
    @Override
    public PageUtils queryList(Integer page, Integer size, CustomerRequest customerRequest) {

        // 时间参数校验
        this.timeCheckout(customerRequest.getCreateTime(), customerRequest.getStartTime(), customerRequest.getEndTime(), customerRequest);
        // 分页参数校验
        if (page == null || page < 1 || size == null || size < 1) {
            page = 1;
            size = 10;
        }
        Page<Object> pages = new Page<>(page, size);
        if(customerRequest.getEndTime()!=null && customerRequest.getEndTime()!="")
            customerRequest.setEndTime(customerRequest.getEndTime()+" 23:59:59");
        // 查询所有客户的基本信息
        IPage<CustomerEntityResponse> customerEntityIPage = customerDao.queryList(pages, customerRequest);
        // 获取所有的客户基本信息
        List<CustomerEntityResponse> customers = customerEntityIPage.getRecords();
        // 获取所有的客户经理id
        Long[] managerIds = new Long[customers.size()];
        for (int i = 0; i < customers.size(); i++) {
            if (customers.get(i).getManager() != null) {
                managerIds[i] = customers.get(i).getManager();
            } else {
                managerIds[i] = 0L;
            }
        }
        // 属性拷贝,将泛型为CustomerEntity类型的IPage的属性拷贝给泛型为CustomerTO类型的IPage,才能赋值给PageUtils
        IPage<CustomerTO> iPage = new Page<>();
        BeanUtils.copyProperties(customerEntityIPage, iPage);
        List<CustomerTO> listResponse = new ArrayList<>();
        // 获取所有的客户经理名称和部门名称
        ResponseResult<List<EmployeeTO>> responseResult = hrmsFeignService.getEmployeeAndDeptById(managerIds);
        List<EmployeeTO> employeeTO = responseResult.getData();
        if (employeeTO != null) {
            // 将客户信息和客户经理及部门信息拼装起来
            listResponse = this.jointCustomerEmployee(customers, employeeTO);
        } else {
            for (CustomerEntityResponse customer : customers) {
                CustomerTO customerTO = new CustomerTO();
                BeanUtils.copyProperties(customer, customerTO);
                listResponse.add(customerTO);
            }
        }
        iPage.setRecords(listResponse);
        return new PageUtils(iPage);
    }

    /**
     * 根据id查询客户的信息
     *
     * @param id 客户id
     * @return
     */
    @Override
    public ResponseResult queryCustomerById(Long id) {

        if (id == null) {
            return new ResponseResult(CommonCode.INVALID_PARAM);
        }
        // 员工基本信息
        CustomerEntity customer = customerDao.queryCustomerById(id);
        if (customer == null) {
            return new ResponseResult(CommonCode.INEXISTENCE);
        }
        // 判断该客户的类型
        if (Objects.equals(customer.getType(), CrmsConstant.CustomerType.PERSONAL_CUSTOMER.getType())) {
            PersonalCustomerEntity personalCustomerEntity = personalCustomerDao.queryPersonalCustomerById(id);
            // 封装个人类型客户信息
            CustomerPersonalDTO customerPersonalDTO = this.setPersonal(customer, personalCustomerEntity);
            return new ResponseResult<CustomerPersonalDTO>().SUCCESS(customerPersonalDTO);
        } else if (Objects.equals(customer.getType(), CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType())) {
            // CompanyCustomerEntity companyCustomerEntity = companyCustomerDao.queryCompanyCustomerById(id);
            CompanyCustomerEntity companyCustomerEntity = companyCustomerDao.selectOne(new QueryWrapper<CompanyCustomerEntity>()
                    .eq("customer_id", id));
            // 封装企业类型客户信息
            CustomerCompanyDTO customerCompanyDTO = this.setCompany(customer, companyCustomerEntity);
            return new ResponseResult<CustomerCompanyDTO>().SUCCESS(customerCompanyDTO);
        } else {
            return new ResponseResult(CommonCode.INEXISTENCE);
        }
    }

    /**
     * 根据id查询客户的信息(导入到excel中)
     */
    public ResponseResult getCustomerById(Long id) {

        if (id == null) {
            return new ResponseResult(CommonCode.INVALID_PARAM);
        }
        // 员工基本信息
        CustomerEntity customer = customerDao.queryCustomerById(id);
        if (customer == null) {
            return new ResponseResult(CommonCode.INEXISTENCE);
        }
        String name = PromptSuccess.NOT_BEING;
        if (customer.getManager() != null) {
            ResponseResult<EmployeeEntity> employee = hrmsFeignService.getEmployeeById(customer.getManager());
            if (employee.getData() != null) {
                // 客户经理姓名
                name = hrmsFeignService.getEmployeeById(customer.getManager()).getData().getName();
            }
        }
        // 判断该客户的类型
        if (Objects.equals(customer.getType(), CrmsConstant.CustomerType.PERSONAL_CUSTOMER.getType())) {
            PersonalCustomerEntity personalCustomerEntity = personalCustomerDao.queryPersonalCustomerById(id);
            // 封装个人类型客户信息
            CustomerPersonalVO customerPersonalVO = this.setPersonal(customer, name, personalCustomerEntity);
            return new ResponseResult<CustomerPersonalVO>().SUCCESS(customerPersonalVO);
        } else if (Objects.equals(customer.getType(), CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType())) {
            CompanyCustomerEntity companyCustomerEntity = companyCustomerDao.queryCompanyCustomerById(id);
            // 封装企业类型客户信息
            CustomerCompanyVO customerCompanyVO = this.setCompany(customer, name, companyCustomerEntity);
            return new ResponseResult<CustomerCompanyVO>().SUCCESS(customerCompanyVO);
        } else {
            return new ResponseResult(CommonCode.INEXISTENCE);
        }
    }

    /**
     * 根据id更新是否删除字段
     *
     * @param ids 客户id集合
     * @return
     */
    @Override
    public List<String> updateCustomerById(Long[] ids) {
        if (ids == null || ids.length == 0) {
            return null;
        }
        //TODO 远程调用审批模块判断所选客户是否在审批,返回正在审批的客户的id集合Long[] idArray
        List<Long> idArray = new ArrayList<>();// 假设为返回的正在审批的客户id集合
        if (idArray == null || idArray.size() == 0) {
            boolean customer = customerDao.updateCustomerById(Arrays.asList(ids));
            List<String> success = new ArrayList<>();
            success.add(PromptSuccess.DELETE_SUCCESS);
            List<String> fail = new ArrayList<>();
            fail.add(PromptSuccess.DELETE_FAIL);
            return customer ? success : fail;
        } else {
            return customerDao.queryCustomerNameById(idArray);
        }
    }

    /**
     * 插入个人类型客户信息
     */
    @Transactional
    @Override
    public boolean saveCustomerPersonal(CustomerPersonalTOI customerPersonalTOI) {
        if (customerPersonalTOI.getName() != null && customerPersonalTOI.getName().length() != 0) {
            // 客户姓名判重
            List<String> names = customerDao.listCustomerName(0); // 个人类型客户名称
            for (String name : names) {
                if (customerPersonalTOI.getName().equals(name)) {
                    ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        PersonalCustomerEntity personalCustomerEntity = new PersonalCustomerEntity();
        BeanUtils.copyProperties(customerPersonalTOI, customerEntity);
        BeanUtils.copyProperties(customerPersonalTOI, personalCustomerEntity);
        // 设置客户编号
        String code = customerDao.getCodeByType(CrmsConstant.CustomerType.PERSONAL_CUSTOMER.getType());
        if (code == null || code.length() == 0) {
            code = PromptSuccess.PERSONAL_START_CODE;
        }
        int codeNumber = Integer.parseInt(code.substring(code.length() - 4));
        customerEntity.setCode(PromptSuccess.PERSONAL_CODE + String.format("%04d", (codeNumber + 1)));
        customerEntity.setCreateTime(new Date());
        customerEntity.setMotifyTime(new Date());
        customerEntity.setType(CrmsConstant.CustomerType.PERSONAL_CUSTOMER.getType());
        boolean customer = customerDao.saveCustomer(customerEntity);
        personalCustomerEntity.setCustomerId(customerEntity.getId());
        // 将自增的客户id返回给工作台
        customerPersonalTOI.setManager(customerEntity.getId());
        boolean personal = personalCustomerDao.savePersonalCustomer(personalCustomerEntity);
        return customer && personal;
    }

    /**
     * 插入企业类型客户信息
     */
    /*@Transactional
    public boolean saveCustomerCompany(CustomerCompanyTOI customerCompanyTOI) {
        CustomerEntity customerEntity = new CustomerEntity();
        CompanyCustomerEntity companyCustomerEntity = new CompanyCustomerEntity();
        BeanUtils.copyProperties(customerCompanyTOI, customerEntity);
        BeanUtils.copyProperties(customerCompanyTOI, companyCustomerEntity);
        //TODO 设置客户编号
        String code = customerDao.getCodeByType(CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType());
        if (code == null || code.length() == 0) {
            code = PromptSuccess.COMPANY_START_CODE;
        }
        int codeNumber = Integer.parseInt(code.substring(code.length() - 4));
        customerEntity.setCode(PromptSuccess.COMPANY_CODE + String.format("%04d", (codeNumber + 1)));
        customerEntity.setCreateTime(new Date());
        customerEntity.setMotifyTime(new Date());
        customerEntity.setType(CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType());
        boolean customer = customerDao.saveCustomer(customerEntity);
        companyCustomerEntity.setCustomerId(customerEntity.getId());
        customerCompanyTOI.setManager(customerEntity.getId());
        boolean company = companyCustomerDao.saveCompanyCustomer(companyCustomerEntity);
        return customer && company;
    }
*/

    /**
     * 更新个人类型客户信息
     */
    @Transactional
    @Override
    public boolean updateCustomerPersonal(CustomerPersonalTOU customerPersonalTOU) {
        // 客户名称判重,注意判断是否与原来相同
        if (customerPersonalTOU.getName() != null && customerPersonalTOU.getName().length() != 0) {
            String customerName = customerDao.getNameByCustomerId(customerPersonalTOU.getId());
            if (!customerPersonalTOU.getName().equals(customerName)) {
                List<String> names = customerDao.listCustomerName(0);
                for (String name : names) {
                    if (customerPersonalTOU.getName().equals(name)) {
                        ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                    }
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        PersonalCustomerEntity personalCustomerEntity = new PersonalCustomerEntity();
        BeanUtils.copyProperties(customerPersonalTOU, customerEntity);
        BeanUtils.copyProperties(customerPersonalTOU, personalCustomerEntity);
        if (customerEntity.getId() == null) {
            return false;
        }
        customerEntity.setMotifyTime(new Date());
        boolean customer = customerDao.updateCustomer(customerEntity);
        personalCustomerEntity.setCustomerId(customerEntity.getId());
        boolean personal = personalCustomerDao.updatePersonalCustomer(personalCustomerEntity);
        return customer && personal;
    }

    /**
     * 更新企业类型客户信息
     */
    /*@Transactional
    public boolean updateCustomerCompany(CustomerCompanyTOU customerCompanyTOU) {
        CustomerEntity customerEntity = new CustomerEntity();
        CompanyCustomerEntity companyCustomerEntity = new CompanyCustomerEntity();
        BeanUtils.copyProperties(customerCompanyTOU, customerEntity);
        BeanUtils.copyProperties(customerCompanyTOU, companyCustomerEntity);
        customerEntity.setMotifyTime(new Date());
        boolean customer = customerDao.updateCustomer(customerEntity);
        companyCustomerEntity.setCustomerId(customerEntity.getId());
        boolean company = companyCustomerDao.updateCompanyCustomer(companyCustomerEntity);
        return customer && company;
    }*/

    /**
     * 导出数据到excel
     *
     * @param response 响应
     */
    @Transactional
    @Override
    public void excelExportCustomer(HttpServletResponse response) {
        //TODO 查询所有的客户详细信息,并按照类型封装,后续有待优化,消耗时间太长
        try {
            List<CustomerCompanyVO> companyList = new ArrayList<>();
            /*List<CustomerPersonalVO> personalList = new ArrayList<>();
            List<Long> longList = customerDao.listCustomerId();
            if (longList != null) {
                for (Long id : longList) {
                    ResponseResult responseResult = this.getCustomerById(id);
                    if ((responseResult.getData()).getClass() == CustomerCompanyVO.class) {
                        companyList.add((CustomerCompanyVO) responseResult.getData());
                    } else {
                        personalList.add((CustomerPersonalVO) responseResult.getData());
                    }
                }
            }
            ExcelUtil.writeExcelWithSheets(response, personalList, "客户资源信息一览表", "个人类型", new CustomerPersonalVO())
                    .write(companyList, "企业类型", new CustomerCompanyVO())
                    .finish();*/
            List<Long> longList = customerDao.listCompanyCustomerIds();
            if (null != longList) {
                for (Long id : longList) {
                    ResponseResult responseResult = this.getCustomerById(id);
                    if (null != responseResult) {
                        companyList.add((CustomerCompanyVO) responseResult.getData());
                    }
                }
            }
            ExcelUtil.writeExcelWithSheets(response, companyList, "客户资源信息一览表", "企业类型", new CustomerCompanyVO())
                    .finish();
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
        }
    }

    /**
     * 导出excel模板
     */
    @Override
    public void excelTemplate(HttpServletResponse response) {
        try {
            List<CompanyTemplate> companyTemplates = new ArrayList<>();
            /*List<PersonalTemplate> personalTemplates = new ArrayList<>();
            ExcelUtil.writeExcelWithSheets(response, personalTemplates, "客户资源信息表模板", "个人类型", new PersonalTemplate())
                    .write(companyTemplates, "企业类型", new CompanyTemplate())
                    .finish();*/
            ExcelUtil.writeExcelWithSheets(response, companyTemplates, "客户资源信息表模板", "企业类型", new CompanyTemplate())
                    .finish();
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORTTEMPLATE_EXSIT);
        }
    }

    /**
     * 导入excel数据
     *
     * @param excel
     */
    @Transactional
    @Override
    public ResponseResult excelImportCustomer(MultipartFile excel) {
        //TODO 导入excel数据到数据库,思路耗时太长,有待优化
        if (excel.isEmpty()) {
            ExceptionCast.cast(CrmsCode.NOT_NULL);
        }
        try {
            List<Object> companyList = ExcelUtil.readExcel(excel, new CompanyTemplate(), 1);
            /*List<Object> personalList = ExcelUtil.readExcel(excel, new PersonalTemplate(), 1);
            List<EmployeeVO> nameList = hrmsFeignService.listEmployeeName().getData();
            CustomerPersonalTOI customerPersonalTOI;
            ManagerName manager; // 客户名称
            // 个人类型
            for (Object personal : personalList) {
                customerPersonalTOI = new CustomerPersonalTOI();
                customerPersonalTOI.setType(0);
                BeanUtils.copyProperties(personal, customerPersonalTOI);
                manager = new ManagerName();
                BeanUtils.copyProperties(personal, manager);
                // 根据客户经理名称找到对应的员工id
                if (nameList != null && manager.getManager() != null) {
                    for (EmployeeVO employeeVO : nameList) {
                        if (Objects.equals(manager.getManager(), employeeVO.getName())) {
                            customerPersonalTOI.setManager(employeeVO.getId());
                        }
                    }
                    if (customerPersonalTOI.getManager() == null) {
                        return new ResponseResult<String>().FAIL("导入数据失败,员工" + manager.getManager() + "不存在");
                    }
                }
                this.saveCustomerPersonal(customerPersonalTOI);
            }*/
            // 企业类型
            for (Object company : companyList) {
                CompanyCustomerRequest companyCustomerRequest = new CompanyCustomerRequest();
                /*manager = new ManagerName();*/
                companyCustomerRequest.setType(1);
                companyCustomerRequest.setIsExistRelated(0);
                BeanUtils.copyProperties(company, companyCustomerRequest);
                /*BeanUtils.copyProperties(company, manager);
                // 根据客户经理名称找到对应的员工id
                if (nameList != null && manager.getManager() != null) {
                    for (EmployeeVO employeeVO : nameList) {
                        if (Objects.equals(manager.getManager(), employeeVO.getName())) {
                            companyCustomerRequest.setManager(employeeVO.getId());
                        }
                    }
                    if (companyCustomerRequest.getManager() == null) {
                        return new ResponseResult<String>().FAIL("导入数据失败,员工" + manager.getManager() + "不存在");
                    }
                }*/
                this.insertCompany(companyCustomerRequest);
                // this.saveCustomerCompany(customerCompanyTOI);
            }
            return ResponseResult.SUCCESS();
        } catch (Exception e) {
            return ResponseResult.FAIL("EXPORT FAIL");
        }
    }

    /**
     * 选择某段时间内客户信息的时间参数校验
     *
     * @param createTime 创建时间
     * @param startTime  起始时间
     * @param endTime    结束时间
     */
    private void timeCheckout(Integer createTime, String startTime, String endTime, CustomerRequest customerRequest) {
        // 没有选择自定义时间且选择了固定的"3个月内、6个月内、9个月内...."时间
        if ((startTime == null || startTime.length() == 0) && (endTime == null || endTime.length() == 0) && createTime != null) {
            if (createTime == 3 || createTime == 6 || createTime == 9 || createTime == 12) {
                String pastTime = this.pastTime(createTime);
                customerRequest.setStartTime(pastTime);
            } else {
                customerRequest.setStartTime(null);
            }
        }
        // 选择了自定义时间且选择了固定时间则固定时间无效
        if (createTime != null && ((startTime != null && startTime.length() != 0) || (endTime != null && endTime.length() != 0))) {
            customerRequest.setCreateTime(null);
        }
    }

    /**
     * 得到过去number月内的时间
     *
     * @param number
     * @return 字符串类型
     */
    private String pastTime(Integer number) {
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, -number);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:ss:mm");
        return sdf.format(calendar.getTime());
    }

    /**
     * 分页参数校验
     *
     * @param page 当前页
     * @param size 每页条数
     */
    /*private Page<Object> pageCheckout(Integer page, Integer size) {
        // 查询客户有效总条数
        Integer count = this.count(new QueryWrapper<CustomerEntity>().eq("del_or_not", CrmsConstant.NOT_DELETE));
        if (size == null || size < 1) {
            size = 10;
        }
        Integer maxPage = count % size == 0 ? count / size : count / size + 1;
        if (page == null || page < 1) {
            page = 1;
        }
        if (page > maxPage) {
            page = maxPage;
        }
        Page<Object> pages = new Page<>(page, size);
        return pages;
    }*/

    /**
     * 拼接客户及其对应的客户经理及部门信息
     *
     * @param customers  客户基本信息集合
     * @param employeeTO 员工及部门信息集合
     * @return
     */
    private List<CustomerTO> jointCustomerEmployee(List<CustomerEntityResponse> customers, List<EmployeeTO> employeeTO) {
        List<CustomerTO> customerTOS = new ArrayList<>();// 用于拼装客户信息和员工及部门信息的集合
        CustomerTO customerTO;
        // 将客户信息和客户经理及部门信息拼装起来
        for (CustomerEntityResponse customer : customers) {
            // 每一个客户对应一个不同的对象
            customerTO = new CustomerTO();
            for (EmployeeTO employee : employeeTO) {
                if (Objects.equals(customer.getManager(), employee.getId())) {
                    customerTO.setManager(employee.getEmpName());
                    customerTO.setDepartments(employee.getDeptNames());
                }
            }
            BeanUtils.copyProperties(customer, customerTO);
            customerTOS.add(customerTO);
        }
        return customerTOS;
    }


    /**
     * 封装个人类型的客户信息
     *
     * @param customer 客户基本信息
     * @param personal 个人类型信息
     * @return
     */
    private CustomerPersonalDTO setPersonal(CustomerEntity customer, PersonalCustomerEntity personal) {
        //BeanUtils能批量封装相同字段名称和类型的属性的值
        CustomerPersonalDTO personalDTO = new CustomerPersonalDTO();
        BeanUtils.copyProperties(customer, personalDTO);
        BeanUtils.copyProperties(personal, personalDTO);
        return personalDTO;
    }

    /**
     * 封装企业类型的客户信息
     *
     * @param customer 客户基本信息
     * @param company  企业类型信息
     * @return
     */
    private CustomerCompanyDTO setCompany(CustomerEntity customer, CompanyCustomerEntity company) {
        CustomerCompanyDTO companyDTO = new CustomerCompanyDTO();
        BeanUtils.copyProperties(customer, companyDTO);
        BeanUtils.copyProperties(company, companyDTO);
        return companyDTO;
    }

    /**
     * 封装企业类型的客户信息(excel)
     */
    private CustomerCompanyVO setCompany(CustomerEntity customer, String name, CompanyCustomerEntity company) {
        CustomerCompanyVO customerCompanyVO = new CustomerCompanyVO();
        BeanUtils.copyProperties(customer, customerCompanyVO);
        BeanUtils.copyProperties(company, customerCompanyVO);
        customerCompanyVO.setManager(name);
        return customerCompanyVO;
    }

    /**
     * 封装个人类型的客户信息(excel)
     */
    private CustomerPersonalVO setPersonal(CustomerEntity customer, String name, PersonalCustomerEntity personal) {
        //BeanUtils能批量封装相同字段名称和类型的属性的值
        CustomerPersonalVO customerPersonalVO = new CustomerPersonalVO();
        BeanUtils.copyProperties(customer, customerPersonalVO);
        BeanUtils.copyProperties(personal, customerPersonalVO);
        customerPersonalVO.setManager(name);
        return customerPersonalVO;
    }

    @Override
    public ResponseResult queryCustomerInfoById(DgApplyAmountInfoRequest applyAmountInfo) {
        CompanyCustomerResponse companyCustomerResponse = companyCustomerDao.queryCompanyInfo(applyAmountInfo);
        return new ResponseResult<CompanyCustomerResponse>().SUCCESS(companyCustomerResponse);
    }

    public List<CustomerEntity> queryCompanyCodeAndName(int type) {
        List<CustomerEntity> list = customerDao.queryCompanyCodeAndName(type);
        return list;
    }

    /**
     * 插入企业类型客户信息
     */
    @Transactional
    public boolean insertCompany(CompanyCustomerRequest companyCustomerReq) {
        // 客户名称判重
        if (companyCustomerReq.getName() != null && companyCustomerReq.getName().length() != 0) {
            List<String> names = customerDao.listCustomerName(1); // 企业类型客户名称
            for (String name : names) {
                if (companyCustomerReq.getName().equals(name)) {
                    ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                }
            }
        }
        // 客户社会统一代码判重
        if (companyCustomerReq.getSocialUnifiedCode() != null && companyCustomerReq.getSocialUnifiedCode().length() != 0) {
            List<String> socialUnifiedCodes = customerDao.listSocialUnifiedCode();
            for (String socialUnifiedCode : socialUnifiedCodes) {
                if (companyCustomerReq.getSocialUnifiedCode().equals(socialUnifiedCode)) {
                    ExceptionCast.cast(CrmsCode.CUSTOMER_SOCIAL_UNIFIED_CODE_REPETITION);
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        CompanyCustomerEntity companyCustomerEntity = new CompanyCustomerEntity();
        BeanUtils.copyProperties(companyCustomerReq, customerEntity);
        BeanUtils.copyProperties(companyCustomerReq, companyCustomerEntity);
        //TODO 设置客户编号
        String code = customerDao.getCodeByType(CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType());
        if (code == null || code.length() == 0) {
            code = PromptSuccess.COMPANY_START_CODE;
        }
        int codeNumber = Integer.parseInt(code.substring(code.length() - 4));
        customerEntity.setCode(PromptSuccess.COMPANY_CODE + String.format("%04d", (codeNumber + 1)));
        customerEntity.setMotifyTime(new Date());
        customerEntity.setCreateTime(new Date());
        customerEntity.setManager(companyCustomerReq.getManager());
        customerEntity.setType(CrmsConstant.CustomerType.COMPANY_CUSTOMER.getType());
        boolean customer = customerDao.saveCustomer(customerEntity);
        companyCustomerEntity.setCustomerId(customerEntity.getId());
        int company = companyCustomerDao.insert(companyCustomerEntity);
        if (companyCustomerReq.getIsExistRelated() == 1) {//有关联人的情况下才绑定关联关系
            //绑定关联关系
            List<CrmsCustomerRelated> relatedList = new ArrayList<>();
            List<Integer> intList = companyCustomerReq.getRelatedId();

            if (null != intList && intList.size() != 0) {
                for (int i = 0; i < intList.size(); i++) {
                    CrmsCustomerRelated crmsCustomerRelated = new CrmsCustomerRelated();
                    Integer integer = intList.get(i);
                    crmsCustomerRelated.setCustomerId(companyCustomerEntity.getId().intValue());//企业id
                    crmsCustomerRelated.setRelatedId(integer);//关联人/企业id
                    relatedList.add(crmsCustomerRelated);
                }
                crmsCustomerRelatedService.insertCustomerRelated(relatedList);
            }
        }

        return customer && company > 0;
    }

    /**
     * 更新企业类型客户信息
     */
    @Transactional
    public boolean updateCompanyNew(CompanyCustomerRequest companyCustomerReq) {
        // 客户名称判重,注意判断是否与原来相同
        if (companyCustomerReq.getName() != null && companyCustomerReq.getName().length() != 0) {
            String customerName = customerDao.getNameByCustomerId(companyCustomerReq.getId());
            if (!companyCustomerReq.getName().equals(customerName)) {
                List<String> names = customerDao.listCustomerName(1);
                for (String name : names) {
                    if (companyCustomerReq.getName().equals(name)) {
                        ExceptionCast.cast(CrmsCode.CUSTOMER_NAME_REPETITION);
                    }
                }
            }
        }
        // 社会统一代码判重,注意判断是否与原来相同
        if (companyCustomerReq.getSocialUnifiedCode() != null && companyCustomerReq.getSocialUnifiedCode().length() != 0) {
            String newSocialUnifiedCode = customerDao.getSocialUnifiedCodeByCustomerId(companyCustomerReq.getId());
            if (!companyCustomerReq.getSocialUnifiedCode().equals(newSocialUnifiedCode)) {
                List<String> socialUnifiedCodes = customerDao.listSocialUnifiedCode();
                for (String socialUnifiedCode : socialUnifiedCodes) {
                    if (companyCustomerReq.getSocialUnifiedCode().equals(socialUnifiedCode)) {
                        ExceptionCast.cast(CrmsCode.CUSTOMER_SOCIAL_UNIFIED_CODE_REPETITION);
                    }
                }
            }
        }
        CustomerEntity customerEntity = new CustomerEntity();
        CompanyCustomerEntity companyCustomerEntity = new CompanyCustomerEntity();
        BeanUtils.copyProperties(companyCustomerReq, customerEntity);
        BeanUtils.copyProperties(companyCustomerReq, companyCustomerEntity);
        customerEntity.setMotifyTime(new Date());
        customerEntity.setManager(companyCustomerReq.getManager());
        boolean customer = customerDao.updateCustomer(customerEntity);

        companyCustomerEntity.setCustomerId(customerEntity.getId());
        int company = companyCustomerDao.update(companyCustomerEntity, new QueryWrapper<CompanyCustomerEntity>()
                .eq("customer_id", companyCustomerEntity.getCustomerId()));

        //根据id查询crms_company_customer表里面的customerId
        CompanyCustomerEntity entity = companyCustomerDao.selectOne(new QueryWrapper<CompanyCustomerEntity>()
                .eq("customer_id", customerEntity.getId()));
        companyCustomerEntity.setCustomerId(entity.getId());

        //根据企业id删除关联关系
        crmsCustomerRelatedMapper.delete(new QueryWrapper<CrmsCustomerRelated>().eq("customer_id", entity.getId()));//companyCustomerEntity.getId()

        //绑定关联关系
        List<CrmsCustomerRelated> relatedList = new ArrayList<>();
        List<Integer> intList = companyCustomerReq.getRelatedId();

        if (null != intList && intList.size() != 0) {
            for (int i = 0; i < intList.size(); i++) {
                CrmsCustomerRelated crmsCustomerRelated = new CrmsCustomerRelated();
                Integer integer = intList.get(i);
                crmsCustomerRelated.setCustomerId(companyCustomerEntity.getCustomerId().intValue());//企业id  companyCustomerEntity.getId().intValue()
                crmsCustomerRelated.setRelatedId(integer);//关联人/企业id
                relatedList.add(crmsCustomerRelated);
            }
        }
        if (relatedList.size() > 0) {
            crmsCustomerRelatedService.insertCustomerRelated(relatedList);
        }
        return true;
    }
}