package com.daqing.financial.crms.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.crms.dao.CustomerWorkbenchDao;
import com.daqing.financial.crms.feign.HrmsFeignService;
import com.daqing.financial.crms.model.request.CompanyCustomerRequest;
import com.daqing.financial.crms.service.CustomerService;
import com.daqing.financial.crms.service.CustomerWorkbenchService;
import com.daqing.framework.domain.crms.CustomerEntity;
import com.daqing.framework.domain.crms.ext.*;
import com.daqing.framework.domain.crms.request.CustomerWorkbenchRequest;
import com.daqing.framework.domain.crms.response.CustomerEntityResponse;
import com.daqing.framework.domain.crms.response.CustomerWorkbenchListVO;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeTO;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @auther River
 * @date 2020/10/19 10:44
 */
@Service
public class CustomerWorkbenchServiceImpl extends ServiceImpl<CustomerWorkbenchDao, CustomerEntity>
        implements CustomerWorkbenchService {

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Autowired
    private CustomerService customerService;

    /**
     * 列表
     */
    @Override
    public PageUtils list(Integer page, Integer size, CustomerWorkbenchRequest cwr) {
        // 分页参数
        if (page == null || size == null) {
            page = 1;
            size = 10;
        }
        // 校验时间参数
        this.timeCheckout(cwr.getCreateTime(), cwr.getStartTime(), cwr.getEndTime(), cwr);
        //IPage<CustomerEntity> customerEntityIPage = this.getBaseMapper().queryList(new Page(page, size), cwr);
        //List<CustomerEntity> customers = customerEntityIPage.getRecords();

        if(cwr.getEndTime()!=null)
            cwr.setEndTime(cwr.getEndTime()+" 23:59:59");

        IPage<CustomerEntityResponse> customerEntityIPage = this.getBaseMapper().queryList(new Page(page, size), cwr);
        List<CustomerEntityResponse> customers = customerEntityIPage.getRecords();
        // 获取所有的客户经理id
        Long[] managerIds = new Long[customers.size()];
        for (int i = 0; i < customers.size(); i++) {
            if (customers.get(i).getManager() != null) {
                managerIds[i] = customers.get(i).getManager();
            } else {
                managerIds[i] = 0L;
            }
        }
        // 获取所有的客户id
        Long[] customerIds = customers.stream().map(CustomerEntityResponse::getId).toArray(Long[]::new);
        // 返回列表实体类集合
        List<CustomerWorkbenchListVO> listResponse = new ArrayList<>();
        // 远程调用查询客户对应的客户经理信息
        ResponseResult<List<EmployeeTO>> responseResult = hrmsFeignService.getEmployeeAndDeptById(managerIds);
        List<EmployeeTO> employeeTOs = responseResult.getData();
        if (employeeTOs != null) {
            listResponse = this.jointCustomerEmployee(customers, employeeTOs);
        } else {
            for (CustomerEntityResponse customer : customers) {
                CustomerWorkbenchListVO workbenchListVO = new CustomerWorkbenchListVO();
                BeanUtils.copyProperties(customer, workbenchListVO);
                listResponse.add(workbenchListVO);
            }
        }
        if (customerIds.length != 0) {
            // TODO:远程调用担保服务查询每个客户的审批状态
            for (CustomerWorkbenchListVO customerWorkbenchListVO : listResponse) {
                customerWorkbenchListVO.setStatus(2);
            }
            if (cwr.getStatus() != null && (cwr.getStatus() == 0 || cwr.getStatus() == 1 || cwr.getStatus() == 2)) {
                // 筛选状态
                listResponse.removeIf(customerWorkbenchListVO -> !Objects.equals(customerWorkbenchListVO.getStatus(), cwr.getStatus()));
            }
        }
        // 属性拷贝,将泛型为CustomerEntity类型的IPage的属性拷贝给泛型为CustomerTO类型的IPage,才能赋值给PageUtils
        IPage<CustomerWorkbenchListVO> iPage = new Page<>();
        BeanUtils.copyProperties(customerEntityIPage, iPage);
        iPage.setRecords(listResponse);

        return new PageUtils(iPage);
    }

    /**
     * 详情
     */
    @Override
    public ResponseResult info(Long id) {

        return customerService.queryCustomerById(id);
    }

    /**
     * 删除
     */
    @Transactional
    @Override
    public List<String> delete(Long[] ids) {

        return customerService.updateCustomerById(ids);
    }

    /**
     * 新增个人
     */
    @Transactional
    @Override
    public Boolean savePersonal(CustomerPersonalTOI customerPersonalTOI) {

        Long userId = Long.parseLong(this.getUserId());
        customerPersonalTOI.setManager(userId); // 从redis当前登录用户
        boolean result = customerService.saveCustomerPersonal(customerPersonalTOI);
        // TODO 将当前客户状态和自增的客户id新增到担保表中
        // customerId = customerPersonalTOI.getManager();
        // status = 2;
        // Boolean result = save(customerId,status);
        return result;
    }

    /**
     * 新增企业
     */
   /* @Transactional
    @Override
    public Boolean saveCompany(CustomerCompanyTOI customerCompanyTOI) {

        Long userId = Long.parseLong(this.getUserId());
        customerCompanyTOI.setManager(userId); // 从redis获取当前登录用户
        boolean result = customerService.saveCustomerCompany(customerCompanyTOI);
        // TODO 将当前客户状态和自增的客户id新增到担保表中
        // customerId = customerPersonalTOI.getManager();
        // status = 2;
        // Boolean result = save(customerId,status);
        return result;
    }*/

    /**
     * 修改个人
     */
    @Transactional
    @Override
    public Boolean updatePersonal(CustomerPersonalTOU customerPersonalTOU) {

        Long userId = Long.parseLong(this.getUserId());
        customerPersonalTOU.setManager(userId);
        return customerService.updateCustomerPersonal(customerPersonalTOU);
    }

    /**
     * 修改企业
     */
    /*@Transactional
    @Override
    public Boolean updateCompany(CustomerCompanyTOU customerCompanyTOU) {

        Long userId = Long.parseLong(this.getUserId());
        customerCompanyTOU.setManager(userId);
        return customerService.updateCustomerCompany(customerCompanyTOU);
    }*/

    /**
     * 通过审核
     *
     * @param id      当前客户id
     * @param manager 客户经理id
     * @return
     */
    @Override
    public Boolean pass(Long id, Long manager) {
        if (id == null || manager == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        return this.getBaseMapper().passById(id, manager);
    }

    /**
     * 不通过审核
     *
     * @param id    当前客户id
     * @param cause 原因
     * @return
     */
    @Override
    public Boolean noPass(Long id, String cause) {

        // TODO 将当前客户id、不通过原因、客户状态status = 1(不通过)更新到担保业务的表中
        return true;
    }

    /**
     * 获取当前客户部门经理的部门下所有客户经理
     */
    @Override
    public List<EmployeeEntity> getCustomerManager() {

        Long userId = Long.parseLong(this.getUserId());
        ResponseResult<List<EmployeeEntity>> employees = hrmsFeignService.listEmployeeById(userId);
        return employees.getData();
    }

    /**
     * 选择某段时间内客户信息的时间参数校验
     */
    private void timeCheckout(Integer createTime, String startTime, String endTime, CustomerWorkbenchRequest customerWorkbenchRequest) {
        // 没有选择自定义时间且选择了固定的"3个月内、6个月内、9个月内...."时间
        if ((startTime == null || startTime.length() == 0) && (endTime == null || endTime.length() == 0) && createTime != null) {
            if (createTime == 3 || createTime == 6 || createTime == 9 || createTime == 12) {
                String pastTime = this.pastTime(createTime);
                customerWorkbenchRequest.setStartTime(pastTime);
            } else {
                customerWorkbenchRequest.setStartTime(null);
            }
        }
        // 选择了自定义时间且选择了固定时间则固定时间无效
        if (createTime != null && ((startTime != null && startTime.length() != 0) || (endTime != null && endTime.length() != 0))) {
            customerWorkbenchRequest.setCreateTime(null);
        }
    }

    /**
     * 得到过去number月内的时间
     */
    private String pastTime(Integer number) {
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, -number);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:ss:mm");
        return sdf.format(calendar.getTime());
    }

    /**
     * 拼接客户及其对应的客户经理及部门信息
     */
    private List<CustomerWorkbenchListVO> jointCustomerEmployee(List<CustomerEntityResponse> customers, List<EmployeeTO> employeeTO) {
        List<CustomerWorkbenchListVO> listResponse = new ArrayList<>();// 用于拼装客户信息和员工及部门信息的集合
        CustomerWorkbenchListVO customerWorkbenchListVO;
        // 将客户信息和客户经理及部门信息拼装起来
        for (CustomerEntityResponse customer : customers) {
            // 每一个客户对应一个不同的对象
            customerWorkbenchListVO = new CustomerWorkbenchListVO();
            for (int i = 0; i < employeeTO.size(); i++) {
                if (customer.getManager() != null && customer.getManager().equals(employeeTO.get(i).getId())) {
                    customerWorkbenchListVO.setManager(employeeTO.get(i).getEmpName());
                    customerWorkbenchListVO.setDepartments(employeeTO.get(i).getDeptNames());
                }
            }
            BeanUtils.copyProperties(customer, customerWorkbenchListVO);
            listResponse.add(customerWorkbenchListVO);
        }
        return listResponse;
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:" + token);
    }

    /**
     * 新增企业
     */
    @Transactional
    public Boolean insertCompany(CompanyCustomerRequest companyCustomerReq) {

        Long userId = Long.parseLong(this.getUserId());
        companyCustomerReq.setManager(userId); // 从redis获取当前登录用户
        boolean result = customerService.insertCompany(companyCustomerReq);
        return result;
    }

    /**
     * 修改企业
     */
    @Transactional
    public Boolean updateCompanyNew(CompanyCustomerRequest companyCustomerReq) {

        Long userId = Long.parseLong(this.getUserId());
        companyCustomerReq.setManager(userId);
        return customerService.updateCompanyNew(companyCustomerReq);
    }
}
