package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.AlCollectionListRequest;
import com.daqing.financial.guarantee.model.request.AlCollectionRequest;
import com.daqing.financial.guarantee.model.response.AlCollectionListResponse;
import com.daqing.financial.guarantee.model.response.AlCollectionResponse;
import com.daqing.financial.guarantee.model.response.MoreOverdueDetailResponse;
import com.daqing.financial.guarantee.model.response.OverdueDetailResponse;
import com.daqing.financial.guarantee.service.IAlCollectionService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.guarantee.AlCollection;
import com.daqing.framework.domain.guarantee.AlRepaymentEntry;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.ArraysUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.util.List;

/**
 * <p>
 * 保后催收 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2021-03-17
 */
@RestController
@RequestMapping("/al-collection")
@Api(value = "催收管理", tags = "催收管理")
public class AlCollectionController {

    @Autowired
    private IAlCollectionService alCollectionService;

    @GetMapping("/list")
    @ApiOperation(value = "催收管理列表", notes = "催收管理列表", response = AlCollectionResponse.class)
    public ResponseResult list(
            @RequestParam("page") Integer page, @RequestParam("size") Integer size,
            @RequestParam(name = "codeOrName", required = false) String codeOrName,
            @RequestParam(name = "status", required = false) Integer status) {
        return ResponseResult.SUCCESS(alCollectionService.list(page, size, codeOrName, status));
    }

    @PostMapping("/collection")
    @ApiOperation(value = "催收", notes = "新增催收")
    public ResponseResult collection(@RequestBody @Valid AlCollectionRequest alCollectionRequest) {
        Boolean result = alCollectionService.collection(alCollectionRequest);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    @Log(detail = "更新催收信息",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.UPDATE)
    @PostMapping("/updateCollection")
    @ApiOperation(value = "编辑催收信息")
    public ResponseResult updateCollection(@RequestBody @Valid AlCollectionRequest alCollectionRequest){
        AlCollection collection = new AlCollection();
        BeanUtils.copyProperties(alCollectionRequest, collection);
        collection.setEnclosureFile(ArraysUtil.toString(alCollectionRequest.getEnclosureFiles()));
        boolean update = alCollectionService.updateById(collection);
        return update ? ResponseResult.SUCCESS("编辑成功！"):ResponseResult.FAIL(60007,"编辑失败！");
    }

    @Log(detail = "删除催收信息",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.DELETE)
    @PostMapping("/delCollection")
    @ApiOperation(value = "删除催收信息")
    public ResponseResult delCollection(@RequestParam String id){
        boolean remove = alCollectionService.removeById(id);
        return remove ? ResponseResult.SUCCESS("删除成功！"):ResponseResult.FAIL(60008,"删除失败！");
    }

    @GetMapping("/overdue/detail")
    @ApiOperation(value = "催收时展示的详情", notes = "催收时展示的详情", response = OverdueDetailResponse.class)
    public ResponseResult overdueDetail(@RequestParam("insuranceId") Integer insuranceId) {
        return ResponseResult.SUCCESS(alCollectionService.overdueDetail(insuranceId));
    }

    @PostMapping("/collection/list")
    @ApiOperation(value = "催收列表", notes = "催收列表", response = AlCollectionListResponse.class)
    public ResponseResult collectionList(@RequestBody AlCollectionListRequest alCollectionListRequest) {
        return ResponseResult.SUCCESS(alCollectionService.collectionList(alCollectionListRequest));
    }

    @GetMapping("/examine/detail")
    @ApiOperation(value = "查看时的催收详情", notes = "查看时的催收详情", response = MoreOverdueDetailResponse.class)
    public ResponseResult moreOverdueDetail(@RequestParam("id") Integer id) {
        return ResponseResult.SUCCESS(alCollectionService.moreOverdueDetail(id));
    }

    @ApiOperation(value = "催收管理列表导出", notes = "催收管理列表导出")
    @GetMapping("/list/export/excel")
    public ResponseResult listExportExcel(
            @RequestParam("insuranceIds") List<Integer> insuranceIds,
            HttpServletResponse response) throws IOException {
        alCollectionService.listExportExcel(insuranceIds, response);
        return ResponseResult.SUCCESS();
    }

    @ApiOperation(value = "催收列表导出", notes = "催收列表导出")
    @GetMapping("/collection/list/export/excel")
    public ResponseResult collectionListExportExcel(
            @RequestParam("ids") List<Integer> ids,
            @RequestParam("insuranceId")Integer insuranceId,
            HttpServletResponse response) throws IOException {
        alCollectionService.collectionListExportExcel(ids,insuranceId, response);
        return ResponseResult.SUCCESS();
    }
}
