package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.CollectionProgressReq;
import com.daqing.financial.guarantee.model.request.EditStatisticsListReq;
import com.daqing.financial.guarantee.model.request.GenerateReq;
import com.daqing.financial.guarantee.model.request.StatisticsListReq;
import com.daqing.financial.guarantee.model.response.CollectionGroupListResp;
import com.daqing.financial.guarantee.model.response.CollectionProgressResp;
import com.daqing.financial.guarantee.model.response.Personnel;
import com.daqing.financial.guarantee.model.response.StatisticsGroupVo;
import com.daqing.financial.guarantee.service.AlCollectionStatisticsService;
import com.daqing.financial.guarantee.util.R;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.guarantee.AlCollectionStatistics;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author lr
 * @since 2021-08-19
 */
@RestController
@RequestMapping("/al-collection-statistics")
@Api(value = "保后数据统计", tags = "清收情况统计和进度查询")
public class AlCollectionStatisticsController {

    @Autowired
    private AlCollectionStatisticsService collectionStatisticsService;

    @Log(detail = "清收进度查询",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @PostMapping("/queryCollectionProgress")
    @ApiOperation(value = "清收进度查询",response = CollectionProgressResp.class)
    public R queryCollectionProgress(@RequestBody CollectionProgressReq collectionProgressReq){
        List<CollectionProgressResp> list = collectionStatisticsService.queryCollectionProgress(collectionProgressReq);
        int total = collectionStatisticsService.selectCollectionProgressPageTotal(collectionProgressReq);
        return R.ok().data("list",list).data("total",total);
    }

    @Log(detail = "清收进度查询导出",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/collectionProgressExport")
    @ApiOperation(value = "清收进度查询导出")
    public void collectionProgressExport(
            @ApiParam(name = "ids", value = "业务id", required = true)
            @RequestParam("ids") String ids, HttpServletResponse response){
        try {
            collectionStatisticsService.batchExport(response,ids);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @ApiOperation(value = "清收组成员列表")
    @PostMapping("/collectionGroupList")
    public R collectionGroupList(){
        List<Personnel> list = collectionStatisticsService.collectionGroupList();
        return R.ok().data("list",list);
    }

    @Log(detail = "生成年度统计表",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.INSERT)
    @PostMapping("/generateAnnualStatistics")
    @ApiOperation(value = "生成年度统计表")
    public R generateAnnualStatistics(@Valid @RequestBody GenerateReq generateReq){
        boolean ok = collectionStatisticsService.selectYear(generateReq.getYear());
        if (!ok){
            return R.error().message("生成失败，该年度下无数据！");
        }
        boolean exist = collectionStatisticsService.selectGenerateYear(generateReq.getYear());
        if (exist && generateReq.getFlag() == 1){
            return R.error().message("该年度已存在，如需增减人员请在人员选择处选择！");
        }
        collectionStatisticsService.generateStatistics(generateReq);
        return R.ok();
    }

    @Log(detail = "年度统计列表",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @PostMapping("/annualStatisticsList")
    @ApiOperation(value = "年度统计列表",response = AlCollectionStatistics.class)
    public R annualStatisticsList(@Valid @RequestBody StatisticsListReq statisticsListReq){
        List<AlCollectionStatistics> list = collectionStatisticsService.annualStatisticsList(statisticsListReq);
        int total = collectionStatisticsService.annualStatisticsListTotal(statisticsListReq);
        //数据分组
        StatisticsGroupVo groupList = new StatisticsGroupVo();
        Map<Integer, List<AlCollectionStatistics>> collect = list.stream().collect(Collectors.groupingBy(AlCollectionStatistics::getGroupId));
        groupList.setCollect(collect);
        return R.ok().data("list",list).data("groupList",groupList).data("total",total);
    }

    @Log(detail = "编辑年度统计列表",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.UPDATE)
    @PostMapping("/editAnnualStatistics")
    @ApiOperation(value = "编辑年度统计列表")
    public R editAnnualStatistics(
            @ApiParam(name = "req", value = "年度统计信息", required = true)
            @RequestBody EditStatisticsListReq req){
        boolean update = collectionStatisticsService.updateBatchById(req.getCollectionStatistics());
        return update ? R.ok() : R.error();
    }

    @Log(detail = "年度统计列表导出",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/annualStatisticsListExport")
    @ApiOperation(value = "年度统计列表导出")
    public void annualStatisticsListExport(
            @ApiParam(name = "year", value = "年份", required = true)
            @RequestParam("year") String year,
            @ApiParam(name = "flag", value = "0为组排名、1为公司排名", required = true)
            @RequestParam("flag") String flag, HttpServletResponse response){
        try {
            collectionStatisticsService.annualStatisticsListExport(year,flag,response);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Log(detail = "年度列表",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/annualList")
    @ApiOperation(value = "年度列表")
    public R annualList(){
        List<String> list = collectionStatisticsService.annualList();
        return R.ok().data("list",list);
    }

    @Log(detail = "人员选择",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/personnelSelection")
    @ApiOperation(value = "人员选择")
    public R personnelSelection(
            @ApiParam(name = "year", value = "年份", required = true)
            @RequestParam("year") String year){
        List<Integer> list = collectionStatisticsService.personnelSelection(year);
        return R.ok().data("list",list);
    }

    @Log(detail = "年度统计删除",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.DELETE)
    @PostMapping("/annualStatisticsDeleted")
    @ApiOperation(value = "年度统计删除")
    public R annualStatisticsDeleted(
            @ApiParam(name = "year", value = "年份", required = true)
            @RequestParam("year") String year){
        boolean delete = collectionStatisticsService.annualStatisticsDeleted(year);
        return delete ? R.ok() : R.error();
    }

}

