package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.InsuranceSaveReq;
import com.daqing.financial.guarantee.service.IAlInsuranceListService;
import com.daqing.financial.guarantee.util.R;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 保后管理列表 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2021-03-17
 */
@Api(tags = {"保后管理"})
@RestController
@RequestMapping("/al-insurance-list")
public class AlInsuranceListController {

    @Autowired
    private IAlInsuranceListService alInsuranceListService;

    @Log(detail = "保后管理列表",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/insuranceList")
    @ApiOperation(value = "保后管理列表")
    public ResponseResult insuranceList(
            @RequestParam(value="page",required=false) Integer page,
            @RequestParam(value="size",required=false) Integer size,
            @RequestParam(value = "CustomerNumberOrName", required = false) String CustomerNumberOrName,
            @RequestParam(value = "paymentStatus", required = false) Integer paymentStatus){
        PageUtils data = alInsuranceListService.queryPage(page, size, CustomerNumberOrName, paymentStatus);
        return ResponseResult.SUCCESS(data);
    }

    @Log(detail = "添加保后业务",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.INSERT)
    @ApiOperation(value = "添加保后业务")
    @PostMapping("/saveInsurance")
    public R saveInsurance(
            @ApiParam(name = "insuranceSaveReq", value = "保后业务信息", required = true)
            @RequestBody InsuranceSaveReq insuranceSaveReq) {
        boolean save = alInsuranceListService.saveInsurance(insuranceSaveReq);
        return save ? R.ok() : R.error();
    }

    @Log(detail = "编辑保后业务",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.UPDATE)
    @ApiOperation(value = "编辑保后业务")
    @PostMapping("/updateInsurance")
    public R updateInsurance(
            @ApiParam(name = "insuranceSaveReq", value = "保后业务信息", required = true)
            @RequestBody InsuranceSaveReq insuranceSaveReq) {
        Integer insuranceId = alInsuranceListService.updateInsurance(insuranceSaveReq);
        return R.ok().data("insuranceId",insuranceId);
    }

    @Log(detail = "删除保后业务",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.DELETE)
    @ApiOperation(value = "删除保后业务")
    @PostMapping("/deleteInsurance")
    public R deleteInsurance(
            @ApiParam(name = "insuranceId", value = "保后业务id", required = true)
            @RequestParam Integer insuranceId) {
        boolean delete = alInsuranceListService.deleteInsurance(insuranceId);
        return delete ? R.ok() : R.error();
    }

    @Log(detail = "担保详细",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/insuranceDetail")
    @ApiOperation(value = "担保详细")
    public R insuranceDetail(Integer id){
        return alInsuranceListService.insuranceDetail(id);
    }

    @Log(detail = "代偿情况",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/compensationSituation")
    @ApiOperation(value = "代偿情况")
    public R compensationSituation(String date,Integer insuranceId){
        return alInsuranceListService.compensationSituation(date,insuranceId);
    }

    @Log(detail = "导出保后管理列表",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出保后管理列表")
    public void excelExport(
            @ApiParam(name = "ids", value = "保后id")
            @RequestParam("ids")List<Integer> ids, HttpServletResponse response) {
        alInsuranceListService.excelExport(ids,response);
    }

    @Log(detail = "办理结项",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.UPDATE)
    @GetMapping("/updateStatus")
    @ApiOperation(value = "办理结项")
    public ResponseResult updateStatus(@RequestParam("id") Integer id){
        int result = alInsuranceListService.updateStatus(id);
        return result>0 ? ResponseResult.SUCCESS("编辑成功！"):ResponseResult.FAIL(60007,"编辑失败！");
    }

    @PostMapping("/importInsurance")
    @ApiOperation("批量导入保后业务")
    public R importQuestion(@RequestParam(name = "file") MultipartFile file ,HttpServletResponse response) throws IOException {
        try {
            Map<String, String> importInsurance = alInsuranceListService.importInsurance(file);
            return R.ok().data("import",importInsurance);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return R.ok();
    }

    /**
     * 批量导入保后业务失败数据导出
     * @throws Exception e
     */
    @ApiOperation(value = "批量导入保后业务失败数据导出")
    @GetMapping("/exportFailure")
    public void exportFailureRecord(
            @ApiParam(name = "exportCode", value = "importInsurance接口返回的exportCode", required = true)
            @RequestParam String exportCode,
            HttpServletResponse response) throws Exception {
        alInsuranceListService.exportFailureRecord(response, exportCode);
    }

}
