package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.service.IAlRepaymentEntryService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.guarantee.AlRepaymentEntry;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 还款记录表 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2021-03-17
 */
@Api(tags = {"还款记录"})
@RestController
@RequestMapping("/al-repayment-entry")
public class AlRepaymentEntryController {

    @Autowired
    private IAlRepaymentEntryService alRepaymentEntryService;

    /**
     * 还款录入
     */
    @Log(detail = "还款录入",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.INSERT)
    @PostMapping("/repaymentEntry")
    @ApiOperation(value = "还款录入")
    public ResponseResult repaymentEntry(@RequestBody @Valid AlRepaymentEntry alRepaymentEntryReq){
        int result = alRepaymentEntryService.repaymentEntry(alRepaymentEntryReq);
        return result>0 ? ResponseResult.SUCCESS("录入成功"):ResponseResult.FAIL(60006,"录入失败");
    }

    /**
     * 还款详细
     */
    @Log(detail = "还款详细",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/repaymentDetail")
    @ApiOperation(value = "还款详细")
    public ResponseResult repaymentDetail(Integer id){
        AlRepaymentEntry response = alRepaymentEntryService.repaymentDetail(id);
        return ResponseResult.SUCCESS(response);
    }

    /**
     * 还款记录列表
     */
    @Log(detail = "还款记录列表",level = 3,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/repaymentList")
    @ApiOperation(value = "还款记录列表")
    public ResponseResult repaymentList(@RequestParam(value="page",required=false) Integer page, @RequestParam(value="size",required=false) Integer size,@RequestParam(value = "insuranceId") Integer insuranceId,
                                        @RequestParam(value = "CustomerNumberOrName", required = false) String CustomerNumberOrName, @RequestParam(value="status",required=false) Integer status){

        PageUtils data = alRepaymentEntryService.queryPage(page, size, CustomerNumberOrName,status,insuranceId);

        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 编辑还款信息
     */
    @Log(detail = "编辑还款信息",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.UPDATE)
    @PostMapping("/updateRepayment")
    @ApiOperation(value = "编辑还款信息")
    public ResponseResult updateRepayment(@RequestBody @Valid AlRepaymentEntry alRepaymentEntryReq){
        int result = alRepaymentEntryService.updateRepayment(alRepaymentEntryReq);
        return result>0 ? ResponseResult.SUCCESS("编辑成功！"):ResponseResult.FAIL(60007,"编辑失败！");
    }

    /**
     * 批量删除还款信息
     */
    @Log(detail = "批量删除还款信息",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.DELETE)
    @PostMapping("/delRepayment")
    @ApiOperation(value = "批量删除还款信息")
    public ResponseResult delRepayment(@RequestBody List<Integer> ids){
        int result = alRepaymentEntryService.delRepayment(ids);
        return result>0 ? ResponseResult.SUCCESS("删除成功！"):ResponseResult.FAIL(60008,"删除失败！");
    }

    /**
     * 导出
     */
    @Log(detail = "导出还款记录列表",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出还款记录列表")
    public ResponseResult excelExport(@RequestParam("ids") List<String> ids,@RequestParam("insuranceId") Integer insuranceId, HttpServletResponse response) {

        Boolean result = alRepaymentEntryService.excelExport(ids,insuranceId,response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 还款统计
     * @param insuranceId 保后Id guaranteeAmount 担保额度
     */
    @Log(detail = "还款统计",level = 4,operationUnit = OperationUnit.INSURANCE,operationType = OperationType.SELECT)
    @GetMapping("/repaymentStatistics")
    @ApiOperation(value = "还款统计")
    public ResponseResult repaymentStatistics(@RequestParam("insuranceId") Integer insuranceId){
        Map map = alRepaymentEntryService.repaymentStatistics(insuranceId);
        return ResponseResult.SUCCESS(map);
    }

    /**
     * 定时任务处理还款记录内容 朝九晚六工作时间每隔十分钟执行一次
     */
    /*@Scheduled(cron ="0 0/10 9-18 * * ?")
    @GetMapping("/updateRepaymentEntry")
    public void updateRepaymentEntry() {
        alRepaymentEntryService.updateRepaymentEntry();
    }*/

}
