package com.daqing.financial.guarantee.controller;

import com.alibaba.fastjson.JSONObject;
import com.daqing.financial.guarantee.DgApplyAmountInfoControllerApi;
import com.daqing.financial.guarantee.feign.CrmsFeignService;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.model.request.ApplyAmountListRequest;
import com.daqing.financial.guarantee.model.request.ApprovalBusinessRequest;
import com.daqing.financial.guarantee.model.request.BusinessApplicationRequest;
import com.daqing.financial.guarantee.model.response.BusinessApplicationDetailResponse;
import com.daqing.financial.guarantee.model.response.BusinessApplicationExportResponse;
import com.daqing.financial.guarantee.model.response.BusinessApplicationListResponse;
import com.daqing.financial.guarantee.service.IDgApplyAmountInfoService;
import com.daqing.financial.guarantee.service.IDgApplyAmountListService;
import com.daqing.financial.guarantee.util.MoneyUtil;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.crms.request.DgApplyAmountInfoRequest;
import com.daqing.framework.domain.guarantee.DgApplyField;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.JwtUtils;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.util.*;

/**
 * <p>
 * 申请贷款信息 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Api(tags = {"申请贷款信息"})
@Slf4j
@RestController
@RequestMapping("/dg-apply-amount-info")
public class DgApplyAmountInfoController implements DgApplyAmountInfoControllerApi {

    @Resource
    private CrmsFeignService crmsFeignService;

    @Autowired
    private IDgApplyAmountInfoService applyAmountInfoService;

    @Resource
    private HrmsFeignService hrmsFeignService;

    @Autowired
    private IDgApplyAmountListService applyAmountListService;


    /**
     * 获取当前用户角色
     */
    @GetMapping("/getRoles")
    @ApiOperation(value = "获取当前用户角色")
    public ResponseResult getRoles(){
        String roleIds = applyAmountInfoService.getRoles();
        //List list = applyAmountInfoService.getPermission(roleIds);
        return ResponseResult.SUCCESS(roleIds);
    }


    /**
     * 根据客户编号/客户名称获取企业信息
     * @param applyAmountInfo
     * @return
     */
    @PostMapping("/companyInfoBySth")
    @ApiOperation(value = "根据客户编号/客户名称获取企业信息")
    public ResponseResult companyInfoBySth(@RequestBody DgApplyAmountInfoRequest applyAmountInfo){
        //远程调用crms查询企业信息
        return crmsFeignService.queryCustomerInfoById(applyAmountInfo);
    }

    /**
     * 业务申请
     * @param businessApplication
     * @return
     */
    @Log(detail = "业务申请",level = 3,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.INSERT)
    @PostMapping("/businessApplication")
    @ApiOperation(value = "业务申请")
    public ResponseResult businessApplication(@RequestBody BusinessApplicationRequest businessApplication){
        boolean legal = MoneyUtil.judgeTwoDecimal(businessApplication.getApplyAmount());
        if(legal==true){
            int result = applyAmountInfoService.businessApplication(businessApplication);
            return result>0 ? ResponseResult.SUCCESS("上传成功"):ResponseResult.FAIL(40004,"上传失败");
        }else{
            return ResponseResult.FAIL(40009,"申请额度格式错误！");
        }

    }

    /**
     * 上传文件
     * @param files
     * @return
     */
    @PostMapping("/uploadFile")// 浏览器地址栏输入的路径
    @ResponseBody
    public ResponseResult uploadFile(@RequestParam("file")MultipartFile[] files) throws IOException {
        System.out.println(" 文件名称： 来了来了");
        /**
         * MultipartFile类是用来接收前台传过来的文件，常用的方法如下：
         * String getContentType()          //获取文件MIME类型
         * InputStream getInputStream()     //获取文件流
         * String getName()                 //获取表单中文件组件的名字
         * String getOriginalFilename()     //获取上传文件的原名
         * long getSize()                   //获取文件的字节大小，单位byte，除以1024就是kb
         * boolean isEmpty()                //是否为空
         * void transferTo(File dest)       //保存到一个目标文件中。
         */

        List<String> list = new ArrayList<String>();
        for (MultipartFile file : files) { //因为有上传多个文件，所以用的是MultipartFile[]数组，所以要遍历数组保存里面的每一个文件
            String filePath =PromptSuccess.FILE_URL_PATH;
            System.out.println(" 文件名称： " + file.getOriginalFilename());
            System.out.println(" 文件大小： " + file.getSize() / 1024D + "Kb");
            System.out.println(" 文件类型： " + file.getContentType());
            System.out.println(" 属性名称： " + file.getName());
            System.out.println();
            //在这里执行调用文件保存的方法....
            String randomId = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 16);
            filePath = filePath + randomId+file.getOriginalFilename();
            file.transferTo(new File(filePath));
            filePath = "";
            list.add(PromptSuccess.ENCLOSUREFILE_URL_PATH+randomId+file.getOriginalFilename());

        }
        return ResponseResult.SUCCESS(list);
    }

    /**
     * 下载单个文件
     *
     * @param response
     * @param fileName
     * @return
     */
    @RequestMapping("/download")
    public String download(HttpServletResponse response, @RequestParam("fileName") String fileName) {
        File file = new File(PromptSuccess.FILE_URL_PATH + fileName);
        if (file.exists()) {
            response.setContentType("application/force-download;charset=UTF-8");
            response.setCharacterEncoding("UTF-8");
            response.setHeader("Content-Disposition", "attachment;fileName =" + fileName);
            byte[] buffer = new byte[1024];

            try (FileInputStream inputStream = new FileInputStream(file);
                 OutputStream outStream = response.getOutputStream();
                 BufferedInputStream buffStream = new BufferedInputStream(inputStream)) {
                int i = 0;
                while ((i = buffStream.read(buffer)) != -1) {
                    outStream.write(buffer);
                    outStream.write(buffer, 0, i);
                }
                return "download success";
            } catch (Exception e) {
                log.error("download error {}", e.getMessage());
            }
        }
        return null;
    }

    /**
     * 删除服务上的文件
     * @param filePath 路径
     * @return
     */
    @GetMapping("/deleteServerFile")
    public boolean deleteServerFile(@RequestParam("filePath")String filePath){
        boolean delete_flag = false;
        String certainPath =PromptSuccess.FILE_URL_PATH;
        filePath = certainPath + filePath;
        File file = new File(filePath);
        if (!file.exists()) {
            return delete_flag;
        }
        try{
            delete_flag = file.delete();
        }catch (Exception e){
            e.printStackTrace();
        }
        return delete_flag;
    }


    /**
     * 业务申请列表
     * @param
     * @return
     */
    @Log(detail = "业务申请列表",level = 3,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.SELECT)
    @GetMapping("/businessApplicationList")
    @ApiOperation(value = "业务申请列表")
    public ResponseResult businessApplicationList(@RequestParam(value="page",required=false) Integer page,@RequestParam(value="size",required=false) Integer size,
            @RequestParam(value = "CustomerNumberOrName", required = false) String CustomerNumberOrName,@RequestParam(value = "status", required = false) Integer status){
        //获取当前登录用户id
        String userId = getUserId();

        //根据角色查询资产部调查列表
        String roleIds = RedisUtil.get("dq:userRole:" + userId);
        PageUtils data = null;
        if(roleIds != null) {
            String[] ids = roleIds.split(",");
            List<String> roleIdlist = Arrays.asList(ids);
            data = applyAmountListService.queryPage(page, size, CustomerNumberOrName, roleIdlist, userId, status);

        }

        //遍历data里面的数据提单人id,找出匹配id,将用户名以及部门名称赋值进去
        if(data != null){
            List<BusinessApplicationListResponse> list = (List<BusinessApplicationListResponse>) data.getList();
            if(list.size()>0){
                List<Integer> arr = new ArrayList<>();
                for (BusinessApplicationListResponse res : list) {
                    arr.add(res.getPresenterId());
                }
                //根据提单人id查询其部门名称
                ResponseResult responseResult = hrmsFeignService.getAccountAndDeptNameById(arr);
                List<LinkedHashMap> employeeMessage = null;
                if(responseResult.getData() != null){
                    employeeMessage = (List<LinkedHashMap>) responseResult.getData();
                }
                System.out.println("employeeMessage==================="+employeeMessage);

                if(employeeMessage!= null && employeeMessage.size()>0){
                    for (BusinessApplicationListResponse response:list) {
                        for(LinkedHashMap res : employeeMessage){
                            if (response.getPresenterId()!=null){
                                if(response.getPresenterId().equals(res.get("id"))){//如果提单人id相同情况下，就往对象里面赋值
                                    response.setAccount(JSONObject.toJSONString(res.get("account")).replace("\"",""));
                                    response.setDeptName(JSONObject.toJSONString(res.get("deptName")).replace("\"",""));
                                }
                            }
                            if(response != null){
                                if(response.getAccount()==null){
                                    response.setAccount("客户");
                                }
                            }
                        }
                    }
                }else{
                    for (BusinessApplicationListResponse response:list) {
                        if(response != null){
                            if(response.getAccount()==null){
                                response.setAccount("客户");
                            }
                        }
                    }
                }
            }
        }
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 根据业务id查询业务申请详情
     * @param id
     * @return
     */
    @Log(detail = "业务申请详情",level = 3,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.SELECT)
    @GetMapping("/businessApplicationDetail")
    @ApiOperation(value = "根据业务id查询业务申请详情")
    public ResponseResult businessApplicationDetail(Integer id){
        BusinessApplicationDetailResponse response = applyAmountInfoService.businessApplicationDetail(id);
        return ResponseResult.SUCCESS(response);
    }

    /**
     * 根据业务id修改业务申请内容
     * @param businessApplication
     * @return
     */
    @Log(detail = "修改业务申请",level = 4,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.UPDATE)
    @PostMapping("/updateBusinessApplication")
    @ApiOperation(value = "根据业务id修改业务申请内容")
    public ResponseResult updateBusinessApplication(@RequestBody @Valid BusinessApplicationRequest businessApplication){
        int result = applyAmountInfoService.updateBusinessApplication(businessApplication);
        return result>0 ? ResponseResult.SUCCESS("修改成功！"):ResponseResult.FAIL(40005,"修改失败！");
    }

    /**
     * 根据业务id撤销业务申请内容
     * @param id
     * @return
     */
    @Log(detail = "撤销业务申请",level = 4,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.UPDATE)
    @GetMapping("/revokeBusinessApplication")
    @ApiOperation(value = "根据业务id撤销业务申请内容")
    public ResponseResult revokeBusinessApplication(@RequestParam("id") Integer id){
        int result = applyAmountInfoService.revokeBusinessApplication(id);
        return result>0 ? ResponseResult.SUCCESS("撤销成功！"):ResponseResult.FAIL(40006,"撤销失败！");
    }

    /**
     * 导出
     */
    @Log(detail = "导出业务申请列表",level = 4,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出业务申请内容")
    public ResponseResult excelExport(@RequestParam("ids") List<String> ids,@RequestParam("userId")Integer userId,HttpServletResponse response) {

        Boolean result = applyAmountInfoService.excelExport(ids,userId,response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    @Log(detail = "导出业务申请列表",level = 4,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.SELECT)
    @ApiOperation(value = "导出业务申请列表数据")
    @PostMapping("/applyAmountListExport")
    public void applyAmountListExport(
            @ApiParam(name = "applyAmountListRequest", value = "业务申请列表数据", required = true)
            @RequestBody ApplyAmountListRequest applyAmountListRequest, HttpServletResponse response){
        //获取业务申请列表数据
        List<BusinessApplicationExportResponse> businessAppListResList = applyAmountListRequest.getBusinessAppListResList();
        try {
            //导出excel文件
            EasyExcelUtil.download(response, BusinessApplicationExportResponse.class, businessAppListResList,"业务申请列表","第一页");
        } catch (IOException e) {
            e.printStackTrace();

        }
    }

    /**
     * 根据业务id审核业务申请内容
     * @param approvalBusinessReq
     * @return
     */
    @Log(detail = "审核业务申请",level = 4,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.UPDATE)
    @PostMapping("/approvalBusinessApplication")
    @ApiOperation(value = "根据业务id审核业务申请内容")
    public ResponseResult approvalBusinessApplication(@RequestBody ApprovalBusinessRequest approvalBusinessReq){
        Integer id=approvalBusinessReq.getId();
        Integer status=approvalBusinessReq.getStatus();
        String remark=approvalBusinessReq.getRemark();
        Integer companyId=approvalBusinessReq.getCompanyId();
        int result = applyAmountInfoService.approvalBusinessApplication(id,status,remark,companyId);
        return result>0 ? ResponseResult.SUCCESS("审核成功！"):ResponseResult.FAIL(40006,"审核失败！");
    }

    /**
     * 获取当前登录用户信息
     */
    public static String getUserId(){
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:"+token);
    }

    /**
     * 获取当前登录用户名称
     */
    public static String getAccount()  {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        Claims claims = null;
        try {
            claims = JwtUtils.parseJWT(token);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return claims.getIssuer();
    }

    /**
     * 获取当前登录用户名称
     */
    public static Map getUserIdAndName()  {
        Map<String,String> map = new HashMap<String,String>();
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        Claims claims = null;
        try {
            claims = JwtUtils.parseJWT(token);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String userId = RedisUtil.get("dq:token:"+token);

        map.put("name", claims.getIssuer());
        map.put("userId",userId);
        return map;
    }


    /**
     * 获取业务申请所有字段
     */
    @GetMapping("/queryFieldField")
    @ApiOperation(value = "获取业务申请所有字段",notes = "获取业务申请所有字段", response = DgApplyField.class)
    public ResponseResult queryApplyField(){

        return ResponseResult.SUCCESS(applyAmountInfoService.queryApplyField());
    }

    /**
     * 修改业务申请字段
     */
    @ApiOperation(value = "修改业务申请字段")
    @PostMapping("/updateApplyField")
    public ResponseResult updateApplyField(@RequestBody DgApplyField dgApplyField){

        Boolean result = applyAmountInfoService.updateApplyField(dgApplyField);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 根据业务id查询业务申请详情
     * @param id
     * @return
     */
    @Log(detail = "小程序业务申请详情",level = 3,operationUnit = OperationUnit.APPLYAMOUNTINFO,operationType = OperationType.SELECT)
    @GetMapping("/xcxBusinessAppDetail")
    @ApiOperation(value = "小程序业务申请详情")
    public ResponseResult xcxBusinessAppDetail(Integer id){
        BusinessApplicationDetailResponse response = applyAmountInfoService.xcxBusinessAppDetail(id);
        return ResponseResult.SUCCESS(response);
    }
}
