package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.*;
import com.daqing.financial.guarantee.model.response.AssetsDetailResponse;
import com.daqing.financial.guarantee.service.IDgAssetsInvestigationService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;


import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 资产调查 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-23
 */
@Api(tags = {"资产部调查信息"})
@RestController
@RequestMapping("/dg-assets-investigation")
public class DgAssetsInvestigationController {

    @Resource
    private IDgAssetsInvestigationService dgAssetsInvestigationService;

    @Resource
    private DgApplyAmountInfoController dgApplyAmountInfoController;

    /**
     * 资产部调查列表
     * @param
     * @return
     */
    @Log(detail = "资产部调查列表",level = 3,operationUnit = OperationUnit.ASSETS,operationType = OperationType.SELECT)
    @PostMapping("/assetsList")
    @ApiOperation(value = "资产部调查列表")
    public ResponseResult assetsList(@RequestBody AssetsInvestigationRequest assetsInvestigationRequest){
        Integer page=assetsInvestigationRequest.getPage();
        Integer size=assetsInvestigationRequest.getSize();
        String CustomerNumberOrName=assetsInvestigationRequest.getCustomerNumberOrName();
        Integer status=assetsInvestigationRequest.getStatus();
        //获取当前登录用户userId
        String userId = DgApplyAmountInfoController.getUserId();
        //根据角色查询资产部调查列表
        String roleIds = RedisUtil.get("dq:userRole:" + userId);
        PageUtils data = null;
        if(roleIds != null) {
            String[] ids = roleIds.split(",");
            List<String> roleIdlist = Arrays.asList(ids);
            data = dgAssetsInvestigationService.queryPage(page, size, CustomerNumberOrName, roleIdlist, userId, status);

        }
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 导出
     */
    @Log(detail = "导出资产部调查列表",level = 4,operationUnit = OperationUnit.ASSETS,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出资产部调查内容")
    public ResponseResult excelExport(@RequestParam("ids") List<String> ids,@RequestParam("userId")Integer userId, HttpServletResponse response) {

        Boolean result = dgAssetsInvestigationService.excelExport(ids,userId,response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 指派资产部调查员
     * @param
     * @return
     */
    @Log(detail = "指派资产部调查员",level = 4,operationUnit = OperationUnit.ASSETS,operationType = OperationType.UPDATE)
    @PostMapping("/assignCorners")
    @ApiOperation(value = "指派资产部调查员")
    @Transactional
    public ResponseResult assignCorners(@RequestBody AssetsRequest assetsRequest){

        Boolean b = dgAssetsInvestigationService.assignCorners(assetsRequest);
        return new ResponseResult<>().SUCCESS(b);
    }

    /**
     * 根据资产部id查询资产部记录详情
     * @param id
     * @return
     */
    @Log(detail = "资产部调查详情",level = 4,operationUnit = OperationUnit.ASSETS,operationType = OperationType.SELECT)
    @GetMapping("/assetsDetail")
    @ApiOperation(value = "查看详情")
    public ResponseResult assetsDetail(Integer id){
        AssetsDetailResponse response = dgAssetsInvestigationService.assetsDetail(id);
        return ResponseResult.SUCCESS(response);
    }

    /**
     * 根据资产部id调查资产部记录
     * @param investigateAssetsRequest
     * @return
     */
    @Log(detail = "资产部调查",level = 4,operationUnit = OperationUnit.ASSETS,operationType = OperationType.UPDATE)
    @PostMapping("/investigateAssets")
    @ApiOperation(value = "调查")
    public ResponseResult investigateAssets(@RequestBody InvestigateAssetsRequest investigateAssetsRequest){
        int result = dgAssetsInvestigationService.investigateGuarantee(investigateAssetsRequest);
        return result>0 ? ResponseResult.SUCCESS("调查成功！"):ResponseResult.FAIL(41001,"调查失败！");
    }

    /**
     * 根据资产部id审核资产部记录，包括资产部经理审核以及资产部分管领导审核
     * @param approvalAssetsRequest
     * @return
     */
    @Log(detail = "资产部调查审核",level = 4,operationUnit = OperationUnit.ASSETS,operationType = OperationType.UPDATE)
    @PostMapping("/approvalAssets")
    @ApiOperation(value = "审核")
    public ResponseResult approvalAssets(@RequestBody ApprovalAssetsRequest approvalAssetsRequest){
        int result = dgAssetsInvestigationService.approvalAssets(approvalAssetsRequest);
        return result>0 ? ResponseResult.SUCCESS("审核成功！"):ResponseResult.FAIL(41002,"审核失败！");
    }

}
