package com.daqing.financial.guarantee.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.model.response.ApprovalNum;
import com.daqing.financial.guarantee.model.response.TodayApprovalRecordResponse;
import com.daqing.financial.guarantee.service.IDgAuditProcessService;
import com.daqing.financial.guarantee.util.R;
import com.daqing.financial.guarantee.util.TaskNodeUtil;
import com.daqing.framework.domain.guarantee.DgAuditProcess;
import com.daqing.framework.domain.guarantee.po.ProcessPO;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.LinkedHashMap;
import java.util.List;

/**
 * <p>
 * 审核流程进程 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Api(tags = {"业务流程进程"})
@RestController
@RequestMapping("/dg-audit-process")
public class DgAuditProcessController {

    @Autowired
    private IDgAuditProcessService auditProcessService;

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @ApiOperation(value = "审核流程进程列表")
    @GetMapping("/auditProcessList")
    public ResponseResult processList(
            @ApiParam(name = "businessId", value = "业务id", required = true)
            @RequestParam String businessId,
            @ApiParam(name = "processId", value = "流程id", required = false)
            @RequestParam(required = false) String processId){
        List<DgAuditProcess> processList = auditProcessService.queryProcessList(businessId,processId);
        ResponseResult responseResult = hrmsFeignService.roleList();

        List<LinkedHashMap> message = null;
        if(responseResult.getData() != null){
            message = (List<LinkedHashMap>) responseResult.getData();
        }
        for (DgAuditProcess process:processList){
            for(LinkedHashMap res : message){
                if(process.getRoleId().equals(res.get("id"))){//如果id相同情况下，就往对象里面赋值
                    process.setDeptName(JSONObject.toJSONString(res.get("name")).replace("\"",""));
                }
            }
        }
        return ResponseResult.SUCCESS(processList);
    }


    @ApiOperation(value = "审核流程进程")
    @GetMapping("/getProcessId")
    public ResponseResult getProcessId(
            @ApiParam(name = "businessId", value = "业务id", required = true)
            @RequestParam String businessId){
        List<Integer> processIds = auditProcessService.getProcessId(businessId);
        QueryWrapper<DgAuditProcess> wrapper = new QueryWrapper<>();
        wrapper.eq("business_id",businessId).orderByDesc("create_time").last("limit 1");
        DgAuditProcess process = auditProcessService.getOne(wrapper);
        Integer processId = process.getProcessId();
        ProcessPO processPO = new ProcessPO();
        processPO.setProcessIds(processIds);
        processPO.setProcessId(processId);
        return ResponseResult.SUCCESS(processPO);
    }

    @ApiOperation(value = "今日审批记录",response = TodayApprovalRecordResponse.class)
    @GetMapping("todayApprovalRecord")
    public R nowadaysAuditProcess(){
        List<TodayApprovalRecordResponse> approvalRecords = auditProcessService.todayApprovalRecord();
        ApprovalNum approvalNum = new ApprovalNum();
        //统计状态个数
        approvalRecords.forEach(todayApprovalRecordResponse -> {
            if (todayApprovalRecordResponse.getStatus()!=null){
                String taskNode = todayApprovalRecordResponse.getTaskNode();
                if (taskNode!=null){
                    String converter = TaskNodeUtil.converter(taskNode);
                    todayApprovalRecordResponse.setTaskNode(converter);
                }
                switch (todayApprovalRecordResponse.getStatus()){
                    case "0":
                        approvalNum.setInitiateNum(approvalNum.getInitiateNum()+1);
                        break;
                    case "1":
                        approvalNum.setAgreed(approvalNum.getAgreed()+1);
                        break;
                    case "2":
                        approvalNum.setAgreed(approvalNum.getAgreed()+1);
                        break;
                    case "3":
                        approvalNum.setRejected(approvalNum.getRejected()+1);
                        break;
                    case "4":
                        approvalNum.setTurnDown(approvalNum.getTurnDown()+1);
                        break;
                }
            }
        });
        return R.ok().data("approvalRecords",approvalRecords).data("approvalNum",approvalNum);
    }

}
