package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.DgComplianceInvestigationCheckRequest;
import com.daqing.financial.guarantee.model.request.DgComplianceInvestigationRequest;
import com.daqing.financial.guarantee.model.response.DgComplianceInvestigationInfoResponse;
import com.daqing.financial.guarantee.model.response.DgComplianceInvestigationResponse;
import com.daqing.financial.guarantee.service.IDgComplianceInvestigationService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 合规调查 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@RestController
@RequestMapping("/compliance/investigation")
@Api(value = "合规调查", tags = "合规调查")
public class DgComplianceInvestigationController {

    @Autowired
    private IDgComplianceInvestigationService iDgComplianceInvestigationService;

    /**
     * 列表
     */
    @Log(detail = "合规调查列表",level = 3,operationUnit = OperationUnit.COMPLIANCE,operationType = OperationType.SELECT)
    @GetMapping("/list")
    @ApiOperation(value = "列表", notes = "列表", response = DgComplianceInvestigationResponse.class)
    public ResponseResult list(@RequestParam("page") Integer page,
                               @RequestParam("size") Integer size, DgComplianceInvestigationRequest dgComplianceInvestigationRequest) {
        return ResponseResult.SUCCESS(iDgComplianceInvestigationService.list(page, size, dgComplianceInvestigationRequest));
    }

    /**
     * 指派
     */
    @Log(detail = "指派法规C角",level = 4,operationUnit = OperationUnit.COMPLIANCE,operationType = OperationType.UPDATE)
    @GetMapping("/assign")
    @ApiOperation(value = "指派", notes = "指派")
    public ResponseResult assign(@RequestParam("iid") Integer iid, @RequestParam("userId") Integer userId) {
        Boolean result = iDgComplianceInvestigationService.assign(iid, userId);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 调查
     */
    @Log(detail = "合规调查",level = 4,operationUnit = OperationUnit.COMPLIANCE,operationType = OperationType.UPDATE)
    @PostMapping("/survey")
    @ApiOperation(value = "调查", notes = "调查")
    public ResponseResult survey(@RequestBody DgComplianceInvestigationCheckRequest dgComplianceInvestigationCheckRequest) {
        Boolean result = iDgComplianceInvestigationService.survey(dgComplianceInvestigationCheckRequest.getIid(),
                iDgComplianceInvestigationService.getFileUrl(dgComplianceInvestigationCheckRequest.getFileUrls()),
                dgComplianceInvestigationCheckRequest.getOpinion(), dgComplianceInvestigationCheckRequest.getStatus());
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 审核
     */
    @Log(detail = "合规调查审核",level = 4,operationUnit = OperationUnit.COMPLIANCE,operationType = OperationType.UPDATE)
    @ApiOperation(value = "审核", notes = "审核")
    @PostMapping("/check")
    public ResponseResult check(@RequestBody DgComplianceInvestigationCheckRequest dgComplianceInvestigationCheckRequest) {
        Boolean result = iDgComplianceInvestigationService.check(dgComplianceInvestigationCheckRequest.getIid(),
                dgComplianceInvestigationCheckRequest.getOpinion(), dgComplianceInvestigationCheckRequest.getStatus());
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 查看
     */
    @Log(detail = "合规调查详情",level = 4,operationUnit = OperationUnit.COMPLIANCE,operationType = OperationType.SELECT)
    @GetMapping("/query")
    @ApiOperation(value = "查看", notes = "查看", response = DgComplianceInvestigationInfoResponse.class)
    public ResponseResult query(@RequestParam("iid") Integer iid) {

        return ResponseResult.SUCCESS(iDgComplianceInvestigationService.query(iid));
    }

    /**
     * 导出
     */
    @Log(detail = "导出合规调查",level = 4,operationUnit = OperationUnit.COMPLIANCE,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出", notes = "导出")
    public ResponseResult query(HttpServletResponse response, @RequestParam("ids") List<String> ids,@RequestParam("userId")Integer userId) {
        Boolean result = iDgComplianceInvestigationService.excelExport(response, ids,userId);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }
}
