package com.daqing.financial.guarantee.controller;

import com.alibaba.fastjson.JSONObject;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.model.request.CopyForMeRequest;
import com.daqing.financial.guarantee.model.request.DgEfficiencyTeamRequest;
import com.daqing.financial.guarantee.model.request.PersonalEfficiencyListRequest;
import com.daqing.financial.guarantee.model.response.BusinessApplicationListResponse;
import com.daqing.financial.guarantee.model.response.CopyForMeResponse;
import com.daqing.financial.guarantee.model.response.NumResponse;
import com.daqing.financial.guarantee.model.response.PersonalEfficiencyListResponse;
import com.daqing.financial.guarantee.service.IDgCopyUserService;
import com.daqing.financial.guarantee.service.IDgEfficiencyService;
import com.daqing.financial.guarantee.util.DateUtils;
import com.daqing.financial.guarantee.util.R;
import com.daqing.financial.guarantee.util.TaskNodeUtil;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import com.daqing.framework.xss.SQLFilter;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 流程效率
 *
 * @auther River
 * @date 2020/11/5 14:20
 */
@RestController
@RequestMapping("/guarantee/efficiency")
@Api(value = "流程效率", tags = "流程效率")
public class DgEfficiencyController {

    @Autowired
    private IDgEfficiencyService dgEfficiencyService;

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Autowired
    private IDgCopyUserService copyUserService;

    /**
     * 团队效率
     */
    @Log(detail = "团队效率",level = 3,operationUnit = OperationUnit.EFFICIENCY,operationType = OperationType.SELECT)
    @PostMapping("/team")
    @ApiOperation(value = "团队效率", notes = "团队效率")
    public ResponseResult queryList(@RequestBody DgEfficiencyTeamRequest dgEfficiencyTeamRequest) {
        return ResponseResult.SUCCESS(dgEfficiencyService.queryList(dgEfficiencyTeamRequest));
    }

    /**
     * 统计不同业务状态下业务数量
     */
    @GetMapping("/businessCount")
    @ApiOperation(value = "统计不同业务状态下业务数量", notes = "统计不同业务状态下业务数量")
    public ResponseResult getBusinessCount() {

        return ResponseResult.SUCCESS(dgEfficiencyService.getBusinessCount());
    }

    /**
     * 首页-近十天的审批数目
     */
    @Log(detail = "首页-近十天的审批数目",level = 3,operationUnit = OperationUnit.EFFICIENCY,operationType = OperationType.SELECT)
    @GetMapping("/approvalNum")
    public ResponseResult approvalNum() {
        //查询审批数目的值的数组
        Map result = dgEfficiencyService.approvalNum();
        return ResponseResult.SUCCESS(result);
    }

    /**
     * 首页-当前月份每天新建审批数量
     */
    @GetMapping("/getNewBusinessCount")
    @ApiOperation(value = "当前月份每天新建审批数量")
    public ResponseResult getNewBusinessCount() {

        return ResponseResult.SUCCESS(dgEfficiencyService.getNewBusinessCount());
    }

    /**
     * 首页-今天新建审批数量并按员工和客户分类
     */
    @GetMapping("/getTodayBusinessCount")
    @ApiOperation(value = "今天新建审批数量并按员工和客户分类")
    public ResponseResult getTodayBusinessCount() {

        return ResponseResult.SUCCESS(dgEfficiencyService.getTodayBusinessCount());
    }

    /**
     * 首页-通知
     * @Param title 标题
     */
    @Log(detail = "首页-通知",level = 3,operationUnit = OperationUnit.EFFICIENCY,operationType = OperationType.SELECT)
    @GetMapping("/notice")
    public ResponseResult notice() {
        //查询审批数目的值的数组
        List result = dgEfficiencyService.notice();
        return ResponseResult.SUCCESS(result);
    }

    @Log(detail = "个人效率记录列表",level = 3,operationUnit = OperationUnit.EFFICIENCY,operationType = OperationType.SELECT)
    @ApiOperation(value = "个人效率记录列表")
    @PostMapping("/personalEfficiencyList")
    public R personalEfficiencyList(
            @ApiParam(name = "personalEfficiencyListRequest", value = "个人效率列表查询对象")
            @RequestBody PersonalEfficiencyListRequest personalEfficiencyListRequest) {
        List<PersonalEfficiencyListResponse> responseList = dgEfficiencyService.queryPersonalEfficiencyList();

        if (responseList.size() > 0) {
            List<Integer> arr = new ArrayList<>();
            for (PersonalEfficiencyListResponse res : responseList) {
                arr.add(res.getApplicantId());
            }
            //根据提单人id查询其部门名称
            ResponseResult responseResult = hrmsFeignService.getAccountAndDeptNameById(arr);
            List<LinkedHashMap> employeeMessage = null;
            if (responseResult.getData() != null) {
                employeeMessage = (List<LinkedHashMap>) responseResult.getData();
            }

            for (PersonalEfficiencyListResponse response : responseList) {
                for (LinkedHashMap res : employeeMessage) {
                    if (response.getApplicantId().equals(res.get("id"))) {//如果提单人id相同情况下，就往对象里面赋值
                        response.setApplicant(JSONObject.toJSONString(res.get("account")).replace("\"", ""));
                    }
                }
            }
        }

        // 时间筛选(固定时间)，先根据固定时间筛选，再根据自选时间筛选
        if (personalEfficiencyListRequest.getCreateFixedTime() != null ) {
            switch (personalEfficiencyListRequest.getCreateFixedTime()) {
                case 0:
                    personalEfficiencyListRequest.setStartTime(DateUtils.getDayBegin());
                    personalEfficiencyListRequest.setEndTime(DateUtils.getDayEnd());// 今天
                    break;
                case 1:
                    personalEfficiencyListRequest.setStartTime(DateUtils.getBeginDayOfYesterday());
                    personalEfficiencyListRequest.setEndTime(DateUtils.getEndDayOfYesterDay());// 昨天
                    break;
                case 2:
                    personalEfficiencyListRequest.setStartTime(DateUtils.getBeginDayOfWeek());
                    personalEfficiencyListRequest.setEndTime(DateUtils.getEndDayOfWeek());// 本周
                    break;
                case 3:
                    personalEfficiencyListRequest.setStartTime(DateUtils.getBeginDayOfMonth());
                    personalEfficiencyListRequest.setEndTime(DateUtils.getEndDayOfMonth());// 本月
                    break;
                case 4:
                    personalEfficiencyListRequest.setStartTime(DateUtils.getBeginDayOfYear());
                    personalEfficiencyListRequest.setEndTime(DateUtils.getEndDayOfYear());// 本年
                    break;
            }
        //自选时间筛选
        }else if (personalEfficiencyListRequest.getStartTime() != null && personalEfficiencyListRequest.getEndTime() != null){
            personalEfficiencyListRequest.setStartTime(personalEfficiencyListRequest.getStartTime());
            personalEfficiencyListRequest.setEndTime(personalEfficiencyListRequest.getEndTime());
        }else {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date = null;
            try {
                date = sdf.parse("2020-04-16 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            personalEfficiencyListRequest.setStartTime(date);
            personalEfficiencyListRequest.setEndTime(new Date());
        }
        //获取毫秒数
        long longStart = personalEfficiencyListRequest.getStartTime().getTime();
        long longEnd = personalEfficiencyListRequest.getEndTime().getTime();
        //迭代器进行筛选(过滤不在指定范围的时间数据)
        Iterator<PersonalEfficiencyListResponse> iterator = responseList.iterator();
        while (iterator.hasNext()) {
            PersonalEfficiencyListResponse next = iterator.next();
            String applicationDate = next.getApplicationDate();
            long longDate = getLongDate(applicationDate);
            //当前日期小于开始日期或当前日期大于结束日期，删除该元素
            if (longDate < longStart || longDate > longEnd) {
                iterator.remove();
            }
        }

        String userId = DgApplyAmountInfoController.getUserId();
        CopyForMeRequest copyForMeRequest = new CopyForMeRequest();
        copyForMeRequest.setUserId(Integer.parseInt(userId));
        Integer total = copyUserService.queryTotal(copyForMeRequest);
        NumResponse numResponse = new NumResponse();
        numResponse.setCopySendNum(total);
        //转换任务节点，并统计操作状态个数
        responseList.forEach(personalEfficiencyListResponse -> {
            //耗时为空，默认为0
            if (StringUtils.isEmpty(personalEfficiencyListResponse.getTimeConsuming())){
                personalEfficiencyListResponse.setTimeConsuming("0天0小时0分钟");
            }
            //提单人为空，默认赋值客户
            if (StringUtils.isEmpty(personalEfficiencyListResponse.getApplicant())){
                personalEfficiencyListResponse.setApplicant("客户");
            }
            String taskNode = personalEfficiencyListResponse.getTaskNode();
            if (taskNode != null) {
                String converter = TaskNodeUtil.converter(taskNode);
                personalEfficiencyListResponse.setTaskNode(converter);
            }
            if (personalEfficiencyListResponse.getOperatingStatus() != null) {
                if (personalEfficiencyListResponse.getOperatingStatus()==0){
                    numResponse.setInitiateNum(numResponse.getInitiateNum() + 1);
                }else if (personalEfficiencyListResponse.getOperatingStatus()==1){
                    numResponse.setPendingNum(numResponse.getPendingNum() + 1);
                }else if (personalEfficiencyListResponse.getOperatingStatus()==2){
                    numResponse.setProcessedNum(numResponse.getProcessedNum() + 1);
                }
            }
        });
        //根据条件筛选数据
        //迭代器进行筛选
        if (personalEfficiencyListRequest.getOperatingStatus() != null) {
            // 已发起
            if (personalEfficiencyListRequest.getOperatingStatus().equals(StatusCode.CZ_LAUNCH)) {
                responseList.removeIf(next -> !next.getOperatingStatus().equals(StatusCode.CZ_LAUNCH));
                // 待处理
            } else if (personalEfficiencyListRequest.getOperatingStatus().equals(StatusCode.CZ_ON_HAND)) {
                responseList.removeIf(next -> !next.getOperatingStatus().equals(StatusCode.CZ_ON_HAND));
                // 已处理
            } else if (personalEfficiencyListRequest.getOperatingStatus().equals(StatusCode.CZ_PROCESSED)) {
                responseList.removeIf(next -> !next.getOperatingStatus().equals(StatusCode.CZ_PROCESSED));
            }
        }

        //按时间降序排列
        List<PersonalEfficiencyListResponse> collect = responseList.stream().sorted((o1, o2) -> {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date dt1 = format.parse(o1.getApplicationDate());
                Date dt2 = format.parse(o2.getApplicationDate());
                // 默认降序
                return Long.compare(dt2.getTime(), dt1.getTime());
            } catch (Exception e) {
                e.printStackTrace();
            }
            return 0;
        }).collect(Collectors.toList());
        return R.ok().data("responseList", collect).data("statistics", numResponse);
    }

    public static long getLongDate(String stringDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            Date date = sdf.parse(stringDate);
            return date.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return 0L;
    }

    @Log(detail = "个人效率记录列表导出",level = 3,operationUnit = OperationUnit.EFFICIENCY,operationType = OperationType.SELECT)
    @ApiOperation(value = "个人效率记录列表导出")
    @GetMapping("/personalEfficiencyListExport")
    public void personalEfficiencyListExport(HttpServletResponse httpServletResponse) {
        List<PersonalEfficiencyListResponse> responseList = dgEfficiencyService.queryPersonalEfficiencyList();
        if (responseList.size() > 0) {
            List<Integer> arr = new ArrayList<>();
            for (PersonalEfficiencyListResponse res : responseList) {
                arr.add(res.getApplicantId());
            }
            //根据提单人id查询其部门名称
            ResponseResult responseResult = hrmsFeignService.getAccountAndDeptNameById(arr);
            List<LinkedHashMap> employeeMessage = null;
            if (responseResult.getData() != null) {
                employeeMessage = (List<LinkedHashMap>) responseResult.getData();
            }

            if(employeeMessage!= null && employeeMessage.size()>0){
                for (PersonalEfficiencyListResponse response : responseList) {
                    for (LinkedHashMap res : employeeMessage) {
                        if (response.getApplicantId().equals(res.get("id"))) {//如果提单人id相同情况下，就往对象里面赋值
                            response.setApplicant(JSONObject.toJSONString(res.get("account")).replace("\"", ""));
                        }
                        if(response != null){
                            if(response.getApplicant()==null){
                                response.setApplicant("客户");
                            }
                        }
                    }
                }
            }else{
                for (PersonalEfficiencyListResponse response:responseList) {
                    if(response != null){
                        if(response.getApplicant()==null){
                            response.setApplicant("客户");
                        }
                    }
                }
            }

        }
        try {
            EasyExcelUtil.download(httpServletResponse, PersonalEfficiencyListResponse.class, responseList, "个人效率记录列表", "第一页");
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

}
