package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.ApprovalGuaranteeRequest;
import com.daqing.financial.guarantee.model.request.GuaranteeRequest;
import com.daqing.financial.guarantee.model.request.InvestigateGuaranteeRequest;
import com.daqing.financial.guarantee.model.response.GuaranteeDetailResponse;
import com.daqing.financial.guarantee.service.IDgGuaranteeAssignUserService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 担保部记录 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Api(tags = {"担保部调查信息"})
@RestController
@RequestMapping("/dg-guarantee-assign-user")
public class DgGuaranteeAssignUserController {

    @Autowired
    private DgApplyAmountInfoController dgApplyAmountInfoController;

    @Autowired
    private IDgGuaranteeAssignUserService dgGuaranteeAssignUserService;

    /**
     * 担保部调查列表
     * @param
     * @return
     */
    @Log(detail = "担保部调查列表",level = 3,operationUnit = OperationUnit.GUARANTEE,operationType = OperationType.SELECT)
    @GetMapping("/guaranteeList")
    @ApiOperation(value = "担保部调查列表")
    public ResponseResult businessApplicationList(@RequestParam(value="page",required=false) Integer page, @RequestParam(value="size",required=false) Integer size,
                                                  @RequestParam(value = "CustomerNumberOrName", required = false) String CustomerNumberOrName,
                                                  @RequestParam(value = "status", required = false) Integer status){
        //获取当前登录用户userId
        String userId = DgApplyAmountInfoController.getUserId();
        //根据角色查询担保部调查列表
        String roleIds = RedisUtil.get("dq:userRole:" + userId);
        PageUtils data = null;
        if(roleIds != null) {
            String[] ids = roleIds.split(",");
            List<String> roleIdlist = Arrays.asList(ids);

            data = dgGuaranteeAssignUserService.queryPage(page, size, CustomerNumberOrName, roleIdlist, userId, status);
        }

        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 导出
     */
    @Log(detail = "导出担保部调查列表",level = 4,operationUnit = OperationUnit.GUARANTEE,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出担保部调查内容")
    public ResponseResult excelExport(@RequestParam("ids") List<String> ids,@RequestParam("userId")Integer userId, HttpServletResponse response) {

        Boolean result = dgGuaranteeAssignUserService.excelExport(ids,userId,response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 指派A,B角
     * @param
     * @return
     */
    @Log(detail = "指派担保部调查员",level = 4,operationUnit = OperationUnit.GUARANTEE,operationType = OperationType.UPDATE)
    @PostMapping("/assignCorners")
    @ApiOperation(value = "指派A,B角")
    @Transactional
    public ResponseResult assignCorners(@RequestBody GuaranteeRequest guaranteeRequest){

        return dgGuaranteeAssignUserService.assignCorners(guaranteeRequest);
    }


    /**
     * 根据担保id查询担保部记录详情
     * @param id
     * @return
     */
    @Log(detail = "担保部调查详情",level = 4,operationUnit = OperationUnit.GUARANTEE,operationType = OperationType.SELECT)
    @GetMapping("/guaranteeDetail")
    @ApiOperation(value = "根据担保id查询担保部记录详情")
    public ResponseResult guaranteeDetail(Integer id){
        GuaranteeDetailResponse response = dgGuaranteeAssignUserService.guaranteeDetail(id);
        return ResponseResult.SUCCESS(response);
    }

    /**
     * 根据担保id调查担保记录
     * @param investigateGuaranteeRequest
     * @return
     */
    @Log(detail = "担保部调查",level = 4,operationUnit = OperationUnit.GUARANTEE,operationType = OperationType.UPDATE)
    @PostMapping("/investigateGuarantee")
    @ApiOperation(value = "根据担保id调查担保记录")
    public ResponseResult investigateGuarantee(@RequestBody InvestigateGuaranteeRequest investigateGuaranteeRequest){
        int result = dgGuaranteeAssignUserService.investigateGuarantee(investigateGuaranteeRequest);
        return result>0 ? ResponseResult.SUCCESS("调查成功！"):ResponseResult.FAIL(41001,"调查失败！");
    }

    /**
     * 根据担保id审核担保记录，包括担保部经理审核以及担保部分管领导审核
     * @param approvalGuaranteeRequest
     * @return
     */
    @Log(detail = "担保部调查审核",level = 4,operationUnit = OperationUnit.GUARANTEE,operationType = OperationType.UPDATE)
    @PostMapping("/approvalGuarantee")
    @ApiOperation(value = "根据担保id审核担保记录，包括担保部经理审核以及担保部分管领导审核")
    public ResponseResult approvalGuarantee(@RequestBody ApprovalGuaranteeRequest approvalGuaranteeRequest){
        int result = dgGuaranteeAssignUserService.approvalGuarantee(approvalGuaranteeRequest);
        return result>0 ? ResponseResult.SUCCESS("审核成功！"):ResponseResult.FAIL(41002,"审核失败！");
    }
}
