package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.GuaranteeLetterQueryRequest;
import com.daqing.financial.guarantee.model.request.GuaranteeLetterUpdateStatusRequest;
import com.daqing.financial.guarantee.model.response.GuaranteeLetterDetailRes;
import com.daqing.financial.guarantee.model.response.GuaranteeLetterListResponse;
import com.daqing.financial.guarantee.service.IDgGuaranteeLetterAssignUserService;
import com.daqing.financial.guarantee.util.R;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.guarantee.DgGuaranteeLetterAssignUser;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.DateUtil;
import com.daqing.framework.utils.PageUtils;
import com.deepoove.poi.XWPFTemplate;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.HashMap;

/**
 * <p>
 * 担保函记录 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-12
 */
@Api(tags = {"担保函记录"})
@RestController
@RequestMapping("/dg-guarantee-letter-assign-user")
public class DgGuaranteeLetterAssignUserController {

    @Autowired
    private IDgGuaranteeLetterAssignUserService guaranteeLetterAssignUserService;

    @Log(detail = "担保函记录列表",level = 3,operationUnit = OperationUnit.GUARANTEELETTER,operationType = OperationType.SELECT)
    @ApiOperation(value = "担保函记录列表",response = GuaranteeLetterListResponse.class)
    @PostMapping("/guaranteeLetterList")
    public ResponseResult guaranteeLetterList(
            @ApiParam(name = "guaranteeLetterRequest", value = "担保函列表查询条件", required = true)
            @RequestBody GuaranteeLetterQueryRequest guaranteeLetterQueryRequest){
        PageUtils data = guaranteeLetterAssignUserService.queryPage(guaranteeLetterQueryRequest);
        new ResponseResult<PageUtils>();
        return ResponseResult.SUCCESS(data);
    }

    @Log(detail = "导出担保函列表数据",level = 4,operationUnit = OperationUnit.GUARANTEELETTER,operationType = OperationType.SELECT)
    @ApiOperation(value = "导出担保函列表数据")
    @GetMapping("/guaranteeLetterListExport")
    public void guaranteeLetterListExport(
            @ApiParam(name = "ids", value = "多个id", required = true)
            @RequestParam("ids") String ids,@RequestParam("userId")Integer userId,HttpServletResponse response){
        try {
            guaranteeLetterAssignUserService.batchExport(response,ids,userId);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Log(detail = "查询担保函详情",level = 4,operationUnit = OperationUnit.GUARANTEELETTER,operationType = OperationType.SELECT)
    @ApiOperation(value = "查询担保函详情")
    @GetMapping("/guaranteeLetterDetail")
    public R guaranteeLetterDetail(
            @ApiParam(name = "id", value = "主键id", required = true)
            @RequestParam("id") Integer id){
        GuaranteeLetterDetailRes res = guaranteeLetterAssignUserService.guaranteeLetterDetail(id);
        return R.ok().data("data",res);
    }

    @Log(detail = "添加担保函记录",level = 4,operationUnit = OperationUnit.GUARANTEELETTER,operationType = OperationType.INSERT)
    @ApiOperation(value = "贷审会通过后添加担保函记录")
    @PostMapping("/addGuaranteeLetter")
    public ResponseResult addGuaranteeLetter(
            @ApiParam(name = "guaranteeLetterAddRequest", value = "担保函对象", required = true)
            @RequestBody DgGuaranteeLetterAssignUser guaranteeLetterAssignUser) {
        boolean result = guaranteeLetterAssignUserService.saveGuaranteeLetter(guaranteeLetterAssignUser);
        return result ? ResponseResult.SUCCESS("添加成功！"):ResponseResult.FAIL(40005,"添加失败！");
    }

    @Log(detail = "修改担保函记录",level = 4,operationUnit = OperationUnit.GUARANTEELETTER,operationType = OperationType.UPDATE)
    @ApiOperation(value = "更新角色或用户确认状态及审核意见")
    @PostMapping("/updateStatus")
    public ResponseResult updateStatus(
            @ApiParam(name = "guaranteeLetterUpdateStatusRequest", value = "更新确认状态请求", required = true)
            @RequestBody GuaranteeLetterUpdateStatusRequest guaranteeLetterUpdateStatusRequest) {
        //如果为驳回操作且无审核意见，抛出异常
        if (StatusCode.SP_REJECT.equals(guaranteeLetterUpdateStatusRequest.getStatus()) && guaranteeLetterUpdateStatusRequest.getAuditOpinion()==null){
            return ResponseResult.FAIL(44444,"请填写审核意见！");
        }
        boolean result = guaranteeLetterAssignUserService.updateGuaranteeLetterStatus(guaranteeLetterUpdateStatusRequest);
        return result ? ResponseResult.SUCCESS("确认成功！"):ResponseResult.FAIL(40005,"确认失败！");
    }

    @Log(detail = "导出担保函",level = 4,operationUnit = OperationUnit.GUARANTEELETTER,operationType = OperationType.SELECT)
    @ApiOperation(value = "导出担保函")
    @GetMapping("/exportGuaranteeLetter")
    public void guaranteeLetterList(
            @ApiParam(name = "businessId", value = "业务id", required = true)
            @RequestParam String businessId,
            @ApiParam(name = "roleId", value = "角色id", required = true)
            @RequestParam String roleId,HttpServletResponse httpServletResponse){
        GuaranteeLetterListResponse response = guaranteeLetterAssignUserService.selectGuaranteeLetter(businessId,roleId);
        String bank = response.getBank();
        String name = response.getName();
        Double loanMoney = response.getLoanMoney();
        String loanTern = response.getLoanTern();
        Date passingTime = response.getPassingTime();
        String time = DateUtil.dateFormat(passingTime);
        //本地测试开启
        // XWPFTemplate template = XWPFTemplate.compile("D:\\template\\template.docx").render(
        //服务器测试
        XWPFTemplate template = XWPFTemplate.compile("//usr//local//template//template.docx").render(
                new HashMap<String, Object>(){{
                    put("bankName", bank);
                    put("clientName", name);
                    put("passTime", time);
                    put("loanMoney", loanMoney);
                    put("loanTerm", loanTern);
                }});
        try {
            httpServletResponse.setContentType("application/msword");
            String filePath = name + "担保函申请.docx";
            String fileName = new String(filePath.getBytes(), StandardCharsets.ISO_8859_1);
            httpServletResponse.addHeader("Content-Disposition", "filename=" + fileName);
            template.write(httpServletResponse.getOutputStream());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
