package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.DgLoanCommitteeConsiderRequest;
import com.daqing.financial.guarantee.model.response.DgLoanCommitteeConsiderResponse;
import com.daqing.financial.guarantee.service.IDgLoanCommitteeConsiderService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 贷审会审议 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@RestController
@RequestMapping("/committee/consider")
@Api(value = "贷审会", tags = "贷审会")
public class DgLoanCommitteeConsiderController {

    @Autowired
    private IDgLoanCommitteeConsiderService dgLoanCommitteeConsiderService;

    /**
     * 列表
     */
    @Log(detail = "贷审会列表",level = 3,operationUnit = OperationUnit.LOANCOMMITTEE,operationType = OperationType.SELECT)
    @GetMapping("/list")
    @ApiOperation(value = "列表", notes = "列表", response = DgLoanCommitteeConsiderResponse.class)
    public ResponseResult list(@RequestParam("page") Integer page,
                               @RequestParam("size") Integer size, String codeOrName,@RequestParam(value = "status", required = false) Integer status) {
        return ResponseResult.SUCCESS(dgLoanCommitteeConsiderService.list(page, size, codeOrName,status));
    }

    /**
     * 审议
     */
    @Log(detail = "贷审会审议",level = 4,operationUnit = OperationUnit.LOANCOMMITTEE,operationType = OperationType.UPDATE)
    @PostMapping("/check")
    @ApiOperation(value = "审议", notes = "审议")
    public ResponseResult check(@RequestBody @Valid DgLoanCommitteeConsiderRequest dgLoanCommitteeConsiderRequest) {
        Boolean result = dgLoanCommitteeConsiderService.check(dgLoanCommitteeConsiderRequest);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 获取抽取的评委
     */
    @Log(detail = "获取抽取的评委",level = 4,operationUnit = OperationUnit.LOANCOMMITTEE,operationType = OperationType.SELECT)
    @GetMapping("/theJudgesDrawn")
    @ApiOperation(value = "获取抽取的评委", notes = "获取抽取的评委")
    public ResponseResult theJudgesDrawn(@RequestParam("id") Integer id) {
        return ResponseResult.SUCCESS(dgLoanCommitteeConsiderService.getJudges(id));
    }

    /**
     * 查看
     */
    @Log(detail = "贷审会详情",level = 4,operationUnit = OperationUnit.LOANCOMMITTEE,operationType = OperationType.SELECT)
    @GetMapping("/query")
    @ApiOperation(value = "查看", notes = "查看")
    public ResponseResult query(@RequestParam("id") Integer id) {
        return ResponseResult.SUCCESS(dgLoanCommitteeConsiderService.query(id));
    }

    /**
     * 导出
     */
    @Log(detail = "导出贷审会列表",level = 4,operationUnit = OperationUnit.LOANCOMMITTEE,operationType = OperationType.SELECT)
    @GetMapping("excelExport")
    @ApiOperation(value = "导出", notes = "导出")
    public ResponseResult excelExport(HttpServletResponse response, @RequestParam("ids") List<String> ids,@RequestParam("userId")Integer userId) {
        Boolean result = dgLoanCommitteeConsiderService.excelExport(response, ids,userId);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }
}
