package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.*;
import com.daqing.financial.guarantee.model.response.GuaranteeLetterDetailRes;
import com.daqing.financial.guarantee.model.response.LoanNoticeDetailRes;
import com.daqing.financial.guarantee.model.response.LoanNoticeListResponse;
import com.daqing.financial.guarantee.service.IDgLoanNoticeService;
import com.daqing.financial.guarantee.util.R;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.NumberToCN;
import com.daqing.framework.utils.PageUtils;
import com.deepoove.poi.XWPFTemplate;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;

/**
 * <p>
 * 放款通知 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Api(tags = {"放款通知"})
@RestController
@RequestMapping("/dg-loan-notice")
public class DgLoanNoticeController {

    @Autowired
    private IDgLoanNoticeService loanNoticeService;

    @Log(detail = "放款通知记录列表",level = 3,operationUnit = OperationUnit.LOANNOTICE,operationType = OperationType.SELECT)
    @ApiOperation(value = "放款通知记录列表",response = LoanNoticeListResponse.class)
    @PostMapping("/loanNoticeList")
    public ResponseResult loanNoticeList(
            @ApiParam(name = "loanNoticeQueryRequest", value = "放款通知列表查询条件", required = true)
            @RequestBody LoanNoticeQueryRequest loanNoticeQueryRequest){
        PageUtils data = loanNoticeService.queryPage(loanNoticeQueryRequest);
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    @Log(detail = "导出放款通知列表数据",level = 4,operationUnit = OperationUnit.LOANNOTICE,operationType = OperationType.SELECT)
    @ApiOperation(value = "导出放款通知列表数据")
    @GetMapping("/loanNoticeListExport")
    public void loanNoticeListExport(
            @ApiParam(name = "ids", value = "多个id", required = true)
            @RequestParam("ids") String ids,@RequestParam("userId")Integer userId, HttpServletResponse response){
        try {
            loanNoticeService.batchExport(response,ids,userId);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Log(detail = "查询放款通知详情",level = 4,operationUnit = OperationUnit.GUARANTEELETTER,operationType = OperationType.SELECT)
    @ApiOperation(value = "查询放款通知详情")
    @GetMapping("/loanNoticeDetail")
    public R loanNoticeDetail(
            @ApiParam(name = "id", value = "主键id", required = true)
            @RequestParam("id") Integer id){
        LoanNoticeDetailRes res = loanNoticeService.loanNoticeDetail(id);
        return R.ok().data("data",res);
    }

    @Log(detail = "更新放款通知状态",level = 4,operationUnit = OperationUnit.LOANNOTICE,operationType = OperationType.UPDATE)
    @ApiOperation(value = "更新放款通知状态")
    @PostMapping("/updateLoanNotice")
    public ResponseResult updateLoanNotice(
            @ApiParam(name = "loanNoticeUpdateRequest", value = "更新放款通知请求", required = true)
            @RequestBody LoanNoticeUpdateRequest loanNoticeUpdateRequest) {
        //如果为驳回操作且无审核意见，抛出异常
        if (StatusCode.SP_REJECT.equals(loanNoticeUpdateRequest.getStatus()) && loanNoticeUpdateRequest.getAuditOpinion()==null){
            return ResponseResult.FAIL(44444,"请填写审核意见！");
        }
        boolean result = loanNoticeService.updateLoanNotice(loanNoticeUpdateRequest);
        return result ? ResponseResult.SUCCESS("更新成功！"):ResponseResult.FAIL(40005,"更新失败！");
    }

    @Log(detail = "导出放款通知",level = 4,operationUnit = OperationUnit.LOANNOTICE,operationType = OperationType.SELECT)
    @ApiOperation(value = "导出放款通知")
    @GetMapping("/exportLoanNotice")
    public void exportLoanNotice(
            @ApiParam(name = "businessId", value = "业务id", required = true)
            @RequestParam String businessId,
            @ApiParam(name = "roleId", value = "角色id", required = true)
            @RequestParam String roleId,HttpServletResponse httpServletResponse){
        LoanNoticeListResponse response = loanNoticeService.selectLoanNotice(businessId,roleId);
        String bank = response.getBank();
        String name = response.getName();
        Double loanMoney = response.getLoanMoney();
        BigDecimal numberOfMoney = new BigDecimal(loanMoney*10000);
        String money = NumberToCN.number2CNMontrayUnit(numberOfMoney);
        //本地测试开启
        // XWPFTemplate template = XWPFTemplate.compile("D:\\template\\templateTwo.docx").render(
        //服务器测试
        XWPFTemplate template = XWPFTemplate.compile("//usr//local//template//templateTwo.docx").render(
                new HashMap<String, Object>(){{
                    put("bankName", bank);
                    put("clientName", name);
                    put("loanMoney", money);
                    put("money", loanMoney);
                }});
        try {
            httpServletResponse.setContentType("application/msword");
            String filePath = name + "放款通知.docx";
            String fileName = new String(filePath.getBytes(), StandardCharsets.ISO_8859_1);
            httpServletResponse.addHeader("Content-Disposition", "filename=" + fileName);
            template.write(httpServletResponse.getOutputStream());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }



}
