package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.*;
import com.daqing.financial.guarantee.model.response.MessageDetailResponse;
import com.daqing.financial.guarantee.service.IDgMessageInvestigationService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 信息调查 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Api(tags = {"信息部调查信息"})
@RestController
@RequestMapping("/dg-message-investigation")
public class DgMessageInvestigationController {

    @Resource
    private IDgMessageInvestigationService dgMessageInvestigationService;

    @Resource
    private DgApplyAmountInfoController dgApplyAmountInfoController;

    /**
     * 信息部调查列表
     * @param
     * @return
     */
    @Log(detail = "信息部调查列表",level = 3,operationUnit = OperationUnit.MESSAGE,operationType = OperationType.SELECT)
    @GetMapping("/messageList")
    @ApiOperation(value = "信息部调查列表")
    public ResponseResult messageList(MessageInvestigationRequest messageInvestigationRequest){
        Integer page=messageInvestigationRequest.getPage();
        Integer size=messageInvestigationRequest.getSize();
        String CustomerNumberOrName=messageInvestigationRequest.getCustomerNumberOrName();
        Integer status=messageInvestigationRequest.getStatus();

        //获取当前登录用户userId
        String userId = DgApplyAmountInfoController.getUserId();
        //根据角色查询担保部调查列表
        String roleIds = RedisUtil.get("dq:userRole:" + userId);
        PageUtils data = null;
        if(roleIds != null) {
            String[] ids = roleIds.split(",");
            List<String> roleIdlist = Arrays.asList(ids);

            data = dgMessageInvestigationService.queryPage(page, size, CustomerNumberOrName, roleIdlist, userId, status);
        }
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 导出
     */
    @Log(detail = "导出信息部调查列表",level = 4,operationUnit = OperationUnit.MESSAGE,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出信息部调查内容")
    public ResponseResult excelExport(@RequestParam("ids") List<String> ids,@RequestParam("userId")Integer userId, HttpServletResponse response) {

        Boolean result = dgMessageInvestigationService.excelExport(ids,userId,response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 指派信息部调查员
     * @param
     * @return
     */
    @Log(detail = "指派信息部调查员",level = 4,operationUnit = OperationUnit.MESSAGE,operationType = OperationType.INSERT)
    @PostMapping("/assignCorners")
    @ApiOperation(value = "指派信息部调查员")
    @Transactional
    public ResponseResult assignCorners(@RequestBody MessageRequest messageRequest){

        Boolean b = dgMessageInvestigationService.assignCorners(messageRequest);
        return new ResponseResult<>().SUCCESS(b);
    }

    /**
     * 根据信息部id查询信息部记录详情
     * @param id
     * @return
     */
    @Log(detail = "信息部调查详情",level = 4,operationUnit = OperationUnit.MESSAGE,operationType = OperationType.SELECT)
    @GetMapping("/messageDetail")
    @ApiOperation(value = "查看详情")
    public ResponseResult messageDetail(Integer id){
        MessageDetailResponse response = dgMessageInvestigationService.messageDetail(id);
        return ResponseResult.SUCCESS(response);
    }

    /**
     * 根据信息部id调查信息部记录
     * @param investigateMessageRequest
     * @return
     */
    @Log(detail = "信息部调查",level = 4,operationUnit = OperationUnit.MESSAGE,operationType = OperationType.UPDATE)
    @PostMapping("/investigateMessage")
    @ApiOperation(value = "调查")
    public ResponseResult investigateMessage(@RequestBody InvestigateMessageRequest investigateMessageRequest){
        int result = dgMessageInvestigationService.investigateMessage(investigateMessageRequest);
        return result>0 ? ResponseResult.SUCCESS("调查成功！"):ResponseResult.FAIL(41001,"调查失败！");
    }


    /**
     * 根据信息部id审核信息部记录，包括信息部经理审核以及信息部分管领导审核
     * @param approvalMessageRequest
     * @return
     */
    @Log(detail = "信息部调查审核",level = 4,operationUnit = OperationUnit.MESSAGE,operationType = OperationType.UPDATE)
    @PostMapping("/approvalMessage")
    @ApiOperation(value = "审核")
    public ResponseResult approvalMessage(@RequestBody ApprovalMessageRequest approvalMessageRequest){
        int result = dgMessageInvestigationService.approvalMessage(approvalMessageRequest);
        return result>0 ? ResponseResult.SUCCESS("审核成功！"):ResponseResult.FAIL(41002,"审核失败！");
    }

}
