package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.PaymentConfirmationQueryRequest;
import com.daqing.financial.guarantee.model.request.PaymentConfirmationUpdateRequest;
import com.daqing.financial.guarantee.model.response.GuaranteeLetterDetailRes;
import com.daqing.financial.guarantee.model.response.PaymentConfirmationDetailRes;
import com.daqing.financial.guarantee.model.response.PaymentConfirmationListResponse;
import com.daqing.financial.guarantee.service.IDgPaymentConfirmationConsiderService;
import com.daqing.financial.guarantee.util.R;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * <p>
 * 回款确认 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Api(tags = {"财务确认"})
@RestController
@RequestMapping("/dg-payment-confirmation-consider")
public class DgPaymentConfirmationConsiderController {

    @Autowired
    private IDgPaymentConfirmationConsiderService paymentConfirmationConsiderService;

    @Log(detail = "财务确认记录列表",level = 3,operationUnit = OperationUnit.PAYMENTCONFIRMATION,operationType = OperationType.SELECT)
    @ApiOperation(value = "回款确认记录列表",response = PaymentConfirmationListResponse.class)
    @PostMapping("/paymentConfirmationList")
    public ResponseResult paymentConfirmationList(
            @ApiParam(name = "paymentConfirmationQueryRequest", value = "回款确认列表查询条件", required = true)
            @RequestBody PaymentConfirmationQueryRequest paymentConfirmationQueryRequest){
        PageUtils data = paymentConfirmationConsiderService.queryPage(paymentConfirmationQueryRequest);
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    @Log(detail = "导出财务确认记录列表",level = 4,operationUnit = OperationUnit.PAYMENTCONFIRMATION,operationType = OperationType.SELECT)
    @ApiOperation(value = "导出回款确认列表数据")
    @GetMapping("/paymentConfirmationListExport")
    public void paymentConfirmationListExport(
            @ApiParam(name = "ids", value = "多个id", required = true)
            @RequestParam("ids") String ids,@RequestParam("userId")Integer userId, HttpServletResponse response){
        try {
            paymentConfirmationConsiderService.batchExport(response,ids,userId);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Log(detail = "查询财务确认详情",level = 4,operationUnit = OperationUnit.PAYMENTCONFIRMATION,operationType = OperationType.SELECT)
    @ApiOperation(value = "查询财务确认详情")
    @GetMapping("/paymentConfirmationDetail")
    public R paymentConfirmationDetail(
            @ApiParam(name = "id", value = "主键id", required = true)
            @RequestParam("id") Integer id){
        PaymentConfirmationDetailRes res = paymentConfirmationConsiderService.paymentConfirmationDetail(id);
        return R.ok().data("data",res);
    }

    @Log(detail = "更新财务确认",level = 4,operationUnit = OperationUnit.PAYMENTCONFIRMATION,operationType = OperationType.UPDATE)
    @ApiOperation(value = "更新回款确认")
    @PostMapping("/updatePaymentConfirmation")
    public ResponseResult updatePaymentConfirmation(
            @ApiParam(name = "paymentConfirmationUpdateRequest", value = "更新回款确认请求", required = true)
            @RequestBody PaymentConfirmationUpdateRequest paymentConfirmationUpdateRequest) {
        //如果为驳回操作且无审核意见，抛出异常
        if (StatusCode.SP_REJECT.equals(paymentConfirmationUpdateRequest.getStatus()) && paymentConfirmationUpdateRequest.getAuditOpinion()==null){
            return ResponseResult.FAIL(44444,"请填写审核意见！");
        }
        boolean result = paymentConfirmationConsiderService.updatePaymentConfirmation(paymentConfirmationUpdateRequest);
        return result ? ResponseResult.SUCCESS("更新成功！"):ResponseResult.FAIL(40005,"更新失败！");
    }

}
