package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.ProcessManageRequest;
import com.daqing.financial.guarantee.model.request.UpdateUserVisualReq;
import com.daqing.financial.guarantee.service.IDgProcessManageService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.guarantee.DgProcessManage;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;


import java.util.*;

/**
 * <p>
 * 流程管理表 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2021-01-15
 */
@RestController
@Api(tags = {"流程管理"})
@RequestMapping("/dg-process-manage")
public class DgProcessManageController {

    @Autowired
    private DgApplyAmountInfoController dgApplyAmountInfoController;
    @Autowired
    private IDgProcessManageService iDgProcessManageService;

    /**
     * 流程管理列表(可见范围内的人可以查看列表)
     * @param
     * @return
     */
    @Log(detail = "流程管理列表",level = 3,operationUnit = OperationUnit.PROCESSMANAGE,operationType = OperationType.SELECT)
    @GetMapping("/processManageList")
    @ApiOperation(value = "流程管理列表")
    public ResponseResult processManageList(){
        //获取当前登录用户id
        String userId = DgApplyAmountInfoController.getUserId();
        //查询可见范围内的人可见列表
        List<DgProcessManage> list = iDgProcessManageService.processManageList(userId);
        return new ResponseResult<PageUtils>().SUCCESS(list);
    }

    /**
     * 查询审批模板对应的可见人员
     * @param
     * @return
     */
    @Log(detail = "查询审批模板对应的可见人员",level = 3,operationUnit = OperationUnit.PROCESSMANAGE,operationType = OperationType.SELECT)
    @GetMapping("/processVisualList")
    @ApiOperation(value = "查询审批模板对应的可见人员")
    public ResponseResult processVisualList(@RequestParam(value = "id") Integer id){
        //查询可见范围内的人可见列表
        List<EmployeeVO> list = iDgProcessManageService.processVisualList(id);
        return new ResponseResult<PageUtils>().SUCCESS(list);
    }

    /**
     * 修改人员可以看到审批模板
     * @param updateUserVisualReq
     * @return
     */
    @Log(detail = "修改人员可以看到审批模板",level = 3,operationUnit = OperationUnit.PROCESSMANAGE,operationType = OperationType.UPDATE)
    @PostMapping("/updateUserVisual")
    @ApiOperation(value = "修改人员可以看到审批模板")
    @Transactional
    public ResponseResult updateUserVisual(@RequestBody UpdateUserVisualReq updateUserVisualReq){
        Integer id = updateUserVisualReq.getId();
        List<Integer>ids = updateUserVisualReq.getIds();
        Boolean a=false;
        if(ids.size()>0){
            //查询可见范围内的人可见列表
            a= iDgProcessManageService.updateUserVisual(id,ids);
        }else{
            ExceptionCast.cast(HrmsCode.AT_LEAST_ONE);
        }
        return new ResponseResult<PageUtils>().SUCCESS(a);
    }

    /**
     * 修改基础设置
     * @param processManageReq
     * @return
     */
    @Log(detail = "修改基础设置",level = 3,operationUnit = OperationUnit.PROCESSMANAGE,operationType = OperationType.UPDATE)
    @PostMapping("/updateProcessManage")
    @ApiOperation(value = "修改基础设置")
    @Transactional
    public ResponseResult updateProcessManage(@RequestBody ProcessManageRequest processManageReq){
        processManageReq.setUpdateTime(new Date());
        //根据id修改基础设置
        int a= iDgProcessManageService.updateProcessManage(processManageReq);
        return a>0 ? ResponseResult.SUCCESS("修改成功！"):ResponseResult.FAIL(40008,"修改失败！");

    }

    /**
     * 设置超管人员可以看到审批模板
     * @param
     * @return
     */
    @Log(detail = "设置超管人员可以看到审批模板",level = 3,operationUnit = OperationUnit.PROCESSMANAGE,operationType = OperationType.UPDATE)
    @PostMapping("/updateManagerVisual")
    @ApiOperation(value = "设置超管人员可以看到审批模板")
    @Transactional
    public ResponseResult updateManagerVisual(@RequestBody Integer userId){

        boolean a = iDgProcessManageService.queryAllModelId(userId);

        return new ResponseResult<>().SUCCESS(a);
    }

    /**
     * 根据指定用户删除可见审批模板
     * @param
     * @return
     */
    @Log(detail = "根据指定用户删除可见审批模板",level = 3,operationUnit = OperationUnit.PROCESSMANAGE,operationType = OperationType.DELETE)
    @PostMapping("/delUserVisual")
    @ApiOperation(value = "根据指定用户删除可见审批模板")
    @Transactional
    public ResponseResult delUserVisual(@RequestBody Integer userId){

        boolean a = iDgProcessManageService.delUserVisual(userId);

        return new ResponseResult<>().SUCCESS(a);
    }

    /**
     * 批量删除用户可见审批模板
     * @param
     * @return
     */
    @Log(detail = "批量删除用户可见审批模板",level = 3,operationUnit = OperationUnit.PROCESSMANAGE,operationType = OperationType.DELETE)
    @GetMapping("/delBatchUserVisual")
    @ApiOperation(value = "批量删除用户可见审批模板")
    @Transactional
    public ResponseResult delBatchUserVisual(@RequestParam("userId") List<Integer> userId){

        boolean a = iDgProcessManageService.delBatchUserVisual(userId);

        return new ResponseResult<>().SUCCESS(a);
    }

}
