package com.daqing.financial.guarantee.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.model.request.CopySendUserRequest;
import com.daqing.financial.guarantee.service.IDgProcessUserService;
import com.daqing.financial.guarantee.util.R;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.guarantee.DgProcessUser;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * <p>
 * 业务模块选择抄送用户表 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2021-01-15
 */
@Api(tags = {"流程选择抄送人"})
@RestController
@RequestMapping("/dg-process-user")
public class DgProcessUserController {

    @Autowired
    private IDgProcessUserService processUserService;

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Log(detail = "根据进程id查询抄送用户",level = 4,operationUnit = OperationUnit.COPYFOR,operationType = OperationType.SELECT)
    @ApiOperation("根据进程id查询抄送用户")
    @GetMapping("/queryCopySendUser")
    public R queryCopySendUser(
            @ApiParam(name = "processId", value = "进程id")
            @RequestParam Integer processId){
        QueryWrapper<DgProcessUser> wrapper = new QueryWrapper<>();
        wrapper.eq("process_id",processId);
        List<DgProcessUser> list = processUserService.list(wrapper);
        List<Integer> arr = new ArrayList<>();
        for (DgProcessUser res : list) {
            arr.add(res.getUserId());
        }
        //根据提单人id查询其部门名称
        ResponseResult responseResult = hrmsFeignService.getAccountAndDeptNameById(arr);
        List<LinkedHashMap> employeeMessage = null;
        if (responseResult.getData() != null) {
            employeeMessage = (List<LinkedHashMap>) responseResult.getData();
        }
        for (DgProcessUser response : list) {
            for (LinkedHashMap res : employeeMessage) {
                if (response.getUserId().equals(res.get("id"))) {//如果提单人id相同情况下，就往对象里面赋值
                    response.setUserName(JSONObject.toJSONString(res.get("account")).replace("\"", ""));
                }
            }
        }
        return R.ok().data("userList",list);
    }

    @Log(detail = "更新流程的抄送用户",level = 4,operationUnit = OperationUnit.COPYFOR,operationType = OperationType.UPDATE)
    @ApiOperation("更新流程的抄送用户")
    @PostMapping("/updateCopySendUser")
    public R updateCopySendUser(@RequestBody CopySendUserRequest copySendUserRequest){
        boolean save = processUserService.saveCopySendUser(copySendUserRequest);
        return save ? R.ok() : R.error();
    }

}
