package com.daqing.financial.guarantee.controller;

import com.daqing.financial.guarantee.model.request.DgStatisticsRequest;
import com.daqing.financial.guarantee.model.response.DgStatisticsListRefuseResponse;
import com.daqing.financial.guarantee.model.response.DgStatisticsListResponse;
import com.daqing.financial.guarantee.service.IDgStatisticsService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;

/**
 * 业务统计
 *
 * @auther River
 * @date 2020/11/5 14:18
 */
@RestController
@RequestMapping("/guarantee/statistics")
@Api(value = "担保业务统计", tags = "提供业务统计增删改查的方法")
public class DgStatisticsController {

    @Autowired
    private IDgStatisticsService iDgStatisticsService;

    /**
     * 列表
     */
    @Log(detail = "业务统计列表",level = 3,operationUnit = OperationUnit.STATISTICS,operationType = OperationType.SELECT)
    @GetMapping("/list")
    @ApiOperation(value = "业务统计列表", notes = "业务统计列表", response = DgStatisticsListResponse.class)
    public ResponseResult list(@RequestParam("page") Integer page,
                               @RequestParam("size") Integer size, DgStatisticsRequest dgStatisticsRequest) {
        return ResponseResult.SUCCESS(iDgStatisticsService.list(page, size, dgStatisticsRequest));
    }

    /**
     * 列表导出
     */
    @Log(detail = "导出业务统计列表",level = 4,operationUnit = OperationUnit.STATISTICS,operationType = OperationType.SELECT)
    @ApiOperation(value = "统计列表导出", notes = "统计列表导出")
    @GetMapping("/excelExport")
    public ResponseResult excelExport(HttpServletResponse response) {

        Boolean result = iDgStatisticsService.excelExport(response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 拒绝列表
     */
    @Log(detail = "拒绝列表",level = 3,operationUnit = OperationUnit.STATISTICS,operationType = OperationType.SELECT)
    @GetMapping("/listRefuse")
    @ApiOperation(value = "业务统计拒绝列表", notes = "业务统计拒绝列表", response = DgStatisticsListRefuseResponse.class)
    public ResponseResult listRefuse(@RequestParam("page") Integer page,
                                     @RequestParam("size") Integer size, DgStatisticsRequest dgStatisticsRequest) {
        return ResponseResult.SUCCESS(iDgStatisticsService.listRefuse(page, size, dgStatisticsRequest));
    }

    /**
     * 拒绝列表导出
     */
    @Log(detail = "导出拒绝列表",level = 4,operationUnit = OperationUnit.STATISTICS,operationType = OperationType.SELECT)
    @ApiOperation(value = "统计拒绝列表导出", notes = "统计拒绝列表导出")
    @GetMapping("/excelExportRefuse")
    public ResponseResult excelExportRefuse(HttpServletResponse response) {

        Boolean result = iDgStatisticsService.excelExportRefuse(response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL(70001,"导出数据为空！");
    }
}
