package com.daqing.financial.guarantee.controller;


import com.daqing.financial.guarantee.model.request.DgWorkConferenceConsiderRequest;
import com.daqing.financial.guarantee.service.IDgWorkConferenceConsiderService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 工作会审议 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@RestController
@RequestMapping("/work/conference")
@Api(value = "工作会", tags = "工作会")
public class DgWorkConferenceConsiderController {

    @Autowired
    private IDgWorkConferenceConsiderService dgWorkConferenceConsiderService;

    /**
     * 列表
     */
    @Log(detail = "工作会审议列表",level = 3,operationUnit = OperationUnit.WORKCONFERENCE,operationType = OperationType.SELECT)
    @GetMapping("/list")
    @ApiOperation(value = "列表", notes = "列表")
    public ResponseResult list(@RequestParam("page") Integer page,
                               @RequestParam("size") Integer size, String codeOrName,@RequestParam(value = "status", required = false) Integer status) {
        return ResponseResult.SUCCESS(dgWorkConferenceConsiderService.list(page, size, codeOrName,status));
    }

    /**
     * 审议
     */
    @Log(detail = "工作会审议",level = 4,operationUnit = OperationUnit.WORKCONFERENCE,operationType = OperationType.UPDATE)
    @PostMapping("/check")
    @ApiOperation(value = "审议", notes = "审议")
    public ResponseResult check(@RequestBody DgWorkConferenceConsiderRequest dgWorkConferenceConsiderRequest) {
        Boolean result = dgWorkConferenceConsiderService.check(dgWorkConferenceConsiderRequest.getId(),
                dgWorkConferenceConsiderRequest.getOpinion(), dgWorkConferenceConsiderRequest.getStatus(),
                dgWorkConferenceConsiderRequest.getEmpNames());
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 评委抽取
     */
    @Log(detail = "评委抽取",level = 4,operationUnit = OperationUnit.WORKCONFERENCE,operationType = OperationType.INSERT)
    @PostMapping("/theJudgesDrawn")
    @ApiOperation(value = "评委抽取", notes = "评委抽取")
    public ResponseResult theJudgesDrawn(@RequestBody DgWorkConferenceConsiderRequest dgWorkConferenceConsiderRequest) {
        Boolean result = dgWorkConferenceConsiderService.theJudgesDrawn(dgWorkConferenceConsiderRequest.getId(),
                dgWorkConferenceConsiderRequest.getEmpNames());
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 获取所有的审核意见、审核人及时间
     */
    @GetMapping("/getAuditOpinion")
    @ApiOperation(value = "获取所有的审核意见、审核人及时间", notes = "获取所有的审核意见、审核人及时间")
    public ResponseResult getAuditOpinion(@RequestParam("businessId") Integer businessId) {
        return ResponseResult.SUCCESS(dgWorkConferenceConsiderService.getAuditOpinion(businessId));
    }

    /**
     * 查看
     */
    @Log(detail = "工作会详情",level = 4,operationUnit = OperationUnit.WORKCONFERENCE,operationType = OperationType.SELECT)
    @GetMapping("/query")
    @ApiOperation(value = "查看", notes = "查看")
    public ResponseResult query(@RequestParam("id") Integer id) {
        return ResponseResult.SUCCESS(dgWorkConferenceConsiderService.query(id));
    }

    /**
     * 导出
     */
    @Log(detail = "导出工作会列表",level = 4,operationUnit = OperationUnit.WORKCONFERENCE,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @ApiOperation(value = "导出", notes = "导出")
    public ResponseResult excelExport(HttpServletResponse response, @RequestParam("ids") List<String> ids,@RequestParam("userId")Integer userId){
        Boolean result = dgWorkConferenceConsiderService.excelExport(response, ids,userId);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }
}
