package com.daqing.financial.guarantee.feign;

import com.daqing.framework.domain.guarantee.po.DgGuaranteePO;
import com.daqing.framework.domain.guarantee.response.EmployeeMessageResponse;
import com.daqing.framework.domain.hrms.PermissionEntity;
import com.daqing.framework.domain.hrms.RolePermissionEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeTO;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.model.response.ResponseResult;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * 这是一个声明式的远程调用
 */
@FeignClient(value = "dq-financial-hrms", contextId = "guaranteeToHrms")
public interface HrmsFeignService {

    /**
     * 根据客户编号/客户名称获取企业信息
     */
//    @GetMapping("/crms/customer/list")
//    ResponseResult list(@RequestParam Map<String, Object> params);

    /**
     * 根据id集合获取对应的员工id、姓名、部门名称
     */
    @GetMapping("/hrms/employee/getEmployeeAndDeptById") // 远程调用传递集合类型的参数必须以数组封装，不然会导致数据接收不完整
    ResponseResult<List<EmployeeTO>> getEmployeeAndDeptById(@RequestParam("ids") Long[] ids);

    /**
     * 根据userId获取用户名以及部门名称
     */
    @GetMapping("hrms/employee/getAccountAndDeptNameByIds")
    ResponseResult<List<EmployeeTO>> getAccountAndDeptNameByIds(@RequestParam("ids") Long[] ids);

    /**
     * 根据部门id获取该部门下所有员工信息
     */
    @GetMapping("/hrms/dept/listEmployeeByDeptId")
    ResponseResult<List<EmployeeVO>> listEmployeeByDeptId(@RequestParam("deptId") Long deptId);

    /**
     * 根据userId获取用户名以及部门名称
     */
    @PostMapping("hrms/employee/getAccountAndDeptNameById")
    ResponseResult getAccountAndDeptNameById(@RequestBody List<Integer> ids);

    /**
     * 新增用户角色
     */
    @GetMapping("hrms/employee/insertUserRole")
    ResponseResult insertUserRole(@RequestParam("userId") Long userId,@RequestParam("roleId")Long roleId);

    /**
     * 给用户赋予一个角色
     */
    @GetMapping("hrms/employee/saveUserRole")
    ResponseResult saveUserRole(@RequestParam("userId") Long userId,@RequestParam("roleId")Long roleId);

    @PostMapping("hrms/employee/insertUsrRole")
    ResponseResult insertUsrRole(@RequestBody List<DgGuaranteePO> list);

    /**
     * 根据用户id获取对应的员工名称
     */
    @GetMapping("/hrms/employee/listEmpNameByUserId")
    ResponseResult<List<EmployeeVO>> listEmpNameByUserId(@RequestParam("userIds") Integer[] userIds);

    @GetMapping("hrms/employee/selectUserRole")
    ResponseResult selectUserRole(@RequestParam("userId") Long userId,@RequestParam("roleId")Long roleId);

    /**
     * 根据用户id获取所在部门ids
     */
    @GetMapping("/hrms/employee/getDepartmentIds")
    String getDepartmentIds(@RequestParam("userId") Integer userId);

/*    *//**
     * 根据角色id查询所有权限
     *//*
    @GetMapping("/hrms/permission/queryPermissionsByRoleIds")
    ResponseResult queryPermissionsByRoleIds(@RequestParam("ids") Long[] ids);

    *//**
     * 根据主键id查询权限
     * @return
     *//*
    @GetMapping("/hrms/permission/queryPermissionsByIds")
    ResponseResult<List<PermissionEntity>> queryPermissionsByIds(@RequestParam("ids") Integer[] ids);

    @GetMapping("/hrms/permission/info")
    ResponseResult info(@RequestParam("id") Long id);*/

    @GetMapping("/hrms/role/roleList")
    ResponseResult roleList();
}
