package com.daqing.financial.guarantee.model.request;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;

import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Data
@ToString
public class BusinessApplicationRequest implements Serializable {

    /**
     * 主键id
     */
    @ApiModelProperty(value = "id")
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 业务id
     */
    @ApiModelProperty(value = "业务id")
    private Integer businessId;

    /**
     * 企业id
     */
    @ApiModelProperty(value = "企业id")
    private Integer companyId;

    /**
     * 提单人id
     */
    @ApiModelProperty(value = "提单人id")
    private Integer presenterId;

    /**
     * 申请额度
     */
    //@Pattern(regexp = "^(([1-9]{1}\\d*)|(0{1}))(\\.\\d{1,2})?$", message = "申请额度格式错误！")
    @ApiModelProperty(value = "申请额度")
    private BigDecimal applyAmount;

    /**
     * 申请期限
     */
    @ApiModelProperty(value = "申请期限")
    private String applyTime;

    /**
     * 贷款银行（用中划线隔开）
     */
    @ApiModelProperty(value = "贷款银行（用中划线隔开）")
    private String bank;

    /**
     * 贷款用途
     */
    @ApiModelProperty(value = "贷款用途")
    private String amountWide;

    /**
     * 其他贷款用途
     */
    @ApiModelProperty(value = "其他贷款用途")
    private String otherAmountWide;

    /**
     * 反担保措施（中划线隔开）
     */
    @ApiModelProperty(value = "反担保措施（中划线隔开）")
    private String[] enGuaranteeMeasures;

    /**
     * 反担保措施描述
     */
    @ApiModelProperty(value = "反担保措施描述")
    private String description;

    @ApiModelProperty(value = "客户类型")
    private String customerType;

    /**
     * 业务类型
     */
    @ApiModelProperty(value = "业务类型")
    private String businessType;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    private Date createTime;

    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间")
    private Date updateTime;


    /**
     * 营业执照复印件
     */
    @ApiModelProperty(value = "营业执照复印件")
    private String[] businessLicense;

    /**
     * 上传法定代表人夫妻及企业实际经营者身份证复印件
     */
    @ApiModelProperty(value = "上传法定代表人夫妻及企业实际经营者身份证复印件")
    private String[] legalCardCopy;

    /**
     * 法定代表人夫妻户口本，结婚证（离婚证）复印件
     */
    @ApiModelProperty(value = "法定代表人夫妻户口本，结婚证（离婚证）复印件")
    private String[] marriageCopy;

    /**
     * 法定代表人身份证明
     */
    @ApiModelProperty(value = "法定代表人身份证明")
    private String[] legalCopy;

    /**
     * 公司章程
     */
    @ApiModelProperty(value = "公司章程")
    private String[] companyConstitution;

    /**
     * 会计师事务所审计的上一年度及本年度审计报告
     */
    @ApiModelProperty(value = "会计师事务所审计的上一年度及本年度审计报告")
    private String[] accountingFirm;

    /**
     * 企业信用报告
     */
    @ApiModelProperty(value = "企业信用报告")
    private String[] companyCredit;

    /**
     * 企业法人夫妇信用报告，实际经营者信用报告
     */
    @ApiModelProperty(value = "企业法人夫妇信用报告，实际经营者信用报告")
    private String[] creditReport;

    /**
     * 征信业务授权书，承诺书
     */
    @ApiModelProperty(value = "征信业务授权书，承诺书")
    private String[] certificateAuthorization;

    /**
     * 增值税纳税申报表首表复印件/完税证明
     */
    @ApiModelProperty(value = "增值税纳税申报表首表复印件/完税证明")
    private String[] taxCertificate;

    /**
     * 股东会会议纪要
     */
    @ApiModelProperty(value = "股东会会议纪要")
    private String[] meetingMinutes;

    /**
     * 反担保资料和评估报告
     */
    @ApiModelProperty(value = "反担保资料和评估报告")
    private String[] assessmentReport;

    /**
     * 谈话纪要
     */
    @ApiModelProperty(value = "谈话纪要")
    private String[] conversationSummary;

    /**
     * 考察照片
     */
    @ApiModelProperty(value = "考察照片")
    private String[] inspectionPhotos;

    /**
     * 区分保存和提交按钮  1->commit
     */
    @ApiModelProperty(value = "区分保存和提交按钮")
    private Integer isSaveOrCommit;

    /**
     * 关联人id
     */
    @ApiModelProperty(value = "关联人id")
    private List<Integer> relatedId;

    /**
     * 是否存在关联人
     */
    @ApiModelProperty(value = "是否存在关联人")
    private Integer isExistRelated;

    @ApiModelProperty(value = "申请类型：0->员工申请；1->客户申请")
    private Integer applyType;

    /**
     * 注册资金（万元）
     */
    @ApiModelProperty(value = "注册资金（万元）")
    private String registeredCapital;

    /**
     * 社会统一代码
     */
    @ApiModelProperty(value = "社会统一代码")
    private String socialUnifiedCode;

    /**
     * 员工人数
     */
    @ApiModelProperty(value = "员工人数")
    private String empNum;

    /**
     * 联系人
     */
    @ApiModelProperty(value = "联系人")
    private String linkman;

    /**
     * 联系人的联系电话
     */
    @ApiModelProperty(value = "联系人的联系电话")
    private String linkPhone;

    /**
     * 业务来源
     */
    @ApiModelProperty(value = "业务来源")
    private String businessSource;

    /**
     * 经营范围
     */
    @ApiModelProperty(value = "经营范围")
    private String businessScope;

    /**
     * 经营地址
     */
    @ApiModelProperty(value = "经营地址")
    private String businessAddr;

    /**
     * 注册时间
     */
    @ApiModelProperty(value = "注册时间")
    private Date registerTime;

    /**
     * 注册地址
     */
    @ApiModelProperty(value = "注册地址")
    private String registerAddr;

    /**
     * 法人-身份证号
     */
    @ApiModelProperty(value = "法人-身份证号")
    private String legalIdNumber;

    /**
     * 法人-户口所在地
     */
    @ApiModelProperty(value = "法人-户口所在地")
    private String legalHukouAddr;

    /**
     * 法人-联系电话
     */
    @ApiModelProperty(value = "法人-联系电话")
    private String legalPhone;

    /**
     * 法人-家庭住址
     */
    @ApiModelProperty(value = "法人-家庭住址")
    private String legalHomeAddr;

    /**
     * 法人-性别 1、男，0、女
     */
    @ApiModelProperty(value = "法人-性别 1、男，0、女")
    private Integer legalGender;

    /**
     * 法人-姓名
     */
    @ApiModelProperty(value = "法人-姓名")
    private String legalName;

    /**
     * 客户编号
     */
    @ApiModelProperty(value = "客户编号")
    private String code;

    /**
     * 客户类型：1、企业类型，0：个人类型
     */
    @ApiModelProperty(value = "客户类型：1、企业类型，0：个人类型")
    private Integer type;

    /**
     * 客户名称
     */
    @ApiModelProperty(value = "客户名称")
    private String name;

    /**
     * 联系地址(指个人的)
     */
    @ApiModelProperty(value = "联系地址(指个人的)")
    private String addr;

    /**
     * 联系电话（指个人或企业的）
     */
    @ApiModelProperty(value = "联系电话（指个人或企业的）")
    private String phone;

    @ApiModelProperty(value = "客户基本信息表id")
    private Long customerId;

}
