package com.daqing.financial.guarantee.model.response;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.metadata.BaseRowModel;
import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.*;
import com.daqing.framework.utils.excel.RepaymentStatusConverter;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import lombok.ToString;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 还款记录表
 * </p>
 *
 * @author Qyq
 * @since 2021-03-17
 */
@ToString
@Data
@TableName("al_repayment_entry")
public class AlRepaymentRes extends BaseRowModel implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @ExcelIgnore
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 保后外键id
     */
    @ExcelIgnore
    private Integer insuranceId;

    /**
     * 应还款日
     */
    @ExcelProperty(value = "应还款日",index = 0)
    private Date repaymentDate;

    /**
     * 实际还款日
     */
    @ExcelProperty(value = "实际还款日",index = 1)
    private Date actualRepaymentDate;

    @ExcelProperty(value = "担保额度（元）",index = 2)
    private BigDecimal guaranteeAmount;

    /**
     * 逾期天数
     */
    @ExcelProperty(value = "逾期天数",index = 8)
    private Integer overdueDays;

    /**
     * 还款总额（元）
     */
    @ExcelProperty(value = "还款总额（元）",index = 6)
    private BigDecimal totalRepayment;

    /**
     * 本次还款（元）
     */
    @ExcelProperty(value = "本次还款（元）",index = 3)
    private BigDecimal currentRepayment;

    /**
     * 利息（元）
     */
    @ExcelProperty(value = "本次利息（元）",index = 4)
    private BigDecimal interest;

    /**
     * 其他费用（元）
     */
    @ExcelProperty(value = "其他费用（元）",index = 5)
    private BigDecimal otherExpenses;

    /**
     * 减免金额（元）
     */
    @ExcelProperty(value = "减免金额（元）",index = 7)
    private BigDecimal deductionAmount;

    /**
     * 还款备注
     */
    @ExcelProperty(value = "还款备注",index = 12)
    private String repaymentNotes;

    /**
     * 还款状态：1->待还款；2->已还款；3->已逾期；4->未到期；
     */
    @ExcelProperty(value = "状态",index = 9,converter = RepaymentStatusConverter.class)
    private Integer repaymentStatus;

    /**
     * 提交人id
     */
    @ExcelIgnore
    private Integer submitterId;

    /**
     * 提交人名称
     */
    @ExcelProperty(value = "提交人名称",index = 10)
    private String submitterName;

    /**
     * 创建时间
     */
    @ExcelProperty(value = "提交时间",index = 11)
    @TableField(fill= FieldFill.INSERT)
    private Date createTime;

    /**
     * 修改时间
     */
    @ExcelIgnore
    @TableField(fill= FieldFill.INSERT_UPDATE)
    private Date updateTime;
}
