package com.daqing.financial.guarantee.model.response;

import com.baomidou.mybatisplus.annotation.*;
import com.daqing.framework.domain.guarantee.DgAssetsAssessmentAssetsLog;
import com.daqing.framework.domain.guarantee.DgAssetsMoney;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 资产调查
 * </p>
 *
 * @author Qyq
 * @since 2020-11-23
 */
@Data
@TableName("dg_assets_investigation")
public class DgAssetsInvestigationResponse implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 业务id
     */
    private Integer businessId;

    /**
     * 企业id
     */
    private Integer companyId;

    /**
     * 调查者id
     */
    private Integer empId;

    /**
     * 角色id
     */
    private Integer roleId;

    /**
     * 类型（1：第一审批人 2：指派人AB角 3：经理）
     */
    private Integer type;

    /**
     * 状态 0：待指派；1：审核中；2：已审核；3：拒绝；4：驳回；5：已撤销；6：草稿；
     */
    private Integer status;

    /**
     * 操作状态: 0->已发起；1->待处理；2->已处理；
     */
    private Integer operatingStatus;

    /**
     * 资产评估报告（名称）
     */
    private String assetsName;

    /**
     * 资产评估报告编号
     */
    private String assetsCode;

    /**
     * 评估日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date assetsTime;

    /**
     * 评估价值
     */
    private String assetsAmount;

    private List<String> assetsAmountList;
    /**
     * 合计
     */
    private BigDecimal amountSum;

    /**
     * 审计报告（名称）
     */
    private String auditName;

    /**
     * 审计编号
     */
    private String auditCode;

    /**
     * 审计日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date auditTime;

    /**
     * 审计价值（万元）
     */
    private BigDecimal auditAmount;

    /**
     * 反担保措施
     */
    private String guaranteeMeasures;

    /**
     * 附件
     */
    private String file;

    /**
     * 附件
     */
    private List<String> fileList;

    /**
     * 审核意见
     */
    private String applyContent;

    /**
     * 审核金额
     */
    private BigDecimal amount;

    /**
     * 期限
     */
    private String term;

    /**
     * 补充调查原因
     */
    private String remark;

    private List<String> remarkList;

    /**
     * 其他原因
     */
    private String otherRemark;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 修改时间
     */
    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty(value = "评估价值列表")
    private List<DgAssetsAssessmentAssetsLog> assessmentList;

    @ApiModelProperty(value = "评估价值对象")
    private DgAssetsMoney dgAssetsMoney;

    @ApiModelProperty(value = "资产部调查专员")
    private String cName;
}
