package com.daqing.financial.guarantee.service.impl;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.mapper.*;
import com.daqing.financial.guarantee.model.request.CollectionProgressReq;
import com.daqing.financial.guarantee.model.request.GenerateReq;
import com.daqing.financial.guarantee.model.request.StatisticsListReq;
import com.daqing.financial.guarantee.model.response.*;
import com.daqing.financial.guarantee.service.AlCollectionStatisticsService;
import com.daqing.framework.domain.guarantee.*;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author lr
 * @since 2021-08-19
 */
@Service
public class AlCollectionStatisticsServiceImpl extends ServiceImpl<AlCollectionStatisticsMapper, AlCollectionStatistics> implements AlCollectionStatisticsService {

    @Autowired
    private AlRepaymentEntryMapper repaymentEntryMapper;

    @Autowired
    private AlReimbursementMapper reimbursementMapper;

    @Autowired
    private AlInsuranceListMapper insuranceListMapper;

    @Autowired
    private AlCollectionGroupMapper collectionGroupMapper;

    @Override
    public List<CollectionProgressResp> queryCollectionProgress(CollectionProgressReq collectionProgressReq) {
        collectionProgressReq.setPage((collectionProgressReq.getPage()-1)*collectionProgressReq.getSize());
        List<CollectionProgressResp> pageList = baseMapper.selectCollectionProgressPage(collectionProgressReq);
        for (CollectionProgressResp collectionProgressResp : pageList) {
            //查询业务还款信息
            QueryWrapper<AlRepaymentEntry> wrapper = new QueryWrapper<>();
            wrapper.eq("insurance_id", collectionProgressResp.getId());
            List<AlRepaymentEntry> repaymentEntryList = repaymentEntryMapper.selectList(wrapper);
            ArrayList<String> repaymentNotes = Lists.newArrayList();
            for (AlRepaymentEntry repaymentEntry : repaymentEntryList) {
                if (repaymentEntry.getRepaymentCash() != null && repaymentEntry.getRepaymentCash().compareTo(BigDecimal.ZERO) != 0) {
                    if (!repaymentNotes.contains("现金")) {
                        repaymentNotes.add("现金");
                    }
                }
                if (repaymentEntry.getRepaymentAssetsAmount() != null && repaymentEntry.getRepaymentAssetsAmount().compareTo(BigDecimal.ZERO) != 0) {
                    if (!repaymentNotes.contains("资产")) {
                        repaymentNotes.add("资产");
                    }
                }
                if (repaymentEntry.getRepaymentOtherAmount() != null && repaymentEntry.getRepaymentOtherAmount().compareTo(BigDecimal.ZERO) != 0) {
                    if (!repaymentNotes.contains("其他")) {
                        repaymentNotes.add("其他");
                    }
                }
            }
            collectionProgressResp.setRepaymentMethod(repaymentNotes.toString());
        }
        return pageList;
    }

    @Override
    public int selectCollectionProgressPageTotal(CollectionProgressReq collectionProgressReq) {
        return baseMapper.selectCollectionProgressPageTotal(collectionProgressReq);
    }

    @Override
    public void batchExport(HttpServletResponse response, String ids) throws IOException {
        List<String> idList= new ArrayList<>();
        if(ids!=null && !"".equals(ids)){
            String[] id = ids.split(",");
            idList = Arrays.asList(id);
        }
        List<CollectionProgressResp> excelDataList = baseMapper.selectBatchData(idList);
        for (CollectionProgressResp collectionProgressResp : excelDataList) {
            //查询业务还款信息
            QueryWrapper<AlRepaymentEntry> wrapper = new QueryWrapper<>();
            wrapper.eq("insurance_id", collectionProgressResp.getId());
            List<AlRepaymentEntry> repaymentEntryList = repaymentEntryMapper.selectList(wrapper);
            String note = "";
            for (AlRepaymentEntry repaymentEntry : repaymentEntryList) {
                ArrayList<String> repaymentNotes = Lists.newArrayList();
                if (repaymentEntry.getRepaymentCash() != null && repaymentEntry.getRepaymentCash().compareTo(BigDecimal.ZERO) != 0) {
                    if (!repaymentNotes.contains("现金")) {
                        repaymentNotes.add("现金");
                    }
                }
                if (repaymentEntry.getRepaymentAssetsAmount() != null && repaymentEntry.getRepaymentAssetsAmount().compareTo(BigDecimal.ZERO) != 0) {
                    if (!repaymentNotes.contains("资产")) {
                        repaymentNotes.add("资产");
                    }
                }
                if (repaymentEntry.getRepaymentOtherAmount() != null && repaymentEntry.getRepaymentOtherAmount().compareTo(BigDecimal.ZERO) != 0) {
                    if (!repaymentNotes.contains("其他")) {
                        repaymentNotes.add("其他");
                    }
                }
                note = repaymentNotes.toString();
            }
            collectionProgressResp.setRepaymentMethod(note);
        }
        EasyExcelUtil.download(response,CollectionProgressResp.class,excelDataList,"清收工作进度列表","第一页");
    }

    @Override
    @Transactional
    public void generateStatistics(GenerateReq generateReq) {
        //根据年份和负责人id筛选数据
        String year = generateReq.getYear();
        List<Integer> personLiableIds = generateReq.getPersonLiableIds();

        //有人员id生成数据
        if (personLiableIds.size() > 0){
            List<AlReimbursement> reimbursementList = reimbursementMapper.selectListByYear(year,personLiableIds);
            for (AlReimbursement reimbursement : reimbursementList){
                AlCollectionStatistics collectionStatistics = new AlCollectionStatistics();
                Integer collectionGroupId = reimbursement.getCollectionGroupId();
                Integer personLiableId = reimbursement.getPersonLiableId();
                String personLiable = reimbursement.getPersonLiable();
                Integer insuranceId = reimbursement.getInsuranceId();
                //查询工号
                String jobNumber = baseMapper.selectJobNumber(reimbursement.getPersonLiableId());
                //查询企业
                AlInsuranceList alInsuranceList = insuranceListMapper.selectById(insuranceId);
                String customerName = alInsuranceList.getCustomerName();
                //业务担保额度
                // BigDecimal guaranteeAmount = alInsuranceList.getGuaranteeAmount();
                //统计业务还款记录清收现金和资产
                QueryWrapper<AlRepaymentEntry> repaymentEntryWrapper = new QueryWrapper<>();
                repaymentEntryWrapper.eq("insurance_id",insuranceId).last("and YEAR(actual_repayment_date) = '"+year+"' ");
                List<AlRepaymentEntry> repaymentEntryList = repaymentEntryMapper.selectList(repaymentEntryWrapper);
                //清收现金
                BigDecimal repaymentCash = new BigDecimal(0);
                //清收资产
                BigDecimal repaymentAssets = new BigDecimal(0);
                StringBuilder note = new StringBuilder();
                for (AlRepaymentEntry repaymentEntry: repaymentEntryList){
                    //无实际还款日不加入年度统计或不在此年度不加入该年度统计
                    if (!year.equals(repaymentEntry.getActualRepaymentDate().substring(0,4))){
                        continue;
                    }
                    if (repaymentEntry.getRepaymentCash()!=null){
                        repaymentCash = repaymentCash.add(repaymentEntry.getRepaymentCash());
                    }
                    if (repaymentEntry.getRepaymentAssetsAmount()!=null){
                        repaymentAssets = repaymentAssets.add(repaymentEntry.getRepaymentAssetsAmount());
                    }
                    if (repaymentEntry.getRepaymentOtherAmount()!=null){
                        //其他金额加入清收资产中
                        repaymentAssets = repaymentAssets.add(repaymentEntry.getRepaymentOtherAmount());
                    }
                    collectionStatistics.setCollectionTime(repaymentEntry.getActualRepaymentDate());
                    if (repaymentEntry.getRepaymentNotes()!=null){
                        note.append(repaymentEntry.getRepaymentNotes());
                    }
                }

                collectionStatistics.setGroupId(collectionGroupId);
                collectionStatistics.setInsuranceId(insuranceId);
                collectionStatistics.setActualRepaymentDate(year);
                collectionStatistics.setPersonLiable(personLiable);
                collectionStatistics.setPersonLiableId(personLiableId);
                collectionStatistics.setJobNumber(jobNumber);
                collectionStatistics.setEnterprise(customerName);
                collectionStatistics.setCash(repaymentCash);
                collectionStatistics.setAssets(repaymentAssets);
                collectionStatistics.setDescription(note.toString());
                baseMapper.insert(collectionStatistics);
            }
        }
    }

    @Override
    public List<Personnel> collectionGroupList() {
        /*List<CollectionGroupListResp> groupListResp = new ArrayList<>();
        List<AlCollectionGroup> groups = collectionGroupMapper.selectList(null);
        for (AlCollectionGroup group : groups){
            CollectionGroupListResp collectionGroupListResp = new CollectionGroupListResp();
            Integer id = group.getId();
            collectionGroupListResp.setId(id+"group");
            collectionGroupListResp.setName(group.getGroupName());
            QueryWrapper<AlReimbursement> wrapper = new QueryWrapper<>();
            wrapper.eq("collection_group_id",id).select("person_liable","person_liable_id");
            List<AlReimbursement> repaymentEntryList = reimbursementMapper.selectList(wrapper);
            List<Personnel> personnelList = new ArrayList<>();
            for (AlReimbursement reimbursement : repaymentEntryList){
                Personnel personnel = new Personnel();
                Integer personLiableId = reimbursement.getPersonLiableId();
                String personLiable = reimbursement.getPersonLiable();
                personnel.setId(id+","+personLiableId);
                personnel.setName(personLiable);
                if(!personnelList.contains(personnel)){
                    personnelList.add(personnel);
                }
            }
            collectionGroupListResp.setPersonnel(personnelList);
            groupListResp.add(collectionGroupListResp);
        }*/

        List<Personnel> personnel = reimbursementMapper.selectPersonnelList();
        return personnel;
    }

    @Override
    public List<AlCollectionStatistics> annualStatisticsList(StatisticsListReq statisticsListReq) {
        return baseMapper.selectAnnualStatisticsList(statisticsListReq);
    }

    @Override
    public int annualStatisticsListTotal(StatisticsListReq statisticsListReq) {
        return baseMapper.selectAnnualStatisticsListTotal(statisticsListReq);
    }

    @Override
    public List<String> annualList() {
        return baseMapper.selectAnnualList();
    }

    @Override
    public void annualStatisticsListExport(String year,String flag,HttpServletResponse response) throws IOException {
        StatisticsListReq statisticsListReq = new StatisticsListReq();
        statisticsListReq.setYear(year);
        List<AlCollectionStatistics> statisticsList = baseMapper.selectAnnualStatisticsList(statisticsListReq);
        List<AlCollectionStatistics> statisticsLists = new ArrayList<>();
        if ("0".equals(flag)){
            //根据组进行排名
            Map<Integer, List<AlCollectionStatistics>> collect = statisticsList.stream().
                    sorted(Comparator.comparing(AlCollectionStatistics::getTotalAmount).reversed()).
                    collect(Collectors.groupingBy(AlCollectionStatistics::getGroupId));
            int sort = 0;
            for (Integer s : collect.keySet()){
                List<AlCollectionStatistics> statisticsList1 = collect.get(s);
                ++sort;
                for (AlCollectionStatistics alCollectionStatistics : statisticsList1){
                    alCollectionStatistics.setSort(sort);
                    String proportion = alCollectionStatistics.getProportion();
                    if (proportion!=null){
                        float v1 = alCollectionStatistics.getTotalAmount().floatValue();
                        float v2 = Float.parseFloat(proportion);
                        DecimalFormat df = new DecimalFormat("##.##%");//传入格式模板
                        String result = df.format(v1/v2);
                        alCollectionStatistics.setProportion(result);
                    }
                }
                statisticsLists.addAll(statisticsList1);
            }
        }else {
            //根据公司进行排名...
            List<AlCollectionStatistics> annualStatisticsListByPersonSort = baseMapper.selectAnnualStatisticsListByPersonSort();
            int sort = 0;
            for (AlCollectionStatistics alCollectionStatistics : annualStatisticsListByPersonSort){
                alCollectionStatistics.setSort(++sort);
                String proportion = alCollectionStatistics.getProportion();
                if (proportion!=null){
                    //获取分母
                    String denominator = proportion.substring(proportion.lastIndexOf(",")+1);
                    float v1 = alCollectionStatistics.getTotalAmount().floatValue();
                    float v2 = Float.parseFloat(denominator);
                    DecimalFormat df = new DecimalFormat("##.##%");//传入格式模板
                    String result = df.format(v1/v2);
                    alCollectionStatistics.setProportion(result);
                }
            }
            statisticsLists.addAll(annualStatisticsListByPersonSort);
        }

        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("年度统计列表", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), AlCollectionStatistics.class, statisticsLists);
        workbook.write(response.getOutputStream());
    }

    /**
     * 返回该年份已经存在的负责人id
     * @param year 年份
     * @return 负责人id
     */
    @Override
    public List<Integer> personnelSelection(String year) {
        List<Integer> list = baseMapper.selectYearPerson(year);
        /*ArrayList<String> ids = Lists.newArrayList();
        list.forEach(id -> {
            List<Integer> groupList = baseMapper.selectGroupList(id);
            groupList.forEach(g -> {
                String s = g + ","+id;
                ids.add(s);
            });
        });*/
        return list;
    }

    @Override
    public boolean annualStatisticsDeleted(String year) {
        int delete = baseMapper.deleteAnnualStatistics(year);
        return delete > 0;
    }

    @Override
    public boolean selectYear(String year) {
        int have = baseMapper.selectStatisticsYear(year);
        return have > 0;
    }

    @Override
    public boolean selectGenerateYear(String year) {
        int have = baseMapper.selectGenerateYear(year);
        return have > 0;
    }

    /**
     * 还款添加或编辑或删除，根据还款记录id查询保后业务数据更新年度清收数据
     */
    @Override
    public void updateCompleteStatistics(Integer insuranceId,String year) {
        //查询清收代偿信息
        QueryWrapper<AlReimbursement> wrapper = new QueryWrapper<>();
        wrapper.eq("insurance_id",insuranceId);
        AlReimbursement reimbursement = reimbursementMapper.selectOne(wrapper);
        Integer groupId = reimbursement.getCollectionGroupId();
        Integer personLiableId = reimbursement.getPersonLiableId();
        String personLiable = reimbursement.getPersonLiable();
        //查询该人员是否在统计列表中，在统计列表中，计算该业务下的年度统计情况
        ArrayList<Integer> list = baseMapper.selectPerson();
        boolean contains = list.contains(personLiableId);
        if (contains){
            AlCollectionStatistics collectionStatistics = new AlCollectionStatistics();
            //查询工号
            String jobNumber = baseMapper.selectJobNumber(reimbursement.getPersonLiableId());
            //查询企业
            AlInsuranceList alInsuranceList = insuranceListMapper.selectById(insuranceId);
            String customerName = alInsuranceList.getCustomerName();
            //统计该业务下还款记录清收现金和资产
            QueryWrapper<AlRepaymentEntry> repaymentEntryWrapper = new QueryWrapper<>();
            repaymentEntryWrapper.eq("insurance_id",insuranceId).last("and YEAR(actual_repayment_date) = '"+year+"' ");
            List<AlRepaymentEntry> repaymentEntryList = repaymentEntryMapper.selectList(repaymentEntryWrapper);
            //无还款记录则删除该年度业务统计
            if (ObjectUtil.isNull(repaymentEntryList)||repaymentEntryList.size()==0){
                QueryWrapper<AlCollectionStatistics> deleteWrapper = new QueryWrapper<>();
                deleteWrapper.eq("insurance_id",insuranceId).eq("group_id",groupId).eq("actual_repayment_date",year);
                baseMapper.delete(deleteWrapper);
            }else {
                //先清空之前数据
                QueryWrapper<AlCollectionStatistics> deleteWrapper = new QueryWrapper<>();
                deleteWrapper.eq("insurance_id",insuranceId).eq("group_id",groupId).eq("actual_repayment_date",year);
                baseMapper.delete(deleteWrapper);
                //清收现金
                BigDecimal repaymentCash = new BigDecimal(0);
                //清收资产
                BigDecimal repaymentAssets = new BigDecimal(0);

                StringBuilder note = new StringBuilder();
                for (AlRepaymentEntry repayment: repaymentEntryList){
                    //无实际还款日不加入年度统计或不在此年度不加入该年度统计
                    if (!year.equals(repayment.getActualRepaymentDate().substring(0,4))){
                        continue;
                    }
                    if (repayment.getRepaymentCash()!=null){
                        repaymentCash = repaymentCash.add(repayment.getRepaymentCash());
                    }
                    if (repayment.getRepaymentAssetsAmount()!=null){
                        repaymentAssets = repaymentAssets.add(repayment.getRepaymentAssetsAmount());
                    }
                    if (repayment.getRepaymentOtherAmount()!=null){
                        //其他金额加入清收资产中
                        repaymentAssets = repaymentAssets.add(repayment.getRepaymentOtherAmount());
                    }
                    collectionStatistics.setCollectionTime(repayment.getActualRepaymentDate());
                    if (repayment.getRepaymentNotes()!=null){
                        note.append(repayment.getRepaymentNotes());
                    }
                }
                //添加年度数据
                collectionStatistics.setGroupId(groupId);
                collectionStatistics.setInsuranceId(insuranceId);
                collectionStatistics.setActualRepaymentDate(year);
                collectionStatistics.setPersonLiable(personLiable);
                collectionStatistics.setPersonLiableId(personLiableId);
                collectionStatistics.setJobNumber(jobNumber);
                collectionStatistics.setEnterprise(customerName);
                collectionStatistics.setCash(repaymentCash);
                collectionStatistics.setAssets(repaymentAssets);
                collectionStatistics.setDescription(note.toString());
                baseMapper.insert(collectionStatistics);
            }
        }
    }
}
