package com.daqing.financial.guarantee.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.mapper.*;
import com.daqing.financial.guarantee.model.request.ExcelImpInsuranceVO;
import com.daqing.financial.guarantee.model.request.InsuranceSaveReq;
import com.daqing.financial.guarantee.model.response.LegalVo;
import com.daqing.financial.guarantee.model.response.PersonLiableResp;
import com.daqing.financial.guarantee.service.AlCollectionStatisticsService;
import com.daqing.financial.guarantee.service.IAlInsuranceListService;
import com.daqing.financial.guarantee.service.IAlRepaymentEntryService;
import com.daqing.financial.guarantee.util.ExcelImportHelper;
import com.daqing.financial.guarantee.util.R;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.*;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 保后管理列表 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2021-03-17
 */
@Service
public class AlInsuranceListServiceImpl extends ServiceImpl<AlInsuranceListMapper, AlInsuranceList> implements IAlInsuranceListService {

    @Autowired
    private AlRepaymentEntryMapper alRepaymentEntryMapper;

    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @Autowired
    private AlCollectionGroupMapper collectionGroupMapper;

    @Autowired
    private AlCollectionMapper collectionMapper;

    @Autowired
    private AlCollectionStatisticsMapper collectionStatisticsMapper;

    @Autowired
    private AlCollectionStatisticsService collectionStatisticsService;

    @Autowired
    private AlReimbursementMapper reimbursementMapper;

    @Autowired
    private AlRepaymentEntryServiceImpl repaymentEntryService;

    @Override
    public PageUtils queryPage(Integer page, Integer size, String customerNumberOrName, Integer paymentStatus) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<AlInsuranceList> positionVO = this.getBaseMapper().pageByCondition(new Page(page, size), customerNumberOrName, paymentStatus);

        return new PageUtils(positionVO);
    }

    @Override
    public R insuranceDetail(Integer id) {
        AlInsuranceList insuranceList = baseMapper.selectById(id);
        QueryWrapper<AlReimbursement> wrapper = new QueryWrapper<>();
        wrapper.eq("insurance_id", insuranceList.getId());
        AlReimbursement reimbursement = reimbursementMapper.selectOne(wrapper);
        return R.ok().data("insuranceList", insuranceList).data("reimbursement", reimbursement);
    }

    @Override
    public void excelExport(List<Integer> ids, HttpServletResponse response) {
        List<AlInsuranceList> alInsuranceList = null;
        //为空默认导出全部数据
        if (ids.size()==0){
            alInsuranceList = baseMapper.selectList(null);
        }else {
            alInsuranceList = baseMapper.selectBatchIds(ids);
        }
        try {
            EasyExcelUtil.download(response, AlInsuranceList.class,alInsuranceList,"保后管理一览表","第一页");
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
        }
    }

    @Override
    public int updateStatus(Integer id) {
        AlInsuranceList alInsuranceList = new AlInsuranceList();
        alInsuranceList.setPaymentStatus(4);//已结项
        return this.baseMapper.update(alInsuranceList, new QueryWrapper<AlInsuranceList>().eq("id", id));
    }

    @Override
    @Transactional
    public boolean saveInsurance(InsuranceSaveReq insuranceSaveReq) {
        AlInsuranceList alInsuranceList = insuranceSaveReq.getAlInsuranceList();
        AlReimbursement alReimbursement = insuranceSaveReq.getAlReimbursement();
        long time = System.currentTimeMillis();
        alInsuranceList.setBusinessCode("SD-"+time);
        //添加固定为手动录入
        alInsuranceList.setSource(1);
        alInsuranceList.setApplyTime(new Date());
        //默认数据
        alInsuranceList.setRepaymentAmount(new BigDecimal(0));
        alInsuranceList.setRemainAmount(alInsuranceList.getGuaranteeAmount());
        alInsuranceList.setRepaymentTime(0);
        int insert = baseMapper.insert(alInsuranceList);
        int ok = 0;
        //添加担保信息
        if (insert > 0) {
            alReimbursement.setInsuranceId(alInsuranceList.getId());
            ok = reimbursementMapper.insert(alReimbursement);
        }
        return ok > 0;
    }

    @Override
    @Transactional
    public Integer updateInsurance(InsuranceSaveReq insuranceSaveReq) {
        //获取更新数据
        AlInsuranceList alInsuranceList = insuranceSaveReq.getAlInsuranceList();
        AlReimbursement alReimbursement = insuranceSaveReq.getAlReimbursement();
        //更新业务信息
        baseMapper.updateById(alInsuranceList);
        //查询清收代偿信息
        QueryWrapper<AlReimbursement> wrapper = new QueryWrapper<>();
        wrapper.eq("insurance_id",alInsuranceList.getId());
        AlReimbursement reimbursement = reimbursementMapper.selectOne(wrapper);
        //数据存在及更新,流转即添加
        if (ObjectUtil.isNotNull(reimbursement)){
            //之前清收负责人或清收组数据和本次更新不一致，清除之前数据
            if (!reimbursement.getCollectionGroupId().equals(alReimbursement.getCollectionGroupId())||!reimbursement.getPersonLiableId().equals(alReimbursement.getPersonLiableId())){
                //查询之前人员是否在统计列表中，存在则清除
                ArrayList<Integer> list = collectionStatisticsMapper.selectPerson();
                boolean contains = list.contains(reimbursement.getPersonLiableId());
                if (contains){
                    //清空之前统计该业务数据
                    QueryWrapper<AlCollectionStatistics> deleteWrapper = new QueryWrapper<>();
                    deleteWrapper.eq("insurance_id",alInsuranceList.getId());
                    int delete = collectionStatisticsMapper.delete(deleteWrapper);
                    // System.out.println("删除条数"+delete);
                    /*if (delete>0){
                        //根据业务id更新年度统计
                        collectionStatisticsService.updateCompleteStatistics(alInsuranceList.getId());
                    }*/
                }
            }
            //更新清收代偿数据
            alReimbursement.setId(reimbursement.getId());
            reimbursementMapper.updateById(alReimbursement);
        }else {
            reimbursementMapper.insert(alReimbursement);
        }
        return alInsuranceList.getId();
    }

    @Override
    @Transactional
    public boolean deleteInsurance(Integer insuranceId) {
        //删除保后
        int delete = baseMapper.deleteById(insuranceId);
        QueryWrapper<AlReimbursement> wrapper = new QueryWrapper<>();
        wrapper.eq("insurance_id",insuranceId);
        //删除清收代偿
        reimbursementMapper.delete(wrapper);
        QueryWrapper<AlRepaymentEntry> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("insurance_id",insuranceId);
        //删除还款记录
        alRepaymentEntryMapper.delete(queryWrapper);
        //删除催收记录
        QueryWrapper<AlCollection> collectionQueryWrapper = new QueryWrapper<>();
        collectionQueryWrapper.eq("insurance_id",insuranceId);
        collectionMapper.delete(collectionQueryWrapper);
        //删除清收年度统计数据
        QueryWrapper<AlCollectionStatistics> statisticsQueryWrapper = new QueryWrapper<>();
        statisticsQueryWrapper.eq("insurance_id",insuranceId);
        collectionStatisticsMapper.delete(statisticsQueryWrapper);
        return delete > 0;
    }

    /**
     * 代偿情况，根据代偿时间返回该日期之后的本次还款金额+其他费用+利息，用于计算代偿余额相关信息
     * @param date 代偿时间
     * @param insuranceId 业务id
     * @return 还款金额
     */
    @Override
    public R compensationSituation(String date,Integer insuranceId) {
        Integer amount = alRepaymentEntryMapper.compensationSituation(date,insuranceId);
        return R.ok().data("amount",amount);
    }

    @Override
    public Map<String, String> importInsurance(MultipartFile file) throws Exception {
        //获取文件数据
        List<ExcelImpInsuranceVO> impInsuranceVOList = ExcelImportHelper.readInsuranceManagement(file);
        if (impInsuranceVOList.size() <= 0) {
            //小于等于0为上传空模板的情况下 抛出异常
            throw new Exception("模板为空");
        }
        //导入失败数据
        List<ExcelImpInsuranceVO> impInsuranceFailVo = new ArrayList<>();
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (org.apache.commons.lang3.StringUtils.isBlank(file.getOriginalFilename())
                || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            throw new Exception("只能上传.xlsx后缀的文件");
        }
        Map<String, String> map = new HashMap<>();
        Long ii = 1L;//用于记录序列号
        Integer countNum = 0;//用于失败原因排序 eg:1、XXXXX 2、XXXXXXX

        try {
            Integer countSuccess = 0;//统计成功次数
            Integer countError = 0;//统计失败次数

            for (ExcelImpInsuranceVO impInsuranceVO : impInsuranceVOList) {
                boolean ret = true;
                ++ii;

                String errMsg = "";
                //去除Excel中的全角半角空格,判断空字符再截取
                String customerName = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getCustomerName())){
                    customerName = impInsuranceVO.getCustomerName().replaceAll("\\u00A0", "").trim();
                }

                String bank = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getBank())){
                    bank = impInsuranceVO.getBank().replaceAll("\\u00A0", "").trim();
                }

                String guaranteeAmount = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getGuaranteeAmount())){
                    guaranteeAmount = impInsuranceVO.getGuaranteeAmount().replaceAll("\\u00A0", "").trim();
                }

                String loanStartTime = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getLoanStartTime())){
                    loanStartTime = impInsuranceVO.getLoanStartTime().replaceAll("\\u00A0", "").trim();
                }

                String loanEndTime = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getLoanEndTime())){
                    loanEndTime = impInsuranceVO.getLoanEndTime().replaceAll("\\u00A0", "").trim();
                }

                String guaranteeTime = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getGuaranteeTime())){
                    guaranteeTime = impInsuranceVO.getGuaranteeTime().replaceAll("\\u00A0", "").trim();
                }

                String businessType = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getBusinessType())){
                    businessType = impInsuranceVO.getBusinessType().replaceAll("\\u00A0", "").trim();
                }

                String amountWide = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getAmountWide())){
                    amountWide = impInsuranceVO.getAmountWide().replaceAll("\\u00A0", "").trim();
                }

                String compensationAmount = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getCompensationAmount())){
                    compensationAmount = impInsuranceVO.getCompensationAmount().replaceAll("\\u00A0", "").trim();
                }

                String compensatoryBalance = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getCompensatoryBalance())){
                    compensatoryBalance = impInsuranceVO.getCompensatoryBalance().replaceAll("\\u00A0", "").trim();
                }

                String compensatoryTime = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getCompensatoryTime())){
                    compensatoryTime = impInsuranceVO.getCompensatoryTime().replaceAll("\\u00A0", "").trim();
                }

                String personLiable = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getPersonLiable())){
                    personLiable = impInsuranceVO.getPersonLiable().replaceAll("\\u00A0", "").trim();
                }

                String collectionGroup = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getCollectionGroup())){
                    collectionGroup = impInsuranceVO.getCollectionGroup().replaceAll("\\u00A0", "").trim();
                }

                String area = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getArea())){
                    area = impInsuranceVO.getArea().replaceAll("\\u00A0", "").trim();
                }

                String operation = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getOperation())){
                    operation = impInsuranceVO.getOperation().replaceAll("\\u00A0", "").trim();
                }

                String litigation = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getLitigation())){
                    litigation = impInsuranceVO.getLitigation().replaceAll("\\u00A0", "").trim();
                }

                String mortgage = "";
                if (StringUtils.isNotBlank(impInsuranceVO.getMortgage())){
                    mortgage = impInsuranceVO.getMortgage().replaceAll("\\u00A0", "").trim();
                }
                String legalName = "";
                String legalPhone = "";

                ExcelImpInsuranceVO failVo = new ExcelImpInsuranceVO();
                failVo.setIndex(ii);
                //检验客户
                if ("".equals(customerName)) {
                    ++countNum;
                    failVo.setCustomerName(customerName + " *必填项:(客户名称不能为空) ");
                    errMsg += countNum + "必填项:(客户名称不能为空)";
                    ret = false;
                } else {
                    //查询企业信息
                    LegalVo legalVo = baseMapper.selectLegal(customerName);
                    if (ObjectUtil.isNull(legalVo)) {
                        failVo.setCustomerName(customerName + " *必填项:(客户不存在) ");
                        errMsg += countNum + "必填项:(客户不存在)";
                        ret = false;
                    } else {
                        //法定代表人
                        legalName = legalVo.getLegalName();
                        //联系电话
                        legalPhone = legalVo.getLegalPhone();
                    }
                    failVo.setCustomerName(customerName);
                }

                if ("".equals(bank)) {
                    ++countNum;
                    failVo.setBank(bank + " *必填项:(银行不能为空)");
                    errMsg += countNum + " *必填项:(银行不能为空)";
                    ret = false;
                } else {
                    int ok = baseMapper.selectBank(bank);
                    if (ok == 0) {
                        failVo.setBank(bank + " *必填项:(银行不存在) ");
                        errMsg += countNum + "必填项:(银行不存在)";
                        ret = false;
                    }
                    failVo.setBank(bank);
                }

                if ("".equals(guaranteeAmount)) {
                    ++countNum;
                    failVo.setGuaranteeAmount(guaranteeAmount + " *必填项:(担保额度不能为空)");
                    errMsg += countNum + " *必填项:(担保额度不能为空)";
                    ret = false;
                }else {
                    boolean numeric = StringUtils.isNumeric(guaranteeAmount);
                    if(!numeric){
                        ++countNum;
                        failVo.setGuaranteeAmount(guaranteeAmount + " *必填项:(担保额度不能字符)");
                        errMsg += countNum + " *必填项:(担保额度不能字符)";
                        ret = false;
                    }
                    failVo.setGuaranteeAmount(guaranteeAmount);
                }

                if ("".equals(loanStartTime)) {
                    ++countNum;
                    failVo.setLoanStartTime(loanStartTime + " *必填项:(贷款开始日期不能为空)");
                    errMsg += countNum + " *必填项:(贷款开始日期不能为空)";
                    ret = false;
                }else {
                    boolean validDate = isValidDate(loanStartTime);
                    if (!validDate){
                        ++countNum;
                        failVo.setLoanStartTime(loanStartTime + " *必填项:(贷款开始日期格式错误)");
                        errMsg += countNum + " *必填项:(贷款开始日期格式错误)";
                        ret = false;
                    }
                    failVo.setLoanStartTime(loanStartTime);
                }

                if ("".equals(loanEndTime)) {
                    ++countNum;
                    failVo.setLoanEndTime(loanEndTime + " *必填项:(贷款结束日期不能为空)");
                    errMsg += countNum + " *必填项:(贷款结束日期不能为空)";
                    ret = false;
                }else {
                    boolean validDate = isValidDate(loanEndTime);
                    if (!validDate){
                        ++countNum;
                        failVo.setLoanEndTime(loanEndTime + " *必填项:(贷款结束日期格式错误)");
                        errMsg += countNum + " *必填项:(贷款结束日期格式错误)";
                        ret = false;
                    }
                    failVo.setLoanEndTime(loanEndTime);
                }

                if ("".equals(guaranteeTime)) {
                    ++countNum;
                    failVo.setGuaranteeTime(guaranteeTime + " *必填项:(担保期限不能为空)");
                    errMsg += countNum + " *必填项:(担保期限不能为空)";
                    ret = false;
                }else {
                    boolean numeric = StringUtils.isNumeric(guaranteeTime);
                    if(!numeric){
                        ++countNum;
                        failVo.setGuaranteeTime(guaranteeTime + " *必填项:(担保期限格式错误,请输入数字)");
                        errMsg += countNum + " *必填项:(担保期限格式错误,请输入数字)";
                        ret = false;
                    }
                    failVo.setGuaranteeTime(guaranteeTime);
                }

                if ("".equals(businessType)) {
                    ++countNum;
                    failVo.setBusinessType(businessType + " *必填项:(业务类别不能为空)");
                    errMsg += countNum + " *必填项:(业务类别不能为空)";
                    ret = false;
                } else {
                    switch (businessType.trim()) {
                        case "融资担保":
                        case "担保":
                        case "借贷":
                        case "小额贷":
                        case "企业大贷":
                        case "其他":
                            break;
                        default:
                            ++countNum;
                            failVo.setBusinessType(businessType + " *必填项:(业务类别不存在)");
                            errMsg += countNum + " *必填项:(业务类别不存在)";
                            ret = false;
                    }
                    failVo.setBusinessType(businessType);
                }

                if ("".equals(amountWide)) {
                    ++countNum;
                    failVo.setAmountWide(amountWide + " *必填项:(贷款用途不能为空)");
                    errMsg += countNum + " *必填项:(贷款用途不能为空)";
                    ret = false;
                } else {
                    switch (amountWide.trim()) {
                        case "短期周转":
                        case "购房贷款":
                        case "装修贷款":
                        case "购车贷款":
                        case "教育贷款":
                        case "投资创业":
                        case "其他":
                            break;
                        default:
                            ++countNum;
                            failVo.setAmountWide(amountWide + " *必填项:(贷款用途不存在)");
                            errMsg += countNum + " *必填项:(贷款用途不存在)";
                            ret = false;
                    }
                    failVo.setAmountWide(amountWide);
                }
                Integer personLiableId = 0;
                if ("".equals(personLiable)) {
                    ++countNum;
                    failVo.setPersonLiable(personLiable + " *必填项:(清收负责人不能为空)");
                    errMsg += countNum + " *必填项:(清收负责人不能为空)";
                    ret = false;
                } else {
                    PersonLiableResp personLiableResp = baseMapper.selectPersonLiable(personLiable);
                    if (ObjectUtil.isNull(personLiableResp)) {
                        failVo.setPersonLiable(personLiable + " *必填项:(清收负责人不存在) ");
                        errMsg += countNum + "必填项:(清收负责人不存在)";
                        ret = false;
                    }else {
                        personLiableId = personLiableResp.getId();
                    }
                    failVo.setPersonLiable(personLiable);
                }

                int collectionGroupId = 0;
                if ("".equals(collectionGroup)) {
                    ++countNum;
                    failVo.setCollectionGroup(collectionGroup + " *必填项:(清收组不能为空)");
                    errMsg += countNum + " *必填项:(清收组不能为空)";
                    ret = false;
                } else {
                    QueryWrapper<AlCollectionGroup> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("group_name", collectionGroup);
                    AlCollectionGroup one = collectionGroupMapper.selectOne(queryWrapper);
                    if (ObjectUtil.isNull(one)) {
                        failVo.setCollectionGroup(collectionGroup + " *必填项:(清收组不存在) ");
                        errMsg += countNum + "必填项:(清收组不存在)";
                        ret = false;
                    }else {
                        collectionGroupId = one.getId();
                    }
                    failVo.setCollectionGroup(collectionGroup);
                }

                if ("".equals(area)) {
                    ++countNum;
                    failVo.setArea(area + " *必填项:(所在区域不能为空)");
                    errMsg += countNum + " *必填项:(所在区域不能为空)";
                    ret = false;
                } else {
                    switch (area.trim()) {
                        case "萨尔图区":
                        case "高新区":
                        case "龙凤区":
                        case "让胡同路区":
                        case "大同区":
                        case "肇县区":
                            break;
                        default:
                            ++countNum;
                            failVo.setArea(area + " *必填项:(所在区域不存在)");
                            errMsg += countNum + " *必填项:(所在区域不存在)";
                            ret = false;
                    }
                    failVo.setArea(area);
                }

                //效验都通过添加数据
                if (ret) {
                    InsuranceSaveReq insuranceSaveReq = new InsuranceSaveReq();
                    AlInsuranceList insuranceList = new AlInsuranceList();
                    AlReimbursement reimbursement = new AlReimbursement();

                    insuranceList.setCustomerName(customerName);
                    insuranceList.setBank(bank);
                    insuranceList.setLegalName(legalName);
                    insuranceList.setPhone(legalPhone);
                    insuranceList.setBusinessType(businessType);
                    insuranceList.setGuaranteeAmount(new BigDecimal(guaranteeAmount));
                    insuranceList.setGuaranteeTime(guaranteeTime);
                    insuranceList.setLoanStartTime(DateUtil.parse(loanStartTime));
                    insuranceList.setLoanEndTime(DateUtil.parse(loanEndTime));
                    insuranceList.setAmountWide(amountWide);

                    if (!"".equals(compensationAmount)) {
                        reimbursement.setCompensationAmount(new BigDecimal(compensationAmount));
                    }
                    if (!"".equals(compensatoryBalance)) {
                        reimbursement.setCompensatoryBalance(new BigDecimal(compensatoryBalance));
                    }

                    if (!"".equals(compensatoryTime)) {
                        reimbursement.setCompensatoryTime(DateUtil.parse(compensatoryTime));
                    }

                    reimbursement.setPersonLiable(personLiable);
                    reimbursement.setPersonLiableId(personLiableId);
                    reimbursement.setCollectionGroupId(collectionGroupId);
                    reimbursement.setArea(area);
                    reimbursement.setOperation(operation);
                    reimbursement.setLitigation(litigation);
                    reimbursement.setMortgage(mortgage);

                    insuranceSaveReq.setAlInsuranceList(insuranceList);
                    insuranceSaveReq.setAlReimbursement(reimbursement);
                    this.saveInsurance(insuranceSaveReq);
                    countSuccess++;
                } else {
                    countError++;
                    failVo.setFailureMsg(errMsg);
                    //添加失败数据
                    impInsuranceFailVo.add(failVo);
                }
            }
            String exportCode = "";
            // 有导入失败的数据，才会存入redis
            if (countError > 0) {
                //生成token
                exportCode = "FAILURE_IMPORT" + UUID.randomUUID().toString().replace("-", "");
                ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
                String failureVOJson = JSON.toJSONString(impInsuranceFailVo);
                ops.set(exportCode, failureVOJson, 30 * 60, TimeUnit.SECONDS);
            }
            map.put("exportCode", exportCode);//返回导出code
            map.put("successNum", countSuccess + "");//本次新增成功数量
            map.put("failureNum", countError + "");//本次新增失败数量
            return map;
        } catch (RuntimeException e) {
            throw new RuntimeException();
        }

    }

    @Override
    public void exportFailureRecord(HttpServletResponse response, String exportCode) {
        if (StringUtils.isEmpty(exportCode)) {
            return;
        }
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        //获取数据
        String record = ops.get(exportCode);
        if (StringUtils.isEmpty(record)) {
            return;
        }
        List<ExcelImpInsuranceVO> parse = JSON.parseObject(record, new TypeReference<List<ExcelImpInsuranceVO>>() {
        });

        parse.sort(Comparator.comparing(ExcelImpInsuranceVO::getIndex));

        try {
            EasyExcelUtil.download(response, ExcelImpInsuranceVO.class,parse,"导入失败数据表","第一页");
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
        }
    }

    public static boolean isValidDate(String str) {
        boolean convertSuccess = true;
        // 指定日期格式为四位年/两位月份/两位日期，注意yyyy/MM/dd区分大小写；
        SimpleDateFormat format = new SimpleDateFormat("yyyy/MM/dd");
        SimpleDateFormat format1 = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        if (str.contains("-")) {
            str=str.replace("-", "/");
        }
        try {
            // 设置lenient为false. 否则SimpleDateFormat会比较宽松地验证日期，比如2007/02/29会被接受，并转换成2007/03/01
            if (str.length()==8) {
                str=str.substring(0, 5)+"0"+str.substring(5, 7)+"0"+str.charAt(7);
                format.setLenient(false);
                format.parse(str);
            }else if(str.length()==9){
                str=str.substring(0, 5)+"0"+str.substring(5, 9);
                format.setLenient(false);
                format.parse(str);
            }else if(str.length()==10){
                format.setLenient(false);
                format.parse(str);
            }else {
                format1.setLenient(false);
                format1.parse(str);
            }
        } catch (ParseException e) {
            // e.printStackTrace();
            // 如果throw java.text.ParseException或者NullPointerException，就说明格式不对
            convertSuccess = false;
        }
        return convertSuccess;
    }

}
