package com.daqing.financial.guarantee.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.mapper.AlInsuranceListMapper;
import com.daqing.financial.guarantee.mapper.AlReimbursementMapper;
import com.daqing.financial.guarantee.mapper.AlRepaymentEntryMapper;
import com.daqing.financial.guarantee.model.response.AlRepaymentEntryListRes;
import com.daqing.financial.guarantee.model.response.AlRepaymentEntryRes;
import com.daqing.financial.guarantee.model.response.AlRepaymentRes;
import com.daqing.financial.guarantee.service.AlCollectionStatisticsService;
import com.daqing.financial.guarantee.service.IAlRepaymentEntryService;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.AlInsuranceList;
import com.daqing.framework.domain.guarantee.AlReimbursement;
import com.daqing.framework.domain.guarantee.AlRepaymentEntry;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 还款记录表 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2021-03-17
 */
@Service
public class AlRepaymentEntryServiceImpl extends ServiceImpl<AlRepaymentEntryMapper, AlRepaymentEntry> implements IAlRepaymentEntryService {

    @Autowired
    private AlInsuranceListMapper alInsuranceListMapper;

    @Autowired
    private AlReimbursementMapper reimbursementMapper;

    @Autowired
    private AlRepaymentEntryMapper alRepaymentEntryMapper;

    @Autowired
    private AlCollectionStatisticsService collectionStatisticsService;

    @Override
    public int repaymentEntry(AlRepaymentEntry alRepaymentEntry) {
        //效验信息
        BigDecimal repaymentCash = alRepaymentEntry.getRepaymentCash();
        BigDecimal repaymentAssetsAmount = alRepaymentEntry.getRepaymentAssetsAmount();
        BigDecimal repaymentOtherAmount = alRepaymentEntry.getRepaymentOtherAmount();
        if (repaymentCash==null&&repaymentAssetsAmount==null&&repaymentOtherAmount==null){
            ExceptionCast.cast(HrmsCode.REQUIRED);
        }
        Map map = DgApplyAmountInfoController.getUserIdAndName();
        String name = map.get("name").toString();
        Integer userId = Integer.parseInt(map.get("userId").toString());
        alRepaymentEntry.setSubmitterId(userId);
        alRepaymentEntry.setSubmitterName(name);
        int insert = baseMapper.insert(alRepaymentEntry);
        Integer insuranceId = alRepaymentEntry.getInsuranceId();
        //更新代偿信息
        updateReimbursementData(insuranceId);
        //更新业务状态
        updateRepaymentEntry(insuranceId);
        //更新统计还款记录数据
        repaymentStatistics(insuranceId);
        String actualRepaymentDate = alRepaymentEntry.getActualRepaymentDate();
        //添加无实际还款不进行年度统计
        if (actualRepaymentDate!=null){
            //获取实际还款日年份
            String year = actualRepaymentDate.substring(0,4);
            //更新年度统计
            collectionStatisticsService.updateCompleteStatistics(insuranceId,year);
        }
        return insert;
    }

    @Override
    public void updateReimbursementData(Integer insuranceId) {
        QueryWrapper<AlReimbursement> wrapper = new QueryWrapper<>();
        wrapper.eq("insurance_id",insuranceId);
        AlReimbursement reimbursement = reimbursementMapper.selectOne(wrapper);
        if (ObjectUtil.isNotNull(reimbursement)){
            //代偿金额和代偿时间不为空，则计算代偿余额
            if (reimbursement.getCompensationAmount()!=null&&reimbursement.getCompensatoryTime()!=null){
                //还款记录都删除，则余额与金额相同
                QueryWrapper<AlRepaymentEntry> repaymentEntryQueryWrapper = new QueryWrapper<>();
                repaymentEntryQueryWrapper.eq("insurance_id",insuranceId);
                Integer integer = alRepaymentEntryMapper.selectCount(repaymentEntryQueryWrapper);
                if (integer==0){
                    reimbursement.setCompensatoryBalance(reimbursement.getCompensationAmount());
                }else {
                    Integer amount = alRepaymentEntryMapper.compensationSituation(DateUtil.formatDate(reimbursement.getCompensatoryTime()), insuranceId);
                    BigDecimal compensationAmount = reimbursement.getCompensationAmount();
                    compensationAmount = compensationAmount.subtract(new BigDecimal(amount));
                    reimbursement.setCompensatoryBalance(compensationAmount);
                }
                reimbursementMapper.updateById(reimbursement);
            }
        }
    }

    @Override
    public AlRepaymentEntry repaymentDetail(Integer id) {
        return baseMapper.selectById(id);
    }

    @Override
    public PageUtils queryPage(Integer page, Integer size, String customerNumberOrName, Integer status, Integer insuranceId) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<AlRepaymentEntryListRes> positionVO = this.getBaseMapper().pageByCondition(new Page(page, size),customerNumberOrName,status,insuranceId);

        return new PageUtils(positionVO);
    }

    @Override
    public int updateRepayment(AlRepaymentEntry alRepaymentEntryReq) {
        String actualRepaymentDate = alRepaymentEntryReq.getActualRepaymentDate();

        //查询更新之前的年份
        String before = "";
        AlRepaymentEntry repaymentEntry = baseMapper.selectById(alRepaymentEntryReq.getId());
        before = repaymentEntry.getActualRepaymentDate();
        before = before.substring(0,4);

        int update = baseMapper.updateById(alRepaymentEntryReq);

        // if (actualRepaymentDate==null){
        //     //查询更新之前数据
        //     AlRepaymentEntry repaymentEntry1 = baseMapper.selectById(alRepaymentEntryReq.getId());
        //     actualRepaymentDate = repaymentEntry1.getActualRepaymentDate();
        // }else {
        // }
        //查询更新之前数据进行更新
        collectionStatisticsService.updateCompleteStatistics(alRepaymentEntryReq.getInsuranceId(),before);
        //更新代偿信息
        updateReimbursementData(alRepaymentEntryReq.getInsuranceId());
        //更新业务状态
        updateRepaymentEntry(alRepaymentEntryReq.getInsuranceId());
        //更新统计还款记录数据
        repaymentStatistics(alRepaymentEntryReq.getInsuranceId());

        //无实际还款不进行年度统计
        if (actualRepaymentDate!=null){
            //获取实际还款日年份
            String year = actualRepaymentDate.substring(0,4);
            //更新年度统计
            collectionStatisticsService.updateCompleteStatistics(alRepaymentEntryReq.getInsuranceId(),year);
        }
        return update;
    }

    @Override
    public int delRepayment(List<Integer> ids) {
        if (ids.size()>0){
            AlRepaymentEntry repaymentEntry = baseMapper.selectById(ids.get(0));

            //删除前更新年度统计
            for (Integer id : ids){
                AlRepaymentEntry repaymentEntry1 = baseMapper.selectById(id);
                String actualRepaymentDate = repaymentEntry1.getActualRepaymentDate();
                //添加无实际还款不进行年度统计
                if (actualRepaymentDate!=null){
                    //获取实际还款日年份
                    String year = actualRepaymentDate.substring(0,4);
                    //更新年度统计
                    collectionStatisticsService.updateCompleteStatistics(repaymentEntry.getInsuranceId(),year);
                }
            }

            int i = baseMapper.deleteBatchIds(ids);
            //更新统计还款记录数据
            repaymentStatistics(repaymentEntry.getInsuranceId());
            //更新代偿信息
            updateReimbursementData(repaymentEntry.getInsuranceId());
            return i;
        }
        return 0;
    }

    @Override
    public Boolean excelExport(List<String> ids,Integer insuranceId, HttpServletResponse response) {

        List<AlRepaymentRes>repaymentList = this.baseMapper.selectListByIds(ids,insuranceId);
        try {
            ExcelUtil.writeExcelWithSheets(response, repaymentList, "还款记录一览表", "repaymentEntry", new AlRepaymentRes())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    @Override
    public Map repaymentStatistics(Integer insuranceId) {
        Map<String, Object> map = new HashMap<>();
        //统计还款期数（统计状态为已还款期数）
        int count1 = this.baseMapper.selectCount(new QueryWrapper<AlRepaymentEntry>()
                .eq("repayment_status",2).eq("insurance_id",insuranceId));
        map.put("repaymentSum",count1);

        //统计逾期期数(统计状态为已逾期期数)
        int count2 = this.baseMapper.selectCount(new QueryWrapper<AlRepaymentEntry>()
                .eq("repayment_status",3).eq("insurance_id",insuranceId));
        map.put("overdueSum",count2);

        //统计逾期利息(计算状态为已逾期的所有利息的合计数)
        List<AlRepaymentEntry> list = this.baseMapper.selectList(new QueryWrapper<AlRepaymentEntry>()
                .eq("repayment_status", 3).eq("insurance_id",insuranceId));
        Double interestSum = 0.00;
        Double currentSum = 0.00;
        for (AlRepaymentEntry entry:list) {
            BigDecimal interest = entry.getInterest();
            interestSum=interestSum+interest.doubleValue();

            BigDecimal currentRepayment = entry.getCurrentRepayment();
            currentSum=currentSum+currentRepayment.doubleValue();
        }
        map.put("overdueInterest",interestSum);

        //统计逾期额度（元）(计算状态为已逾期的所有本次还款的合计数)
        map.put("currentSum",currentSum);

        //统计还款额度（元）(统计状态为已还款的本次还款)
        List<AlRepaymentEntry> list2 = this.baseMapper.selectList(new QueryWrapper<AlRepaymentEntry>()
                .eq("repayment_status", 2).eq("insurance_id",insuranceId));
        Double alreadyPaymentSum = 0.00;
        for (AlRepaymentEntry entry:list2) {
            BigDecimal currentRepayment = entry.getCurrentRepayment();
            alreadyPaymentSum=alreadyPaymentSum+currentRepayment.doubleValue();
        }
        map.put("alreadyPaymentSum",alreadyPaymentSum);
        //统计剩余额度（元）(担保额度-还款额度)
        AlInsuranceList insuranceList = alInsuranceListMapper.selectById(insuranceId);
        if (ObjectUtil.isNotNull(insuranceList)){
            BigDecimal guaranteeAmount = insuranceList.getGuaranteeAmount();
            map.put("company",insuranceList.getCustomerName());

            //剩余额度
            BigDecimal remainSum = guaranteeAmount.subtract(new BigDecimal(alreadyPaymentSum));
            map.put("remainSum",remainSum);

            if (remainSum.compareTo(BigDecimal.ZERO) <= 0) {//剩余额度为0.00,状态为已还清
                insuranceList.setPaymentStatus(3);//已还清
            }
            insuranceList.setRemainAmount(remainSum);//剩余额度
            //额度更新
            insuranceList.setRepaymentAmount(new BigDecimal(alreadyPaymentSum));//还款额度
            insuranceList.setRepaymentTime(count1);//还款期数
            alInsuranceListMapper.updateById(insuranceList);
        }

        QueryWrapper<AlReimbursement> wrapper = new QueryWrapper<>();
        wrapper.eq("insurance_id",insuranceId);
        AlReimbursement reimbursement = reimbursementMapper.selectOne(wrapper);
        if (ObjectUtil.isNotNull(reimbursement)){
            map.put("compensationAmount",reimbursement.getCompensationAmount());
            map.put("compensatoryBalance",reimbursement.getCompensatoryBalance());
        }

        return map;
    }

    @Override
    public void updateRepaymentEntry(Integer insuranceId) {
        //根据业务id查询该业务下的还款记录
        QueryWrapper<AlRepaymentEntry> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("insurance_id",insuranceId);
        List<AlRepaymentEntry> repaymentEntryList = baseMapper.selectList(queryWrapper);
        if (repaymentEntryList.size()>0){
            AlInsuranceList alInsuranceList = alInsuranceListMapper.selectById(insuranceId);
            int status = 1;
            //统计还款数据更新业务的相关状态
            for (AlRepaymentEntry repaymentEntry : repaymentEntryList){
                long times1 = new Date().getTime();//当前时间
                long times2 = repaymentEntry.getRepaymentDate().getTime();//应还款日
                long times3 = 0L;

                if(repaymentEntry.getActualRepaymentDate()!=null){
                    times3 = DateUtil.parseDate(repaymentEntry.getActualRepaymentDate()).getTime();//实际还款日
                }
                //如果存在当前日期超过应还款日时的数据，则计算逾期天数,并且状态改为已逾期，同时要处理保后管理模块内容
                if(repaymentEntry.getActualRepaymentDate()==null && times1 > times2){
                    int days = (int)((times1-times2)/1000/60/60/24);
                    repaymentEntry.setOverdueDays(days);
                    repaymentEntry.setRepaymentStatus(3);//已逾期
                    baseMapper.updateById(repaymentEntry);//更新逾期天数和状态
                    //更新保后管理列表处的状态为已逾期，并且is_overdue设置为0
                    // alInsuranceList.setPaymentStatus(2);//已逾期
                    // alInsuranceList.setIsOverdue(0);//是否逾期过：0->是；1->否；
                }else if(repaymentEntry.getActualRepaymentDate()==null && times1==times2){//应还款日=当前日期，则显示状态为待还款
                    repaymentEntry.setRepaymentStatus(1);//待还款
                    baseMapper.updateById(repaymentEntry);//更新状态
                }else if(repaymentEntry.getActualRepaymentDate()!=null && times3 <= times1){//实际还款日不超过当前日期时，状态为已还款
                    repaymentEntry.setRepaymentStatus(2);//已还款
                    baseMapper.updateById(repaymentEntry);//更新状态
                }else if(repaymentEntry.getActualRepaymentDate()==null && times1 <= times2){//当前日期未到应还款日,状态为未到期
                    repaymentEntry.setRepaymentStatus(4);//未到期
                    baseMapper.updateById(repaymentEntry);//更新状态
                }else if (alInsuranceList.getRemainAmount()!=null && repaymentEntry.getActualRepaymentDate()==null && alInsuranceList.getRemainAmount().doubleValue()>0){
                    //如果存在剩余额度,保后管理列表状态为还款中
                    alInsuranceList.setPaymentStatus(1);//还款中
                }

                //有逾期数据更换数值
                if (repaymentEntry.getRepaymentStatus()==3){
                    status = 3;
                }
            }
            if (status==3){
                alInsuranceList.setPaymentStatus(2);//已逾期
                alInsuranceList.setIsOverdue(0);//是否逾期过：0->是；1->否；
            }else {
                alInsuranceList.setPaymentStatus(1);//还款中
            }
            alInsuranceListMapper.updateById(alInsuranceList);
        }


        /*List<AlRepaymentEntryRes> list = this.baseMapper.selectBind();


        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat formatter2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date four = null;
        Date three = null;
        Date two = null;
        Date now = new Date();

        try {
            four = formatter2.parse(formatter.format(now)+" 00:00:00");
        } catch (ParseException e) {
            e.printStackTrace();
        }
        long times1 = four.getTime();//当前时间

        //遍历list取出应还款日，计算逾期天数=当前时间-应还款日
        for (AlRepaymentEntryRes res:list) {
            try {
                two = formatter2.parse(formatter.format(res.getRepaymentDate())+" 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            long times2 = two.getTime();//应还款日
            //long times2 = res.getRepaymentDate().getTime();//应还款日
            long times3=0L;
            if(res.getActualRepaymentDate()!=null){

                try {
                    three = formatter2.parse(formatter.format(res.getActualRepaymentDate())+" 00:00:00");
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                times3 = three.getTime();//实际还款日
                //times3 = res.getActualRepaymentDate().getTime();//实际还款日
            }

            QueryWrapper<AlRepaymentEntry> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id",res.getId());
            AlRepaymentEntry entry = baseMapper.selectOne(queryWrapper);

            QueryWrapper<AlInsuranceList> wrapper = new QueryWrapper<>();
            wrapper.eq("id",res.getInsuranceId());
            AlInsuranceList alInsuranceList = alInsuranceListMapper.selectOne(wrapper);
            //如果存在当前日期超过应还款日时的数据，则计算逾期天数,并且状态改为已逾期，同时要处理保后管理模块内容
            if(res.getActualRepaymentDate()==null && times1 > times2){
                int days = (int)((times1-times2)/1000/60/60/24);
                entry.setOverdueDays(days);
                entry.setRepaymentStatus(3);//已逾期
                this.baseMapper.updateById(entry);//更新逾期天数和状态
                //更新保后管理列表处的状态为已逾期，并且is_overdue设置为0
                alInsuranceList.setPaymentStatus(2);//已逾期
                alInsuranceList.setIsOverdue(0);//是否逾期过：0->是；1->否；
                alInsuranceListMapper.updateById(alInsuranceList);
            }else if(res.getActualRepaymentDate()==null && times1==times2){//应还款日=当前日期，则显示状态为待还款
                entry.setRepaymentStatus(1);//待还款
                this.baseMapper.updateById(entry);//更新状态
            }else if(res.getActualRepaymentDate()!=null && times3 <= times1){//实际还款日不超过当前日期时，状态为已还款
                entry.setRepaymentStatus(2);//已还款
                this.baseMapper.updateById(entry);//更新状态
            }else if(res.getActualRepaymentDate()==null && times1 <= times2){//当前日期未到应还款日,状态为未到期
                entry.setRepaymentStatus(4);//未到期
                this.baseMapper.updateById(entry);//更新状态
            }else if (res.getRemainAmount()!=null && res.getActualRepaymentDate()==null && res.getRemainAmount().doubleValue()>0){
                //如果存在剩余额度,保后管理列表状态为还款中
                alInsuranceList.setPaymentStatus(1);//还款中
                alInsuranceListMapper.updateById(alInsuranceList);
            }
        }*/
    }
}
