package com.daqing.financial.guarantee.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.feign.CrmsFeignService;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.*;
import com.daqing.financial.guarantee.model.request.BusinessApplicationRequest;
import com.daqing.financial.guarantee.model.response.BusinessApplicationDetailResponse;
import com.daqing.financial.guarantee.model.response.BusinessApplicationListResponse;
import com.daqing.financial.guarantee.model.response.DgApplyAmountInfoResponse;
import com.daqing.financial.guarantee.model.response.DgEnclosureInfoResponse;
import com.daqing.financial.guarantee.service.IDgApplyAmountInfoService;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.util.ArraysUtil;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.crms.request.DgApplyAmountInfoRequest;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.*;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RandomUtil;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 申请贷款信息 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Slf4j
@Service
public class DgApplyAmountInfoServiceImpl extends ServiceImpl<DgApplyAmountInfoMapper, DgApplyAmountInfo>
        implements IDgApplyAmountInfoService {

    @Resource
    private DgApplyAmountInfoMapper dgApplyAmountInfoMapper;

    @Resource
    private DgEnclosureInfoMapper dgEnclosureInfoMapper;

    @Resource
    private CrmsFeignService crmsFeignService;

    @Resource
    private HrmsFeignService hrmsFeignService;

    @Resource
    private DgGuaranteeAssignUserMapper dgGuaranteeAssignUserMapper;

    @Resource
    private DgAssetsInvestigationMapper dgAssetsInvestigationMapper;

    @Resource
    private  DgMessageInvestigationMapper dgMessageInvestigationMapper;

    @Resource
    private DgApplyAmountListMapper dgApplyAmountListMapper;

    @Resource
    private DgBusinessProcessStatusMapper dgBusinessProcessStatusMapper;
    @Resource
    private DgApplyAmountInfoController dgApplyAmountInfoController;
    @Resource
    private DgBusinessCompanyMapper dgBusinessCompanyMapper;

    @Resource
    private IDgCopyForService dgCopyForService;

    @Override
    public String getRoles() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        //获取当前用户拥有的角色ids
        return RedisUtil.get("dq:userRole:" + userId);
    }

    /**
     * 查询业务申请所有字段
     */
    @Override
    public DgApplyField queryApplyField() {

        return this.getBaseMapper().queryApplyField();
    }

    /**
     * 修改业务申请字段名称
     */
    @Override
    public Boolean updateApplyField(DgApplyField dgApplyField) {

        dgApplyField.setId(1);
        dgApplyField.setUpdateTime(new Date());

        return this.getBaseMapper().updateApplyField(dgApplyField);
    }

    @Override
    @Transactional
    public int businessApplication(BusinessApplicationRequest businessApplication) {
        //获取当前登录的用户信息
        String userId = null;
        try{
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            String token = request.getHeader("token");

            userId = RedisUtil.get("dq:token:"+token);
            System.out.println(userId);
        } catch (Exception e) {
            log.error("LogAspect 操作失败：" + e.getMessage());
            e.printStackTrace();
        }

        DgApplyAmountList dgApplyAmountList = new DgApplyAmountList();
        if(businessApplication.getIsSaveOrCommit()==1){//提交
            //根据社会统一信用代码判断是否可以申请业务，同一客户如果有一笔业务在业务流程在进行中，则不能再申请
            this.repeatBusiness(businessApplication);

            dgApplyAmountList.setStatus(StatusCode.SP_IN_REVIEW);//审批状态 设置为审核中
            dgApplyAmountList.setOperatingStatus(StatusCode.CZ_LAUNCH);//操作状态设置为已发起
        }else{//保存
            dgApplyAmountList.setStatus(StatusCode.SP_DRAFT);//审批状态 保存为草稿
            dgApplyAmountList.setOperatingStatus(StatusCode.CZ_TO_BE_SUBMITTED);//操作状态设置为待提交
        }

        //申请贷款相关信息
        DgApplyAmountInfo dgApplyAmountInfo = new DgApplyAmountInfo();
        dgApplyAmountInfo.setCompanyId(businessApplication.getCompanyId());
        dgApplyAmountInfo.setPresenterId(Integer.parseInt(userId));
        dgApplyAmountInfo.setApplyAmount(businessApplication.getApplyAmount());//申请额度
        dgApplyAmountInfo.setApplyTime(businessApplication.getApplyTime());//申请期限
        dgApplyAmountInfo.setBank(businessApplication.getBank());//贷款银行
        dgApplyAmountInfo.setAmountWide(businessApplication.getAmountWide());//贷款用途
        dgApplyAmountInfo.setOtherAmountWide(businessApplication.getOtherAmountWide());//其他贷款用途
        dgApplyAmountInfo.setEnGuaranteeMeasures(ArraysUtil.toString(businessApplication.getEnGuaranteeMeasures()));//反担保措施
        dgApplyAmountInfo.setDescription(businessApplication.getDescription());//反担保措施描述
        dgApplyAmountInfo.setBusinessType(businessApplication.getBusinessType());//业务类型
        dgApplyAmountInfo.setCustomerType(businessApplication.getCustomerType());//客户类型
        dgApplyAmountInfo.setApplyType(businessApplication.getApplyType());//申请类型
        //设置业务编号，查询今天是否申请过业务
        String businessCode=null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String dateTime=sdf.format(new Date());
        String timeCode=dateTime.replaceAll("-","");
        int count = dgApplyAmountInfoMapper.selectBusinessTaday(dateTime);
        if(count ==0){//今天没有申请过业务，所以编号从001开始
            if(businessApplication.getBusinessType()!= null && businessApplication.getBusinessType().equals("融资担保")){//融资业务：RZ-20201231001
                businessCode="RZ-"+timeCode+"001";
            }else{//非融资业务：FRZ-20201231001
                businessCode="FRZ-"+timeCode+"001";
            }
        }else{
            //查询最大业务编号
            String code = dgApplyAmountInfoMapper.selectMaxBusinessTaday(dateTime);
            int codeNumber = Integer.parseInt(code.substring(code.length() - 3));
            String newBusinessCode = String.format("%03d", (codeNumber + 1));
            if(businessApplication.getBusinessType()!= null && businessApplication.getBusinessType().equals("融资担保")){//融资业务：RZ-20201231001
                businessCode="RZ-"+timeCode+newBusinessCode;
            }else{//非融资业务：FRZ-20201231001
                businessCode="FRZ-"+timeCode+newBusinessCode;
            }
        }
        dgApplyAmountInfo.setBusinessCode(businessCode);

        int insert = dgApplyAmountInfoMapper.insert(dgApplyAmountInfo);

        //新增到业务列表
        dgApplyAmountList.setBusinessId(dgApplyAmountInfo.getId());
        dgApplyAmountList.setPresenterId(Integer.parseInt(userId));
        dgApplyAmountList.setEmpId(Integer.parseInt(userId));
        dgApplyAmountList.setRoleId(PromptSuccess.DB_YWY_ID);
        dgApplyAmountList.setType(1);//类型为担保业务员

        dgApplyAmountListMapper.insert(dgApplyAmountList);


        //修改是否存在关联人
        crmsFeignService.updateIsExistRelated(dgApplyAmountInfo.getCompanyId(),businessApplication.getIsExistRelated());

        //上传附件信息
        DgEnclosureInfo dgEnclosureInfo = new DgEnclosureInfo();
        dgEnclosureInfo.setBusinessId(dgApplyAmountInfo.getId());//业务id
        dgEnclosureInfo.setCompanyId(businessApplication.getCompanyId());//企业Id
        getEnclosure(businessApplication,dgEnclosureInfo);
        dgEnclosureInfoMapper.insert(dgEnclosureInfo);

        //根据企业id查询企业详情
        DgApplyAmountInfoRequest applyAmountInfo = new DgApplyAmountInfoRequest();
        applyAmountInfo.setCompanyId(dgApplyAmountInfo.getCompanyId());
        ResponseResult result = crmsFeignService.queryCustomerInfoById(applyAmountInfo);
        LinkedHashMap linkedList = (LinkedHashMap)result.getData();
        Object o1 = linkedList.get("linkman");
        Object o2 = linkedList.get("linkPhone");

        //备份企业信息，以防企业信息改动从而影响业务信息变更
        DgBusinessCompany dgBusinessCompany = new DgBusinessCompany();
        BeanUtils.copyProperties(businessApplication,dgBusinessCompany);
        dgBusinessCompany.setBusinessId(dgApplyAmountInfo.getId());
        dgBusinessCompany.setLinkman(o1.toString());
        dgBusinessCompany.setLinkPhone(o2.toString());
        dgBusinessCompanyMapper.insert(dgBusinessCompany);

        //如果审批状态为审核中，操作状态为已发起，则往担保部经理添加一条待审核的记录
        if(dgApplyAmountList.getStatus().equals(StatusCode.SP_IN_REVIEW) && dgApplyAmountList.getOperatingStatus().equals(StatusCode.CZ_LAUNCH)){
            DgApplyAmountList applyAmountList = new DgApplyAmountList();
            applyAmountList.setBusinessId(dgApplyAmountInfo.getId());//主键id
            applyAmountList.setPresenterId(Integer.parseInt(userId));// 提单人id
            applyAmountList.setRoleId(PromptSuccess.DB_JL_ID);//担保部经理角色id
            applyAmountList.setType(2);//设置类型
            applyAmountList.setStatus(StatusCode.SP_IN_REVIEW);//状态设置为审核中
            applyAmountList.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
            applyAmountList.setVer(1);//设置担保部经理的初始版本号
            dgApplyAmountListMapper.insert(applyAmountList);

            //同时往业务流程状态表新增一条记录
            DgBusinessProcessStatus status = new DgBusinessProcessStatus();
            status.setBusinessId(dgApplyAmountInfo.getId());
            status.setBusinessStatus(StatusCode.YW_IN_PROGRESS);//业务状态设置为进行中
            dgBusinessProcessStatusMapper.insert(status);
        }
        //如果不为保存就往审核流程进程表新增一条记录
        if(businessApplication.getIsSaveOrCommit()==1){
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(userId));
            AuditProcessUtil.save(Integer.parseInt(userId),dgApplyAmountInfo.getId(),account,
                    departmentId,"",1,PromptSuccess.DB_YWY_ID,"","","",StatusCode.CZ_LAUNCH);
        }
        return 1;
    }

    public void getEnclosure(BusinessApplicationRequest businessApplication,DgEnclosureInfo dgEnclosureInfo){
        //上传附件信息
//        if(businessApplication.getBusinessLicense()!=null)
            dgEnclosureInfo.setBusinessLicense(ArraysUtil.toString(businessApplication.getBusinessLicense()));//营业执照复印件
//        if(businessApplication.getLegalCardCopy()!=null)
            dgEnclosureInfo.setLegalCardCopy(ArraysUtil.toString(businessApplication.getLegalCardCopy()));//上传法定代表人夫妻及企业实际经营者身份证复印件
//        if(businessApplication.getMarriageCopy()!=null)
            dgEnclosureInfo.setMarriageCopy(ArraysUtil.toString(businessApplication.getMarriageCopy()));//法定代表人夫妻户口本，结婚证（离婚证）复印件
//        if(businessApplication.getLegalCopy()!=null)
            dgEnclosureInfo.setLegalCopy(ArraysUtil.toString(businessApplication.getLegalCopy()));//法定代表人身份证明
//        if(businessApplication.getCompanyConstitution()!=null)
            dgEnclosureInfo.setCompanyConstitution(ArraysUtil.toString(businessApplication.getCompanyConstitution()));//公司章程
//        if(businessApplication.getAccountingFirm()!=null)
            dgEnclosureInfo.setAccountingFirm(ArraysUtil.toString(businessApplication.getAccountingFirm()));//会计师事务所审计的上一年度及本年度审计报告
//        if(businessApplication.getCompanyCredit()!=null)
            dgEnclosureInfo.setCompanyCredit(ArraysUtil.toString(businessApplication.getCompanyCredit()));//企业信用报告
//        if(businessApplication.getCreditReport()!=null)
            dgEnclosureInfo.setCreditReport(ArraysUtil.toString(businessApplication.getCreditReport()));//企业法人夫妇信用报告，实际经营者信用报告
//        if(businessApplication.getCertificateAuthorization()!=null)
            dgEnclosureInfo.setCertificateAuthorization(ArraysUtil.toString(businessApplication.getCertificateAuthorization()));//征信业务授权书，承诺书
//        if(businessApplication.getTaxCertificate()!=null)
            dgEnclosureInfo.setTaxCertificate(ArraysUtil.toString(businessApplication.getTaxCertificate()));//增值税纳税申报表首表复印件/完税证明
//        if(businessApplication.getMeetingMinutes()!=null)
            dgEnclosureInfo.setMeetingMinutes(ArraysUtil.toString(businessApplication.getMeetingMinutes()));//股东会会议纪要
//        if(businessApplication.getAssessmentReport()!=null)
            dgEnclosureInfo.setAssessmentReport(ArraysUtil.toString(businessApplication.getAssessmentReport()));//反担保资料和评估报告
//        if(businessApplication.getConversationSummary()!=null)
            dgEnclosureInfo.setConversationSummary(ArraysUtil.toString(businessApplication.getConversationSummary()));
//        if(businessApplication.getInspectionPhotos()!=null)
            dgEnclosureInfo.setInspectionPhotos(ArraysUtil.toString(businessApplication.getInspectionPhotos()));

    }

    @Override
    public PageUtils queryPage(Integer page, Integer size, String CustomerNumberOrName) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<BusinessApplicationListResponse> positionVO = this.getBaseMapper().pageByCondition(new Page(page, size),CustomerNumberOrName);

        return new PageUtils(positionVO);
    }

    @Override
    public BusinessApplicationDetailResponse businessApplicationDetail(Integer id) {
        //根据id查询业务id
        DgApplyAmountList dgApplyAmountList = dgApplyAmountListMapper.selectById(id);
        BusinessApplicationDetailResponse response = publicBusinessCommon(id);

        //根据业务id查询备份企业信息
        DgBusinessCompany dgBusinessCompany = dgBusinessCompanyMapper.selectOne(new QueryWrapper<DgBusinessCompany>()
                .eq("business_id", dgApplyAmountList.getBusinessId()));

        response.setDgBusinessCompany(dgBusinessCompany);
        return response;
    }

    public BusinessApplicationDetailResponse publicBusinessCommon(Integer id) {
        //根据id查询业务id
        DgApplyAmountList dgApplyAmountList = dgApplyAmountListMapper.selectById(id);
        //根据主键id查询业务申请详细信息
        //DgApplyAmountInfo dgApplyAmountInfo = this.baseMapper.selectById(id);
        DgApplyAmountInfoResponse dgApplyAmountInfo = this.baseMapper.selectApplyAmountInfoById(dgApplyAmountList.getBusinessId());
        String measures=dgApplyAmountInfo.getEnGuaranteeMeasures();
        if(measures != null){
            if(measures.length()>0){
                String a = measures.substring(1,measures.length()-1);
                String[] arry =a.split(",");
                List<String> demoList = Arrays.asList(arry);
                dgApplyAmountInfo.setEnGuaranteeMeasure(demoList);
            }
        }
        dgApplyAmountInfo.setRemark(dgApplyAmountList.getApplyContent());

        //查询附件详细，附件关联业务主键id
        DgEnclosureInfoResponse dgEnclosureInfo = dgEnclosureInfoMapper.selectByCompanyId(dgApplyAmountList.getBusinessId());

        List<String>list = new ArrayList<>();
        System.out.println(dgEnclosureInfo.getAccountingFirm());
        list.add(dgEnclosureInfo.getAccountingFirm());
        list.add(dgEnclosureInfo.getAssessmentReport());
        list.add(dgEnclosureInfo.getBusinessLicense());
        list.add(dgEnclosureInfo.getCertificateAuthorization());
        list.add(dgEnclosureInfo.getCompanyConstitution());
        list.add(dgEnclosureInfo.getCompanyCredit());
        list.add(dgEnclosureInfo.getConversationSummary());
        list.add(dgEnclosureInfo.getCreditReport());
        list.add(dgEnclosureInfo.getInspectionPhotos());
        list.add(dgEnclosureInfo.getLegalCardCopy());
        list.add(dgEnclosureInfo.getLegalCopy());
        list.add(dgEnclosureInfo.getTaxCertificate());
        list.add(dgEnclosureInfo.getMeetingMinutes());
        list.add(dgEnclosureInfo.getMarriageCopy());

        List arr1List2 = new ArrayList<>();

        for(int i=0;i<list.size();i++){
            List<String> arr1List;
            if(list.get(i)!=null){
                String a1 = list.get(i).substring(1,list.get(i).length()-1);
                String[] arr1 =a1.split(",");
                arr1List = Arrays.asList(arr1);
                arr1List2.add(i,arr1List);
            }else{
                arr1List2.add(i,null);
            }
        }
/*        if(arr1List2.size()>0){
            if(!arr1List2.get(0).toString().equals("[]"))*/
        dgEnclosureInfo.setAccountingFirmList((List<String>) arr1List2.get(0));
        dgEnclosureInfo.setAssessmentReportList((List<String>) arr1List2.get(1));
        dgEnclosureInfo.setBusinessLicenseList((List<String>) arr1List2.get(2));
        dgEnclosureInfo.setCertificateAuthorizationList((List<String>) arr1List2.get(3));
        dgEnclosureInfo.setCompanyConstitutionList((List<String>) arr1List2.get(4));
        dgEnclosureInfo.setCompanyCreditList((List<String>) arr1List2.get(5));
        dgEnclosureInfo.setConversationSummaryList((List<String>) arr1List2.get(6));
        dgEnclosureInfo.setCreditReportList((List<String>) arr1List2.get(7));
        dgEnclosureInfo.setInspectionPhotosList((List<String>) arr1List2.get(8));
        dgEnclosureInfo.setLegalCardCopyList((List<String>) arr1List2.get(9));
        dgEnclosureInfo.setLegalCopyList((List<String>) arr1List2.get(10));
        dgEnclosureInfo.setTaxCertificateList((List<String>) arr1List2.get(11));
        dgEnclosureInfo.setMeetingMinutesList((List<String>) arr1List2.get(12));
        dgEnclosureInfo.setMarriageCopyList((List<String>) arr1List2.get(13));
//        }

        BusinessApplicationDetailResponse businessApplicationDetail= new BusinessApplicationDetailResponse();
        businessApplicationDetail.setDgApplyAmountInfo(dgApplyAmountInfo);
        businessApplicationDetail.setDgEnclosureInfo(dgEnclosureInfo);
        return businessApplicationDetail;
    }

    @Override
    public BusinessApplicationDetailResponse xcxBusinessAppDetail(Integer id) {
        //根据id查询业务id
        DgApplyAmountList dgApplyAmountList = dgApplyAmountListMapper.selectById(id);
        BusinessApplicationDetailResponse response = publicBusinessCommon(id);

        DgApplyAmountInfo dgApplyAmountInfo = dgApplyAmountInfoMapper.selectById(dgApplyAmountList.getBusinessId());
        DgApplyAmountInfoRequest applyAmountInfo = new DgApplyAmountInfoRequest();
        applyAmountInfo.setCompanyId(dgApplyAmountInfo.getCompanyId());
        //根据企业id查询企业详情
        ResponseResult result = crmsFeignService.queryCustomerInfoById(applyAmountInfo);

        LinkedHashMap linkedList = (LinkedHashMap)result.getData();
        response.setLinkedHashMap(linkedList);
        return response;
    }

    @Override
    public int updateBusinessApplication(BusinessApplicationRequest businessApplication) {
        //根据业务id以及type查询对应的记录
        DgApplyAmountList dgApplyAmountList = dgApplyAmountListMapper.selectOne(new QueryWrapper<DgApplyAmountList>()
                .eq("business_id", businessApplication.getBusinessId()).eq("type", 1));
        //审批状态为草稿或已撤销的情况下进行判断
        if(dgApplyAmountList.getStatus().equals(StatusCode.SP_DRAFT) || dgApplyAmountList.getStatus().equals(StatusCode.SP_RESCINDED)){
            //根据社会统一信用代码判断是否可以申请业务，同一客户如果有一笔业务在业务流程在进行中，则不能再申请
            this.repeatBusiness(businessApplication);
        }

        //申请贷款相关信息
        DgApplyAmountInfo dgApplyAmountInfo = new DgApplyAmountInfo();
        dgApplyAmountInfo.setId(businessApplication.getBusinessId());
        dgApplyAmountInfo.setCompanyId(businessApplication.getCompanyId());
        dgApplyAmountInfo.setApplyAmount(businessApplication.getApplyAmount());//申请额度
        dgApplyAmountInfo.setApplyTime(businessApplication.getApplyTime());//申请期限
        dgApplyAmountInfo.setBank(businessApplication.getBank());//贷款银行
        dgApplyAmountInfo.setAmountWide(businessApplication.getAmountWide());//贷款用途
        dgApplyAmountInfo.setOtherAmountWide(businessApplication.getOtherAmountWide());//其他贷款用途
        dgApplyAmountInfo.setEnGuaranteeMeasures(ArraysUtil.toString(businessApplication.getEnGuaranteeMeasures()));//反担保措施
        dgApplyAmountInfo.setDescription(businessApplication.getDescription());//反担保措施描述
        dgApplyAmountInfo.setBusinessType(businessApplication.getBusinessType());//业务类型
        dgApplyAmountInfo.setCustomerType(businessApplication.getCustomerType());//客户类型
        dgApplyAmountInfo.setApplyType(businessApplication.getApplyType());//申请类型
        //根据业务id修改申请贷款信息
        this.baseMapper.updateById(dgApplyAmountInfo);

        //业务员操作即可处理-------------------------------

        //草稿状态下，新增业务状态到业务状态表,同时新增一条记录到担保经理处理
        if(dgApplyAmountList.getStatus().equals(StatusCode.SP_DRAFT)){
            DgBusinessProcessStatus processStatus = new DgBusinessProcessStatus();
            processStatus.setBusinessId(businessApplication.getBusinessId());//业务id
            processStatus.setBusinessStatus(1);//进行中
            dgBusinessProcessStatusMapper.insert(processStatus);

            DgApplyAmountList applyAmountList = new DgApplyAmountList();
            applyAmountList.setBusinessId(dgApplyAmountInfo.getId());//主键id
            applyAmountList.setPresenterId(dgApplyAmountList.getPresenterId());// 提单人id
            applyAmountList.setRoleId(PromptSuccess.DB_JL_ID);//担保部经理角色id
            applyAmountList.setType(2);//设置类型
            applyAmountList.setStatus(StatusCode.SP_IN_REVIEW);//状态设置为审核中
            applyAmountList.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
            applyAmountList.setVer(1);//设置担保部经理的初始版本号
            dgApplyAmountListMapper.insert(applyAmountList);
        }

        //撤销状态下,修改业务状态
        if(dgApplyAmountList.getStatus().equals(StatusCode.SP_RESCINDED)){
            DgBusinessProcessStatus processStatus = new DgBusinessProcessStatus();
            processStatus.setBusinessStatus(1);//进行中
            dgBusinessProcessStatusMapper.update(processStatus,new QueryWrapper<DgBusinessProcessStatus>()
                    .eq("business_id",businessApplication.getBusinessId()));
        }
        //将业务员状态改为审核中，操作状态改为已处理
        dgApplyAmountList.setId(dgApplyAmountList.getId());
        dgApplyAmountList.setStatus(StatusCode.SP_IN_REVIEW);//审核中
        dgApplyAmountList.setOperatingStatus(StatusCode.CZ_PROCESSED);//已处理
        dgApplyAmountListMapper.updateById(dgApplyAmountList);
        //同时将担保部经理状态改为审核中，操作状态改为待处理
        DgApplyAmountList applyAmountList = dgApplyAmountListMapper.selectOne(new QueryWrapper<DgApplyAmountList>()
                .eq("business_id", businessApplication.getBusinessId()).eq("type", 2));
        if(applyAmountList != null){
            applyAmountList.setId(applyAmountList.getId());
            applyAmountList.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            applyAmountList.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理
            dgApplyAmountListMapper.updateById(applyAmountList);
        }
        //结束---------------------------------------------

        //业务申请重新绑定关联人
        //修改是否存在关联人
        crmsFeignService.updateIsExistRelated(businessApplication.getCompanyId(),businessApplication.getIsExistRelated());

        //上传附件信息
        DgEnclosureInfo dgEnclosureInfo = new DgEnclosureInfo();
        dgEnclosureInfo.setBusinessId(businessApplication.getBusinessId());//业务id
        dgEnclosureInfo.setCompanyId(businessApplication.getCompanyId());//企业Id
        getEnclosure(businessApplication,dgEnclosureInfo);

        //根据业务id查询对应的附件信息
        DgEnclosureInfo dgEnclosureInfo1 = dgEnclosureInfoMapper.selectOne(new QueryWrapper<DgEnclosureInfo>().eq("business_id", businessApplication.getBusinessId()));
        dgEnclosureInfo.setConversationSummary(dgEnclosureInfo1.getConversationSummary());//谈话纪要
        dgEnclosureInfo.setInspectionPhotos(dgEnclosureInfo1.getInspectionPhotos());//考察照片
        dgEnclosureInfo.setOtherFile(dgEnclosureInfo1.getOtherFile());//其他

        //根据业务id删除对应的附件信息
        dgEnclosureInfoMapper.delete(new QueryWrapper<DgEnclosureInfo>().eq("business_id",businessApplication.getBusinessId()));

        //根据企业id查询企业详情
        DgApplyAmountInfoRequest applyAmountInfo = new DgApplyAmountInfoRequest();
        applyAmountInfo.setCompanyId(dgApplyAmountInfo.getCompanyId());
        ResponseResult result = crmsFeignService.queryCustomerInfoById(applyAmountInfo);
        LinkedHashMap linkedList = (LinkedHashMap)result.getData();
        Object o1 = linkedList.get("linkman");
        Object o2 = linkedList.get("linkPhone");
        //修改备份企业信息
        DgBusinessCompany dgBusinessCompany = new DgBusinessCompany();
        BeanUtils.copyProperties(businessApplication,dgBusinessCompany);
        dgBusinessCompany.setLinkPhone(String.valueOf(o2));
        dgBusinessCompany.setLinkman(String.valueOf(o1));
        System.out.println("dgBusinessCompany============="+dgBusinessCompany);
        dgBusinessCompanyMapper.update(dgBusinessCompany,new QueryWrapper<DgBusinessCompany>()
                .eq("business_id",businessApplication.getBusinessId()));

        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),businessApplication.getBusinessId(),account,
                departmentId,"",1,PromptSuccess.DB_YWY_ID,"","","",StatusCode.CZ_LAUNCH);

        //附件新增
        return dgEnclosureInfoMapper.insert(dgEnclosureInfo);
    }

    @Override
    public int revokeBusinessApplication(Integer id) {
        //根据id查询业务id
        DgApplyAmountList dgApplyAmountList1 = dgApplyAmountListMapper.selectById(id);
        //根据id修改当前业务状态
        DgApplyAmountList dgApplyAmountList = new DgApplyAmountList();
        dgApplyAmountList.setId(id);
        dgApplyAmountList.setStatus(StatusCode.SP_RESCINDED);//状态为已撤销
        dgApplyAmountList.setOperatingStatus(StatusCode.CZ_TO_BE_SUBMITTED);//操作状态为待提交
        dgApplyAmountListMapper.updateById(dgApplyAmountList);
        //修改担保部经理审核状态

        DgApplyAmountList applyAmountList = new DgApplyAmountList();
        applyAmountList.setBusinessId(dgApplyAmountList1.getBusinessId());//业务id
        applyAmountList.setType(2);//类型为担保部经理审核
        applyAmountList.setStatus(StatusCode.SP_RESCINDED);//状态为已撤销
        applyAmountList.setOperatingStatus(StatusCode.CZ_TO_BE_DEFINED);//操作状态为待定义，目前前端不展示
        dgApplyAmountListMapper.update(applyAmountList,new QueryWrapper<DgApplyAmountList>()
                .eq("business_id",applyAmountList.getBusinessId())
                .eq("type",applyAmountList.getType()));
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),dgApplyAmountList1.getBusinessId(),
                account,departmentId,"",1,PromptSuccess.DB_YWY_ID,"","","",StatusCode.CZ_LAUNCH);

        //将业务流程状态设置为已撤销

        DgBusinessProcessStatus status = new DgBusinessProcessStatus();
        status.setBusinessId(dgApplyAmountList1.getBusinessId());
        status.setBusinessStatus(StatusCode.YW_RESCINDED);//业务状态设置为已撤销
        return dgBusinessProcessStatusMapper.update(status,new QueryWrapper<DgBusinessProcessStatus>()
                .eq("business_id",status.getBusinessId()));
    }

    @Transactional
    @Override
    public Boolean excelExport(List<String> ids, Integer userId, HttpServletResponse response3) {
        //根据ids查询业务申请列表信息
        List<BusinessApplicationListResponse>applyList = this.baseMapper.selectListByIds(ids,userId);
        System.out.println("applyList==========================="+applyList);
        if(applyList != null){
            if(applyList.size()>0){
                List<Integer> arr = new ArrayList<>();
                for (BusinessApplicationListResponse res : applyList) {
                    arr.add(res.getPresenterId());
                }
                //根据提单人id查询其部门名称
                ResponseResult responseResult = hrmsFeignService.getAccountAndDeptNameById(arr);
                List<LinkedHashMap> employeeMessage = null;
                if(responseResult.getData() != null){
                    employeeMessage = (List<LinkedHashMap>) responseResult.getData();
                }

                if(employeeMessage!= null && employeeMessage.size()>0){
                    for (BusinessApplicationListResponse response:applyList) {
                        for(LinkedHashMap res : employeeMessage){
                            if(response.getPresenterId().equals(res.get("id"))){//如果提单人id相同情况下，就往对象里面赋值
                                response.setAccount(JSONObject.toJSONString(res.get("account")).replace("\"",""));
                                response.setDeptName(JSONObject.toJSONString(res.get("deptName")).replace("\"",""));
                            }
                            if(response != null){
                                if(response.getAccount()==null){
                                    response.setAccount("客户");
                                }
                            }
                        }
                    }
                }else{
                    for (BusinessApplicationListResponse response:applyList) {
                        if(response != null){
                            if(response.getAccount()==null){
                                response.setAccount("客户");
                            }
                        }
                    }
                }

//                for (BusinessApplicationListResponse response2:applyList) {
//                    for(LinkedHashMap res : employeeMessage){
//                        if(response2.getPresenterId().equals(res.get("id"))){//如果提单人id相同情况下，就往对象里面赋值
//                            response2.setAccount(JSONObject.toJSONString(res.get("account")).replace("\"",""));
//                            response2.setDeptName(JSONObject.toJSONString(res.get("deptName")).replace("\"",""));
//                        }
//                    }
//                }
            }
        }
        try {

            ExcelUtil.writeExcelWithSheets(response3, applyList, "业务贷款一览表", "businessApplication", new BusinessApplicationListResponse())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    @Transactional
    @Override
    public int approvalBusinessApplication(Integer id, Integer status, String remark, Integer companyId) {
        //获取当前登录用户(审批人)id
        String userId = null;
        try{
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            String token = request.getHeader("token");
            userId = RedisUtil.get("dq:token:"+token);
            System.out.println(userId);
        } catch (Exception e) {
            e.printStackTrace();
        }
        //根据id查询业务id
        DgApplyAmountList dgApplyAmountList1 = dgApplyAmountListMapper.selectById(id);

        //根据dg_apply_amount_list表的id修改当前业务状态
        DgApplyAmountList dgApplyAmountList = new DgApplyAmountList();
        dgApplyAmountList.setStatus(status);
        dgApplyAmountList.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态改为已处理
        dgApplyAmountList.setApplyContent(remark);
        dgApplyAmountList.setEmpId(Integer.parseInt(userId));
        dgApplyAmountList.setVer(dgApplyAmountList1.getVer()+1);//设置担保部经理的版本号+1
        int update = dgApplyAmountListMapper.update(dgApplyAmountList, new QueryWrapper<DgApplyAmountList>()
                .eq("id", id).eq("ver", dgApplyAmountList1.getVer()));
        if(update<=0)
            return 0;

        //修改专员那一条记录审核状态
        DgApplyAmountList applyAmountList = new DgApplyAmountList();
        applyAmountList.setStatus(status);
        dgApplyAmountListMapper.update(applyAmountList,new QueryWrapper<DgApplyAmountList>()
                .eq("business_id",dgApplyAmountList1.getBusinessId()));

        //如果是A,B角驳回的情况，此处审核通过之后不经过指派A,B角的操作，直接到调查
        //根据业务id查询A,B角的操作状态是否是已处理，如果是，则为A,B角已驳回的数据
        DgGuaranteeAssignUser assignUser = dgGuaranteeAssignUserMapper.selectOne(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", dgApplyAmountList1.getBusinessId()).eq("type", 1));
        DgGuaranteeAssignUser assignUser2 = dgGuaranteeAssignUserMapper.selectOne(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", dgApplyAmountList1.getBusinessId()).eq("type", 2));

        if(status.equals(StatusCode.SP_REFUSE)){//如果为拒绝，业务状态则为已否决
            DgBusinessProcessStatus processStatus = new DgBusinessProcessStatus();
            processStatus.setBusinessId(dgApplyAmountList1.getBusinessId());
            processStatus.setBusinessStatus(StatusCode.YW_REJECTED);//业务状态设置为已否决
            dgBusinessProcessStatusMapper.update(processStatus,new QueryWrapper<DgBusinessProcessStatus>()
                    .eq("business_id",processStatus.getBusinessId()));
        }else if(status.equals(StatusCode.SP_REJECT)){//状态为驳回情况
            //担保业务员状态改为驳回，操作状态为已处理
            DgApplyAmountList applyAmount = new DgApplyAmountList();
            applyAmount.setStatus(status);
            applyAmount.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态改为待处理
            dgApplyAmountListMapper.update(applyAmount,new QueryWrapper<DgApplyAmountList>()
                    .eq("business_id",dgApplyAmountList1.getBusinessId()).eq("type",1));
        }else if(assignUser != null && assignUser2 != null){
            if((assignUser.getOperatingStatus().equals(StatusCode.CZ_PROCESSED)&& assignUser.getStatus().equals(StatusCode.SP_REJECT))
                    ||(assignUser2.getOperatingStatus().equals(StatusCode.CZ_PROCESSED)) && assignUser2.getStatus().equals(StatusCode.SP_REJECT)){
                //此处为AB角驳回情况，如果审核通过，则修改A,B角操作状态为待处理,状态为审核中
                if(status.equals(StatusCode.SP_REVIEWED)){
                    DgGuaranteeAssignUser user = new DgGuaranteeAssignUser();
                    user.setOperatingStatus(StatusCode.CZ_ON_HAND);
                    user.setStatus(StatusCode.SP_IN_REVIEW);
                    dgGuaranteeAssignUserMapper.update(user,new QueryWrapper<DgGuaranteeAssignUser>()
                            .eq("business_id",dgApplyAmountList1.getBusinessId()).eq("type",1));
                    dgGuaranteeAssignUserMapper.update(user,new QueryWrapper<DgGuaranteeAssignUser>()
                            .eq("business_id",dgApplyAmountList1.getBusinessId()).eq("type",2));
                    //担保部经理处，驳回状态改为审核中（现改为所有该业务下状态均改为审核中）
                    DgGuaranteeAssignUser user2 = new DgGuaranteeAssignUser();
                    user2.setStatus(StatusCode.SP_IN_REVIEW);
                    dgGuaranteeAssignUserMapper.update(user2,new QueryWrapper<DgGuaranteeAssignUser>()
                            .eq("business_id",dgApplyAmountList1.getBusinessId()));//.eq("type",3)
                }
            }
        }else if(dgApplyAmountList.getStatus().equals(StatusCode.SP_REVIEWED) && dgApplyAmountList.getOperatingStatus().equals(StatusCode.CZ_PROCESSED)){
            //如果审批状态为已审核，并且操作状态为已处理，则同时新增数据到担保部，资产部和信息部，让这三个模块去审
            //新增一条记录到担保部
            DgGuaranteeAssignUser guaranteeAssignUser = new DgGuaranteeAssignUser();
            guaranteeAssignUser.setBusinessId(dgApplyAmountList1.getBusinessId());//业务id
            guaranteeAssignUser.setCompanyId(companyId);//企业id
            guaranteeAssignUser.setRoleId(PromptSuccess.DB_JL_ID);//担保部经理角色id
            guaranteeAssignUser.setType(3);//角色类型
            guaranteeAssignUser.setStatus(StatusCode.SP_TO_BE_ASSIGNED);//状态为待指派
            guaranteeAssignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态为待处理
            dgGuaranteeAssignUserMapper.insert(guaranteeAssignUser);

            //新增一条记录到资产部
            DgAssetsInvestigation dgAssetsInvestigation = new DgAssetsInvestigation();
            dgAssetsInvestigation.setBusinessId(dgApplyAmountList1.getBusinessId());//业务id
            dgAssetsInvestigation.setCompanyId(companyId);//企业id
            dgAssetsInvestigation.setRoleId(PromptSuccess.ZC_JL_ID);//资产部经理角色id
            dgAssetsInvestigation.setType(2);//角色类型
            dgAssetsInvestigation.setStatus(StatusCode.SP_TO_BE_ASSIGNED);//状态为待指派
            dgAssetsInvestigation.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态为待处理
            dgAssetsInvestigationMapper.insert(dgAssetsInvestigation);

            //新增一条记录到信息部
            DgMessageInvestigation dgMessageInvestigation = new DgMessageInvestigation();
            dgMessageInvestigation.setBusinessId(dgApplyAmountList1.getBusinessId());//业务id
            dgMessageInvestigation.setCompanyId(companyId);//企业id
            dgMessageInvestigation.setRoleId(PromptSuccess.XX_JL_ID);//信息部经理角色id
            dgMessageInvestigation.setType(2);//角色类型
            dgMessageInvestigation.setStatus(StatusCode.SP_TO_BE_ASSIGNED);//状态为待指派
            dgMessageInvestigation.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态为待处理
            dgMessageInvestigationMapper.insert(dgMessageInvestigation);
        }
        Integer type = dgApplyAmountList1.getType();
        String deptName;
        Integer roleId;
        if(type==1){
            deptName="担保部业务员";
            roleId = PromptSuccess.DB_YWY_ID;
        }else{
            deptName="担保部经理";
            roleId = PromptSuccess.DB_JL_ID;
        }
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),dgApplyAmountList1.getBusinessId(),
                account,departmentId,remark,1,roleId,"","","",status);

        boolean b=false;
        //根据业务为经理处理以及是否驳回决定是否抄送
        if(roleId.equals(PromptSuccess.DB_JL_ID) && !status.equals(StatusCode.SP_REJECT)){//经理，驳回
            try {
                b = dgCopyForService.copySendUser(dgApplyAmountList1.getBusinessId(), 1, id);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return 1;
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        return userId;
    }

    public void repeatBusiness(BusinessApplicationRequest businessApplication){
        //根据社会统一信用代码判断是否可以申请业务，同一客户如果有一笔业务在业务流程在进行中，则不能再申请
        //首先根据统一社会信用代码查询备份企业业务关联表中的业务id
        List<DgBusinessCompany> businessCompanyList = dgBusinessCompanyMapper.selectList(new QueryWrapper<DgBusinessCompany>()
                .eq("social_unified_code", businessApplication.getSocialUnifiedCode()));
        List<Integer> businessIds = new ArrayList<Integer>();
        for (DgBusinessCompany businessCompany:businessCompanyList) {
            businessIds.add(businessCompany.getBusinessId());
        }
        //根据业务id查询dg_business_process_status表中是否存在业务为进行中的记录
        Integer businessStatus=StatusCode.YW_IN_PROGRESS;//进行中
        if(businessIds.size()>0){
            int count2 = dgBusinessProcessStatusMapper.selectBybusinessIdAndStatus(businessIds,businessStatus);
            //如果存在，则不能再申请业务
            if(count2>0){
                ExceptionCast.cast(GuaranteeCode.REPEAT_BUSINESS);
            }
        }
    }

}
