package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.feign.CrmsFeignService;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.*;
import com.daqing.financial.guarantee.model.request.ApprovalAssetsRequest;
import com.daqing.financial.guarantee.model.request.AssetsRequest;
import com.daqing.financial.guarantee.model.request.InvestigateAssetsRequest;
import com.daqing.financial.guarantee.model.response.*;
import com.daqing.financial.guarantee.service.IDgAssetsInvestigationService;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.util.ArraysUtil;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.crms.request.DgApplyAmountInfoRequest;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.*;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * <p>
 * 资产调查 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-23
 */
@Service
public class DgAssetsInvestigationServiceImpl extends ServiceImpl<DgAssetsInvestigationMapper, DgAssetsInvestigation> implements IDgAssetsInvestigationService {

    @Resource
    private DgApplyAmountInfoController dgApplyAmountInfoController;

    @Resource
    private DgAssetsInvestigationMapper dgAssetsInvestigationMapper;

    @Resource
    private HrmsFeignService hrmsFeignService;

    @Resource
    private DgApplyAmountInfoMapper dgApplyAmountInfoMapper;

    @Resource
    private CrmsFeignService crmsFeignService;

    @Resource
    private DgAssetsAssessmentAssetsLogMapper dgAssetsAssessmentAssetsLogMapper;

    @Resource
    private DgAssetsMoneyMapper dgAssetsMoneyMapper;
    @Resource
    private DgGuaranteeAssignUserMapper dgGuaranteeAssignUserMapper;
    @Resource
    private DgMessageInvestigationMapper dgMessageInvestigationMapper;
    @Resource
    private DgComplianceInvestigationMapper dgComplianceInvestigationMapper;
    @Resource
    private DgBusinessProcessStatusMapper dgBusinessProcessStatusMapper;
    @Resource
    private DgBusinessCompanyMapper dgBusinessCompanyMapper;

    @Resource
    private IDgCopyForService dgCopyForService;

    @Override
    public PageUtils queryPage(Integer page, Integer size, String customerNumberOrName, List<String> roleIdlist, String userId, Integer status) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<AssetsInvestigationListResponse> positionVO = this.getBaseMapper().pageByCondition(new Page(page, size),customerNumberOrName,roleIdlist,userId,status);

        return new PageUtils(positionVO);
    }

    @Transactional
    @Override
    public Boolean assignCorners(AssetsRequest assetsRequest) {
        //判断即将被指派成A角的用户id以及B角的用户id是否在当前业务中已被指派
        //查询empId在担保部调查中是否被指派
        Integer count1 = dgGuaranteeAssignUserMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", assetsRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", assetsRequest.getEmpId()));

        Integer count = dgGuaranteeAssignUserMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", assetsRequest.getBusinessId())
                .eq("type", 2).eq("emp_id", assetsRequest.getEmpId()));
        if(count1 >0 || count >0){//已被指派情况下直接返回
            ExceptionCast.cast(HrmsCode.EMP_TO_BE_CHOOSE);
        }

        //查询empId在信息部调查中是否被指派
        Integer count2 = dgMessageInvestigationMapper.selectCount(new QueryWrapper<DgMessageInvestigation>()
                .eq("business_id", assetsRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", assetsRequest.getEmpId()));

        if(count2 >0 ){//已被指派情况下直接返回
            ExceptionCast.cast(HrmsCode.EMP_TO_BE_CHOOSE);
        }

        //根据主键id修改当前记录状态为审核中，操作状态为已处理,设置审批人id为当前用户id
        //获取当前登录用户userId
        String userId = DgApplyAmountInfoController.getUserId();
        DgAssetsInvestigation assetsInvestigation = new DgAssetsInvestigation();
        assetsInvestigation.setStatus(StatusCode.SP_IN_REVIEW);//状态设置为审核中
        assetsInvestigation.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
        assetsInvestigation.setEmpId(Integer.parseInt(userId));
        assetsInvestigation.setId(assetsRequest.getId());
        dgAssetsInvestigationMapper.updateById(assetsInvestigation);

        //新增资产部调查员记录到资产部记录表
        DgAssetsInvestigation dgAssetsInvestigation = new DgAssetsInvestigation();
        dgAssetsInvestigation.setBusinessId(assetsRequest.getBusinessId());//业务id
        dgAssetsInvestigation.setCompanyId(assetsRequest.getCompanyId());//企业id
        dgAssetsInvestigation.setEmpId(assetsRequest.getEmpId());//资产部调查员id
        dgAssetsInvestigation.setRoleId(PromptSuccess.ZC_ZY_ID);//角色设置为资产部专员
        dgAssetsInvestigation.setType(1);//专员操作
        dgAssetsInvestigation.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
        dgAssetsInvestigation.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
        dgAssetsInvestigationMapper.insert(dgAssetsInvestigation);
        //查询用户角色表是否已绑定，如果没有绑定，则去往绑定页面
        ResponseResult i = hrmsFeignService.selectUserRole(assetsRequest.getEmpId().longValue(),PromptSuccess.ZC_ZY_ID.longValue());
        int a = (int) i.getData();
        if(a==0) {
            //绑定角色关系
            hrmsFeignService.insertUserRole(assetsRequest.getEmpId().longValue(), PromptSuccess.ZC_ZY_ID.longValue());
        }
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),assetsRequest.getBusinessId(),
                account, departmentId, "", 3,PromptSuccess.ZC_JL_ID,"","","",StatusCode.SP_REVIEWED);

        return true;
    }

    @Transactional
    @Override
    public AssetsDetailResponse assetsDetail(Integer id) {
        //根据主键id查询资产调查详情
        DgAssetsInvestigationResponse assetsResponse = this.getBaseMapper().selectInfoById(id);

        String remarks = assetsResponse.getRemark();
        if(remarks != null){
            if(remarks.length()>0){
                String a = remarks.substring(1,remarks.length()-1);
                String[] arry =a.split(",");
                List<String> demoList = Arrays.asList(arry);
                assetsResponse.setRemarkList(demoList);
            }
        }

        //根据业务id以及type为1查询资产详细
        DgAssetsInvestigation assetsInvestigation = this.baseMapper.selectOne(new QueryWrapper<DgAssetsInvestigation>()
                .eq("business_id", assetsResponse.getBusinessId()).eq("type", 1));
        if(assetsInvestigation != null){
            String assetsAmounts=assetsInvestigation.getAssetsAmount();
            if(assetsAmounts != null){
                if(assetsAmounts.length()>0){
                    String a = assetsAmounts.substring(1,assetsAmounts.length()-1);
                    String[] arry =a.split(",");
                    List<String> demoList = Arrays.asList(arry);
                    assetsResponse.setAssetsAmountList(demoList);
                }
            }

            AssetsInvestigationResponse assetsInvestigationRes = new AssetsInvestigationResponse();
            if(assetsInvestigation != null){
                BeanUtils.copyProperties(assetsInvestigation,assetsInvestigationRes);
                BeanUtils.copyProperties(assetsInvestigationRes,assetsResponse);
            }
        }
        //根据业务id查询评估价值List
/*        List<DgAssetsAssessmentAssetsLog> assessmentlist = dgAssetsAssessmentAssetsLogMapper.selectList(
                new QueryWrapper<DgAssetsAssessmentAssetsLog>().eq("business_id",assetsResponse.getBusinessId()));

        assetsResponse.setAssessmentList(assessmentlist);*/

        DgAssetsMoney dgAssetsMoney = dgAssetsMoneyMapper.selectOne(new QueryWrapper<DgAssetsMoney>()
                .eq("business_id",assetsResponse.getBusinessId()));
        assetsResponse.setDgAssetsMoney(dgAssetsMoney);

        String files=assetsResponse.getFile();
        if(files != null){
            if(files.length()>0){
                String a = files.substring(1,files.length()-1);
                String[] arry =a.split(",");
                List<String> demoList = Arrays.asList(arry);
                assetsResponse.setFileList(demoList);
            }
        }

        //根据业务id查询业务申请详细信息
        DgApplyAmountInfoResponse dgApplyAmountInfo = dgApplyAmountInfoMapper.selectApplyAmountInfoById(assetsResponse.getBusinessId());
        String measures=dgApplyAmountInfo.getEnGuaranteeMeasures();
        if(measures.length()>0){
            String a = measures.substring(1,measures.length()-1);
            String[] arry =a.split(",");
            List<String> demoList = Arrays.asList(arry);
            dgApplyAmountInfo.setEnGuaranteeMeasure(demoList);
        }


        DgApplyAmountInfoRequest applyAmountInfo = new DgApplyAmountInfoRequest();
        applyAmountInfo.setCompanyId(dgApplyAmountInfo.getCompanyId());
        //根据企业id查询企业详情
/*        ResponseResult result = crmsFeignService.queryCustomerInfoById(applyAmountInfo);

        LinkedHashMap linkedList = (LinkedHashMap)result.getData();*/

        //根据业务id查询备份企业信息
        DgBusinessCompany dgBusinessCompany = dgBusinessCompanyMapper.selectOne(new QueryWrapper<DgBusinessCompany>()
                .eq("business_id", assetsResponse.getBusinessId()));

        //查询该业务id下对应的C角
        DgAssetsInvestigation assetsInvestigate = this.baseMapper.selectOne(new QueryWrapper<DgAssetsInvestigation>()
                .eq("business_id", assetsResponse.getBusinessId()).eq("type", 1));
        if(assetsInvestigate !=null){
            Integer empCId = assetsInvestigate.getEmpId();
            Integer[] arr = new Integer[]{
                    empCId
            };
            if(arr.length>0){
                ResponseResult<List<EmployeeVO>> listResponseResult = hrmsFeignService.listEmpNameByUserId(arr);
                assetsResponse.setCName(listResponseResult.getData().get(0).getName());
            }
        }

        AssetsDetailResponse assetsDetailResponse = new AssetsDetailResponse();
        assetsDetailResponse.setDgApplyAmountInfo(dgApplyAmountInfo);
        assetsDetailResponse.setDgBusinessCompany(dgBusinessCompany);
        //assetsDetailResponse.setLinkedHashMap(linkedList);
        assetsDetailResponse.setAssetsResponse(assetsResponse);

        return assetsDetailResponse;
    }

    @Transactional
    @Override
    public int investigateGuarantee(InvestigateAssetsRequest investigateAssetsRequest) {
        //根据主键id查询业务id
        DgAssetsInvestigation dgAssetsInvestigation = this.getBaseMapper().selectById(investigateAssetsRequest.getId());

        //如果为驳回情况，通过，则修改资产部调查下该业务审批状态为审核中，设置资产部经理操作状态为待处理
        if(dgAssetsInvestigation.getStatus().equals(StatusCode.SP_REJECT) && investigateAssetsRequest.getStatus().equals(StatusCode.SP_IN_REVIEW)){
            DgAssetsInvestigation assets = new DgAssetsInvestigation();
            assets.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            this.baseMapper.update(assets,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",dgAssetsInvestigation.getBusinessId()));

            DgAssetsInvestigation assetsInvestigation = new DgAssetsInvestigation();
            assetsInvestigation.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理
            this.baseMapper.update(assets,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",dgAssetsInvestigation.getBusinessId()).eq("type",2));

            DgAssetsMoney money = new DgAssetsMoney();
            money.setBusinessId(dgAssetsInvestigation.getBusinessId());
            money.setHouseMoney(investigateAssetsRequest.getHouseMoney());
            money.setCarMoney(investigateAssetsRequest.getCarMoney());
            money.setEquipmentMoney(investigateAssetsRequest.getEquipmentMoney());
            money.setOtherMoney(investigateAssetsRequest.getOtherMoney());
            dgAssetsMoneyMapper.update(money,new QueryWrapper<DgAssetsMoney>()
                    .eq("business_id",dgAssetsInvestigation.getBusinessId()));
        }else{
            //根据业务id批量新增数据到评估价值列表
            //dgAssetsAssessmentAssetsLogMapper.insertAssetsList(investigateAssetsRequest.getAssessmentList());
            DgAssetsMoney money = new DgAssetsMoney();
            money.setBusinessId(dgAssetsInvestigation.getBusinessId());
            money.setHouseMoney(investigateAssetsRequest.getHouseMoney());
            money.setCarMoney(investigateAssetsRequest.getCarMoney());
            money.setEquipmentMoney(investigateAssetsRequest.getEquipmentMoney());
            money.setOtherMoney(investigateAssetsRequest.getOtherMoney());
            dgAssetsMoneyMapper.insert(money);
        }

        //资产部调查员调查，修改审核意见以及状态，修改其他数据
        DgAssetsInvestigation assetsInvestigation = new DgAssetsInvestigation();
        assetsInvestigation.setId(investigateAssetsRequest.getId());//主键id
        assetsInvestigation.setStatus(investigateAssetsRequest.getStatus());//状态
        assetsInvestigation.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
        assetsInvestigation.setApplyContent(investigateAssetsRequest.getApplyContent());//审核意见
        assetsInvestigation.setAssetsAmount(ArraysUtil.toString(investigateAssetsRequest.getAssetsAmount()));
        assetsInvestigation.setAmountSum(investigateAssetsRequest.getAmountSum());//合计
        assetsInvestigation.setAssetsName(investigateAssetsRequest.getAssetsName());
        assetsInvestigation.setAssetsCode(investigateAssetsRequest.getAssetsCode());
        assetsInvestigation.setAssetsTime(investigateAssetsRequest.getAssetsTime());
        assetsInvestigation.setAuditName(investigateAssetsRequest.getAuditName());
        assetsInvestigation.setAuditCode(investigateAssetsRequest.getAuditCode());
        assetsInvestigation.setAuditTime(investigateAssetsRequest.getAuditTime());
        assetsInvestigation.setAuditAmount(investigateAssetsRequest.getAuditAmount());
        assetsInvestigation.setFile(ArraysUtil.toString(investigateAssetsRequest.getFile()));
        this.baseMapper.updateById(assetsInvestigation);

        //根据业务id修改反担保措施
        DgApplyAmountInfo dgApplyAmountInfo = new DgApplyAmountInfo();
        dgApplyAmountInfo.setId(dgAssetsInvestigation.getBusinessId());
        dgApplyAmountInfo.setDescription(investigateAssetsRequest.getDescription());
        dgApplyAmountInfo.setEnGuaranteeMeasures(ArraysUtil.toString(investigateAssetsRequest.getEnGuaranteeMeasures()));
        dgApplyAmountInfoMapper.updateById(dgApplyAmountInfo);

        //判断资产部调查员是否调查通过，如果通过，修改资产部经理操作状态为待处理
        if(investigateAssetsRequest.getStatus()==StatusCode.SP_IN_REVIEW){//状态为审核中
            DgAssetsInvestigation assetsInvestigation2= new DgAssetsInvestigation();
            assetsInvestigation2.setOperatingStatus(StatusCode.CZ_ON_HAND);
            this.baseMapper.update(assetsInvestigation2,new QueryWrapper<DgAssetsInvestigation>().
                    eq("business_id",dgAssetsInvestigation.getBusinessId()).eq("type",2));
        }
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));

        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),dgAssetsInvestigation.getBusinessId(),
                account,departmentId,investigateAssetsRequest.getApplyContent(),
                3,PromptSuccess.ZC_ZY_ID,"","","",investigateAssetsRequest.getStatus());

        return 1;
    }

    @Transactional
    @Override
    public int approvalAssets(ApprovalAssetsRequest approvalAssetsRequest) {

        //根据主键id查询资产部记录
        DgAssetsInvestigation assetsInvestigation = this.baseMapper.selectById(approvalAssetsRequest.getId());

        DgAssetsInvestigation dgAssetsInvestigation = new DgAssetsInvestigation();
        dgAssetsInvestigation.setId(approvalAssetsRequest.getId());
        dgAssetsInvestigation.setEmpId(Integer.parseInt(DgApplyAmountInfoController.getUserId()));
        dgAssetsInvestigation.setRemark(ArraysUtil.toString(approvalAssetsRequest.getRemark()));//补充原因
        dgAssetsInvestigation.setOtherRemark(approvalAssetsRequest.getOtherRemark());//其他原因
        if(assetsInvestigation.getType()==2 && approvalAssetsRequest.getStatus()==2){
            //区分经理，领导审核，此处为经理审核
            dgAssetsInvestigation.setStatus(StatusCode.SP_IN_REVIEW);//审核中
        }else{
            dgAssetsInvestigation.setStatus(approvalAssetsRequest.getStatus());
        }
        dgAssetsInvestigation.setOperatingStatus(StatusCode.CZ_PROCESSED);//状态设置为已处理
        dgAssetsInvestigation.setAmount(approvalAssetsRequest.getAmount());//金额
        dgAssetsInvestigation.setTerm(approvalAssetsRequest.getTerm());//期限
        dgAssetsInvestigation.setApplyContent(approvalAssetsRequest.getApplyContent());//审核意见
        //根据主键id修改资产部记录
        /* this.baseMapper.updateById(dgAssetsInvestigation);*/

        //插入或更新的字段有 空字符串 或者 null
        this.baseMapper.update(
                null,
                Wrappers.<DgAssetsInvestigation>lambdaUpdate()
                        .set(DgAssetsInvestigation::getApplyContent, approvalAssetsRequest.getApplyContent()) //把email设置成null
                        .set(DgAssetsInvestigation::getEmpId, Integer.parseInt(DgApplyAmountInfoController.getUserId()))
                        .set(DgAssetsInvestigation::getRemark, ArraysUtil.toString(approvalAssetsRequest.getRemark()))
                        .set(DgAssetsInvestigation::getOtherRemark, approvalAssetsRequest.getOtherRemark())
                        .set(DgAssetsInvestigation::getStatus, dgAssetsInvestigation.getStatus())//approvalAssetsRequest.getStatus()
                        .set(DgAssetsInvestigation::getOperatingStatus, StatusCode.CZ_PROCESSED)
                        .set(DgAssetsInvestigation::getAmount, approvalAssetsRequest.getAmount())
                        .set(DgAssetsInvestigation::getTerm, approvalAssetsRequest.getTerm())
                        .eq(DgAssetsInvestigation::getId, approvalAssetsRequest.getId())
        );

        DgAssetsInvestigation dgAssets = new DgAssetsInvestigation();
        dgAssets.setStatus(StatusCode.SP_REJECT);//驳回
        dgAssets.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理

        if(approvalAssetsRequest.getStatus().equals(StatusCode.SP_REJECT)){
            //驳回情况
            //修改资产部调查下该业务的所有审批状态为驳回
            DgAssetsInvestigation assets = new DgAssetsInvestigation();
            assets.setStatus(StatusCode.SP_REJECT);//驳回
            this.baseMapper.update(assets,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",assetsInvestigation.getBusinessId()));
        }

        //此处为资产部经理审核的操作
        //如果数据为驳回状态，审核通过，则处理资产部调查下该业务的审批状态为审核中，设置资产部领导的操作状态为待处理
        if(assetsInvestigation.getStatus().equals(StatusCode.SP_REJECT) && approvalAssetsRequest.getStatus().equals(StatusCode.SP_IN_REVIEW) && assetsInvestigation.getType()==2){
            DgAssetsInvestigation assets = new DgAssetsInvestigation();
            assets.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            this.baseMapper.update(assets,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",assetsInvestigation.getBusinessId()));

            DgAssetsInvestigation assets2 = new DgAssetsInvestigation();
            assets2.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理
            this.baseMapper.update(assets2,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",assetsInvestigation.getBusinessId()).eq("type",3));
        }else if(approvalAssetsRequest.getStatus().equals(StatusCode.SP_REFUSE)){//如果为拒绝，业务状态则为已否决
            DgAssetsInvestigation user = new DgAssetsInvestigation();
            user.setStatus(StatusCode.SP_REFUSE);//审批状态为拒绝
            this.baseMapper.update(user, new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id", assetsInvestigation.getBusinessId()));

            DgBusinessProcessStatus processStatus = new DgBusinessProcessStatus();
            processStatus.setBusinessId(assetsInvestigation.getBusinessId());
            processStatus.setBusinessStatus(StatusCode.YW_REJECTED);//业务状态设置为已否决
            dgBusinessProcessStatusMapper.update(processStatus,new QueryWrapper<DgBusinessProcessStatus>()
                    .eq("business_id",processStatus.getBusinessId()));
        }else if(approvalAssetsRequest.getStatus().equals(StatusCode.SP_REJECT) && assetsInvestigation.getType()==2){
            //资产部经理驳回情况
            //资产部专员调查审批状态为驳回，操作状态为待处理
            this.baseMapper.update(dgAssets,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",assetsInvestigation.getBusinessId()).eq("type",1));
        }else if(approvalAssetsRequest.getStatus().equals(StatusCode.SP_REJECT) && assetsInvestigation.getType()==3){
            //资产部领导驳回情况
            //资产部经理审批状态设置为驳回，操作状态设置为待处理
            this.baseMapper.update(dgAssets,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",assetsInvestigation.getBusinessId()).eq("type",2));
        }else if((approvalAssetsRequest.getStatus()==StatusCode.SP_IN_REVIEW||approvalAssetsRequest.getStatus()==StatusCode.SP_REVIEWED) && dgAssetsInvestigation.getOperatingStatus()==StatusCode.CZ_PROCESSED && assetsInvestigation.getType()==2){//资产部经理审核通过
            //根据主键id查询业务id
            DgAssetsInvestigation assetsInvestigation2 = this.getBaseMapper().selectById(approvalAssetsRequest.getId());

            //如果逐级驳回，且资产部经理审核通过，首先查看资产领导是否有数据，如果有数据，则修改不新增
            Integer count = this.baseMapper.selectCount(new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id", assetsInvestigation2.getBusinessId()).eq("type", 3));

            DgAssetsInvestigation assign = new DgAssetsInvestigation();
            assign.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
            //修改该业务下所有审批状态为审核中
            this.baseMapper.update(assign,new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",assetsInvestigation2.getBusinessId()));

            if(count>0){//资产领导有数据，则直接修改领导状态
                DgAssetsInvestigation assignUser = new DgAssetsInvestigation();
                assignUser.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
                assignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态为待处理
                this.baseMapper.update(assignUser,new QueryWrapper<DgAssetsInvestigation>()
                        .eq("business_id",assetsInvestigation2.getBusinessId()).eq("type",3));
            }else {
                //如果资产部经理审核通过，则往担保部分管领导处加一条记录，由分管领导审核
                DgAssetsInvestigation assets = new DgAssetsInvestigation();
                assets.setBusinessId(assetsInvestigation2.getBusinessId());
                assets.setCompanyId(assetsInvestigation2.getCompanyId());
                assets.setRoleId(PromptSuccess.ZC_LD_ID);//资产部分管领导
                assets.setType(3);//分管领导审核
                assets.setStatus(StatusCode.SP_IN_REVIEW);//审批状态设置为审核中
                assets.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                this.baseMapper.insert(assets);
            }
        }else if(approvalAssetsRequest.getStatus()==StatusCode.SP_REVIEWED && dgAssetsInvestigation.getOperatingStatus()==StatusCode.CZ_PROCESSED && assetsInvestigation.getType()==3) {//资产部分管领导审核通过
            //如果担保部分管领导审核通过，同时判断资产部分管领导审核以及信息部分管领导审核是否通过，如果通过就往合规调查中插入一条数据
            //如果担保部领导审核通过，则该子业务流程下的所有审批状态都为已审核状态
            DgAssetsInvestigation investigationAssets = new DgAssetsInvestigation();
            investigationAssets.setStatus(StatusCode.SP_REVIEWED);//已审核
            this.baseMapper.update(investigationAssets,new QueryWrapper<DgAssetsInvestigation>().eq("business_id",assetsInvestigation.getBusinessId()));
            //查看担保部分管领导是否审核通过
            int a=dgGuaranteeAssignUserMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id",assetsInvestigation.getBusinessId())
            .eq("type",4).eq("status",StatusCode.SP_REVIEWED).eq("operating_status",StatusCode.CZ_PROCESSED));

            //查看信息部分管领导是否审核通过
            int b=dgMessageInvestigationMapper.selectCount(new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",assetsInvestigation.getBusinessId())
                    .eq("type",3).eq("status",StatusCode.SP_REVIEWED).eq("operating_status",StatusCode.CZ_PROCESSED));
            int sum = a+b;
            if(sum==2){
                //新增记录到合规调查
                DgComplianceInvestigation compliance =new DgComplianceInvestigation();
                compliance.setCompanyId(assetsInvestigation.getCompanyId());//企业id
                compliance.setRoleId(PromptSuccess.FG_JL_ID);//角色id
                compliance.setBusinessId(assetsInvestigation.getBusinessId());//业务id
                compliance.setOperationStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                compliance.setAuditStatus(StatusCode.SP_TO_BE_ASSIGNED);//审核状态设置为待指派
                dgComplianceInvestigationMapper.insert(compliance);
            }
        }
        Integer type = assetsInvestigation.getType();
        String deptName;
        Integer roleId;
        if(type == 2){
            deptName = "资产部经理";
            roleId=PromptSuccess.ZC_JL_ID;
        }else{
            deptName = "资产部分管领导";
            roleId=PromptSuccess.ZC_LD_ID;
        }
        String remark1 = Arrays.toString(approvalAssetsRequest.getRemark());//补充调查原因
        String remark = remark1.replace("[","").replace("]","");
        String otherRemark = approvalAssetsRequest.getOtherRemark();//其他原因
        if(otherRemark == null){
            otherRemark = "";
        }
        String reason = remark+otherRemark;
        String loanMoney = String.valueOf(approvalAssetsRequest.getAmount());//金额
        if(loanMoney == "null"){
            loanMoney = "";
        }
        String loanTerm = approvalAssetsRequest.getTerm();//期限
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),assetsInvestigation.getBusinessId(),
                account,departmentId,approvalAssetsRequest.getApplyContent(),
                3,roleId,reason,loanMoney,loanTerm,approvalAssetsRequest.getStatus());

        boolean b=false;
        //根据业务为资产部分管领导处理以及是否驳回决定是否抄送
        if(roleId.equals(PromptSuccess.ZC_LD_ID) && !approvalAssetsRequest.getStatus().equals(StatusCode.SP_REJECT)){//经理，驳回
            try {
                b = dgCopyForService.copySendUser(assetsInvestigation.getBusinessId(), 3, approvalAssetsRequest.getId());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return 1;
    }

    @Override
    public Boolean excelExport(List<String> ids,Integer userId, HttpServletResponse response) {
        //根据ids查询业务申请列表信息
        List<AssetsInvestigationListResponse>assetsList = this.baseMapper.selectListByIds(ids,userId);
        try {
            ExcelUtil.writeExcelWithSheets(response, assetsList, "资产调查一览表", "assetsInvestigation", new AssetsInvestigationListResponse())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        return userId;
    }

}
