package com.daqing.financial.guarantee.service.impl;

import com.daqing.financial.guarantee.mapper.DgAuditProcessMapper;
import com.daqing.financial.guarantee.model.response.TodayApprovalRecordResponse;
import com.daqing.financial.guarantee.service.IDgAuditProcessService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.util.DateUtils;
import com.daqing.framework.domain.guarantee.DgAuditProcess;
import com.daqing.framework.util.RedisUtil;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.List;

/**
 * <p>
 * 审核流程进程 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Service
public class DgAuditProcessServiceImpl extends ServiceImpl<DgAuditProcessMapper, DgAuditProcess> implements IDgAuditProcessService {

    @Override
    public List<DgAuditProcess> queryProcessList(String businessId, String processId) {
        List<DgAuditProcess> auditProcessList = baseMapper.selectAuditProcessList(businessId, processId);
        return auditProcessList;
    }

    @Override
    public List<Integer> getProcessId(String businessId) {
        return baseMapper.selectProcessId(businessId);
    }

    @Override
    public List<TodayApprovalRecordResponse> todayApprovalRecord() {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String startTime = dateFormat.format(DateUtils.getDayBegin());
        String endTime = dateFormat.format(DateUtils.getDayEnd());
        String userId = this.getUserId();
        return baseMapper.selectTodayApprovalRecord(userId, startTime, endTime);
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:" + token);
    }
}
