package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.DgAssetsInvestigationMapper;
import com.daqing.financial.guarantee.mapper.DgComplianceInvestigationMapper;
import com.daqing.financial.guarantee.mapper.DgGuaranteeAssignUserMapper;
import com.daqing.financial.guarantee.model.request.DgComplianceInvestigationRequest;
import com.daqing.financial.guarantee.model.response.*;
import com.daqing.financial.guarantee.service.IDgAssetsInvestigationService;
import com.daqing.financial.guarantee.service.IDgComplianceInvestigationService;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.service.IDgGuaranteeAssignUserService;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.DgComplianceInvestigation;
import com.daqing.framework.domain.guarantee.DgWorkConferenceConsider;
import com.daqing.framework.domain.guarantee.GuaranteeCode;
import com.daqing.framework.domain.guarantee.po.AttacheAndABUserIdPO;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 合规调查 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Service
public class DgComplianceInvestigationServiceImpl extends ServiceImpl<DgComplianceInvestigationMapper, DgComplianceInvestigation>
        implements IDgComplianceInvestigationService {

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Autowired
    private DgGuaranteeAssignUserMapper dgGuaranteeAssignUserMapper;

    @Autowired
    private IDgGuaranteeAssignUserService guaranteeAssignUserService;

    @Autowired
    private IDgAssetsInvestigationService dgAssetsInvestigationService;

    @Autowired
    private DgAssetsInvestigationMapper dgAssetsInvestigationMapper;

    @Resource
    private IDgCopyForService dgCopyForService;

    /**
     * 列表
     */
    @Override
    public PageUtils list(Integer page, Integer size, DgComplianceInvestigationRequest dgComplianceInvestigationRequest) {
        if (page == null || size == null || page < 1 || size < 1) {
            page = 1;
            size = 10;
        }
        String userId = this.getUserId();
        // 获取当前登录用户的角色id
        String roleIdString = RedisUtil.get("dq:userRole:" + userId);
        if (roleIdString == null || roleIdString.length() == 0) {
            ExceptionCast.cast(GuaranteeCode.ROLE_IS_NOT);
        }
        String[] roleIds = roleIdString.split(",");
        /*String userId = "1";
        String[] roleIds = {"70"};*/
        List<Integer> roleList = new ArrayList<>();
        for (String roleId : roleIds) {
            roleList.add(Integer.parseInt(roleId));
        }
        IPage<DgComplianceInvestigationResponse> list = this.getBaseMapper().list(new Page(page, size),
                dgComplianceInvestigationRequest, roleList, Integer.parseInt(userId));
        return new PageUtils(list);
    }

    /**
     * 指派
     */
    @Transactional
    @Override
    public Boolean assign(Integer iid, Integer userId) {
        if (iid == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        if (userId == null) {
            ExceptionCast.cast(GuaranteeCode.COMPLIANCE_USER_NOT_NULL);
        }
        // 判断当前用户是否已被指派为信息专员、资产专员、AB角
        AttacheAndABUserIdPO userIdPO = this.getBaseMapper().getAttacheAndABByBusinessId(this.getComplianceInvestigation(iid).getBusinessId());
        if (userIdPO != null) {
            if (userId.equals(userIdPO.getAUserId())
                    || userId.equals(userIdPO.getBUserId())
                    || userId.equals(userIdPO.getAssetUserId())
                    || userId.equals(userIdPO.getMessageUserId())) {
                ExceptionCast.cast(GuaranteeCode.COMPLIANCE_USER_IS_EXIST);
            }
        }
        // 指派
        DgComplianceInvestigation complianceInvestigation = this.getComplianceInvestigation(iid);
        complianceInvestigation.setUserId(userId);
        complianceInvestigation.setRoleId(PromptSuccess.FG_C_ID);
        complianceInvestigation.setOperationStatus(StatusCode.CZ_ON_HAND);
        complianceInvestigation.setAuditStatus(StatusCode.SP_IN_REVIEW);
        complianceInvestigation.setCreateTime(new Date());
        complianceInvestigation.setUpdateTime(new Date());
        Boolean result = this.getBaseMapper().assign(complianceInvestigation);
        if (result) {
            // 绑定该条数据的指派操作人(表示指派完成);
            String id = this.getUserId();
            /*String id = "1";*/
            Boolean result1 = this.getBaseMapper().updateAssign(iid, Integer.parseInt(id), StatusCode.SP_IN_REVIEW, StatusCode.CZ_PROCESSED);
            if (result1) {
                // 在角色表中插入一条数据,与当前被指派的员工(用户)关联
                ResponseResult responseResult = hrmsFeignService.saveUserRole(userId.longValue(), PromptSuccess.FG_C_ID.longValue());
                Boolean result2 = (Boolean) responseResult.getData();
                if (result2) {
                    // 在审核进程表中插入一条指派成功的数据
                    String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                    return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                            departmentId, null, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_JL_ID,"","","", StatusCode.SP_REVIEWED);
                }
            }
        }
        return false;
    }

    /**
     * 调查
     */
    @Transactional
    @Override
    public Boolean survey(Integer iid, String fileUrl, String opinion, Integer status) {
        if (iid == null || status == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        String userId = this.getUserId();
        // String userId = "1";
        // 同意
        if (status == 1) {
            if (fileUrl == null || fileUrl.length() == 0) {
                ExceptionCast.cast(GuaranteeCode.COMPLIANCE_UPLOAD_NOT_NULL);
            }
            // 将合规调查的审核状态为审核中
            this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                    StatusCode.SP_IN_REVIEW);
            // 更新合规调查表调查数据,表示调查完成
            Boolean result = this.getBaseMapper().survey(iid, opinion, new Date(), Integer.parseInt(userId),
                    StatusCode.CZ_PROCESSED, StatusCode.SP_IN_REVIEW, fileUrl);
            // 上传附件到附件表里
            if (result) {
                Boolean result1 = this.getBaseMapper().uploadAccessory(this.getComplianceInvestigation(iid).getBusinessId(), fileUrl);
                // 更新合规调查表里面的审核数据
                if (result1) {
                    Boolean result2 = this.getBaseMapper().updateToAudit(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.CZ_ON_HAND, PromptSuccess.FG_JL_ID);
                    // 在业务流程表中插入一条调查成功的数据
                    if (result2) {
                        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                        return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_C_ID,"","","", StatusCode.SP_REVIEWED);
                    }
                }
            }
        }
        // 驳回
        if (status == 2) {
            if (opinion == null || opinion.length() == 0) {
                ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
            }
            // 先改变合规调查的审核状态为驳回
            Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                    StatusCode.SP_REJECT);
            if (result) {
                // 再改变自己的操作状态为已处理
                Boolean result1 = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                if (result1) {
                    // 改变担保部的审核状态为驳回
                    Boolean result2 = dgGuaranteeAssignUserMapper.updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REJECT);
                    if (result2) {
                        // 修改AB角的操作状态为待处理
                        Boolean result3 = dgGuaranteeAssignUserMapper.updateOperationStatusById(this.getComplianceInvestigation(iid)
                                .getBusinessId(), StatusCode.CZ_ON_HAND);
                        if (result3) {
                            // 在业务流程表中插入一条驳回成功的数据(记得带上驳回的意见)
                            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                            return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                    departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_C_ID,"","","", StatusCode.SP_REJECT);
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 审核
     */
    @Transactional
    @Override
    public Boolean check(Integer iid, String opinion, Integer status) {
        if (iid == null || status == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        String userId = this.getUserId();
        // String userId = "1";
        Integer operationStatus = this.getBaseMapper().getOperationStatusByBusinessId(
                this.getComplianceInvestigation(iid).getBusinessId(), PromptSuccess.FG_JL_ID);
        // 1.待处理,表示业务进行到法规经理审核
        if (Objects.equals(operationStatus, StatusCode.CZ_ON_HAND)) {
            // 查询业务流程表,找出该业务id的最新操作记录的角色名称
            Integer processId = this.getBaseMapper().getAuditProcessLatestProcessIdByBusinessId(this.getComplianceInvestigation(iid).getBusinessId());
            if (PromptSuccess.DB_PROCESS_ID.equals(processId)) {
                // 1.1.若最新操作人为A/B角,表示为驳回给AB角之后直接返回的数据
                // 同意
                if (status == 1) {
                    // 改变自己的状态为已处理
                    Boolean result = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                    if (result) {
                        // 改变C角的这条数据的操作状态为待处理
                        Boolean result1 = this.getBaseMapper().updateToAudit(this.getComplianceInvestigation(iid).getBusinessId(),
                                StatusCode.CZ_ON_HAND, PromptSuccess.FG_C_ID);
                        if (result1) {
                            // 在业务流程表中插入一条法规经理同意的数据
                            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                            return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                    departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_JL_ID,"","","", StatusCode.SP_REVIEWED);
                        }
                    }
                }
                // 驳回
                if (status == 2) {
                    if (opinion == null || opinion.length() == 0) {
                        ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
                    }
                    // 先改变所有的审核状态为驳回
                    Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REJECT);
                    if (result) {
                        // 再改变自己当前的操作状态为已处理
                        Boolean result1 = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                        if (result1) {
                            // 最后改变C角的这条数据的操作状态为待处理
                            Boolean result2 = this.getBaseMapper().updateToAudit(this.getComplianceInvestigation(iid).getBusinessId(),
                                    StatusCode.CZ_ON_HAND, PromptSuccess.FG_C_ID);
                            if (result2) {
                                // 在业务流程表中插入一条法规经理驳回的数据
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                                return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_JL_ID,"","","", StatusCode.SP_REJECT);
                            }
                        }
                    }
                }
                // 拒绝
                if (status == 3) {
                    if (opinion == null || opinion.length() == 0) {
                        ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
                    }
                    // 先改变所有的审核状态为已拒绝
                    Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REFUSE);
                    if (result) {
                        // 再改变自己当前的操作状态为已处理
                        Boolean result1 = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                        if (result1) {
                            // 最后改变该条贷款信息的业务状态为已否决
                            Boolean result2 = this.getBaseMapper().updateBusinessStatusByBusinessId(
                                    this.getComplianceInvestigation(iid).getBusinessId(), StatusCode.YW_REJECTED);
                            if (result2) {
                                // 在业务流程表中插入一条拒绝数据(记得带上拒绝意见)
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                                return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_JL_ID,"","","", StatusCode.SP_REFUSE);
                            }
                        }
                    }
                }
            } else {
                // 1.2.若最新操作人为C角或法规部领导,表示C角调查通过后发送过来的数据或者法规部领导驳回的数据
                // 同意
                if (status == 1) {
                    // 此条业务id下的法规领导数据是否存在
                    Integer count = this.getBaseMapper().countComplianceInvestigation(this.getComplianceInvestigation(iid).getBusinessId(),
                            PromptSuccess.FG_LD_ID);
                    Boolean result = this.getBaseMapper().check(iid, opinion, new Date(), StatusCode.SP_IN_REVIEW,
                            StatusCode.CZ_PROCESSED, Integer.parseInt(userId));
                    if (result) {
                        Boolean result1;
                        if (count > 0) {
                            // 表示为法规部领导驳回的数据,先更新审批状态为审核中
                            Boolean result2 = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                                    StatusCode.SP_IN_REVIEW);
                            if (result2) {
                                // 再更新法规部领导的操作状态为待处理
                                result1 = this.getBaseMapper().updateToAudit(this.getComplianceInvestigation(iid).getBusinessId(), StatusCode.CZ_ON_HAND,
                                        PromptSuccess.FG_LD_ID);
                            } else {
                                result1 = false;
                            }
                        } else {
                            // 表示为C角调查通过后发送过来的数据,法规经理完成审核,新增一条法规领导审核数据(借用插入一条调查数据的接口)
                            DgComplianceInvestigation complianceInvestigation = this.getComplianceInvestigation(iid);
                            complianceInvestigation.setUserId(null);
                            complianceInvestigation.setRoleId(PromptSuccess.FG_LD_ID);
                            complianceInvestigation.setOperationStatus(StatusCode.CZ_ON_HAND);
                            complianceInvestigation.setAuditStatus(StatusCode.SP_IN_REVIEW);
                            complianceInvestigation.setCreateTime(new Date());
                            complianceInvestigation.setUpdateTime(new Date());
                            result1 = this.getBaseMapper().assign(complianceInvestigation);
                        }
                        if (result1) {
                            // 在业务流程表中插入一条法规经理审核成功的数据
                            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                            return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                    departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_JL_ID,"","","", StatusCode.SP_REVIEWED);
                        }
                    }
                }
                // 驳回
                if (status == 2) {
                    if (opinion == null || opinion.length() == 0) {
                        ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
                    }
                    // 先改变所有的审核状态为驳回
                    Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REJECT);
                    if (result) {
                        // 再改变自己当前的操作状态为已处理
                        Boolean result1 = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                        if (result1) {
                            // 最后改变C角的这条数据的操作状态为待处理
                            Boolean result2 = this.getBaseMapper().updateToAudit(this.getComplianceInvestigation(iid).getBusinessId(),
                                    StatusCode.CZ_ON_HAND, PromptSuccess.FG_C_ID);
                            if (result2) {
                                // 在业务流程表中插入一条法规经理驳回的数据
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                                return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_JL_ID,"","","", StatusCode.SP_REJECT);
                            }
                        }
                    }
                }
                // 拒绝
                if (status == 3) {
                    if (opinion == null || opinion.length() == 0) {
                        ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
                    }
                    // 先改变所有的审核状态为已拒绝
                    Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REFUSE);
                    if (result) {
                        // 再改变自己当前的操作状态为已处理
                        Boolean result1 = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                        if (result1) {
                            // 最后改变该条贷款信息的业务状态为已否决
                            Boolean result2 = this.getBaseMapper().updateBusinessStatusByBusinessId(
                                    this.getComplianceInvestigation(iid).getBusinessId(), StatusCode.YW_REJECTED);
                            if (result2) {
                                // 在业务流程表中插入一条拒绝数据(记得带上拒绝意见)
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                                return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_JL_ID,"","","", StatusCode.SP_REFUSE);
                            }
                        }
                    }
                }
            }
        }
        // 2.已处理,表示业务进行到法规领导审核
        if (Objects.equals(operationStatus, StatusCode.CZ_PROCESSED)) {
            // 同意
            if (status == 1) {
                Boolean result = this.getBaseMapper().check(iid, opinion, new Date(), StatusCode.SP_REVIEWED,
                        StatusCode.CZ_PROCESSED, Integer.parseInt(userId));
                if (result) {
                    // 更新当前审批状态为已审核
                    Boolean result2 = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REVIEWED);
                    if (result2) {
                        // 判断当前业务id下的工作会数据是否存在
                        DgComplianceInvestigation complianceInvestigation = this.getComplianceInvestigation(iid);
                        Integer count = this.getBaseMapper().countWorkConferenceConsider(complianceInvestigation.getBusinessId(), PromptSuccess.GZU_ID);
                        Boolean result1;
                        if (count > 0) {
                            // 工作会该条数据存在,表示为工作会驳回数据,更新工作会的数据为审核中和待处理(工作会的业务过程中只会有一条数据)
                            result1 = this.getBaseMapper().updateWorkConferenceConsider(complianceInvestigation.getBusinessId(),
                                    PromptSuccess.GZU_ID, StatusCode.SP_IN_REVIEW, StatusCode.CZ_ON_HAND);
                        } else {
                            // 工作会该条数据不存在，表示不是工作会驳回数据,需要新增一条
                            DgWorkConferenceConsider workConferenceConsider = new DgWorkConferenceConsider();
                            workConferenceConsider.setCompanyId(complianceInvestigation.getCompanyId());
                            workConferenceConsider.setBusinessId(complianceInvestigation.getBusinessId());
                            workConferenceConsider.setRoleId(PromptSuccess.GZU_ID);
                            workConferenceConsider.setAuditStatus(StatusCode.SP_IN_REVIEW);
                            workConferenceConsider.setOperationStatus(StatusCode.CZ_ON_HAND);
                            workConferenceConsider.setCreateTime(new Date());
                            workConferenceConsider.setUpdateTime(new Date());
                            result1 = this.getBaseMapper().insertWorkConferenceConsider(workConferenceConsider);
                        }
                        if (result1) {
                            boolean b=false;
                            try {
                                b=dgCopyForService.copySendUser(this.getComplianceInvestigation(iid).getBusinessId(),5,iid);
                            } catch (IOException e) {
                                e.printStackTrace();
                            }

                            // 在业务流程表中插入一条法规领导审核成功的数据
                            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                            return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                    departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_LD_ID,"","","", StatusCode.SP_REVIEWED);
                        }
                    }
                }
            }
            // 驳回
            if (status == 2) {
                if (opinion == null || opinion.length() == 0) {
                    ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
                }
                // 绑定该条数据的userId,表示该用户操作了该条数据
                Boolean result3 = this.getBaseMapper().updateDgComplianceInvestigationUserIdById(iid, Integer.parseInt(userId));
                if (result3) {
                    // 先改变所有的审核状态为驳回
                    Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REJECT);
                    if (result) {
                        // 再改变自己当前的操作状态为已处理
                        Boolean result1 = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                        if (result1) {
                            // 最后改变法规部经理的这条数据的操作状态为待处理
                            Boolean result2 = this.getBaseMapper().updateToAudit(this.getComplianceInvestigation(iid).getBusinessId(),
                                    StatusCode.CZ_ON_HAND, PromptSuccess.FG_JL_ID);
                            if (result2) {
                                // 在业务流程表中插入一条法规领导驳回的数据
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                                return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_LD_ID,"","","", StatusCode.SP_REJECT);
                            }
                        }
                    }
                }
            }
            // 拒绝
            if (status == 3) {
                if (opinion == null || opinion.length() == 0) {
                    ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
                }
                // 绑定该条数据的userId,表示该用户操作了该条数据
                Boolean result3 = this.getBaseMapper().updateDgComplianceInvestigationUserIdById(iid, Integer.parseInt(userId));
                if (result3) {
                    // 先改变所有的审核状态为已拒绝
                    Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getComplianceInvestigation(iid).getBusinessId(),
                            StatusCode.SP_REFUSE);
                    if (result) {
                        // 再改变自己当前的操作状态为已处理
                        Boolean result1 = this.getBaseMapper().updateOperationStatusById(iid, StatusCode.CZ_PROCESSED, opinion);
                        if (result1) {
                            // 最后改变该条贷款信息的业务状态为已否决
                            Boolean result2 = this.getBaseMapper().updateBusinessStatusByBusinessId(
                                    this.getComplianceInvestigation(iid).getBusinessId(), StatusCode.YW_REJECTED);
                            if (result2) {
                                boolean b=false;
                                try {
                                    b=dgCopyForService.copySendUser(this.getComplianceInvestigation(iid).getBusinessId(),5,iid);
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }

                                // 在业务流程表中插入一条拒绝数据(记得带上拒绝意见)
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                                return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getComplianceInvestigation(iid).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, opinion, PromptSuccess.HG_PROCESS_ID,PromptSuccess.FG_LD_ID,"","","", StatusCode.SP_REFUSE);
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 查看
     */
    @Override
    public DgComplianceInvestigationInfoResponse query(Integer iid) {
        DgComplianceInvestigationInfoResponse dgComplianceInvestigationInfoResponse = new DgComplianceInvestigationInfoResponse();
        // 担保领导的数据id
        Integer guaranteeId = this.getBaseMapper().getGuaranteeId(this.getComplianceInvestigation(iid).getBusinessId(), PromptSuccess.DB_LD_ID);
        if (guaranteeId != null) {
            // 担保详情
            GuaranteeDetailResponse guaranteeDetailResponse = guaranteeAssignUserService.guaranteeDetail(guaranteeId);
            dgComplianceInvestigationInfoResponse.setGuaranteeDetailResponse(guaranteeDetailResponse);
        }
        // 资产领导的数据id
        Integer dgAssetsInvestigationId = this.getBaseMapper().getDgAssetsInvestigationId(this.getComplianceInvestigation(iid).getBusinessId(),
                PromptSuccess.ZC_LD_ID);
        if (dgAssetsInvestigationId != null) {
            // 资产详情
            AssetsDetailResponse assetsDetailResponse = dgAssetsInvestigationService.assetsDetail(dgAssetsInvestigationId);
            // 资产领导审核时填写的审核金额与期限
            DgAssetsInvestigationResponse dgAssetsInvestigationResponse = dgAssetsInvestigationMapper.selectInfoById(dgAssetsInvestigationId);
            if (dgAssetsInvestigationResponse != null) {
                assetsDetailResponse.getAssetsResponse().setAmount(dgAssetsInvestigationResponse.getAmount());
                assetsDetailResponse.getAssetsResponse().setTerm(dgAssetsInvestigationResponse.getTerm());
            }
            dgComplianceInvestigationInfoResponse.setAssetsResponse(assetsDetailResponse.getAssetsResponse());
        }
        // 合规调查附件和意见
        DgComplianceInvestigation dgComplianceInvestigation = this.getBaseMapper().queryDgComplianceInvestigation(iid);
        if (dgComplianceInvestigation != null) {
            if (dgComplianceInvestigation.getFileUrl() != null && dgComplianceInvestigation.getFileUrl().length() != 0) {
                String[] fileUrls = dgComplianceInvestigation.getFileUrl().split(",");
                dgComplianceInvestigationInfoResponse.setFileUrls(fileUrls);
            }
            if (dgComplianceInvestigation.getOpinion() != null && dgComplianceInvestigation.getOpinion().length() != 0) {
                dgComplianceInvestigationInfoResponse.setOpinion(dgComplianceInvestigation.getOpinion());
            }
        }
        // 获取C角姓名
        Integer userId = this.getBaseMapper().getCUserId(this.getComplianceInvestigation(iid).getBusinessId());
        if (userId != null) {
            Integer[] userIds = new Integer[1];
            userIds[0] = userId;
            ResponseResult<List<EmployeeVO>> result = hrmsFeignService.listEmpNameByUserId(userIds);
            if (result.getData() != null && result.getData().size() != 0) {
                dgComplianceInvestigationInfoResponse.setCName(result.getData().get(0).getName());
            }
        }
        return dgComplianceInvestigationInfoResponse;
    }

    /**
     * 导出
     */
    @Override
    public Boolean excelExport(HttpServletResponse response, List<String> ids,Integer userId) {
/*        if (ids == null || ids.size() == 0) {
            ExceptionCast.cast(GuaranteeCode.EXCEL_ID_NOT_NULL);
        }*/
        List<Integer> idList = new ArrayList<>();
        for (String id : ids) {
            idList.add(Integer.parseInt(id));
        }
        List<DgComplianceInvestigationResponse> list = this.getBaseMapper().listExcel(idList,userId);
        try {
            ExcelUtil.writeExcelWithSheets(response, list, "合规调查数据一览表", "NO.1", new DgComplianceInvestigationResponse())
                    .finish();
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
        }
        return false;
    }

    /**
     * 将接收的数组图片拼装成字符串类型
     */
    @Override
    @Transactional
    public String getFileUrl(String[] fileUrls) {
        StringBuilder fileUrl = new StringBuilder();
        if (null != fileUrls && fileUrls.length != 0) {
            for (int i = 0; i < fileUrls.length; i++) {
                if (i == fileUrls.length - 1) {
                    fileUrl.append(fileUrls[i]);
                } else {
                    fileUrl.append(fileUrls[i]).append(",");
                }
            }
        }
        return fileUrl.toString();
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        return userId;
    }

    /**
     * 获取基本信息
     */
    private DgComplianceInvestigation getComplianceInvestigation(Integer iid) {

        return this.getBaseMapper().getComplianceInvestigation(iid);
    }
}
