package com.daqing.financial.guarantee.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.DgCopyUserMapper;
import com.daqing.financial.guarantee.mapper.DgPaymentConfirmationConsiderMapper;
import com.daqing.financial.guarantee.model.request.CopyForMeRequest;
import com.daqing.financial.guarantee.model.response.CopyForMeResponse;
import com.daqing.financial.guarantee.service.IDgCopyUserService;
import com.daqing.financial.guarantee.util.DateUtils;
import com.daqing.financial.guarantee.util.TaskNodeUtil;
import com.daqing.framework.domain.guarantee.DgCopyUser;
import com.daqing.framework.domain.guarantee.DgPaymentConfirmationConsider;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 抄送用户表 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2021-01-15
 */
@Service
public class DgCopyUserServiceImpl extends ServiceImpl<DgCopyUserMapper, DgCopyUser> implements IDgCopyUserService {

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Autowired
    private DgPaymentConfirmationConsiderMapper paymentConfirmationConsiderMapper;

    @Override
    public List<CopyForMeResponse> queryCopyForMe(CopyForMeRequest copyForMeRequest) {
        // 时间筛选(固定时间)
        if (copyForMeRequest.getCreateFixedTime() != null && (copyForMeRequest.getStartTime() == null ||
                copyForMeRequest.getStartTime().length() == 0) && (copyForMeRequest.getEndTime() == null ||
                copyForMeRequest.getEndTime().length() == 0)) {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            switch (copyForMeRequest.getCreateFixedTime()) {
                case 0:
                    copyForMeRequest.setStartTime(dateFormat.format(DateUtils.getDayBegin()));
                    copyForMeRequest.setEndTime(dateFormat.format(DateUtils.getDayEnd()));// 今天
                    break;
                case 1:
                    copyForMeRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfYesterday()));
                    copyForMeRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfYesterDay()));// 昨天
                    break;
                case 2:
                    copyForMeRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfWeek()));
                    copyForMeRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfWeek()));// 本周
                    break;
                case 3:
                    copyForMeRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfMonth()));
                    copyForMeRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfMonth()));// 本月
                    break;
                case 4:
                    copyForMeRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfYear()));
                    copyForMeRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfYear()));// 本年
                    break;
                default:
                    copyForMeRequest.setStartTime(null);
                    copyForMeRequest.setEndTime(null);
            }
        }
        String userId = this.getUserId();
        copyForMeRequest.setUserId(Integer.parseInt(userId));
        copyForMeRequest.setPage((copyForMeRequest.getPage()-1)*copyForMeRequest.getSize());
        List<CopyForMeResponse> copyForMeResponseList = baseMapper.selectCopyForMe(copyForMeRequest);

        if (copyForMeResponseList.size() > 0) {
            // List<Integer> arr = new ArrayList<>();
            // for (CopyForMeResponse res : copyForMeResponseList) {
            //     Integer approveId = res.getApproveId();
            //     if (approveId!=null){
            //         arr.add(res.getApproveId());
            //     }
            //     arr.add(res.getApplicantId());
            // }
            //去重
            // List<Integer> collect = arr.stream().distinct().collect(Collectors.toList());
            //根据提单人id查询其部门名称
            // ResponseResult responseResult = hrmsFeignService.getAccountAndDeptNameById(collect);
            // List<LinkedHashMap> employeeMessage = null;
            // if (responseResult.getData() != null) {
            //     employeeMessage = (List<LinkedHashMap>) responseResult.getData();
            // }
            // if(employeeMessage != null){
            //     for (CopyForMeResponse response : copyForMeResponseList) {
            //         for (LinkedHashMap res : employeeMessage) {
            //             if (response.getApplicantId()!=null && response.getApplicantId().equals(res.get("id"))) {//如果提单人id相同情况下，就往对象里面赋值
            //                 response.setApplicant(JSONObject.toJSONString(res.get("account")).replace("\"", ""));
            //             }
            //         }
            //     }
            // }

            //转换任务节点
            copyForMeResponseList.forEach(copyForMeResponse -> {
                //财务确认根据业务id查询图片文件
                if ("9".equals(copyForMeResponse.getTaskNode())){
                    QueryWrapper<DgPaymentConfirmationConsider> wrapper = new QueryWrapper<>();
                    wrapper.eq("business_id",copyForMeResponse.getBusinessId()).last("limit 1");
                    DgPaymentConfirmationConsider paymentConfirmationConsider = paymentConfirmationConsiderMapper.selectOne(wrapper);
                    String imgFile = paymentConfirmationConsider.getImgFile();
                    copyForMeResponse.setImgFile(imgFile);
                }
                //提单人为空，默认赋值客户
                if (StringUtils.isEmpty(copyForMeResponse.getApplicant())){
                    copyForMeResponse.setApplicant("客户");
                }
                String taskNode = copyForMeResponse.getTaskNode();
                if (taskNode != null) {
                    String converter = TaskNodeUtil.converter(taskNode);
                    copyForMeResponse.setTaskNode(converter);
                }
            });
        }

        return copyForMeResponseList;
    }

    @Override
    public Integer queryTotal(CopyForMeRequest copyForMeRequest) {
        return baseMapper.selectCopyForMeTotal(copyForMeRequest);
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:" + token);
    }

    @Override
    public void exportCopyForMe(HttpServletResponse response) throws IOException {
        CopyForMeRequest copyForMeRequest = new CopyForMeRequest();
        String userId = this.getUserId();
        copyForMeRequest.setUserId(Integer.parseInt(userId));
        List<CopyForMeResponse> copyForMeList = this.queryCopyForMe(copyForMeRequest);
        EasyExcelUtil.download(response,CopyForMeResponse.class,copyForMeList,"我的抄送列表","第一页");
    }
}
