package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.mapper.DgFormDesignMapper;
import com.daqing.financial.guarantee.mapper.DgProcessManageMapper;
import com.daqing.financial.guarantee.model.request.DgFormDesignRequest;
import com.daqing.financial.guarantee.model.response.DgFormDesignListResponse;
import com.daqing.financial.guarantee.service.IDgFormDesignService;
import com.daqing.framework.domain.guarantee.DgFormDesign;
import com.daqing.framework.domain.guarantee.GuaranteeCode;
import com.daqing.framework.domain.guarantee.po.DgFormDesignPO;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.CommonCode;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 表单设计业务层实现类
 *
 * @auther River
 * @date 2021/1/15 10:40
 */
@Service
public class DgFormDesignServiceImpl extends ServiceImpl<DgFormDesignMapper, DgFormDesign> implements IDgFormDesignService {

    @Autowired
    private DgProcessManageMapper dgProcessManageMapper;

    /**
     * 列表
     */
    @Override
    public List<DgFormDesignListResponse> list(Integer processId) {
        if (processId == null) {
            ExceptionCast.cast(GuaranteeCode.FROM_DESIGN_PROCESS_ERROR);
        }
        List<DgFormDesignListResponse> responseList = new ArrayList<>();
        List<DgFormDesignPO> list = this.getBaseMapper().list(processId);
        for (DgFormDesignPO dgFormDesignPO : list) {
            DgFormDesignListResponse dgFormDesignListResponse = new DgFormDesignListResponse();
            // 当可操作节点与当前节点相同时表示可以操作
            if (processId.equals(dgFormDesignPO.getOperationProcessId())) {
                dgFormDesignListResponse.setIfOperation(0);
            } else {
                dgFormDesignListResponse.setIfOperation(1);
            }
            BeanUtils.copyProperties(dgFormDesignPO, dgFormDesignListResponse);
            responseList.add(dgFormDesignListResponse);
        }
        return responseList;
    }

    /**
     * 修改
     */
    @Transactional
    @Override
    public Boolean update(DgFormDesignRequest dgFormDesignRequest) {
        if (dgFormDesignRequest.getId() == null || dgFormDesignRequest.getIfConceal() == null
                || dgFormDesignRequest.getIfRequired() == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        if (dgFormDesignRequest.getName() == null || dgFormDesignRequest.getName().length() == 0) {
            ExceptionCast.cast(GuaranteeCode.FROM_DESIGN_NAME_NOT_NULL);
        }
        // 获取当前字段的操作节点id
        Integer operationProcessId = this.getBaseMapper().getOperationProcessIdById(dgFormDesignRequest.getId());
        if (operationProcessId != null) {
            Boolean result = dgProcessManageMapper.updateTimeByModelId(operationProcessId, new Date());
            if (result) {
                return this.getBaseMapper().update(dgFormDesignRequest);
            }
        }
        return false;
    }
}
