package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.feign.CrmsFeignService;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.*;
import com.daqing.financial.guarantee.model.request.ApprovalGuaranteeRequest;
import com.daqing.financial.guarantee.model.request.GuaranteeRequest;
import com.daqing.financial.guarantee.model.request.InvestigateGuaranteeRequest;
import com.daqing.financial.guarantee.model.response.*;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.service.IDgGuaranteeAssignUserService;
import com.daqing.financial.guarantee.util.ArraysUtil;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.crms.request.DgApplyAmountInfoRequest;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.*;
import com.daqing.framework.domain.guarantee.po.DgGuaranteePO;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * <p>
 * 担保部记录 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Service
public class DgGuaranteeAssignUserServiceImpl extends ServiceImpl<DgGuaranteeAssignUserMapper, DgGuaranteeAssignUser> implements IDgGuaranteeAssignUserService {

    @Autowired
    private DgApplyAmountInfoController dgApplyAmountInfoController;

    @Resource
    private DgApplyAmountInfoMapper dgApplyAmountInfoMapper;

    @Resource
    private CrmsFeignService crmsFeignService;

    @Resource
    private DgEnclosureInfoMapper dgEnclosureInfoMapper;
    @Resource
    private DgAssetsInvestigationMapper dgAssetsInvestigationMapper;
    @Resource
    private DgComplianceInvestigationMapper dgComplianceInvestigationMapper;
    @Resource
    private DgMessageInvestigationMapper dgMessageInvestigationMapper;
    @Resource
    private DgBusinessProcessStatusMapper dgBusinessProcessStatusMapper;
    @Resource
    private DgApplyAmountListMapper dgApplyAmountListMapper;
    @Resource
    private DgLoanCommitteeConsiderMapper dgLoanCommitteeConsiderMapper;

    @Resource
    private DgGuaranteeAssignUserMapper dgGuaranteeAssignUserMapper;
    @Resource
    private HrmsFeignService hrmsFeignService;
    @Resource
    private DgBusinessCompanyMapper dgBusinessCompanyMapper;

    @Resource
    private IDgCopyForService dgCopyForService;


    @Override
    public PageUtils queryPage(Integer page, Integer size, String customerNumberOrName, List<String> roleIdlist, String userId, Integer status) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<GuaranteeListResponse> positionVO = this.getBaseMapper().pageByCondition(new Page(page, size), customerNumberOrName, roleIdlist, userId, status);

        return new PageUtils(positionVO);
    }

    @Override
    public GuaranteeDetailResponse guaranteeDetail(Integer id) {
        //根据主键id查询业务id
        DgGuaranteeAssignUser guaranteeAssignUser = this.getBaseMapper().selectById(id);

        //根据业务id查询业务申请详细信息
        DgApplyAmountInfoResponse dgApplyAmountInfo = dgApplyAmountInfoMapper.selectApplyAmountInfoById(guaranteeAssignUser.getBusinessId());
        String measures = dgApplyAmountInfo.getEnGuaranteeMeasures();
        if(measures != null){
            if (measures.length() > 0) {
                String a = measures.substring(1, measures.length() - 1);
                String[] arry = a.split(",");
                List<String> demoList = Arrays.asList(arry);
                dgApplyAmountInfo.setEnGuaranteeMeasure(demoList);
            }
        }

        DgApplyAmountInfoRequest applyAmountInfo = new DgApplyAmountInfoRequest();
        applyAmountInfo.setCompanyId(dgApplyAmountInfo.getCompanyId());
/*        //根据企业id查询企业详情
        ResponseResult result = crmsFeignService.queryCustomerInfoById(applyAmountInfo);

        LinkedHashMap linkedList = (LinkedHashMap) result.getData();*/

        //根据业务id查询备份企业信息
        DgBusinessCompany dgBusinessCompany = dgBusinessCompanyMapper.selectOne(new QueryWrapper<DgBusinessCompany>()
                .eq("business_id", guaranteeAssignUser.getBusinessId()));

        //查询附件详细，附件关联业务主键id
        DgEnclosureInfoResponse dgEnclosureInfo = dgEnclosureInfoMapper.selectByCompanyId(guaranteeAssignUser.getBusinessId());

        List<String> list = new ArrayList<>();
        list.add(dgEnclosureInfo.getAccountingFirm());
        list.add(dgEnclosureInfo.getAssessmentReport());
        list.add(dgEnclosureInfo.getBusinessLicense());
        list.add(dgEnclosureInfo.getCertificateAuthorization());
        list.add(dgEnclosureInfo.getCompanyConstitution());
        list.add(dgEnclosureInfo.getCompanyCredit());
        list.add(dgEnclosureInfo.getConversationSummary());
        list.add(dgEnclosureInfo.getCreditReport());
        list.add(dgEnclosureInfo.getInspectionPhotos());
        list.add(dgEnclosureInfo.getLegalCardCopy());
        list.add(dgEnclosureInfo.getLegalCopy());
        list.add(dgEnclosureInfo.getTaxCertificate());
        list.add(dgEnclosureInfo.getMeetingMinutes());
        list.add(dgEnclosureInfo.getMarriageCopy());

        List arr1List2 = new ArrayList<>();

        for (int i = 0; i < list.size(); i++) {
            List<String> arr1List;
            if (list.get(i) != null) {
                String a1 = list.get(i).substring(1, list.get(i).length() - 1);
                String[] arr1 = a1.split(",");
                arr1List = Arrays.asList(arr1);
                arr1List2.add(i, arr1List);
            } else {
                arr1List2.add(i, null);
            }
        }
/*        if(arr1List2.size()>0){
            if(!arr1List2.get(0).toString().equals("[]"))*/
                dgEnclosureInfo.setAccountingFirmList((List<String>) arr1List2.get(0));
                dgEnclosureInfo.setAssessmentReportList((List<String>) arr1List2.get(1));
                dgEnclosureInfo.setBusinessLicenseList((List<String>) arr1List2.get(2));
                dgEnclosureInfo.setCertificateAuthorizationList((List<String>) arr1List2.get(3));
                dgEnclosureInfo.setCompanyConstitutionList((List<String>) arr1List2.get(4));
                dgEnclosureInfo.setCompanyCreditList((List<String>) arr1List2.get(5));
                dgEnclosureInfo.setConversationSummaryList((List<String>) arr1List2.get(6));
                dgEnclosureInfo.setCreditReportList((List<String>) arr1List2.get(7));
                dgEnclosureInfo.setInspectionPhotosList((List<String>) arr1List2.get(8));
                dgEnclosureInfo.setLegalCardCopyList((List<String>) arr1List2.get(9));
                dgEnclosureInfo.setLegalCopyList((List<String>) arr1List2.get(10));
                dgEnclosureInfo.setTaxCertificateList((List<String>) arr1List2.get(11));
                dgEnclosureInfo.setMeetingMinutesList((List<String>) arr1List2.get(12));
                dgEnclosureInfo.setMarriageCopyList((List<String>) arr1List2.get(13));
//        }


/*        DgGuaranteeAssignUser assignUser1=null;
        if(guaranteeAssignUser.getType().equals(3)){//查询担保部经理审核的金额和期限
            assignUser1 = this.baseMapper.selectOne(new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type", 3));

        }else{//查询担保部领导审核的金额和期限
            assignUser1 = this.baseMapper.selectOne(new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type", 4));
        }

        guaranteeAssignUser.setAmount(assignUser1.getAmount());
        guaranteeAssignUser.setTerm(assignUser1.getTerm());*/
        DgGuaranteeAssignUserResponse assignUser = new DgGuaranteeAssignUserResponse();
        BeanUtils.copyProperties(guaranteeAssignUser,assignUser);
        //assignUser.setRemarks(assignUser1.getRemark());

        String remarks = guaranteeAssignUser.getRemark();
        if(remarks != null){
            if (remarks.length() > 0) {
                String a = remarks.substring(1, remarks.length() - 1);
                String[] arry = a.split(",");
                List<String> demoList = Arrays.asList(arry);
                assignUser.setRemark(demoList);
            }
        }

        //查询该业务id下对应的A角B角
        DgGuaranteeAssignUser assignUser2 = this.baseMapper.selectOne(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type", 1));

        DgGuaranteeAssignUser assignUser3 = this.baseMapper.selectOne(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type", 2));

        if(assignUser2 !=null || assignUser3 != null){
            Integer empAId = assignUser2.getEmpId();
            Integer empBId = assignUser3.getEmpId();
            Integer[] arr = new Integer[]{
                    empAId,empBId
            };
            if(arr.length>0){
                ResponseResult<List<EmployeeVO>> listResponseResult = hrmsFeignService.listEmpNameByUserId(arr);
                assignUser.setAName(listResponseResult.getData().get(0).getName());
                assignUser.setBName(listResponseResult.getData().get(1).getName());
            }
        }

        GuaranteeDetailResponse guaranteeDetailResponse = new GuaranteeDetailResponse();
        guaranteeDetailResponse.setDgApplyAmountInfo(dgApplyAmountInfo);
        guaranteeDetailResponse.setDgEnclosureInfo(dgEnclosureInfo);
        guaranteeDetailResponse.setDgBusinessCompany(dgBusinessCompany);
        //guaranteeDetailResponse.setLinkedHashMap(linkedList);
        guaranteeDetailResponse.setAssignUser(assignUser);
        return guaranteeDetailResponse;
    }

    @Transactional
    @Override
    public int investigateGuarantee(InvestigateGuaranteeRequest investigateGuaranteeRequest) {
        //根据主键id查询业务id
        DgGuaranteeAssignUser guaranteeAssignUser = this.getBaseMapper().selectById(investigateGuaranteeRequest.getId());

        Integer count = 0;
        Integer count1 = 0;
        //如果是属于被驳回的数据，并且调查通过（同意）
        if (guaranteeAssignUser.getStatus().equals(StatusCode.SP_REJECT) && investigateGuaranteeRequest.getStatus().equals(StatusCode.SP_IN_REVIEW)) {

            //AB角调查，修改审核意见以及状态
            DgGuaranteeAssignUser dgGuaranteeAssignUser = new DgGuaranteeAssignUser();
            /*dgGuaranteeAssignUser.setId(investigateGuaranteeRequest.getId());//主键id
            dgGuaranteeAssignUser.setStatus(StatusCode.SP_REJECT);//状态为驳回(只有A,B角同时同意，状态才改为审核中)
            dgGuaranteeAssignUser.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
            dgGuaranteeAssignUser.setApplyContent(investigateGuaranteeRequest.getApplyContent());//审核意见
            this.baseMapper.updateById(dgGuaranteeAssignUser);*/

            //插入或更新的字段有 空字符串 或者 null
            this.baseMapper.update(
                    null,
                    Wrappers.<DgGuaranteeAssignUser>lambdaUpdate()
                            .set(DgGuaranteeAssignUser::getApplyContent, investigateGuaranteeRequest.getApplyContent()) //把email设置成null
                            .set(DgGuaranteeAssignUser::getOperatingStatus, StatusCode.CZ_PROCESSED)
                            .set(DgGuaranteeAssignUser::getStatus, StatusCode.SP_REJECT)
                            .eq(DgGuaranteeAssignUser::getId, investigateGuaranteeRequest.getId())
            );

            int i = this.baseMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeAssignUser.getBusinessId())
                    .eq("type", 1).eq("status", StatusCode.SP_REJECT).eq("operating_status", StatusCode.CZ_PROCESSED));
            int n = this.baseMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeAssignUser.getBusinessId())
                    .eq("type", 2).eq("status", StatusCode.SP_REJECT).eq("operating_status", StatusCode.CZ_PROCESSED));
            int sum = i + n;
            //如果A或B调查通过，审批状态仍然是驳回，操作状态改为已处理
            if (sum == 1) {
                DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
                assignUser.setId(investigateGuaranteeRequest.getId());//主键id
                assignUser.setStatus(StatusCode.SP_REJECT);//驳回
                this.baseMapper.updateById(assignUser);
            }
            //判断AB角是否都调查通过，如果通过，则将担保部经理的状态改为待处理，同时所有的审批状态都变成审核中
            if (sum == 2) {
                DgGuaranteeAssignUser user = new DgGuaranteeAssignUser();
                user.setStatus(StatusCode.SP_IN_REVIEW);//审核中
                this.baseMapper.update(user, new QueryWrapper<DgGuaranteeAssignUser>()
                        .eq("business_id", guaranteeAssignUser.getBusinessId()));
            }
        } else {
            DgGuaranteeAssignUser dgGuaranteeAssignUser = new DgGuaranteeAssignUser();
            dgGuaranteeAssignUser.setId(investigateGuaranteeRequest.getId());//主键id
            dgGuaranteeAssignUser.setApplyContent(investigateGuaranteeRequest.getApplyContent());//审核意见

            //查询合规调查C角驳回记录
            count = dgComplianceInvestigationMapper.selectCount(new QueryWrapper<DgComplianceInvestigation>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId())
                    .eq("role_id", PromptSuccess.FG_C_ID).eq("audit_status", StatusCode.SP_REJECT));
            //查询贷审会驳回记录
            count1 = dgLoanCommitteeConsiderMapper.selectCount(new QueryWrapper<DgLoanCommitteeConsider>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId())
                    .eq("role_id", PromptSuccess.DSH_ID).eq("status", 4));
            if(count>0 || count1>0){//说明数据是C角或者贷审会驳回回来的
                //AB角调查，修改审核意见以及状态
/*                DgGuaranteeAssignUser dgGuaranteeAssignUser2 = new DgGuaranteeAssignUser();
                dgGuaranteeAssignUser2.setId(investigateGuaranteeRequest.getId());//主键id
                dgGuaranteeAssignUser2.setStatus(investigateGuaranteeRequest.getStatus());//状态为驳回(只有A,B角同时同意，状态才改为审核中)
                dgGuaranteeAssignUser2.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
                dgGuaranteeAssignUser2.setApplyContent(investigateGuaranteeRequest.getApplyContent());//审核意见
                this.baseMapper.updateById(dgGuaranteeAssignUser2);*/

                //插入或更新的字段有 空字符串 或者 null
                this.baseMapper.update(
                        null,
                        Wrappers.<DgGuaranteeAssignUser>lambdaUpdate()
                                .set(DgGuaranteeAssignUser::getApplyContent, investigateGuaranteeRequest.getApplyContent())
                                .set(DgGuaranteeAssignUser::getOperatingStatus, StatusCode.CZ_PROCESSED)
                                .set(DgGuaranteeAssignUser::getStatus, investigateGuaranteeRequest.getStatus())
                                .eq(DgGuaranteeAssignUser::getId, investigateGuaranteeRequest.getId())
                );

                int i = this.baseMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeAssignUser.getBusinessId())
                        .eq("type", 1).eq("status", StatusCode.SP_IN_REVIEW).eq("operating_status", StatusCode.CZ_PROCESSED));
                int n = this.baseMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeAssignUser.getBusinessId())
                        .eq("type", 2).eq("status", StatusCode.SP_IN_REVIEW).eq("operating_status", StatusCode.CZ_PROCESSED));
                int sum = i + n;

                if(count>0){//C角跨级驳回
                    //判断AB角是否都调查通过，如果通过，则将合规调查所有业务审批状态改为审核中，法规经理操作状态为待处理
                    if (sum == 2) {
                        DgComplianceInvestigation user = new DgComplianceInvestigation();
                        user.setAuditStatus(StatusCode.SP_IN_REVIEW);//审核中
                        dgComplianceInvestigationMapper.update(user, new QueryWrapper<DgComplianceInvestigation>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()));

                        user.setOperationStatus(StatusCode.CZ_ON_HAND);//操作状态:待处理
                        dgComplianceInvestigationMapper.update(user, new QueryWrapper<DgComplianceInvestigation>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId())
                                .eq("role_id",PromptSuccess.FG_JL_ID));
                    }
                }
                if(count1>0){//贷审会跨级驳回
                    //判断AB角是否都调查通过，如果通过，则将贷审会所有业务审批状态改为审核中，贷审会操作状态为待处理
                    if (sum == 2) {
                        DgLoanCommitteeConsider user = new DgLoanCommitteeConsider();
                        user.setStatus(StatusCode.SP_IN_REVIEW);//审核中
                        dgLoanCommitteeConsiderMapper.update(user, new QueryWrapper<DgLoanCommitteeConsider>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()));

                        user.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态:待处理
                        dgLoanCommitteeConsiderMapper.update(user, new QueryWrapper<DgLoanCommitteeConsider>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId())
                                .eq("role_id",PromptSuccess.DSH_ID));
                    }
                }
            }else if(investigateGuaranteeRequest.getIsSaveOrCommit() != null && investigateGuaranteeRequest.getIsSaveOrCommit()==1){//提交
                //AB角调查，修改审核意见以及状态
                dgGuaranteeAssignUser.setStatus(investigateGuaranteeRequest.getStatus());//状态为审核中
                dgGuaranteeAssignUser.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
               // this.baseMapper.updateById(dgGuaranteeAssignUser);
                //插入或更新的字段有 空字符串 或者 null
                this.baseMapper.update(
                        null,
                        Wrappers.<DgGuaranteeAssignUser>lambdaUpdate()
                                .set(DgGuaranteeAssignUser::getApplyContent, investigateGuaranteeRequest.getApplyContent()) //把email设置成null
                                .set(DgGuaranteeAssignUser::getOperatingStatus, StatusCode.CZ_PROCESSED)
                                .set(DgGuaranteeAssignUser::getStatus, investigateGuaranteeRequest.getStatus())
                                .eq(DgGuaranteeAssignUser::getId, investigateGuaranteeRequest.getId())
                );
            }else if(investigateGuaranteeRequest.getIsSaveOrCommit() != null && investigateGuaranteeRequest.getIsSaveOrCommit()!=1){//保存
                dgGuaranteeAssignUser.setStatus(guaranteeAssignUser.getStatus());//审批状态 当前是什么状态就是什么状态
                dgGuaranteeAssignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                //this.baseMapper.updateById(dgGuaranteeAssignUser);
                //插入或更新的字段有 空字符串 或者 null
                this.baseMapper.update(
                        null,
                        Wrappers.<DgGuaranteeAssignUser>lambdaUpdate()
                                .set(DgGuaranteeAssignUser::getApplyContent, investigateGuaranteeRequest.getApplyContent()) //把email设置成null
                                .set(DgGuaranteeAssignUser::getOperatingStatus, StatusCode.CZ_ON_HAND)
                                .set(DgGuaranteeAssignUser::getStatus, guaranteeAssignUser.getStatus())
                                .eq(DgGuaranteeAssignUser::getId, investigateGuaranteeRequest.getId())
                );
            }else{
                if(investigateGuaranteeRequest.getStatus().equals(StatusCode.SP_REFUSE)){//如果为拒绝，则修改所有审批状态为拒绝，同时A,B角操作状态均为已处理
                    dgGuaranteeAssignUser.setStatus(StatusCode.SP_REFUSE);//拒绝
                    this.baseMapper.update(dgGuaranteeAssignUser,new QueryWrapper<DgGuaranteeAssignUser>()
                            .eq("business_id",guaranteeAssignUser.getBusinessId()));
                    dgGuaranteeAssignUser.setOperatingStatus(StatusCode.CZ_PROCESSED);//已处理
/*                    this.baseMapper.update(dgGuaranteeAssignUser,new QueryWrapper<DgGuaranteeAssignUser>()
                            .eq("business_id",guaranteeAssignUser.getBusinessId()).eq("type",1));
                    this.baseMapper.update(dgGuaranteeAssignUser,new QueryWrapper<DgGuaranteeAssignUser>()
                            .eq("business_id",guaranteeAssignUser.getBusinessId()).eq("type",2));*/

                    this.baseMapper.update(
                            null,
                            Wrappers.<DgGuaranteeAssignUser>lambdaUpdate()
                                    .set(DgGuaranteeAssignUser::getApplyContent, investigateGuaranteeRequest.getApplyContent()) //把email设置成null
                                    .set(DgGuaranteeAssignUser::getOperatingStatus, StatusCode.CZ_PROCESSED)
                                    .set(DgGuaranteeAssignUser::getStatus, StatusCode.SP_REFUSE)
                                    .eq(DgGuaranteeAssignUser::getBusinessId, guaranteeAssignUser.getBusinessId())
                                    .eq(DgGuaranteeAssignUser::getType, 1)
                    );

                    this.baseMapper.update(
                            null,
                            Wrappers.<DgGuaranteeAssignUser>lambdaUpdate()
                                    .set(DgGuaranteeAssignUser::getApplyContent, investigateGuaranteeRequest.getApplyContent()) //把email设置成null
                                    .set(DgGuaranteeAssignUser::getOperatingStatus, StatusCode.CZ_PROCESSED)
                                    .set(DgGuaranteeAssignUser::getStatus, StatusCode.SP_REFUSE)
                                    .eq(DgGuaranteeAssignUser::getBusinessId, guaranteeAssignUser.getBusinessId())
                                    .eq(DgGuaranteeAssignUser::getType, 2)
                    );
                    //拒绝状态下，修改业务状态为已否决
                    DgBusinessProcessStatus processStatus = new DgBusinessProcessStatus();
                    processStatus.setBusinessStatus(3);//已否决
                    dgBusinessProcessStatusMapper.update(processStatus,new QueryWrapper<DgBusinessProcessStatus>()
                            .eq("business_id",guaranteeAssignUser.getBusinessId()));
                }else{
                    dgGuaranteeAssignUser.setStatus(investigateGuaranteeRequest.getStatus());//审批状态 请求过来是什么状态就是什么
                    dgGuaranteeAssignUser.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
                    //this.baseMapper.updateById(dgGuaranteeAssignUser);

                    this.baseMapper.update(
                            null,
                            Wrappers.<DgGuaranteeAssignUser>lambdaUpdate()
                                    .set(DgGuaranteeAssignUser::getApplyContent, investigateGuaranteeRequest.getApplyContent()) //把email设置成null
                                    .set(DgGuaranteeAssignUser::getOperatingStatus, StatusCode.CZ_PROCESSED)
                                    .set(DgGuaranteeAssignUser::getStatus, investigateGuaranteeRequest.getStatus())
                                    .eq(DgGuaranteeAssignUser::getId, investigateGuaranteeRequest.getId())
                    );
                }
            }
        }

        if (investigateGuaranteeRequest.getStatus().equals(StatusCode.SP_REJECT)) {//驳回情况处理
            //业务申请部分业务专员审批状态设置为驳回，操作状态设置为待处理
            DgApplyAmountList dgApplyAmountList = new DgApplyAmountList();
            dgApplyAmountList.setStatus(StatusCode.SP_REJECT);//状态设置为驳回
            dgApplyAmountList.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
            dgApplyAmountListMapper.update(dgApplyAmountList, new QueryWrapper<DgApplyAmountList>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type", 1));
            //业务申请部分担保部经理审批状态设置为驳回，操作状态设置为已处理
            DgApplyAmountList applyAmountList = new DgApplyAmountList();
            applyAmountList.setStatus(StatusCode.SP_REJECT);//状态设置为驳回
            applyAmountList.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
            dgApplyAmountListMapper.update(applyAmountList, new QueryWrapper<DgApplyAmountList>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type", 2));

            //修改担保部调查下该业务的所有审批状态为驳回
            DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
            assignUser.setStatus(StatusCode.SP_REJECT);//驳回
            this.baseMapper.update(assignUser, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()));

            DgGuaranteeAssignUser assignUser2 = new DgGuaranteeAssignUser();
            assignUser2.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
            this.baseMapper.update(assignUser2, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type",1));
            this.baseMapper.update(assignUser2, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type",2));
        }

        //如果非合规调查和贷审会跨级驳回情况
//        if(count<1 && count1<1){
            //判断A角和B角是否都调查通过，如果都审核通过，修改担保部经理操作状态为待处理
            int a = this.baseMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeAssignUser.getBusinessId())
                    .eq("type", 1).eq("status", StatusCode.SP_IN_REVIEW).eq("operating_status", StatusCode.CZ_PROCESSED));
            int b = this.baseMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeAssignUser.getBusinessId())
                    .eq("type", 2).eq("status", StatusCode.SP_IN_REVIEW).eq("operating_status", StatusCode.CZ_PROCESSED));
            int sum2 = a + b;
            if (sum2 == 2) {

                //如果合规调查C角状态为驳回，并且操作状态为已处理，同时此条数据也是被驳回来的数据，则为合规调查驳回情况处理
                DgComplianceInvestigation investigation = dgComplianceInvestigationMapper.selectOne(new QueryWrapper<DgComplianceInvestigation>()
                        .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("role_id", PromptSuccess.FG_C_ID));

                //如果贷审会状态为驳回，并且操作状态为已处理，同时此条数据也是被驳回来的数据，则为贷审会驳回情况处理
                DgLoanCommitteeConsider consider = dgLoanCommitteeConsiderMapper.selectOne(new QueryWrapper<DgLoanCommitteeConsider>()
                        .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("role_id", PromptSuccess.DSH_ID));

                if ((investigation != null && investigation.getAuditStatus().equals(StatusCode.SP_IN_REVIEW) && investigation.getOperationStatus().equals(StatusCode.CZ_PROCESSED)
                        && guaranteeAssignUser.getStatus().equals(StatusCode.SP_IN_REVIEW)) || (investigation != null && investigation.getAuditStatus().equals(StatusCode.SP_REJECT)
                        && investigation.getOperationStatus().equals(StatusCode.CZ_PROCESSED)
                                && guaranteeAssignUser.getStatus().equals(StatusCode.SP_REJECT))) {//合规调查驳回情况处理

                    //调查通过，修改法规经理操作状态为待处理，其他子业务为审核中
                    if (investigateGuaranteeRequest.getStatus().equals(StatusCode.SP_IN_REVIEW)) {//如果调查通过
                        DgComplianceInvestigation dgCompliance = new DgComplianceInvestigation();
                        dgCompliance.setOperationStatus(StatusCode.CZ_ON_HAND);//待处理
                        dgComplianceInvestigationMapper.update(dgCompliance, new QueryWrapper<DgComplianceInvestigation>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("role_id", PromptSuccess.FG_JL_ID));

                        DgComplianceInvestigation dgInvestigation = new DgComplianceInvestigation();
                        dgInvestigation.setAuditStatus(StatusCode.SP_IN_REVIEW);//审核中
                        dgComplianceInvestigationMapper.update(dgInvestigation, new QueryWrapper<DgComplianceInvestigation>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()));

                        //修改担保业务审批状态为已审核
                        DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
                        assignUser.setStatus(StatusCode.SP_REVIEWED);//已审核
                        this.baseMapper.update(assignUser, new QueryWrapper<DgGuaranteeAssignUser>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()));
                    }
                }else if (consider != null && consider.getStatus().equals(StatusCode.SP_IN_REVIEW) && consider.getOperatingStatus().equals(StatusCode.CZ_ON_HAND)
                        && guaranteeAssignUser.getStatus().equals(StatusCode.SP_IN_REVIEW) || (consider != null && consider.getStatus().equals(StatusCode.SP_REJECT) && consider.getOperatingStatus().equals(StatusCode.CZ_PROCESSED)
                        && guaranteeAssignUser.getStatus().equals(StatusCode.SP_REJECT))) {//贷审会业务流程驳回情况处理
                /*else if (consider != null && consider.getStatus().equals(StatusCode.SP_REJECT) && consider.getOperatingStatus().equals(StatusCode.CZ_PROCESSED)
                        && guaranteeAssignUser.getStatus().equals(StatusCode.SP_REJECT)) {//贷审会业务流程驳回情况处理*/

                    //调查通过，修改贷审会操作状态为待处理，其他子业务为审核中
                    if (investigateGuaranteeRequest.getStatus().equals(StatusCode.SP_IN_REVIEW)) {//如果调查通过
                        DgLoanCommitteeConsider dgLoanCommittee = new DgLoanCommitteeConsider();
                        dgLoanCommittee.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理
                        dgLoanCommitteeConsiderMapper.update(dgLoanCommittee, new QueryWrapper<DgLoanCommitteeConsider>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("role_id", PromptSuccess.DSH_ID));

                        DgLoanCommitteeConsider dgConsider = new DgLoanCommitteeConsider();
                        dgConsider.setStatus(StatusCode.SP_IN_REVIEW);//审核中
                        dgLoanCommitteeConsiderMapper.update(dgConsider, new QueryWrapper<DgLoanCommitteeConsider>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()));

                        //修改担保业务审批状态为已审核
                        DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
                        assignUser.setStatus(StatusCode.SP_REVIEWED);//已审核
                        this.baseMapper.update(assignUser, new QueryWrapper<DgGuaranteeAssignUser>()
                                .eq("business_id", guaranteeAssignUser.getBusinessId()));
                    }
                } else {//正常情况处理
                    DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
                    assignUser.setRoleId(PromptSuccess.DB_JL_ID);//担保部经理
                    assignUser.setStatus(StatusCode.SP_IN_REVIEW);//审批状态设置为审核中
                    assignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                    this.baseMapper.update(assignUser, new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeAssignUser.getBusinessId())
                            .eq("type", 3));
                }
//            }

        }

        //上传附件信息
        DgEnclosureInfo dgEnclosureInfo = new DgEnclosureInfo();
        dgEnclosureInfo.setBusinessId(guaranteeAssignUser.getBusinessId());//业务id
        dgEnclosureInfo.setCompanyId(guaranteeAssignUser.getCompanyId());//企业Id
        dgEnclosureInfo.setBusinessLicense(ArraysUtil.toString(investigateGuaranteeRequest.getBusinessLicense()));//营业执照复印件
        dgEnclosureInfo.setLegalCardCopy(ArraysUtil.toString(investigateGuaranteeRequest.getLegalCardCopy()));//上传法定代表人夫妻及企业实际经营者身份证复印件
        dgEnclosureInfo.setMarriageCopy(ArraysUtil.toString(investigateGuaranteeRequest.getMarriageCopy()));//法定代表人夫妻户口本，结婚证（离婚证）复印件
        dgEnclosureInfo.setLegalCopy(ArraysUtil.toString(investigateGuaranteeRequest.getLegalCopy()));//法定代表人身份证明
        dgEnclosureInfo.setCompanyConstitution(ArraysUtil.toString(investigateGuaranteeRequest.getCompanyConstitution()));//公司章程
        dgEnclosureInfo.setAccountingFirm(ArraysUtil.toString(investigateGuaranteeRequest.getAccountingFirm()));//会计师事务所审计的上一年度及本年度审计报告
        dgEnclosureInfo.setMeetingMinutes(ArraysUtil.toString(investigateGuaranteeRequest.getMeetingMinutes()));//股东会会议纪要
        dgEnclosureInfo.setConversationSummary(ArraysUtil.toString(investigateGuaranteeRequest.getConversationSummary()));
        dgEnclosureInfo.setInspectionPhotos(ArraysUtil.toString(investigateGuaranteeRequest.getInspectionPhotos()));

        //根据业务id查询对应的附件信息
        DgEnclosureInfo dgEnclosureInfo1 = dgEnclosureInfoMapper.selectOne(new QueryWrapper<DgEnclosureInfo>().eq("business_id", guaranteeAssignUser.getBusinessId()));
        dgEnclosureInfo.setOtherFile(dgEnclosureInfo1.getOtherFile());//其他附件
        dgEnclosureInfo.setCompanyCredit(dgEnclosureInfo1.getCompanyCredit());//企业信用报告
        dgEnclosureInfo.setCreditReport(dgEnclosureInfo1.getCreditReport());
        dgEnclosureInfo.setCertificateAuthorization(dgEnclosureInfo1.getCertificateAuthorization());
        dgEnclosureInfo.setTaxCertificate(dgEnclosureInfo1.getTaxCertificate());
        dgEnclosureInfo.setAssessmentReport(dgEnclosureInfo1.getAssessmentReport());

        //根据业务id删除对应的附件信息
        dgEnclosureInfoMapper.delete(new QueryWrapper<DgEnclosureInfo>().eq("business_id", guaranteeAssignUser.getBusinessId()));
        //附件新增
        dgEnclosureInfoMapper.insert(dgEnclosureInfo);

        Integer type = guaranteeAssignUser.getType();
        String deptName;
        Integer roleId;
        if (type == 1) {
            deptName = "担保部A角";
            roleId = PromptSuccess.DB_A_ID;
        } else {
            deptName = "担保部B角";
            roleId = PromptSuccess.DB_B_ID;
        }
        //不为保存的情况则往审核流程进程表新增一条记录
        if(!investigateGuaranteeRequest.getStatus().equals(StatusCode.SP_DRAFT)){
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
            AuditProcessUtil.save(Integer.parseInt(this.getUserId()),guaranteeAssignUser.getBusinessId(),
                    account, departmentId, investigateGuaranteeRequest.getApplyContent(), 2,roleId,
                    "","","",investigateGuaranteeRequest.getStatus());
        }
        return 1;
    }

    @Transactional
    @Override
    public int approvalGuarantee(ApprovalGuaranteeRequest approvalGuaranteeRequest) {

        //根据主键id查询担保部记录
        DgGuaranteeAssignUser guaranteeUser = this.baseMapper.selectById(approvalGuaranteeRequest.getId());

        DgGuaranteeAssignUser dgGuaranteeAssignUser = new DgGuaranteeAssignUser();
        dgGuaranteeAssignUser.setId(approvalGuaranteeRequest.getId());
        dgGuaranteeAssignUser.setEmpId(Integer.parseInt(DgApplyAmountInfoController.getUserId()));
        dgGuaranteeAssignUser.setRemark(ArraysUtil.toString(approvalGuaranteeRequest.getRemark()));//补充调查原因
        dgGuaranteeAssignUser.setOtherRemark(approvalGuaranteeRequest.getOtherRemark());//其他原因
        if(guaranteeUser.getType()==3 && approvalGuaranteeRequest.getStatus()==2){
            //区分经理，领导审核，此处为经理审核
            dgGuaranteeAssignUser.setStatus(StatusCode.SP_IN_REVIEW);//审核中
        }else{
            dgGuaranteeAssignUser.setStatus(approvalGuaranteeRequest.getStatus());
        }
        dgGuaranteeAssignUser.setOperatingStatus(StatusCode.CZ_PROCESSED);//状态设置为已处理
        dgGuaranteeAssignUser.setAmount(approvalGuaranteeRequest.getAmount());//金额
        dgGuaranteeAssignUser.setTerm(approvalGuaranteeRequest.getTerm());//期限
        dgGuaranteeAssignUser.setApplyContent(approvalGuaranteeRequest.getApplyContent());//审核意见
        //根据主键id修改担保部记录
        this.baseMapper.updateById(dgGuaranteeAssignUser);

        DgGuaranteeAssignUser dgAssignUser = new DgGuaranteeAssignUser();
        dgAssignUser.setStatus(StatusCode.SP_REJECT);//驳回
        dgAssignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理

        if (approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_REJECT)) {//驳回情况
            //修改担保部调查下该业务的所有审批状态为驳回
            DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
            assignUser.setStatus(StatusCode.SP_REJECT);//驳回
            this.baseMapper.update(assignUser, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeUser.getBusinessId()));
        }

        //如果是处理驳回数据，如果审核通过，则将担保部调查业务下所有审批状态设置为审核中，担保领导这条记录操作状态设置为待处理
        if (guaranteeUser.getStatus().equals(StatusCode.SP_REJECT) && approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_IN_REVIEW) && guaranteeUser.getType() == 3) {
            DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
            assignUser.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            this.baseMapper.update(assignUser, new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeUser.getBusinessId()));

            DgGuaranteeAssignUser user = new DgGuaranteeAssignUser();
            user.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态为待处理
            this.baseMapper.update(user, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeUser.getBusinessId()).eq("type", 4));
        } else if (approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_REFUSE)) {//如果为拒绝，业务状态则为已否决，设置子业务下所有审批状态为拒绝
            DgGuaranteeAssignUser user = new DgGuaranteeAssignUser();
            user.setStatus(StatusCode.SP_REFUSE);//审批状态为拒绝
            this.baseMapper.update(user, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeUser.getBusinessId()));

            DgBusinessProcessStatus processStatus = new DgBusinessProcessStatus();
            processStatus.setBusinessId(guaranteeUser.getBusinessId());
            processStatus.setBusinessStatus(StatusCode.YW_REJECTED);//业务状态设置为已否决
            dgBusinessProcessStatusMapper.update(processStatus, new QueryWrapper<DgBusinessProcessStatus>()
                    .eq("business_id", processStatus.getBusinessId()));
        } else if (approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_REJECT) && guaranteeUser.getType() == 3) {
            //担保部经理驳回情况
            //A,B角调查部分审批状态统一设置驳回，操作状态设置为待处理
            this.baseMapper.update(dgAssignUser, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeUser.getBusinessId()).eq("type", 1));
            this.baseMapper.update(dgAssignUser, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeUser.getBusinessId()).eq("type", 2));
        } else if (approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_REJECT) && guaranteeUser.getType() == 4) {
            //担保部领导驳回情况
            //担保部经理审批状态设置为驳回，操作状态设置为待处理
            this.baseMapper.update(dgAssignUser, new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeUser.getBusinessId()).eq("type", 3));
        } else if ((approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_IN_REVIEW ) || approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_REVIEWED) )&& dgGuaranteeAssignUser.getOperatingStatus() == StatusCode.CZ_PROCESSED && guaranteeUser.getType() == 3) {//担保部经理审核通过
            //根据主键id查询业务id
            DgGuaranteeAssignUser guaranteeAssignUser = this.getBaseMapper().selectById(approvalGuaranteeRequest.getId());

            //如果逐级驳回，且担保部经理审核通过，首先查看担保领导是否有数据，如果有数据，则修改不新增
            Integer count = this.baseMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id", guaranteeAssignUser.getBusinessId()).eq("type", 4));

            DgGuaranteeAssignUser assign = new DgGuaranteeAssignUser();
            assign.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
            //修改该业务下所有审批状态为审核中
            this.baseMapper.update(assign,new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id",guaranteeAssignUser.getBusinessId()));

            if(count>0){//担保领导有数据，则直接修改领导状态
                DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
                assignUser.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
                assignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态为待处理
                this.baseMapper.update(assignUser,new QueryWrapper<DgGuaranteeAssignUser>()
                        .eq("business_id",guaranteeAssignUser.getBusinessId()).eq("type",4));
            }else{
                //如果担保部经理审核通过，则往担保部分管领导处加一条记录，由分管领导审核
                DgGuaranteeAssignUser assignUser = new DgGuaranteeAssignUser();
                assignUser.setBusinessId(guaranteeAssignUser.getBusinessId());
                assignUser.setCompanyId(guaranteeAssignUser.getCompanyId());
                assignUser.setRoleId(PromptSuccess.DB_LD_ID);//担保部分管领导
                assignUser.setType(4);//分管领导审核
                assignUser.setStatus(StatusCode.SP_IN_REVIEW);//审批状态设置为审核中
                assignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                this.baseMapper.insert(assignUser);
            }
        } else if (approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_REVIEWED) && dgGuaranteeAssignUser.getOperatingStatus() == StatusCode.CZ_PROCESSED && guaranteeUser.getType() == 4) {//担保部领导审核通过
            //如果担保部分管领导审核通过，同时判断资产部分管领导审核以及信息部分管领导审核是否通过，如果通过就往合规调查中插入一条数据
            //如果信息部领导审核通过，则该子业务流程下的所有审批状态都为已审核状态
            DgGuaranteeAssignUser user = new DgGuaranteeAssignUser();
            user.setStatus(StatusCode.SP_REVIEWED);//已审核
            this.baseMapper.update(user, new QueryWrapper<DgGuaranteeAssignUser>().eq("business_id", guaranteeUser.getBusinessId()));
            //查看信息部分管领导是否审核通过
            int a = dgMessageInvestigationMapper.selectCount(new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id", guaranteeUser.getBusinessId())
                    .eq("type", 3).eq("status", StatusCode.SP_REVIEWED).eq("operating_status", StatusCode.CZ_PROCESSED));

            //查看资产部分管领导是否审核通过
            int b = dgAssetsInvestigationMapper.selectCount(new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id", guaranteeUser.getBusinessId())
                    .eq("type", 3).eq("status", StatusCode.SP_REVIEWED).eq("operating_status", StatusCode.CZ_PROCESSED));
            int sum = a + b;
            if (sum == 2) {
                //新增记录到合规调查
                DgComplianceInvestigation compliance = new DgComplianceInvestigation();
                compliance.setCompanyId(guaranteeUser.getCompanyId());//企业id
                compliance.setRoleId(PromptSuccess.FG_JL_ID);//角色id
                compliance.setBusinessId(guaranteeUser.getBusinessId());//业务id
                compliance.setOperationStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                compliance.setAuditStatus(StatusCode.SP_TO_BE_ASSIGNED);//审核状态设置为待指派
                dgComplianceInvestigationMapper.insert(compliance);
            }
        }
        Integer type = guaranteeUser.getType();
        String deptName;
        Integer roleId;
        if (type == 3) {
            deptName = "担保部经理";
            roleId = PromptSuccess.DB_JL_ID;
        } else {
            deptName = "担保部分管领导";
            roleId = PromptSuccess.DB_LD_ID;
        }
        String remark1 = Arrays.toString(approvalGuaranteeRequest.getRemark());//补充调查原因
        String remark = remark1.replace("[","").replace("]","");
        String otherRemark = approvalGuaranteeRequest.getOtherRemark();//其他原因
        if(otherRemark == null){
            otherRemark = "";
        }
        String reason = remark+otherRemark;
        String loanMoney = String.valueOf(approvalGuaranteeRequest.getAmount());//金额

        if(loanMoney == "null"){
            loanMoney = "";
        }
        String loanTerm = approvalGuaranteeRequest.getTerm();//期限
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),guaranteeUser.getBusinessId(),
                account, departmentId, approvalGuaranteeRequest.getApplyContent(),
                2,roleId,reason,loanMoney,loanTerm,approvalGuaranteeRequest.getStatus());

        boolean b=false;
        //根据业务为担保部分管领导处理以及是否驳回决定是否抄送
        if(roleId.equals(PromptSuccess.DB_LD_ID) && !approvalGuaranteeRequest.getStatus().equals(StatusCode.SP_REJECT)){//领导，驳回
            try {
                b = dgCopyForService.copySendUser(guaranteeUser.getBusinessId(), 2, approvalGuaranteeRequest.getId());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return 1;
    }

    @Override
    public Boolean excelExport(List<String> ids, Integer userId, HttpServletResponse response) {
        //根据ids查询业务申请列表信息
        List<GuaranteeListResponse> guaranteeList = this.baseMapper.selectListByIds(ids,userId);
        System.out.println("applyList===========================" + guaranteeList);
        try {
            ExcelUtil.writeExcelWithSheets(response, guaranteeList, "担保调查一览表", "businessApplication", new GuaranteeListResponse())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    @Override
    public ResponseResult assignCorners(GuaranteeRequest guaranteeRequest) {
        //A,B角用户id不能选择同一个人
        if (guaranteeRequest.getEmpAId().equals(guaranteeRequest.getEmpBId())) {
            ExceptionCast.cast(HrmsCode.EMP_SAME);
        }
        //判断即将被指派成A角的用户id以及B角的用户id是否在当前业务中已被指派
        //查询empAId，empBId在资产部调查中是否被指派
        Integer count1 = dgAssetsInvestigationMapper.selectCount(new QueryWrapper<DgAssetsInvestigation>()
                .eq("business_id", guaranteeRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", guaranteeRequest.getEmpAId()));

        Integer count = dgAssetsInvestigationMapper.selectCount(new QueryWrapper<DgAssetsInvestigation>()
                .eq("business_id", guaranteeRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", guaranteeRequest.getEmpBId()));
        if(count1 >0 || count >0){//已被指派情况下直接返回
            ExceptionCast.cast(HrmsCode.EMP_TO_BE_CHOOSE);
        }

        //查询empAId，empBId在信息部调查中是否被指派
        Integer count2 = dgMessageInvestigationMapper.selectCount(new QueryWrapper<DgMessageInvestigation>()
                .eq("business_id", guaranteeRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", guaranteeRequest.getEmpAId()));

        Integer count3 = dgMessageInvestigationMapper.selectCount(new QueryWrapper<DgMessageInvestigation>()
                .eq("business_id", guaranteeRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", guaranteeRequest.getEmpBId()));
        if(count2 >0 || count3 >0){//已被指派情况下直接返回
            ExceptionCast.cast(HrmsCode.EMP_TO_BE_CHOOSE);
        }
        //根据主键id修改当前记录状态为审核中，操作状态为已处理,设置审批人id为当前用户id
        //获取当前登录用户userId
        String userId = DgApplyAmountInfoController.getUserId();
        DgGuaranteeAssignUser guaranteeAssignUser = new DgGuaranteeAssignUser();
        guaranteeAssignUser.setStatus(StatusCode.SP_IN_REVIEW);//状态设置为审核中
        guaranteeAssignUser.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
        guaranteeAssignUser.setEmpId(Integer.parseInt(userId));
        guaranteeAssignUser.setId(guaranteeRequest.getId());
        dgGuaranteeAssignUserMapper.updateById(guaranteeAssignUser);

        //新增A角记录到担保部记录表
        DgGuaranteeAssignUser dgGuaranteeAssignUser = new DgGuaranteeAssignUser();
        dgGuaranteeAssignUser.setBusinessId(guaranteeRequest.getBusinessId());//业务id
        dgGuaranteeAssignUser.setCompanyId(guaranteeRequest.getCompanyId());//企业id
        dgGuaranteeAssignUser.setEmpId(guaranteeRequest.getEmpAId());//用户A角id
        dgGuaranteeAssignUser.setRoleId(PromptSuccess.DB_A_ID);//角色设置为A角
        dgGuaranteeAssignUser.setType(1);//A角操作
        dgGuaranteeAssignUser.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
        dgGuaranteeAssignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
        dgGuaranteeAssignUserMapper.insert(dgGuaranteeAssignUser);
        //新增B角记录到担保部记录表
        DgGuaranteeAssignUser dgGuaranteeAssignUser2 = new DgGuaranteeAssignUser();
        dgGuaranteeAssignUser2.setBusinessId(guaranteeRequest.getBusinessId());//业务id
        dgGuaranteeAssignUser2.setCompanyId(guaranteeRequest.getCompanyId());//企业id
        dgGuaranteeAssignUser2.setEmpId(guaranteeRequest.getEmpBId());//用户B角id
        dgGuaranteeAssignUser2.setRoleId(PromptSuccess.DB_B_ID);//角色设置为B角
        dgGuaranteeAssignUser2.setType(2);//B角操作
        dgGuaranteeAssignUser2.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
        dgGuaranteeAssignUser2.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
        dgGuaranteeAssignUserMapper.insert(dgGuaranteeAssignUser2);

        //查询用户角色表是否已绑定，如果没有绑定，则去往绑定页面
        ResponseResult i = hrmsFeignService.selectUserRole(guaranteeRequest.getEmpAId().longValue(), PromptSuccess.DB_A_ID.longValue());
        ResponseResult u = hrmsFeignService.selectUserRole(guaranteeRequest.getEmpBId().longValue(),PromptSuccess.DB_B_ID.longValue());
        int a = (int) i.getData();
        int b = (int) u.getData();
        int sum = a+b;
        if(sum==0){
            List<DgGuaranteePO> list = new ArrayList();
            DgGuaranteePO dgGuaranteePO1 = new DgGuaranteePO();
            dgGuaranteePO1.setUserId(guaranteeRequest.getEmpAId().longValue());
            dgGuaranteePO1.setRoleId(PromptSuccess.DB_A_ID.longValue());
            DgGuaranteePO dgGuaranteePO2 = new DgGuaranteePO();
            dgGuaranteePO2.setUserId(guaranteeRequest.getEmpBId().longValue());
            dgGuaranteePO2.setRoleId(PromptSuccess.DB_B_ID.longValue());
            list.add(dgGuaranteePO1);
            list.add(dgGuaranteePO2);
            //批量绑定角色关系
            hrmsFeignService.insertUsrRole(list);
        }else{
            if(a==0){
                hrmsFeignService.insertUserRole(guaranteeRequest.getEmpAId().longValue(),PromptSuccess.DB_A_ID.longValue());
            }
            if(b==0){
                hrmsFeignService.insertUserRole(guaranteeRequest.getEmpBId().longValue(),PromptSuccess.DB_B_ID.longValue());
            }
        }
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),guaranteeRequest.getBusinessId(),
                account, departmentId, "", 2,PromptSuccess.DB_JL_ID,"","","",StatusCode.SP_REVIEWED);

        return ResponseResult.SUCCESS();
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        return userId;
    }

}
