package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.mapper.DgComplianceInvestigationMapper;
import com.daqing.financial.guarantee.mapper.DgGuaranteeAssignUserMapper;
import com.daqing.financial.guarantee.mapper.DgLoanCommitteeConsiderMapper;
import com.daqing.financial.guarantee.model.request.DgLoanCommitteeConsiderConsent;
import com.daqing.financial.guarantee.model.request.DgLoanCommitteeConsiderRequest;
import com.daqing.financial.guarantee.model.response.DgLoanCommitteeConsiderInfoResponse;
import com.daqing.financial.guarantee.model.response.DgLoanCommitteeConsiderResponse;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.service.IDgGuaranteeLetterAssignUserService;
import com.daqing.financial.guarantee.service.IDgLoanCommitteeConsiderService;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.DgGuaranteeLetterAssignUser;
import com.daqing.framework.domain.guarantee.DgLoanCommitteeConsider;
import com.daqing.framework.domain.guarantee.GuaranteeCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import io.swagger.models.auth.In;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 贷审会审议 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Service
public class DgLoanCommitteeConsiderServiceImpl extends ServiceImpl<DgLoanCommitteeConsiderMapper, DgLoanCommitteeConsider>
        implements IDgLoanCommitteeConsiderService {

    @Autowired
    private DgComplianceInvestigationMapper dgComplianceInvestigationMapper;

    @Autowired
    private DgComplianceInvestigationServiceImpl dgComplianceInvestigationService;

    @Autowired
    private IDgGuaranteeLetterAssignUserService guaranteeLetterAssignUserService;

    @Autowired
    private DgGuaranteeAssignUserMapper dgGuaranteeAssignUserMapper;

    @Resource
    private IDgCopyForService dgCopyForService;

    /**
     * 列表
     */
    @Override
    public PageUtils list(Integer page, Integer size, String codeOrName, Integer status) {
        if (page == null || size == null || page < 1 || size < 1) {
            page = 1;
            size = 10;
        }
        String userId = this.getUserId();
        // 获取当前登录用户的角色id
        String roleIdString = RedisUtil.get("dq:userRole:" + userId);
        if (roleIdString == null || roleIdString.length() == 0) {
            ExceptionCast.cast(GuaranteeCode.ROLE_IS_NOT);
        }
        String[] roleIds = roleIdString.split(",");
        /*String userId = "1";
        String[] roleIds = {"72"};*/
        List<Integer> roleList = new ArrayList<>();
        for (String roleId : roleIds) {
            roleList.add(Integer.parseInt(roleId));
        }
        IPage<DgLoanCommitteeConsiderResponse> list = this.getBaseMapper().list(new Page(page, size),
                codeOrName, roleList, Integer.parseInt(userId),status);
        return new PageUtils(list);
    }

    /**
     * 审议
     */
    @Override
    public Boolean check(DgLoanCommitteeConsiderRequest dgLoanCommitteeConsiderRequest) {
        if (dgLoanCommitteeConsiderRequest == null || dgLoanCommitteeConsiderRequest.getId() == null
                || dgLoanCommitteeConsiderRequest.getStatus() == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        String userId = this.getUserId();
        // 同意
        if (dgLoanCommitteeConsiderRequest.getStatus() == 1) {
            if (dgLoanCommitteeConsiderRequest.getAdditionalItems() == null || dgLoanCommitteeConsiderRequest.getAdditionalItems().length() == 0
                    || dgLoanCommitteeConsiderRequest.getAgreedItems() == null || dgLoanCommitteeConsiderRequest.getAgreedItems().length() == 0
                    || dgLoanCommitteeConsiderRequest.getLoanMoney() == null || dgLoanCommitteeConsiderRequest.getLoanTern() == null
                    || dgLoanCommitteeConsiderRequest.getLoanTern().length() == 0 || dgLoanCommitteeConsiderRequest.getFileUrls() == null
                    || dgLoanCommitteeConsiderRequest.getFileUrls().length == 0) {
                ExceptionCast.cast(GuaranteeCode.LOAN_MESSAGE_NOT_NULL);
            }
            // 将文件数组拼接为文件字符串形式
            String fileUrl = dgComplianceInvestigationService.getFileUrl(dgLoanCommitteeConsiderRequest.getFileUrls());
            DgLoanCommitteeConsiderConsent consent = new DgLoanCommitteeConsiderConsent();
            BeanUtils.copyProperties(dgLoanCommitteeConsiderRequest, consent);
            // 将同意名单和不同意名单拼接为字符串格式
            consent.setAgreePerson(this.jointName(dgLoanCommitteeConsiderRequest.getAgreePersons()));
            consent.setDisagreePerson(this.jointName(dgLoanCommitteeConsiderRequest.getDisagreePersons()));
            consent.setFileUrl(fileUrl);
            Boolean result = this.getBaseMapper().check(consent, Integer.parseInt(userId), StatusCode.SP_REVIEWED, StatusCode.CZ_PROCESSED, new Date());
            // 判断该业务id下担保函数据是否存在
            Integer count = this.getBaseMapper().countGuaranteeLetterAssignUser(
                    this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                    PromptSuccess.FG_JL_ID);
            if (result) {
                // 更新贷审会的审核状态为已审核
                Boolean result2 = this.getBaseMapper().updateAuditStatusByBusinessId(
                        this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                        StatusCode.SP_REVIEWED);
                if (result2) {
                    Boolean result1;
                    if (count > 0) {
                        // 当前贷审会的数据存在,表示为担保函驳回数据,改变担保函的法规部经理的操作状态为待处理,审核状态为审核中
                        result1 = this.getBaseMapper().updateGuaranteeLetterAssignUser(
                                this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                                PromptSuccess.FG_JL_ID, StatusCode.SP_IN_REVIEW, StatusCode.CZ_ON_HAND,"");
                        // 此时有可能是多级驳回至贷审会,所以要以业务id更新担保函审核状态
                        this.getBaseMapper().updateGuaranteeLetterAuditStatusByBusinessId(
                                this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                                StatusCode.SP_IN_REVIEW);
                    } else {
                        // 当前贷审会的操作状态不为驳回,表示不是担保函驳回数据,在担保函表中插入一条数据
                        DgGuaranteeLetterAssignUser guaranteeLetterAssignUser = new DgGuaranteeLetterAssignUser();
                        guaranteeLetterAssignUser.setBusinessId(this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId())
                                .getBusinessId());
                        guaranteeLetterAssignUser.setCompanyId(this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId())
                                .getCompanyId());
                        result1 = guaranteeLetterAssignUserService.saveGuaranteeLetter(guaranteeLetterAssignUser);
                    }
                    if (result1) {
                        boolean b=false;
                        try {
                            b=dgCopyForService.copySendUser(this.getBaseMapper().getDgLoanCommitteeConsider(
                                    dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),7,dgLoanCommitteeConsiderRequest.getId());
                        } catch (IOException e) {
                            e.printStackTrace();
                        }

                        // 在业务流程表中插入一条审议成功的数据
                        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                        return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getBaseMapper().getDgLoanCommitteeConsider(
                                dgLoanCommitteeConsiderRequest.getId()).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                departmentId, null, PromptSuccess.DSH_PROCESS_ID,PromptSuccess.DSH_ID,"","","", StatusCode.SP_REVIEWED);
                    }
                }
            }
        }
        // 拒绝
        if (dgLoanCommitteeConsiderRequest.getStatus() == 3) {
            if (dgLoanCommitteeConsiderRequest.getRefuseOpinion() == null
                    || dgLoanCommitteeConsiderRequest.getRefuseOpinion().length() == 0) {
                ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
            }
            // 绑定该条数据的userId,表示该用户操作了该条数据
            Boolean result3 = this.getBaseMapper().updateLoanCommitteeConsiderUserIdById(dgLoanCommitteeConsiderRequest.getId(),
                    Integer.parseInt(userId));
            if (result3) {
                // 先改变所有的审核状态为已拒绝
                Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(
                        this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                        StatusCode.SP_REFUSE);
                if (result) {
                    // 再改变自己当前的操作状态为已处理
                    Boolean result1 = this.getBaseMapper().updateOperationStatusById(dgLoanCommitteeConsiderRequest.getId(),
                            StatusCode.CZ_PROCESSED, dgLoanCommitteeConsiderRequest.getRefuseOpinion());
                    if (result1) {
                        // 最后改变该条贷款信息的业务状态为已否决
                        Boolean result2 = dgComplianceInvestigationMapper.updateBusinessStatusByBusinessId(
                                this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                                StatusCode.YW_REJECTED);
                        if (result2) {
                            boolean b=false;
                            try {
                                b=dgCopyForService.copySendUser(this.getBaseMapper().getDgLoanCommitteeConsider(
                                        dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),7,dgLoanCommitteeConsiderRequest.getId());
                            } catch (IOException e) {
                                e.printStackTrace();
                            }

                            // 在业务流程表中插入一条拒绝数据(记得带上拒绝意见)
                            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                            return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getBaseMapper().getDgLoanCommitteeConsider(
                                    dgLoanCommitteeConsiderRequest.getId()).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                    departmentId, dgLoanCommitteeConsiderRequest.getRefuseOpinion(), PromptSuccess.DSH_PROCESS_ID,
                                    PromptSuccess.DSH_ID,"","","", StatusCode.SP_REFUSE);
                        }
                    }
                }
            }
        }
        // 驳回
        if (dgLoanCommitteeConsiderRequest.getStatus() == 2) {
            if (dgLoanCommitteeConsiderRequest.getRefuseOpinion() == null
                    || dgLoanCommitteeConsiderRequest.getRefuseOpinion().length() == 0) {
                ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
            }
            // 绑定该条数据的userId,表示该用户操作了该条数据
            Boolean result4 = this.getBaseMapper().updateLoanCommitteeConsiderUserIdById(dgLoanCommitteeConsiderRequest.getId(),
                    Integer.parseInt(userId));
            if (result4) {
                // 先改变贷审会的审核状态为驳回
                Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(
                        this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                        StatusCode.SP_REJECT);
                if (result) {
                    // 再改变自己的操作状态为已处理
                    Boolean result1 = this.getBaseMapper().updateOperationStatusById(dgLoanCommitteeConsiderRequest.getId(),
                            StatusCode.CZ_PROCESSED, dgLoanCommitteeConsiderRequest.getRefuseOpinion());
                    if (result1) {
                        // 改变担保部的审核状态为驳回
                        Boolean result2 = dgGuaranteeAssignUserMapper.updateAuditStatusByBusinessId(
                                this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                                StatusCode.SP_REJECT);
                        if (result2) {
                            // 修改AB角的操作状态为待处理
                            Boolean result3 = dgGuaranteeAssignUserMapper.updateOperationStatusById(
                                    this.getBaseMapper().getDgLoanCommitteeConsider(dgLoanCommitteeConsiderRequest.getId()).getBusinessId(),
                                    StatusCode.CZ_ON_HAND);
                            if (result3) {
                                // 在业务流程表中插入一条驳回成功的数据(记得带上驳回的意见)
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                                return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getBaseMapper().getDgLoanCommitteeConsider(
                                        dgLoanCommitteeConsiderRequest.getId()).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, dgLoanCommitteeConsiderRequest.getRefuseOpinion(), PromptSuccess.DSH_PROCESS_ID,
                                        PromptSuccess.DSH_ID,"","","", StatusCode.SP_REJECT);
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 获取工作会抽取的评委名单
     */
    @Override
    public List<String> getJudges(Integer id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        List<String> list = new ArrayList<>();
        String judges = this.getBaseMapper().getJudges(id);
        if (judges != null && judges.length() != 0) {
            String[] judge = judges.split(",");
            list.addAll(Arrays.asList(judge));
        }
        return list;
    }

    /**
     * 查看
     */
    @Override
    public DgLoanCommitteeConsiderInfoResponse query(Integer id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        DgLoanCommitteeConsider dgLoanCommitteeConsider = this.getBaseMapper().query(id);
        DgLoanCommitteeConsiderInfoResponse dgLoanCommitteeConsiderInfoResponse = new DgLoanCommitteeConsiderInfoResponse();
        if (dgLoanCommitteeConsider != null) {
            // 文件数组
            if (dgLoanCommitteeConsider.getFile() != null && dgLoanCommitteeConsider.getFile().length() != 0) {
                dgLoanCommitteeConsiderInfoResponse.setFileUrls(dgLoanCommitteeConsider.getFile().split(","));
            }
            // 同意名单数组
            if (dgLoanCommitteeConsider.getAgreePerson() != null && dgLoanCommitteeConsider.getAgreePerson().length() != 0) {
                dgLoanCommitteeConsiderInfoResponse.setAgreePersons(dgLoanCommitteeConsider.getAgreePerson().split(","));
            }
            // 不同意名单数组
            if (dgLoanCommitteeConsider.getDisagreePerson() != null && dgLoanCommitteeConsider.getDisagreePerson().length() != 0) {
                dgLoanCommitteeConsiderInfoResponse.setDisagreePersons(dgLoanCommitteeConsider.getDisagreePerson().split(","));
            }
            // 贷审会通过时间
            if (dgLoanCommitteeConsider.getPassingTime() != null) {
                dgLoanCommitteeConsiderInfoResponse.setTime(dgLoanCommitteeConsider.getPassingTime());
            }
            BeanUtils.copyProperties(dgLoanCommitteeConsider, dgLoanCommitteeConsiderInfoResponse);
        }
        return dgLoanCommitteeConsiderInfoResponse;
    }

    /**
     * 导出
     */
    @Override
    public Boolean excelExport(HttpServletResponse response, List<String> ids, Integer userId) {
/*        if (null == ids || ids.size() == 0) {
            ExceptionCast.cast(GuaranteeCode.EXCEL_ID_NOT_NULL);
        }*/
        List<Integer> idList = new ArrayList<>();
        for (String id : ids) {
            idList.add(Integer.parseInt(id));
        }
        List<DgLoanCommitteeConsiderResponse> list = this.getBaseMapper().excelList(idList,userId);
        try {
            ExcelUtil.writeExcelWithSheets(response, list, "贷审会数据一览表", "NO.1", new DgLoanCommitteeConsiderResponse())
                    .finish();
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
        }
        return false;
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        return userId;
    }

    /**
     * 将名称数组拼接为名字字符串,以逗号隔开
     */
    private String jointName(String[] names) {
        StringBuilder name = new StringBuilder();
        if (null != names && names.length != 0) {
            for (int i = 0; i < names.length; i++) {
                if (i == names.length - 1) {
                    name.append(names[i]);
                } else {
                    name.append(names[i]).append(",");
                }
            }
        }
        return name.toString();
    }
}
