package com.daqing.financial.guarantee.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.*;
import com.daqing.financial.guarantee.model.request.LoanNoticeQueryRequest;
import com.daqing.financial.guarantee.model.request.LoanNoticeUpdateRequest;
import com.daqing.financial.guarantee.model.response.BusinessApplicationListResponse;
import com.daqing.financial.guarantee.model.response.LegalVo;
import com.daqing.financial.guarantee.model.response.LoanNoticeDetailRes;
import com.daqing.financial.guarantee.model.response.LoanNoticeListResponse;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.service.IDgLoanNoticeService;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.guarantee.*;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;

/**
 * <p>
 * 放款通知 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Slf4j
@Service
public class DgLoanNoticeServiceImpl extends ServiceImpl<DgLoanNoticeMapper, DgLoanNotice> implements IDgLoanNoticeService {

    @Autowired
    private DgPaymentConfirmationConsiderMapper paymentConfirmationConsiderMapper;

    @Autowired
    private DgBusinessProcessStatusMapper businessProcessStatusMapper;

    @Autowired
    private DgApplyAmountInfoMapper applyAmountInfoMapper;

    @Autowired
    private DgBusinessCompanyMapper businessCompanyMapper;

    @Autowired
    private DgLoanCommitteeConsiderMapper loanCommitteeConsiderMapper;

    @Autowired
    private AlInsuranceListMapper alInsuranceListMapper;

    @Autowired
    private IDgCopyForService copyForService;

    @Autowired
    private HrmsFeignService hrmsFeignService;
    /**
     * 获取当前登录用户信息
     */
    private String getUserId(){
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:"+token);
    }

    @Override
    public PageUtils queryPage(LoanNoticeQueryRequest loanNoticeQueryRequest) {
        //取出条件
        Integer page = loanNoticeQueryRequest.getPage();
        Integer size = loanNoticeQueryRequest.getSize();
        Integer status = loanNoticeQueryRequest.getStatus();
        String customerNumberOrName = loanNoticeQueryRequest.getCustomerNumberOrName();

        //获取当前登录用户id
        Integer userId = Integer.parseInt(this.getUserId());

        //获取当前用户拥有的角色ids
        String roleIds = RedisUtil.get("dq:userRole:" + userId);
        IPage<LoanNoticeListResponse>  loanNoticeVO = null;
        if(roleIds != null) {
            String[] ids = roleIds.split(",");
            List<String> roleIdList = Arrays.asList(ids);
            //分页条件查询
            loanNoticeVO = baseMapper.pageByCondition(new Page(page,size),roleIdList,userId,status,customerNumberOrName);
        }
        return new PageUtils(loanNoticeVO);
    }

    @Override
    public void batchExport(HttpServletResponse response, String ids,Integer userId) throws IOException {
        List<String> idList= new ArrayList<>();
        if(ids!=null && ids !=""){
            String[] id = ids.split(",");
            idList = Arrays.asList(id);
        }
        List<LoanNoticeListResponse> excelDataList = baseMapper.selectBatchData(idList, userId);
        EasyExcelUtil.download(response,LoanNoticeListResponse.class,excelDataList,"放款通知列表","第一页");
    }

    @Override
    @Transactional
    public boolean updateLoanNotice(LoanNoticeUpdateRequest loanNoticeUpdateRequest) {
        //取出条件
        Integer businessId = loanNoticeUpdateRequest.getBusinessId();
        //获取当前登录用户id
        Integer currentUser = Integer.parseInt(this.getUserId());
        String auditOpinion = loanNoticeUpdateRequest.getAuditOpinion();
        //状态为2表示通过，为4表示驳回
        Integer status = loanNoticeUpdateRequest.getStatus();

        //提交操作，更新数据
        if (StatusCode.SP_REVIEWED.equals(status)){
            QueryWrapper<DgLoanNotice> loanNoticeQueryWrapper = new QueryWrapper<>();
            loanNoticeQueryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
            DgLoanNotice loanNotice = baseMapper.selectOne(loanNoticeQueryWrapper);
            loanNotice.setAuditOpinion(auditOpinion);
            loanNotice.setConsiderId(currentUser);
            loanNotice.setStatus(StatusCode.SP_REVIEWED);
            loanNotice.setOperatingStatus(StatusCode.CZ_PROCESSED);
            int update = baseMapper.updateById(loanNotice);
            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion,10,PromptSuccess.FG_JL_ID,"","","",status);
            //更新成功更新业务流程已完成
            if (update > 0){
                QueryWrapper<DgBusinessProcessStatus> businessProcessStatusQueryWrapper = new QueryWrapper<>();
                businessProcessStatusQueryWrapper.eq("business_id",businessId);
                DgBusinessProcessStatus dgBusinessProcessStatus = businessProcessStatusMapper.selectOne(businessProcessStatusQueryWrapper);
                dgBusinessProcessStatus.setBusinessStatus(StatusCode.YW_COMPLETED);
                businessProcessStatusMapper.updateById(dgBusinessProcessStatus);
                //查询保后管理的相关数据
                DgApplyAmountInfo applyAmountInfo = applyAmountInfoMapper.selectById(businessId);
                String bank = applyAmountInfo.getBank();
                String businessCode = applyAmountInfo.getBusinessCode();
                String businessType = applyAmountInfo.getBusinessType();
                Date createTime = applyAmountInfo.getCreateTime();
                String amountWide = applyAmountInfo.getAmountWide();
                Integer presenterId = applyAmountInfo.getPresenterId();
                DgBusinessCompany businessCompany = businessCompanyMapper.selectOne(new QueryWrapper<DgBusinessCompany>().eq("business_id", businessId));
                String phone = businessCompany.getPhone();
                String name = businessCompany.getName();
                DgLoanCommitteeConsider loanCommitteeConsider = loanCommitteeConsiderMapper.selectOne(new QueryWrapper<DgLoanCommitteeConsider>().eq("business_id", businessId));
                BigDecimal loanMoney = loanCommitteeConsider.getLoanMoney();
                String loanTern = loanCommitteeConsider.getLoanTern();
                //添加数据到保后管理
                AlInsuranceList alInsuranceList = new AlInsuranceList();
                alInsuranceList.setBank(bank);
                alInsuranceList.setBusinessCode(businessCode);
                alInsuranceList.setBusinessType(businessType);
                alInsuranceList.setApplyTime(createTime);
                alInsuranceList.setPhone(phone);
                alInsuranceList.setCustomerName(name);
                alInsuranceList.setLegalName(businessCompany.getLegalName());
                //转换单位
                BigDecimal quota = new BigDecimal("10000");
                alInsuranceList.setGuaranteeAmount(loanMoney.multiply(quota));
                alInsuranceList.setGuaranteeTime(loanTern);
                alInsuranceList.setAmountWide(amountWide);
                alInsuranceList.setPaymentStatus(1);
                //无需所属部门
                /*ArrayList<Integer> arr = new ArrayList<>();
                arr.add(presenterId);
                //根据提单人id查询其部门名称
                ResponseResult responseResult = hrmsFeignService.getAccountAndDeptNameById(arr);
                List<LinkedHashMap> employeeMessage = null;
                if(responseResult.getData() != null){
                    employeeMessage = (List<LinkedHashMap>) responseResult.getData();
                }
                if(employeeMessage!= null){
                    for(LinkedHashMap res : employeeMessage){
                        if(presenterId.equals(res.get("id"))){//如果提单人id相同情况下，就往对象里面赋值
                            alInsuranceList.setDepartment(JSONObject.toJSONString(res.get("deptName")).replace("\"",""));
                        }
                    }
                }*/
                //添加抄送数据
                try {
                    copyForService.copySendUser(businessId,10,null);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                int insert = alInsuranceListMapper.insert(alInsuranceList);
                //添加成功，返回操作结果
                log.info(insert > 0 ? "业务流程已完成,进入保后管理" : "业务流程更新失败");
                return insert > 0;
            }
        }

        //驳回操作，更新数据和法规部经理和财务部经理数据返回上一流程
        if (StatusCode.SP_REJECT.equals(status)){
            QueryWrapper<DgLoanNotice> loanNoticeQueryWrapper = new QueryWrapper<>();
            loanNoticeQueryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
            DgLoanNotice loanNotice = baseMapper.selectOne(loanNoticeQueryWrapper);
            loanNotice.setAuditOpinion(auditOpinion);
            loanNotice.setConsiderId(currentUser);
            loanNotice.setStatus(StatusCode.SP_REJECT);
            loanNotice.setOperatingStatus(StatusCode.CZ_PROCESSED);
            int update = baseMapper.updateById(loanNotice);
            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion,10,PromptSuccess.FG_JL_ID,"","","",status);
            //更新成功，更新回款确认的法规部经理和财务部经理数据
            if (update > 0){
                QueryWrapper<DgPaymentConfirmationConsider> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
                DgPaymentConfirmationConsider paymentConfirmationConsider = paymentConfirmationConsiderMapper.selectOne(queryWrapper);
                paymentConfirmationConsider.setStatus(StatusCode.SP_REJECT);
                paymentConfirmationConsider.setOperatingStatus(StatusCode.CZ_ON_HAND);
                paymentConfirmationConsider.setAuditOpinion("");
                paymentConfirmationConsiderMapper.updateById(paymentConfirmationConsider);

                QueryWrapper<DgPaymentConfirmationConsider> queryWrapperCWB = new QueryWrapper<>();
                queryWrapperCWB.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
                DgPaymentConfirmationConsider paymentConfirmationConsiderCWB = paymentConfirmationConsiderMapper.selectOne(queryWrapperCWB);
                paymentConfirmationConsiderCWB.setStatus(StatusCode.SP_REJECT);
                int updateById = paymentConfirmationConsiderMapper.updateById(paymentConfirmationConsiderCWB);
                //更新成功，返回操作结果
                log.info(updateById > 0 ? "驳回更新财务部经理成功" : "驳回更新财务部经理失败");
                return updateById > 0;
            }
        }
        //一个判断都不走，直接返回false
        log.info("《《《《《数据更新失败，请确保数据完整》》》》》");
        return false;
    }

    @Override
    public LoanNoticeListResponse selectLoanNotice(String businessId, String roleId) {
        return baseMapper.selectLoanNotice(businessId,roleId);
    }

    @Override
    public LoanNoticeDetailRes loanNoticeDetail(Integer id) {
        return baseMapper.selectLoanNoticeDetail(id);
    }
}
