package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.feign.CrmsFeignService;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.*;
import com.daqing.financial.guarantee.model.request.ApprovalMessageRequest;
import com.daqing.financial.guarantee.model.request.InvestigateMessageRequest;
import com.daqing.financial.guarantee.model.request.MessageRequest;
import com.daqing.financial.guarantee.model.response.DgApplyAmountInfoResponse;
import com.daqing.financial.guarantee.model.response.DgEnclosureInfoResponse;
import com.daqing.financial.guarantee.model.response.MessageDetailResponse;
import com.daqing.financial.guarantee.model.response.MessageInvestigationListResponse;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.service.IDgMessageInvestigationService;
import com.daqing.financial.guarantee.util.ArraysUtil;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.crms.request.DgApplyAmountInfoRequest;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.*;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * <p>
 * 信息调查 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Service
public class DgMessageInvestigationServiceImpl extends ServiceImpl<DgMessageInvestigationMapper, DgMessageInvestigation> implements IDgMessageInvestigationService {

    @Resource
    private DgMessageInvestigationMapper dgMessageInvestigationMapper;

    @Resource
    private HrmsFeignService hrmsFeignService;

    @Resource
    private DgApplyAmountInfoMapper dgApplyAmountInfoMapper;

    @Resource
    private CrmsFeignService crmsFeignService;

    @Resource
    private DgEnclosureInfoMapper dgEnclosureInfoMapper;

    @Resource
    private DgApplyAmountInfoController dgApplyAmountInfoController;
    @Resource
    private DgGuaranteeAssignUserMapper dgGuaranteeAssignUserMapper;
    @Resource
    private DgAssetsInvestigationMapper dgAssetsInvestigationMapper;
    @Resource
    private DgComplianceInvestigationMapper dgComplianceInvestigationMapper;
    @Resource
    private DgBusinessProcessStatusMapper dgBusinessProcessStatusMapper;
    @Resource
    private DgBusinessCompanyMapper dgBusinessCompanyMapper;
    @Resource
    private IDgCopyForService dgCopyForService;

    @Override
    public PageUtils queryPage(Integer page, Integer size, String customerNumberOrName, List<String> roleIdlist, String userId, Integer status) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<MessageInvestigationListResponse> positionVO = this.getBaseMapper().pageByCondition(new Page(page, size),customerNumberOrName,roleIdlist,userId,status);

        return new PageUtils(positionVO);
    }

    @Transactional
    @Override
    public Boolean assignCorners(MessageRequest messageRequest) {
        //查询empId在担保部调查中是否被指派
        Integer count1 = dgGuaranteeAssignUserMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", messageRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", messageRequest.getEmpId()));

        Integer count = dgGuaranteeAssignUserMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>()
                .eq("business_id", messageRequest.getBusinessId())
                .eq("type", 2).eq("emp_id", messageRequest.getEmpId()));
        if(count1 >0 || count >0){//已被指派情况下直接返回
            ExceptionCast.cast(HrmsCode.EMP_TO_BE_CHOOSE);
        }

        //查询empId在资产部调查中是否被指派
        Integer count2 = dgAssetsInvestigationMapper.selectCount(new QueryWrapper<DgAssetsInvestigation>()
                .eq("business_id", messageRequest.getBusinessId())
                .eq("type", 1).eq("emp_id", messageRequest.getEmpId()));
        if(count2 >0 ){//已被指派情况下直接返回
            ExceptionCast.cast(HrmsCode.EMP_TO_BE_CHOOSE);
        }

        //根据主键id修改当前记录状态为审核中，操作状态为已处理,设置审批人id为当前用户id
        //获取当前登录用户userId
        String userId = DgApplyAmountInfoController.getUserId();

        DgMessageInvestigation dgMessageInvestigation = new DgMessageInvestigation();
        dgMessageInvestigation.setStatus(StatusCode.SP_IN_REVIEW);//状态设置为审核中
        dgMessageInvestigation.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
        dgMessageInvestigation.setEmpId(Integer.parseInt(userId));
        dgMessageInvestigation.setId(messageRequest.getId());
        dgMessageInvestigationMapper.updateById(dgMessageInvestigation);

        //新增信息部调查员记录到信息部记录表
        DgMessageInvestigation messageInvestigation = new DgMessageInvestigation();
        messageInvestigation.setBusinessId(messageRequest.getBusinessId());//业务id
        messageInvestigation.setCompanyId(messageRequest.getCompanyId());//企业id
        messageInvestigation.setEmpId(messageRequest.getEmpId());//信息部调查员id
        messageInvestigation.setRoleId(PromptSuccess.XX_ZY_ID);//角色设置为信息部专员
        messageInvestigation.setType(1);//专员操作
        messageInvestigation.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
        messageInvestigation.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
        dgMessageInvestigationMapper.insert(messageInvestigation);

        //查询用户角色表是否已绑定，如果没有绑定，则去往绑定页面
        ResponseResult i = hrmsFeignService.selectUserRole(messageRequest.getEmpId().longValue(),PromptSuccess.XX_ZY_ID.longValue());
        int a = (int) i.getData();
        if(a==0) {
            //绑定角色关系
            hrmsFeignService.insertUserRole(messageRequest.getEmpId().longValue(), PromptSuccess.XX_ZY_ID.longValue());
        }

        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),messageRequest.getBusinessId(),
                account, departmentId, "", 4,
                PromptSuccess.XX_JL_ID,"","","",StatusCode.SP_REVIEWED);

        return true;
    }

    @Transactional
    @Override
    public MessageDetailResponse messageDetail(Integer id) {
        //根据主键id查询业务id
        DgMessageInvestigation dgMessageInvestigation = this.getBaseMapper().selectById(id);

        //根据业务id查询业务申请详细信息
        DgApplyAmountInfoResponse dgApplyAmountInfo = dgApplyAmountInfoMapper.selectApplyAmountInfoById(dgMessageInvestigation.getBusinessId());
        String measures=dgApplyAmountInfo.getEnGuaranteeMeasures();
        if(measures != null){
            if(measures.length()>0){
                String a = measures.substring(1,measures.length()-1);
                String[] arry =a.split(",");
                List<String> demoList = Arrays.asList(arry);
                dgApplyAmountInfo.setEnGuaranteeMeasure(demoList);
            }
        }

        MessageDetailResponse messageDetailResponse= new MessageDetailResponse();
        //查询信息部调查专员调查的附件信息
        DgMessageInvestigation messageInvestigation = this.baseMapper.selectOne(new QueryWrapper<DgMessageInvestigation>()
                .eq("business_id", dgMessageInvestigation.getBusinessId())
                .eq("type", 1));
        if(messageInvestigation != null){
            String files=messageInvestigation.getFile();
            if(files!=null){
                String a = files.substring(1,files.length()-1);
                String[] arry =a.split(",");
                List<String> demoList = Arrays.asList(arry);
                messageDetailResponse.setMessageFileList(demoList);
            }
        }

        DgApplyAmountInfoRequest applyAmountInfo = new DgApplyAmountInfoRequest();
        applyAmountInfo.setCompanyId(dgApplyAmountInfo.getCompanyId());
        //根据企业id查询企业详情
/*        ResponseResult result = crmsFeignService.queryCustomerInfoById(applyAmountInfo);

        LinkedHashMap linkedList = (LinkedHashMap)result.getData();*/

        //根据业务id查询备份企业信息
        DgBusinessCompany dgBusinessCompany = dgBusinessCompanyMapper.selectOne(new QueryWrapper<DgBusinessCompany>()
                .eq("business_id", dgMessageInvestigation.getBusinessId()));

        //查询附件详细，附件关联业务主键id
        DgEnclosureInfoResponse dgEnclosureInfo = dgEnclosureInfoMapper.selectByCompanyId(dgMessageInvestigation.getBusinessId());

        List<String>list = new ArrayList<>();
        list.add(dgEnclosureInfo.getAccountingFirm());
        list.add(dgEnclosureInfo.getAssessmentReport());
        list.add(dgEnclosureInfo.getBusinessLicense());
        list.add(dgEnclosureInfo.getCertificateAuthorization());
        list.add(dgEnclosureInfo.getCompanyConstitution());
        list.add(dgEnclosureInfo.getCompanyCredit());
        list.add(dgEnclosureInfo.getConversationSummary());
        list.add(dgEnclosureInfo.getCreditReport());
        list.add(dgEnclosureInfo.getInspectionPhotos());
        list.add(dgEnclosureInfo.getLegalCardCopy());
        list.add(dgEnclosureInfo.getLegalCopy());
        list.add(dgEnclosureInfo.getTaxCertificate());
        list.add(dgEnclosureInfo.getMeetingMinutes());
        list.add(dgEnclosureInfo.getMarriageCopy());

        List arr1List2 = new ArrayList<>();

        for(int i=0;i<list.size();i++){
            List<String> arr1List;
            if(list.get(i)!=null){
                String a1 = list.get(i).substring(1,list.get(i).length()-1);
                String[] arr1 =a1.split(",");
                arr1List = Arrays.asList(arr1);
                arr1List2.add(i,arr1List);
            }else{
                arr1List2.add(i,null);
            }
        }
/*        if(arr1List2.size()>0){
            if(!arr1List2.get(0).toString().equals("[]"))*/
                dgEnclosureInfo.setAccountingFirmList((List<String>) arr1List2.get(0));
                dgEnclosureInfo.setAssessmentReportList((List<String>) arr1List2.get(1));
                dgEnclosureInfo.setBusinessLicenseList((List<String>) arr1List2.get(2));
                dgEnclosureInfo.setCertificateAuthorizationList((List<String>) arr1List2.get(3));
                dgEnclosureInfo.setCompanyConstitutionList((List<String>) arr1List2.get(4));
                dgEnclosureInfo.setCompanyCreditList((List<String>) arr1List2.get(5));
                dgEnclosureInfo.setConversationSummaryList((List<String>) arr1List2.get(6));
                dgEnclosureInfo.setCreditReportList((List<String>) arr1List2.get(7));
                dgEnclosureInfo.setInspectionPhotosList((List<String>) arr1List2.get(8));
                dgEnclosureInfo.setLegalCardCopyList((List<String>) arr1List2.get(9));
                dgEnclosureInfo.setLegalCopyList((List<String>) arr1List2.get(10));
                dgEnclosureInfo.setTaxCertificateList((List<String>) arr1List2.get(11));
                dgEnclosureInfo.setMeetingMinutesList((List<String>) arr1List2.get(12));
                dgEnclosureInfo.setMarriageCopyList((List<String>) arr1List2.get(13));
//        }

        //查询该业务id下对应的C角
        DgMessageInvestigation messageInvestigate = this.baseMapper.selectOne(new QueryWrapper<DgMessageInvestigation>()
                .eq("business_id", dgMessageInvestigation.getBusinessId()).eq("type", 1));
        if(messageInvestigate !=null){
            Integer empCId = messageInvestigate.getEmpId();
            Integer[] arr = new Integer[]{
                    empCId
            };
            if(arr.length>0){
                ResponseResult<List<EmployeeVO>> listResponseResult = hrmsFeignService.listEmpNameByUserId(arr);
                messageDetailResponse.setCName(listResponseResult.getData().get(0).getName());
            }
        }

        messageDetailResponse.setDgApplyAmountInfo(dgApplyAmountInfo);
        messageDetailResponse.setDgEnclosureInfo(dgEnclosureInfo);
        messageDetailResponse.setDgBusinessCompany(dgBusinessCompany);
        //messageDetailResponse.setLinkedHashMap(linkedList);
        messageDetailResponse.setDgMessageInvestigation(dgMessageInvestigation);
        return messageDetailResponse;
    }

    @Transactional
    @Override
    public int investigateMessage(InvestigateMessageRequest investigateMessageRequest) {
        //根据主键id查询业务id
        DgMessageInvestigation messageInvestigation = this.getBaseMapper().selectById(investigateMessageRequest.getId());

        //AB角调查，修改审核意见以及状态
        DgMessageInvestigation dgMessageInvestigation = new DgMessageInvestigation();
        dgMessageInvestigation.setId(investigateMessageRequest.getId());//主键id
        dgMessageInvestigation.setStatus(investigateMessageRequest.getStatus());//状态
        dgMessageInvestigation.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
        dgMessageInvestigation.setRemark(investigateMessageRequest.getRemark());//审核意见
        dgMessageInvestigation.setFile(ArraysUtil.toString(investigateMessageRequest.getFile()));
        this.baseMapper.updateById(dgMessageInvestigation);

        //如果是信息部经理驳回情况
        if(messageInvestigation.getOperatingStatus().equals(StatusCode.CZ_ON_HAND) && messageInvestigation.getStatus().equals(StatusCode.SP_REJECT)){
            //修改所有审批状态为审核中
            DgMessageInvestigation investigation1 = new DgMessageInvestigation();
            investigation1.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            this.baseMapper.update(investigation1,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()));

            //修改信息部调查员的审批状态为审核中，操作状态为已处理
            DgMessageInvestigation investigation = new DgMessageInvestigation();
            investigation.setOperatingStatus(StatusCode.CZ_PROCESSED);//已处理
            investigation.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            this.baseMapper.update(investigation,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()).eq("type",1));

            //修改信息部经理的审核状态为审核中，操作状态为待处理
            DgMessageInvestigation dgInvestigation = new DgMessageInvestigation();
            dgInvestigation.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理
            dgInvestigation.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            this.baseMapper.update(dgInvestigation,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()).eq("type",2));

        }else if(investigateMessageRequest.getStatus().equals(StatusCode.SP_IN_REVIEW)){
            //判断信息部专员是否调查通过，如果通过，修改信息部经理操作状态为待处理
            DgMessageInvestigation messageInvestigation2 = new DgMessageInvestigation();
            messageInvestigation2.setOperatingStatus(StatusCode.CZ_ON_HAND);
            this.baseMapper.update(messageInvestigation2,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId())
                    .eq("type",2));
        }

        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),messageInvestigation.getBusinessId(),
                account,departmentId,investigateMessageRequest.getRemark(),4,
                PromptSuccess.XX_ZY_ID,"","","",investigateMessageRequest.getStatus());
        return 1;
    }

    @Transactional
    @Override
    public int approvalMessage(ApprovalMessageRequest approvalMessageRequest) {

        //根据主键id查询信息部记录
        DgMessageInvestigation messageInvestigation = this.baseMapper.selectById(approvalMessageRequest.getId());

        DgMessageInvestigation dgMessageInvestigation = new DgMessageInvestigation();
        dgMessageInvestigation.setId(approvalMessageRequest.getId());
        dgMessageInvestigation.setEmpId(Integer.parseInt(DgApplyAmountInfoController.getUserId()));
        if(messageInvestigation.getType()==2 && approvalMessageRequest.getStatus()==2){
            //区分经理，领导审核，此处为经理审核
            dgMessageInvestigation.setStatus(StatusCode.SP_IN_REVIEW);//审核中
        }else{
            dgMessageInvestigation.setStatus(approvalMessageRequest.getStatus());//状态
        }
        dgMessageInvestigation.setOperatingStatus(StatusCode.CZ_PROCESSED);//操作状态设置为已处理
        dgMessageInvestigation.setRemark(approvalMessageRequest.getRemark());//审核意见
        //根据主键id修改信息部记录
        /*this.baseMapper.updateById(dgMessageInvestigation);*/
        //插入或更新的字段有 空字符串 或者 null
        this.baseMapper.update(
                null,
                Wrappers.<DgMessageInvestigation>lambdaUpdate()
                        .set(DgMessageInvestigation::getEmpId, Integer.parseInt(DgApplyAmountInfoController.getUserId()))
                        .set(DgMessageInvestigation::getStatus, dgMessageInvestigation.getStatus())//approvalMessageRequest.getStatus()
                        .set(DgMessageInvestigation::getOperatingStatus, StatusCode.CZ_PROCESSED)
                        .set(DgMessageInvestigation::getRemark, approvalMessageRequest.getRemark())
                        .eq(DgMessageInvestigation::getId, approvalMessageRequest.getId())
        );

        DgMessageInvestigation dgMessage = new DgMessageInvestigation();
        dgMessage.setStatus(StatusCode.SP_REJECT);//驳回
        dgMessage.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理

        if(approvalMessageRequest.getStatus().equals(StatusCode.SP_REJECT)){
            //驳回情况
            //修改信息部调查下该业务的所有审批状态为驳回
            DgMessageInvestigation message = new DgMessageInvestigation();
            message.setStatus(StatusCode.SP_REJECT);//驳回
            this.baseMapper.update(message,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()));
        }

        //如果数据为驳回状态，审核通过，则处理信息部调查下该业务的审批状态为审核中，设置信息部领导的操作状态为待处理
        if(messageInvestigation.getStatus().equals(StatusCode.SP_REJECT) && approvalMessageRequest.getStatus().equals(StatusCode.SP_IN_REVIEW) && messageInvestigation.getType()==2){
            DgMessageInvestigation message = new DgMessageInvestigation();
            message.setStatus(StatusCode.SP_IN_REVIEW);//审核中
            this.baseMapper.update(message,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()));

            DgMessageInvestigation message2 = new DgMessageInvestigation();
            message2.setOperatingStatus(StatusCode.CZ_ON_HAND);//待处理
            this.baseMapper.update(message2,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()).eq("type",3 ));
        }else if(approvalMessageRequest.getStatus().equals(StatusCode.SP_REFUSE)){//如果为拒绝，业务状态则为已否决
            DgMessageInvestigation user = new DgMessageInvestigation();
            user.setStatus(StatusCode.SP_REFUSE);//审批状态为拒绝
            this.baseMapper.update(user, new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id", messageInvestigation.getBusinessId()));

            DgBusinessProcessStatus processStatus = new DgBusinessProcessStatus();
            processStatus.setBusinessId(messageInvestigation.getBusinessId());
            processStatus.setBusinessStatus(StatusCode.YW_REJECTED);//业务状态设置为已否决
            dgBusinessProcessStatusMapper.update(processStatus,new QueryWrapper<DgBusinessProcessStatus>()
                    .eq("business_id",processStatus.getBusinessId()));
        }else if(approvalMessageRequest.getStatus().equals(StatusCode.SP_REJECT) && messageInvestigation.getType()==2){
            //信息部经理驳回情况
            //信息部专员审批状态设置为驳回，操作状态为待处理
            this.baseMapper.update(dgMessage,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()).eq("type",1));
        }else if(approvalMessageRequest.getStatus().equals(StatusCode.SP_REJECT) && messageInvestigation.getType()==3){
            //信息部领导驳回情况
            //信息部经理审批状态设置为驳回，操作状态为待处理
            this.baseMapper.update(dgMessage,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()).eq("type",2));
        }else if((approvalMessageRequest.getStatus().equals(StatusCode.SP_IN_REVIEW)||approvalMessageRequest.getStatus().equals(StatusCode.SP_REVIEWED)) && dgMessageInvestigation.getOperatingStatus()==StatusCode.CZ_PROCESSED && messageInvestigation.getType()==2){//信息部经理审核通过
            //根据主键id查询业务id
            DgMessageInvestigation messageInvestigation2 = this.getBaseMapper().selectById(approvalMessageRequest.getId());

            //如果逐级驳回，且资产部经理审核通过，首先查看资产领导是否有数据，如果有数据，则修改不新增
            Integer count = this.baseMapper.selectCount(new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id", messageInvestigation2.getBusinessId()).eq("type", 3));

            DgMessageInvestigation assign = new DgMessageInvestigation();
            assign.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
            //修改该业务下所有审批状态为审核中
            this.baseMapper.update(assign,new QueryWrapper<DgMessageInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId()));

            if(count>0){//资产领导有数据，则直接修改领导状态
                DgMessageInvestigation assignUser = new DgMessageInvestigation();
                assignUser.setStatus(StatusCode.SP_IN_REVIEW);//审批状态为审核中
                assignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态为待处理
                this.baseMapper.update(assignUser,new QueryWrapper<DgMessageInvestigation>()
                        .eq("business_id",messageInvestigation2.getBusinessId()).eq("type",3));
            }else {
                //如果信息部经理审核通过，则往信息部分管领导处加一条记录，由分管领导审核
                DgMessageInvestigation message = new DgMessageInvestigation();
                message.setBusinessId(messageInvestigation2.getBusinessId());
                message.setCompanyId(messageInvestigation2.getCompanyId());
                message.setRoleId(PromptSuccess.XX_LD_ID);//担保部分管领导
                message.setType(3);//分管领导审核
                message.setStatus(StatusCode.SP_IN_REVIEW);//审批状态设置为审核中
                message.setOperatingStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                this.baseMapper.insert(message);
            }
        }else if(approvalMessageRequest.getStatus().equals(StatusCode.SP_REVIEWED) && dgMessageInvestigation.getOperatingStatus()==StatusCode.CZ_PROCESSED && messageInvestigation.getType()==3) {//信息部领导审核通过
            //如果信息部分管领导审核通过，同时判断资产部分管领导审核以及信息部分管领导审核是否通过，如果通过就往合规调查中插入一条数据
            //如果信息部领导审核通过，则该子业务流程下的所有审批状态都为已审核状态
            DgMessageInvestigation investigation = new DgMessageInvestigation();
            investigation.setStatus(StatusCode.SP_REVIEWED);//已审核
            this.baseMapper.update(investigation,new QueryWrapper<DgMessageInvestigation>().eq("business_id",messageInvestigation.getBusinessId()));
            //查看担保部分管领导是否审核通过
            int a=dgGuaranteeAssignUserMapper.selectCount(new QueryWrapper<DgGuaranteeAssignUser>()
                    .eq("business_id",messageInvestigation.getBusinessId())
                    .eq("type",4).eq("status",StatusCode.SP_REVIEWED).eq("operating_status",StatusCode.CZ_PROCESSED));

            //查看资产部分管领导是否审核通过
            int b=dgAssetsInvestigationMapper.selectCount(new QueryWrapper<DgAssetsInvestigation>()
                    .eq("business_id",messageInvestigation.getBusinessId())
                    .eq("type",3).eq("status",StatusCode.SP_REVIEWED).eq("operating_status",StatusCode.CZ_PROCESSED));
            int sum = a+b;
            if(sum==2){
                //新增记录到合规调查
                DgComplianceInvestigation compliance =new DgComplianceInvestigation();
                compliance.setCompanyId(messageInvestigation.getCompanyId());//企业id
                compliance.setRoleId(PromptSuccess.FG_JL_ID);//角色id
                compliance.setBusinessId(messageInvestigation.getBusinessId());//业务id
                compliance.setOperationStatus(StatusCode.CZ_ON_HAND);//操作状态设置为待处理
                compliance.setAuditStatus(StatusCode.SP_TO_BE_ASSIGNED);//审核状态设置为待指派
                dgComplianceInvestigationMapper.insert(compliance);
            }
        }
        Integer type = messageInvestigation.getType();
        String deptName;
        Integer roleId;
        if(type == 2){
            deptName = "信息部经理";
            roleId = PromptSuccess.XX_JL_ID;
        }else{
            deptName = "信息部分管领导";
            roleId = PromptSuccess.XX_LD_ID;
        }
        //同时往审核流程进程表新增一条记录
        String account = DgApplyAmountInfoController.getAccount();
        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
        AuditProcessUtil.save(Integer.parseInt(this.getUserId()),messageInvestigation.getBusinessId(),
                account,departmentId,approvalMessageRequest.getRemark(),4,roleId,
                "","","",approvalMessageRequest.getStatus());

        boolean b=false;
        //根据业务为信息部分管领导处理以及是否驳回决定是否抄送
        if(roleId.equals(PromptSuccess.XX_LD_ID) && !approvalMessageRequest.getStatus().equals(StatusCode.SP_REJECT)){//领导，驳回
            try {
                b = dgCopyForService.copySendUser(messageInvestigation.getBusinessId(), 4, approvalMessageRequest.getId());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return 1;
    }

    @Override
    public Boolean excelExport(List<String> ids,Integer userId, HttpServletResponse response) {
        //根据ids查询业务申请列表信息
        List<MessageInvestigationListResponse>messageList = this.baseMapper.selectListByIds(ids,userId);
        System.out.println("applyList==========================="+messageList);
        try {
            ExcelUtil.writeExcelWithSheets(response, messageList, "信息调查一览表", "messageInvestigation", new MessageInvestigationListResponse())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        return userId;
    }
}
