package com.daqing.financial.guarantee.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.mapper.DgGuaranteeLetterAssignUserMapper;
import com.daqing.financial.guarantee.mapper.DgLoanNoticeMapper;
import com.daqing.financial.guarantee.mapper.DgPaymentConfirmationConsiderMapper;
import com.daqing.financial.guarantee.model.request.PaymentConfirmationQueryRequest;
import com.daqing.financial.guarantee.model.request.PaymentConfirmationUpdateRequest;
import com.daqing.financial.guarantee.model.response.PaymentConfirmationDetailRes;
import com.daqing.financial.guarantee.model.response.PaymentConfirmationListResponse;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.service.IDgPaymentConfirmationConsiderService;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.guarantee.DgGuaranteeLetterAssignUser;
import com.daqing.framework.domain.guarantee.DgLoanNotice;
import com.daqing.framework.domain.guarantee.DgPaymentConfirmationConsider;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 回款确认 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Slf4j
@Service
public class DgPaymentConfirmationConsiderServiceImpl extends ServiceImpl<DgPaymentConfirmationConsiderMapper, DgPaymentConfirmationConsider> implements IDgPaymentConfirmationConsiderService {

    @Autowired
    private DgLoanNoticeMapper dgLoanNoticeMapper;

    @Autowired
    private DgGuaranteeLetterAssignUserMapper guaranteeLetterAssignUserMapper;

    @Autowired
    private IDgCopyForService copyForService;

    /**
     * 获取当前登录用户信息
     */
    private String getUserId(){
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:"+token);
    }

    @Override
    public PageUtils queryPage(PaymentConfirmationQueryRequest paymentConfirmationQueryRequest) {
        //取出条件
        Integer page = paymentConfirmationQueryRequest.getPage();
        Integer size = paymentConfirmationQueryRequest.getSize();
        Integer status = paymentConfirmationQueryRequest.getStatus();
        String customerNumberOrName = paymentConfirmationQueryRequest.getCustomerNumberOrName();

        //获取当前登录用户id
        Integer userId = Integer.parseInt(this.getUserId());

        //获取当前用户拥有的角色ids
        String roleIds = RedisUtil.get("dq:userRole:" + userId);
        IPage<PaymentConfirmationListResponse> paymentConfirmationVO = null;
        if(roleIds != null) {
            String[] ids = roleIds.split(",");
            List<String> roleIdList = Arrays.asList(ids);
            //分页条件查询
            paymentConfirmationVO = baseMapper.pageByCondition(new Page(page,size),roleIdList,userId,status,customerNumberOrName);
            List<PaymentConfirmationListResponse> records = paymentConfirmationVO.getRecords();
            records.forEach(paymentConfirmationListResponse -> {
                //角色为法规部经理则设置值
                if (paymentConfirmationListResponse.getRoleId().equals(PromptSuccess.FG_JL_ID)){
                    paymentConfirmationListResponse.setIsFgJl(1);
                }
            });
        }
        return new PageUtils(paymentConfirmationVO);
    }

    @Override
    public void batchExport(HttpServletResponse response, String ids,Integer userId) throws IOException {
        List<String> idList= new ArrayList<>();
        if(ids!=null && ids !=""){
            String[] id = ids.split(",");
            idList = Arrays.asList(id);
        }
        List<PaymentConfirmationListResponse> excelDataList = baseMapper.selectBatchData(idList,userId);
        EasyExcelUtil.download(response,PaymentConfirmationListResponse.class,excelDataList,"财务确认列表","第一页");
    }

    @Override
    public PaymentConfirmationDetailRes paymentConfirmationDetail(Integer id) {
        return baseMapper.selectPaymentConfirmationDetail(id);
    }

    /**
     * 财务部经理操作
     */
    private boolean financeDepartmentOperating(Integer businessId,Integer currentUser,String auditOpinion,Integer status,String imgFile){
        //回款确认无驳回操作
        //更新之前,获取此数据的审批状态，如果审批状态为驳回，则更新法规部经理数据，如果审批状态为审批中，则添加法规部经理数据—_—
        QueryWrapper<DgPaymentConfirmationConsider> queryWrapperTemp = new QueryWrapper<>();
        queryWrapperTemp.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
        DgPaymentConfirmationConsider paymentConfirmationConsiderTemp = baseMapper.selectOne(queryWrapperTemp);
        Integer approveStatus = paymentConfirmationConsiderTemp.getStatus();

        //之前状态为审批中，同意则添加回款确认数据给法规部经理
        if (StatusCode.SP_REVIEWED.equals(status) && StatusCode.SP_IN_REVIEW.equals(approveStatus)){
            QueryWrapper<DgPaymentConfirmationConsider> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
            DgPaymentConfirmationConsider paymentConfirmationConsider = baseMapper.selectOne(queryWrapper);
            paymentConfirmationConsider.setAuditOpinion(auditOpinion);
            paymentConfirmationConsider.setConsiderId(currentUser);
            paymentConfirmationConsider.setOperatingStatus(StatusCode.CZ_PROCESSED);
            paymentConfirmationConsider.setImgFile(imgFile);
            int update = baseMapper.updateById(paymentConfirmationConsider);
            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion,9,PromptSuccess.CWB_ID,"","","",status);
            Integer companyId = paymentConfirmationConsider.getCompanyId();
            //更新成功，添加法规部经理数据
            if (update > 0){
                int insert = 0;
                QueryWrapper<DgPaymentConfirmationConsider> wrapperTemp = new QueryWrapper<>();
                wrapperTemp.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
                DgPaymentConfirmationConsider paymentConfirmationTemp = baseMapper.selectOne(wrapperTemp);
                if (paymentConfirmationTemp==null){
                    DgPaymentConfirmationConsider dgPaymentConfirmationConsider = new DgPaymentConfirmationConsider();
                    dgPaymentConfirmationConsider.setRoleId(PromptSuccess.FG_JL_ID);
                    dgPaymentConfirmationConsider.setBusinessId(businessId);
                    dgPaymentConfirmationConsider.setCompanyId(companyId);
                    dgPaymentConfirmationConsider.setImgFile(imgFile);
                    insert = baseMapper.insert(dgPaymentConfirmationConsider);
                }else {
                    paymentConfirmationTemp.setAuditOpinion("");
                    paymentConfirmationTemp.setImgFile(imgFile);
                    paymentConfirmationTemp.setStatus(StatusCode.SP_IN_REVIEW);
                    paymentConfirmationTemp.setOperatingStatus(StatusCode.CZ_ON_HAND);
                    insert = baseMapper.updateById(paymentConfirmationTemp);
                }
                //更新成功，返回操作结果
                log.info(insert > 0 ? "财务部经理确认成功" : "财务部经理确认失败");
                return insert > 0;
            }
        }

        //之前状态为驳回，同意则更新回款确认数据给法规部经理
        if (StatusCode.SP_REVIEWED.equals(status) && StatusCode.SP_REJECT.equals(approveStatus)){
            QueryWrapper<DgPaymentConfirmationConsider> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
            DgPaymentConfirmationConsider paymentConfirmationConsider = baseMapper.selectOne(queryWrapper);
            paymentConfirmationConsider.setAuditOpinion(auditOpinion);
            paymentConfirmationConsider.setConsiderId(currentUser);
            paymentConfirmationConsider.setStatus(StatusCode.SP_IN_REVIEW);
            paymentConfirmationConsider.setOperatingStatus(StatusCode.CZ_PROCESSED);
            paymentConfirmationConsider.setImgFile(imgFile);
            int update = baseMapper.updateById(paymentConfirmationConsider);
            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion, 9,PromptSuccess.CWB_ID,"","","",status);
            //更新成功，更新法规部经理数据
            if (update > 0){
                QueryWrapper<DgPaymentConfirmationConsider> wrapperTemp = new QueryWrapper<>();
                wrapperTemp.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
                DgPaymentConfirmationConsider paymentConfirmationTemp = baseMapper.selectOne(wrapperTemp);
                paymentConfirmationTemp.setAuditOpinion("");
                paymentConfirmationTemp.setImgFile(imgFile);
                paymentConfirmationTemp.setStatus(StatusCode.SP_IN_REVIEW);
                paymentConfirmationTemp.setOperatingStatus(StatusCode.CZ_ON_HAND);
                int updateById = baseMapper.updateById(paymentConfirmationTemp);
                //更新成功，返回操作结果
                log.info(updateById > 0 ? "驳回后再次财务部经理确认成功" : "驳回后再次财务部经理确认失败");
                return updateById > 0;
            }
        }

        //驳回操作，更新数据和资产部经理数据及返回上一流程
        if (StatusCode.SP_REJECT.equals(status)){
            QueryWrapper<DgPaymentConfirmationConsider> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
            DgPaymentConfirmationConsider paymentConfirmationConsider = baseMapper.selectOne(queryWrapper);
            paymentConfirmationConsider.setAuditOpinion(auditOpinion);
            paymentConfirmationConsider.setConsiderId(currentUser);
            paymentConfirmationConsider.setStatus(StatusCode.SP_REJECT);
            paymentConfirmationConsider.setOperatingStatus(StatusCode.CZ_PROCESSED);
            paymentConfirmationConsider.setImgFile(imgFile);
            int update = baseMapper.updateById(paymentConfirmationConsider);
            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion,9,PromptSuccess.CWB_ID,"","","",status);
            //更新成功，更新资产部经理数据及其他人的数据
            if (update > 0){
                //更新担保函法规部经理数据
                //根据业务id查询法规部经理数据
                QueryWrapper<DgGuaranteeLetterAssignUser> guaranteeLetterAssignUserQueryWrapperFg = new QueryWrapper<>();
                guaranteeLetterAssignUserQueryWrapperFg.eq("business_id",businessId).eq("role_id",PromptSuccess.FG_JL_ID);
                DgGuaranteeLetterAssignUser dgGuaranteeLetterAssignUserFg = guaranteeLetterAssignUserMapper.selectOne(guaranteeLetterAssignUserQueryWrapperFg);
                dgGuaranteeLetterAssignUserFg.setStatus(StatusCode.SP_REJECT);
                guaranteeLetterAssignUserMapper.updateById(dgGuaranteeLetterAssignUserFg);

                //更新担保函a角数据
                //根据业务id查询A用户数据
                QueryWrapper<DgGuaranteeLetterAssignUser> guaranteeLetterAssignUserQueryWrapperA = new QueryWrapper<>();
                guaranteeLetterAssignUserQueryWrapperA.eq("business_id",businessId).eq("role_id",PromptSuccess.DB_A_ID);
                DgGuaranteeLetterAssignUser dgGuaranteeLetterAssignUserA = guaranteeLetterAssignUserMapper.selectOne(guaranteeLetterAssignUserQueryWrapperA);
                dgGuaranteeLetterAssignUserA.setStatus(StatusCode.SP_REJECT);
                guaranteeLetterAssignUserMapper.updateById(dgGuaranteeLetterAssignUserA);

                //更新担保函b角数据
                //根据业务id查询B用户数据
                QueryWrapper<DgGuaranteeLetterAssignUser> guaranteeLetterAssignUserQueryWrapperB = new QueryWrapper<>();
                guaranteeLetterAssignUserQueryWrapperB.eq("business_id",businessId).eq("role_id",PromptSuccess.DB_B_ID);
                DgGuaranteeLetterAssignUser dgGuaranteeLetterAssignUserB = guaranteeLetterAssignUserMapper.selectOne(guaranteeLetterAssignUserQueryWrapperB);
                dgGuaranteeLetterAssignUserB.setStatus(StatusCode.SP_REJECT);
                guaranteeLetterAssignUserMapper.updateById(dgGuaranteeLetterAssignUserB);

                //更新担保函资产部经理数据
                //根据业务id查询资产部经理数据
                QueryWrapper<DgGuaranteeLetterAssignUser> guaranteeLetterAssignUserQueryWrapper = new QueryWrapper<>();
                guaranteeLetterAssignUserQueryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.ZC_JL_ID);
                DgGuaranteeLetterAssignUser dgGuaranteeLetterAssignUser = guaranteeLetterAssignUserMapper.selectOne(guaranteeLetterAssignUserQueryWrapper);
                dgGuaranteeLetterAssignUser.setStatus(StatusCode.SP_REJECT);
                dgGuaranteeLetterAssignUser.setOperatingStatus(StatusCode.CZ_ON_HAND);
                dgGuaranteeLetterAssignUser.setAuditOpinion("");

                //如果财务确认的法规部经理有数据，更新法规部经理数据
                //法规部数据存在，更新法规部经理财务回款数据
                QueryWrapper<DgPaymentConfirmationConsider> paymentConfirmationConsiderFG = new QueryWrapper<>();
                paymentConfirmationConsiderFG.eq("business_id",businessId).eq("role_id",PromptSuccess.FG_JL_ID);
                DgPaymentConfirmationConsider paymentConfirmationConsiderFg = baseMapper.selectOne(paymentConfirmationConsiderFG);
                //数据存在，进行更新
                if (ObjectUtil.isNotNull(paymentConfirmationConsiderFg)){
                    paymentConfirmationConsiderFg.setStatus(StatusCode.SP_REJECT);
                    //清空上次的审批意见及银行回单
                    paymentConfirmationConsiderFg.setAuditOpinion("");
                    paymentConfirmationConsiderFg.setImgFile("");
                    baseMapper.updateById(paymentConfirmationConsiderFg);
                }

                int updateById = guaranteeLetterAssignUserMapper.updateById(dgGuaranteeLetterAssignUser);
                //更新成功，返回操作结果
                log.info(updateById > 0 ? "驳回更新资产部经理成功" : "驳回更新资产部经理失败");
                return updateById > 0;
            }
        }
        //其他情况均为失败
        return false;
    }

    /**
     * 法规部经理操作
     */
    private boolean regulationDepartmentOperating(Integer businessId,Integer currentUser,String auditOpinion,Integer status,String imgFile){
        //更新之前,获取此数据的审批状态，如果审批状态为驳回，则更新法规部经理数据，如果审批状态为审批中，则添加法规部经理数据—_—
        QueryWrapper<DgPaymentConfirmationConsider> queryWrapperTemp = new QueryWrapper<>();
        queryWrapperTemp.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
        DgPaymentConfirmationConsider paymentConfirmationConsiderTemp = baseMapper.selectOne(queryWrapperTemp);
        Integer approveStatus = paymentConfirmationConsiderTemp.getStatus();

        //之前状态为审批中，同意则添加放款通知数据给法规部经理
        if (StatusCode.SP_REVIEWED.equals(status) && StatusCode.SP_IN_REVIEW.equals(approveStatus)){
            QueryWrapper<DgPaymentConfirmationConsider> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
            DgPaymentConfirmationConsider paymentConfirmationConsider = baseMapper.selectOne(queryWrapper);
            paymentConfirmationConsider.setAuditOpinion(auditOpinion);
            paymentConfirmationConsider.setConsiderId(currentUser);
            paymentConfirmationConsider.setStatus(StatusCode.SP_REVIEWED);
            paymentConfirmationConsider.setOperatingStatus(StatusCode.CZ_PROCESSED);
            paymentConfirmationConsider.setImgFile(imgFile);
            int update = baseMapper.updateById(paymentConfirmationConsider);

            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion,9,PromptSuccess.FG_JL_ID,"","","",status);
            Integer companyId = paymentConfirmationConsider.getCompanyId();
            //更新成功，添加法规部经理数据和更新财务部数据
            if (update > 0){
                QueryWrapper<DgPaymentConfirmationConsider> queryWrapperCwb = new QueryWrapper<>();
                queryWrapperCwb.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
                DgPaymentConfirmationConsider paymentConfirmationConsiderCwb = baseMapper.selectOne(queryWrapperCwb);
                paymentConfirmationConsiderCwb.setStatus(StatusCode.SP_REVIEWED);
                baseMapper.updateById(paymentConfirmationConsiderCwb);

                int insert = 0;
                QueryWrapper<DgLoanNotice> loanNoticeQueryWrapperFG = new QueryWrapper<>();
                loanNoticeQueryWrapperFG.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
                DgLoanNotice loanNoticeFG = dgLoanNoticeMapper.selectOne(loanNoticeQueryWrapperFG);
                if (loanNoticeFG==null){
                    DgLoanNotice loanNotice = new DgLoanNotice();
                    loanNotice.setCompanyId(companyId);
                    loanNotice.setBusinessId(businessId);
                    loanNotice.setRoleId(PromptSuccess.FG_JL_ID);
                    insert = dgLoanNoticeMapper.insert(loanNotice);
                }else {
                    loanNoticeFG.setStatus(StatusCode.SP_IN_REVIEW);
                    loanNoticeFG.setOperatingStatus(StatusCode.CZ_ON_HAND);
                    loanNoticeFG.setAuditOpinion("");
                    insert = dgLoanNoticeMapper.updateById(loanNoticeFG);
                }
                //添加抄送数据
                try {
                    copyForService.copySendUser(businessId,9,null);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                //更新成功，返回操作结果
                log.info(insert > 0 ? "法规部经理确认成功" : "法规部经理确认失败");
                return insert > 0;
            }
        }

        //之前状态为驳回，同意则更新放款通知数据给法规部经理
        if (StatusCode.SP_REVIEWED.equals(status) && StatusCode.SP_REJECT.equals(approveStatus)){
            QueryWrapper<DgPaymentConfirmationConsider> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
            DgPaymentConfirmationConsider paymentConfirmationConsider = baseMapper.selectOne(queryWrapper);
            paymentConfirmationConsider.setAuditOpinion(auditOpinion);
            paymentConfirmationConsider.setConsiderId(currentUser);
            paymentConfirmationConsider.setStatus(StatusCode.SP_REVIEWED);
            paymentConfirmationConsider.setOperatingStatus(StatusCode.CZ_PROCESSED);
            paymentConfirmationConsider.setImgFile(imgFile);
            int update = baseMapper.updateById(paymentConfirmationConsider);
            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion, 9,PromptSuccess.FG_JL_ID,"","","",status);
            //更新成功，更新法规部经理数据和财务部经理数据
            if (update > 0){
                QueryWrapper<DgPaymentConfirmationConsider> queryWrapperCwb = new QueryWrapper<>();
                queryWrapperCwb.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
                DgPaymentConfirmationConsider paymentConfirmationConsiderCwb = baseMapper.selectOne(queryWrapperCwb);
                paymentConfirmationConsiderCwb.setStatus(StatusCode.SP_REVIEWED);
                baseMapper.updateById(paymentConfirmationConsiderCwb);

                QueryWrapper<DgLoanNotice> loanNoticeQueryWrapper = new QueryWrapper<>();
                loanNoticeQueryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
                DgLoanNotice loanNotice = dgLoanNoticeMapper.selectOne(loanNoticeQueryWrapper);
                //清空审批意见
                loanNotice.setAuditOpinion("");
                loanNotice.setStatus(StatusCode.SP_IN_REVIEW);
                loanNotice.setOperatingStatus(StatusCode.CZ_ON_HAND);
                int updateById = dgLoanNoticeMapper.updateById(loanNotice);
                //添加抄送数据
                try {
                    copyForService.copySendUser(businessId,9,null);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                //更新成功，返回操作结果
                log.info(updateById > 0 ? "驳回后再次财务部经理确认成功" : "驳回后再次财务部经理确认失败");
                return updateById > 0;
            }
        }

        //驳回操作，更新数据和财务部经理数据及返回上一流程
        if (StatusCode.SP_REJECT.equals(status)){
            QueryWrapper<DgPaymentConfirmationConsider> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("business_id",businessId).eq("role_id", PromptSuccess.FG_JL_ID);
            DgPaymentConfirmationConsider paymentConfirmationConsider = baseMapper.selectOne(queryWrapper);
            paymentConfirmationConsider.setAuditOpinion(auditOpinion);
            paymentConfirmationConsider.setConsiderId(currentUser);
            paymentConfirmationConsider.setStatus(StatusCode.SP_REJECT);
            paymentConfirmationConsider.setOperatingStatus(StatusCode.CZ_PROCESSED);
            paymentConfirmationConsider.setImgFile(imgFile);
            int update = baseMapper.updateById(paymentConfirmationConsider);
            //同时往审核流程进程表新增一条记录
            String account = DgApplyAmountInfoController.getAccount();
            String departmentId = OrdinaryUtil.getDepartmentId(currentUser);
            AuditProcessUtil.save(currentUser,businessId,account,departmentId,
                    auditOpinion,9,PromptSuccess.FG_JL_ID,"","","",status);
            //更新成功，更新财务部经理数据
            if (update > 0){
                QueryWrapper<DgPaymentConfirmationConsider> queryWrapperT = new QueryWrapper<>();
                queryWrapperT.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
                DgPaymentConfirmationConsider paymentConfirmationConsiderT = baseMapper.selectOne(queryWrapperT);
                paymentConfirmationConsiderT.setOperatingStatus(StatusCode.CZ_ON_HAND);
                paymentConfirmationConsiderT.setStatus(StatusCode.SP_REJECT);
                paymentConfirmationConsiderT.setAuditOpinion("");
                int updateById = baseMapper.updateById(paymentConfirmationConsiderT);
                //更新成功，返回操作结果
                log.info(updateById > 0 ? "驳回更新资产部经理成功" : "驳回更新资产部经理失败");
                return updateById > 0;
            }
        }
        //其他情况均为失败
        return false;
    }

    @Override
    public boolean updatePaymentConfirmation(PaymentConfirmationUpdateRequest paymentConfirmationUpdateRequest) {
        //取出条件
        Integer businessId = paymentConfirmationUpdateRequest.getBusinessId();
        //获取当前登录用户id
        Integer currentUser = Integer.parseInt(this.getUserId());
        String auditOpinion = paymentConfirmationUpdateRequest.getAuditOpinion();
        String imgFile = paymentConfirmationUpdateRequest.getImgFile();
        //状态为2表示通过，为4表示驳回
        Integer status = paymentConfirmationUpdateRequest.getStatus();
        //获取当前用户拥有的角色ids
        String roleIds = RedisUtil.get("dq:userRole:" + currentUser);
        List<String> roleIdList = null;
        if(roleIds != null) {
            String[] ids = roleIds.split(",");
            roleIdList = Arrays.asList(ids);
        }

        //*************************************************根据角色更新数据************************************************

        //角色不为空
        if (roleIdList!=null){

            //角色为财务部经理回款确认操作和驳回操作
            if (roleIdList.contains(String.valueOf(PromptSuccess.CWB_ID))){
                //财务部经理数据是否被操作过
                QueryWrapper<DgPaymentConfirmationConsider> queryWrapperOne = new QueryWrapper<>();
                queryWrapperOne.eq("business_id",businessId).eq("role_id", PromptSuccess.CWB_ID);
                DgPaymentConfirmationConsider paymentConfirmationConsiderOne = baseMapper.selectOne(queryWrapperOne);
                Integer operatingStatus = paymentConfirmationConsiderOne.getOperatingStatus();
                //待处理则财务部经理操作
                if (operatingStatus.equals(1)){
                    return this.financeDepartmentOperating(businessId,currentUser,auditOpinion,status,imgFile);
                }
                //已处理，拥有法规部经理，则法规部经理操作
                if (roleIdList.contains(String.valueOf(PromptSuccess.FG_JL_ID))){
                    return this.regulationDepartmentOperating(businessId,currentUser,auditOpinion,status,imgFile);
                }
            }

            //角色为法规部经理回款确认操作和驳回操作
            if (roleIdList.contains(String.valueOf(PromptSuccess.FG_JL_ID))){
                return this.regulationDepartmentOperating(businessId,currentUser,auditOpinion,status,imgFile);
            }
        }

        //一个判断都不走，直接返回false
        log.info("《《《《《数据更新失败，请确保数据完整》》》》》");
        return false;
    }
}
