package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.DgProcessManageMapper;
import com.daqing.financial.guarantee.mapper.DgProcessManageVisualMapper;
import com.daqing.financial.guarantee.model.request.ProcessManageRequest;
import com.daqing.financial.guarantee.service.IDgProcessManageService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.framework.domain.guarantee.DgProcessManage;
import com.daqing.framework.domain.guarantee.DgProcessManageVisual;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.model.response.ResponseResult;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 流程管理表 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2021-01-15
 */
@Service
public class DgProcessManageServiceImpl extends ServiceImpl<DgProcessManageMapper, DgProcessManage> implements IDgProcessManageService {

    @Autowired
    private DgProcessManageVisualMapper dgProcessManageVisualMapper;

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Override
    public List<DgProcessManage> processManageList(String userId) {

        int usrId = Integer.parseInt(userId);
        return this.baseMapper.processManageList(usrId);
    }

    @Override
    public List<EmployeeVO> processVisualList(Integer id) {
        //根据流程管理id查询可见人id
        List<Integer> visualUserId = dgProcessManageVisualMapper.processVisualList(id);

        Integer[] userIds =visualUserId.toArray(new Integer[visualUserId.size()]);
        ResponseResult<List<EmployeeVO>> listResponseResult = hrmsFeignService.listEmpNameByUserId(userIds);
        List<EmployeeVO> dataList = listResponseResult.getData();
        return dataList;
    }

    @Override
    public Boolean updateUserVisual(Integer id, List<Integer> ids) {

        //查询设置的可见人是否有流程管理页面的权限

        //先根据主键id删除所有对应可见模板员工
        dgProcessManageVisualMapper.delete(new QueryWrapper<DgProcessManageVisual>()
                .eq("process_manage_id",id));

        //批量新增数据到流程管理可见表
        List<DgProcessManageVisual> list = new ArrayList<>();
        for (Integer i:ids) {
            DgProcessManageVisual dgProcessManageVisual = new DgProcessManageVisual();
            dgProcessManageVisual.setProcessManageId(id);
            dgProcessManageVisual.setUserId(i);
            list.add(dgProcessManageVisual);
        }
        int i = dgProcessManageVisualMapper.insertUserVisual(list);
        return i > 0;
    }

    @Override
    public int updateProcessManage(ProcessManageRequest processManageReq) {
        DgProcessManage dgProcessManage = new DgProcessManage();
        BeanUtils.copyProperties(processManageReq,dgProcessManage);
        return this.baseMapper.updateById(dgProcessManage);
    }

    @Override
    public Boolean queryAllModelId(Integer userId) {
        //获取所有的审批模板id
        List<Integer> list = this.baseMapper.queryAllModelId();

        List<DgProcessManageVisual>visualList = new ArrayList();
        for (Integer modelId:list) {
            DgProcessManageVisual dgManageVisualPO = new DgProcessManageVisual();
            dgManageVisualPO.setProcessManageId(modelId);
            dgManageVisualPO.setUserId(userId);
            visualList.add(dgManageVisualPO);
        }

        //添加用户可见审批模板
        int a = dgProcessManageVisualMapper.insertUserVisual(visualList);
        return a > 0;

    }

    @Override
    public Boolean delUserVisual(Integer userId) {
        System.out.println("userId~~~~~~~~~~~~~~~~~~~~~"+userId);
        int count = dgProcessManageVisualMapper.delete(new QueryWrapper<DgProcessManageVisual>().eq("user_id", userId));
        return count > 0;
    }

    @Override
    public Boolean delBatchUserVisual(List<Integer> userId) {
        int count = dgProcessManageVisualMapper.delBatchUserVisual(userId);
        return count > 0;
    }
}
