package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.DgStatisticsMapper;
import com.daqing.financial.guarantee.model.request.DgStatisticsRequest;
import com.daqing.financial.guarantee.model.response.DgStatisticsExcelRefuseResponse;
import com.daqing.financial.guarantee.model.response.DgStatisticsExcelResponse;
import com.daqing.financial.guarantee.model.response.DgStatisticsListRefuseResponse;
import com.daqing.financial.guarantee.model.response.DgStatisticsListResponse;
import com.daqing.financial.guarantee.service.IDgStatisticsService;
import com.daqing.financial.guarantee.util.DateUtils;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.DgApplyAmountInfo;
import com.daqing.framework.domain.guarantee.po.DgStatisticsPO;
import com.daqing.framework.domain.guarantee.po.DgStatisticsRefusePO;
import com.daqing.framework.domain.hrms.ext.EmployeeTO;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @auther River
 * @date 2020/11/5 15:36
 */
@Service
public class DgStatisticsServiceImpl extends ServiceImpl<DgStatisticsMapper, DgApplyAmountInfo> implements IDgStatisticsService {

    @Autowired
    private HrmsFeignService hrmsFeignService;

    /**
     * 列表
     */
    @Override
    public PageUtils list(Integer page, Integer size, DgStatisticsRequest dgStatisticsRequest) {
        if (page == null || size == null) {
            page = 1;
            size = 10;
        }
        // 时间筛选(固定时间)
        if (dgStatisticsRequest.getCreateFixedTime() != null && dgStatisticsRequest.getStartTime() == null &&
                dgStatisticsRequest.getEndTime() == null) {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            switch (dgStatisticsRequest.getCreateFixedTime()) {
                case 0:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getDayBegin()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getDayEnd()));// 今天
                    break;
                case 1:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfYesterday()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfYesterDay()));// 昨天
                    break;
                case 2:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfWeek()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfWeek()));// 本周
                    break;
                case 3:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfMonth()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfMonth()));// 本月
                    break;
                case 4:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfYear()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfYear()));// 本年
                    break;
                default:
                    dgStatisticsRequest.setStartTime(null);
                    dgStatisticsRequest.setEndTime(null);
            }
        }
        // 查询列表
        IPage<DgStatisticsPO> iPage = this.getBaseMapper().list(new Page(page, size), dgStatisticsRequest);
        List<DgStatisticsPO> dgStatisticsPOList = iPage.getRecords();
        if (dgStatisticsPOList == null) {
            return null;
        }
        // 获取所有的提单人id
        Integer[] presenterIds = dgStatisticsPOList.stream().map(DgStatisticsPO::getPresenterId).toArray(Integer[]::new);
        if (presenterIds.length != 0) {
            // 强制类型转换
            Long[] ids = new Long[presenterIds.length];
            for (int i = 0; i < presenterIds.length; i++) {
                ids[i] = presenterIds[i].longValue();
            }
            //ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getEmployeeAndDeptById(ids);
            ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getAccountAndDeptNameByIds(ids);
            if (empAndDeptList.getData() != null) {
                List<DgStatisticsListResponse> listResponse = this.jointStatistics(dgStatisticsPOList, empAndDeptList.getData());
                IPage<DgStatisticsListResponse> iPages = new Page<>();
                BeanUtils.copyProperties(iPage, iPages);
                iPages.setRecords(listResponse);
                return new PageUtils(iPages);
            } else {
                return new PageUtils(iPage);
            }
        } else {
            return new PageUtils(iPage);
        }
    }

    /**
     * 列表导出
     */
    @Override
    public Boolean excelExport(HttpServletResponse response) {

        IPage<DgStatisticsPO> list = this.getBaseMapper().list(new Page(), new DgStatisticsRequest());
        List<DgStatisticsPO> dgStatisticsPOList = list.getRecords();
        List<DgStatisticsExcelResponse> excelResponse = new ArrayList<>();
        if (dgStatisticsPOList.size()>0) {
            Integer[] presenterIds = dgStatisticsPOList.stream().map(DgStatisticsPO::getPresenterId).toArray(Integer[]::new);
            if (presenterIds.length != 0) {
                // 强制类型转换
                Long[] ids = new Long[presenterIds.length];
                for (int i = 0; i < presenterIds.length; i++) {
                    ids[i] = presenterIds[i].longValue();
                }
                //ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getEmployeeAndDeptById(ids);
                ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getAccountAndDeptNameByIds(ids);
                if (empAndDeptList.getData() != null) {
                    List<DgStatisticsListResponse> listResponse = this.jointStatistics(dgStatisticsPOList, empAndDeptList.getData());

                    for (DgStatisticsListResponse dgStatisticsListResponse : listResponse) {
                        DgStatisticsExcelResponse dgStatisticsExcelResponse = new DgStatisticsExcelResponse();
                        StringBuilder stringBuilder = new StringBuilder();
                        if (dgStatisticsListResponse.getDepartment() != null && dgStatisticsListResponse.getDepartment().size() != 0) {
                            for (int i = 0; i < dgStatisticsListResponse.getDepartment().size(); i++) {
                                if (i == dgStatisticsListResponse.getDepartment().size() - 1) {
                                    stringBuilder.append(dgStatisticsListResponse.getDepartment().get(i));
                                } else {
                                    stringBuilder.append(dgStatisticsListResponse.getDepartment().get(i)).append(",");
                                }
                            }
                        }
                        dgStatisticsExcelResponse.setDepartment(stringBuilder.toString());
                        BeanUtils.copyProperties(dgStatisticsListResponse, dgStatisticsExcelResponse);
                        excelResponse.add(dgStatisticsExcelResponse);
                    }

                }
            }
        }
        try {
            ExcelUtil.writeExcelWithSheets(response, excelResponse, "业务统计信息一览表", "statistics", new DgStatisticsExcelResponse())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }

    }

    /**
     * 拒绝列表
     */
    @Override
    public PageUtils listRefuse(Integer page, Integer size, DgStatisticsRequest dgStatisticsRequest) {
        if (page == null || size == null) {
            page = 1;
            size = 10;
        }
        // 时间筛选(固定时间)
        if (dgStatisticsRequest.getCreateFixedTime() != null && dgStatisticsRequest.getStartTime() == null &&
                dgStatisticsRequest.getEndTime() == null) {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            switch (dgStatisticsRequest.getCreateFixedTime()) {
                case 0:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getDayBegin()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getDayEnd()));// 今天
                    break;
                case 1:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfYesterday()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfYesterDay()));// 昨天
                    break;
                case 2:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfWeek()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfWeek()));// 本周
                    break;
                case 3:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfMonth()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfMonth()));// 本月
                    break;
                case 4:
                    dgStatisticsRequest.setStartTime(dateFormat.format(DateUtils.getBeginDayOfYear()));
                    dgStatisticsRequest.setEndTime(dateFormat.format(DateUtils.getEndDayOfYear()));// 本年
                    break;
                default:
                    dgStatisticsRequest.setStartTime(null);
                    dgStatisticsRequest.setEndTime(null);
            }
        }
        // 拒绝列表
        IPage<DgStatisticsRefusePO> iPage = this.getBaseMapper().listRefuse(new Page(page, size), dgStatisticsRequest);
        List<DgStatisticsRefusePO> dgStatisticsRefusePOList = iPage.getRecords();
        if (dgStatisticsRefusePOList == null) {
            return null;
        }
        // 部门筛选
        /*if (dgStatisticsRequest.getDeptId() != null && dgStatisticsRequest.getEmpId() == null) {
            ResponseResult<List<EmployeeVO>> result = hrmsFeignService.listEmployeeByDeptId(dgStatisticsRequest.getDeptId().longValue());
            if (result.getData() == null || result.getData().size() == 0) {
                return null;
            }
            for (int i = 0; i < dgStatisticsRefusePOList.size(); i++) {
                boolean flag = false;
                for (EmployeeVO employeeVO : result.getData()) {
                    if (Objects.equals(dgStatisticsRefusePOList.get(i).getPresenterId().longValue(), employeeVO.getId())) {
                        flag = true;
                    }
                }
                if (!flag) {
                    dgStatisticsRefusePOList.remove(i);
                    i--; // 索引要减一,不然会触发"快速失败"异常
                }
            }
            iPage.setTotal(dgStatisticsRefusePOList.size());
            iPage.setSize(iPage.getTotal() % size == 0 ? iPage.getTotal() / size : iPage.getTotal() / size + 1);
        }*/
        // 获取所有的提单人id
        Integer[] presenterIds = dgStatisticsRefusePOList.stream().map(DgStatisticsRefusePO::getPresenterId).toArray(Integer[]::new);
        if (presenterIds.length != 0) {
            // 强制类型转换
            Long[] ids = new Long[presenterIds.length];
            for (int i = 0; i < presenterIds.length; i++) {
                ids[i] = presenterIds[i].longValue();
            }
            //ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getEmployeeAndDeptById(ids);
            ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getAccountAndDeptNameByIds(ids);
            if (empAndDeptList.getData() != null) {
                List<DgStatisticsListRefuseResponse> listResponse = this.jointRefuseStatistics(dgStatisticsRefusePOList, empAndDeptList.getData());
                IPage<DgStatisticsListRefuseResponse> iPages = new Page<>();
                BeanUtils.copyProperties(iPage, iPages);
                iPages.setRecords(listResponse);
                return new PageUtils(iPages);
            } else {
                return new PageUtils(iPage);
            }
        } else {
            return new PageUtils(iPage);
        }
    }

    /**
     * 拒绝列表导出
     */
    @Override
    public Boolean excelExportRefuse(HttpServletResponse response) {
        // 列表查询
        IPage<DgStatisticsRefusePO> listRefuse = this.getBaseMapper().listRefuse(new Page(), new DgStatisticsRequest());
        List<DgStatisticsRefusePO> statisticsRefusePOList = listRefuse.getRecords();
        List<DgStatisticsExcelRefuseResponse> excelResponse = new ArrayList<>();
        if (statisticsRefusePOList.size()>0) {

            Integer[] presenterIds = statisticsRefusePOList.stream().map(DgStatisticsRefusePO::getPresenterId).toArray(Integer[]::new);
            if (presenterIds.length != 0) {
                // 强制类型转换
                Long[] ids = new Long[presenterIds.length];
                for (int i = 0; i < presenterIds.length; i++) {
                    ids[i] = presenterIds[i].longValue();
                }
                //ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getEmployeeAndDeptById(ids);
                ResponseResult<List<EmployeeTO>> empAndDeptList = hrmsFeignService.getAccountAndDeptNameByIds(ids);
                if (empAndDeptList.getData() != null) {
                    // 提单人姓名、部门拼装
                    List<DgStatisticsListRefuseResponse> listResponse = this.jointRefuseStatistics(statisticsRefusePOList, empAndDeptList.getData());

                    for (DgStatisticsListRefuseResponse listRefuseResponse : listResponse) {
                        DgStatisticsExcelRefuseResponse excelRefuseResponse = new DgStatisticsExcelRefuseResponse();
                        StringBuilder stringBuilder = new StringBuilder();
                        if (listRefuseResponse.getDepartment() != null && listRefuseResponse.getDepartment().size() != 0) {
                            for (int i = 0; i < listRefuseResponse.getDepartment().size(); i++) {
                                if (i == listRefuseResponse.getDepartment().size() - 1) {
                                    stringBuilder.append(listRefuseResponse.getDepartment().get(i));
                                } else {
                                    stringBuilder.append(listRefuseResponse.getDepartment().get(i)).append(",");
                                }
                            }
                        }
                        excelRefuseResponse.setDepartment(stringBuilder.toString());
                        BeanUtils.copyProperties(listRefuseResponse, excelRefuseResponse);
                        excelResponse.add(excelRefuseResponse);
                    }

                }
            }
        }
        try {
            ExcelUtil.writeExcelWithSheets(response, excelResponse, "业务统计拒绝信息一览表", "statisticsRefuse", new DgStatisticsExcelRefuseResponse())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    private List<DgStatisticsListResponse> jointStatistics(List<DgStatisticsPO> dgStatisticsPOList, List<EmployeeTO> empAndDeptList) {
        // 用于拼装业务统计列表
        List<DgStatisticsListResponse> listResponse = new ArrayList<>();
        DgStatisticsListResponse dgStatisticsListResponse;
        for (DgStatisticsPO dgStatisticsPO : dgStatisticsPOList) {
            // 每一个客户对应一个不同的对象
            dgStatisticsListResponse = new DgStatisticsListResponse();
            for (EmployeeTO employeeTO : empAndDeptList) {
                if (Objects.equals(dgStatisticsPO.getPresenterId().longValue(), employeeTO.getId())) {
                    dgStatisticsListResponse.setEmployeeName(employeeTO.getEmpName());
                    dgStatisticsListResponse.setDepartment(employeeTO.getDeptNames());
                }
            }
            if(dgStatisticsListResponse.getEmployeeName()== null){
                dgStatisticsListResponse.setEmployeeName("客户");
            }
            BeanUtils.copyProperties(dgStatisticsPO, dgStatisticsListResponse);
            listResponse.add(dgStatisticsListResponse);
        }
        return listResponse;
    }

    private List<DgStatisticsListRefuseResponse> jointRefuseStatistics(List<DgStatisticsRefusePO> dgStatisticsRefusePOList, List<EmployeeTO> empAndDeptList) {
        // 用于拼装业务统计列表
        List<DgStatisticsListRefuseResponse> listResponse = new ArrayList<>();
        DgStatisticsListRefuseResponse dgStatisticsListRefuseResponse;
        for (DgStatisticsRefusePO dgStatisticsRefusePO : dgStatisticsRefusePOList) {
            // 每一个客户对应一个不同的对象
            dgStatisticsListRefuseResponse = new DgStatisticsListRefuseResponse();
            for (EmployeeTO employeeTO : empAndDeptList) {
                if (Objects.equals(dgStatisticsRefusePO.getPresenterId().longValue(), employeeTO.getId())) {
                    dgStatisticsListRefuseResponse.setEmployeeName(employeeTO.getEmpName());
                    dgStatisticsListRefuseResponse.setDepartment(employeeTO.getDeptNames());
                }
            }
            if(dgStatisticsListRefuseResponse.getEmployeeName()== null){
                dgStatisticsListRefuseResponse.setEmployeeName("客户");
            }
            BeanUtils.copyProperties(dgStatisticsRefusePO, dgStatisticsListRefuseResponse);
            listResponse.add(dgStatisticsListRefuseResponse);
        }
        return listResponse;
    }
}
