package com.daqing.financial.guarantee.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.guarantee.controller.DgApplyAmountInfoController;
import com.daqing.financial.guarantee.feign.HrmsFeignService;
import com.daqing.financial.guarantee.mapper.DgComplianceInvestigationMapper;
import com.daqing.financial.guarantee.mapper.DgLoanCommitteeConsiderMapper;
import com.daqing.financial.guarantee.mapper.DgWorkConferenceConsiderMapper;
import com.daqing.financial.guarantee.model.response.*;
import com.daqing.financial.guarantee.service.IDgCopyForService;
import com.daqing.financial.guarantee.service.IDgGuaranteeAssignUserService;
import com.daqing.financial.guarantee.service.IDgWorkConferenceConsiderService;
import com.daqing.financial.guarantee.util.AuditProcessUtil;
import com.daqing.financial.guarantee.util.OrdinaryUtil;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.DgLoanCommitteeConsider;
import com.daqing.framework.domain.guarantee.DgWorkConferenceConsider;
import com.daqing.framework.domain.guarantee.GuaranteeCode;
import com.daqing.framework.domain.guarantee.po.AuditOpinionPO;
import com.daqing.framework.domain.guarantee.po.DgWorkConferencePO;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.StatusCode;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.EasyExcelUtil;
import com.daqing.framework.utils.excel.ExcelUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 工作会审议 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2020-11-05
 */
@Service
public class DgWorkConferenceConsiderServiceImpl extends ServiceImpl<DgWorkConferenceConsiderMapper, DgWorkConferenceConsider>
        implements IDgWorkConferenceConsiderService {

    @Autowired
    private DgLoanCommitteeConsiderMapper dgLoanCommitteeConsiderMapper;

    @Autowired
    private DgComplianceInvestigationMapper dgComplianceInvestigationMapper;

    @Autowired
    private IDgGuaranteeAssignUserService guaranteeAssignUserService;

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Resource
    private IDgCopyForService dgCopyForService;

    /**
     * 列表
     */
    @Override
    public PageUtils list(Integer page, Integer size, String codeOrName,Integer status) {
        if (page == null || size == null || page < 1 || size < 1) {
            page = 1;
            size = 10;
        }
        String userId = this.getUserId();
        // 获取当前登录用户的角色id
        String roleIdString = RedisUtil.get("dq:userRole:" + userId);
        if (roleIdString == null || roleIdString.length() == 0) {
            ExceptionCast.cast(GuaranteeCode.ROLE_IS_NOT);
        }
        String[] roleIds = roleIdString.split(",");
        /*String userId = "1";
        String[] roleIds = {"71"};*/
        List<Integer> roleList = new ArrayList<>();
        for (String roleId : roleIds) {
            roleList.add(Integer.parseInt(roleId));
        }
        IPage<DgWorkConferencePO> list = this.getBaseMapper().list(new Page(page, size), codeOrName, roleList, Integer.parseInt(userId),status);
        List<DgWorkConferencePO> dgWorkConferencePOS = list.getRecords();
        List<DgWorkConferenceResponse> listResponse = new ArrayList<>();
        if (dgWorkConferencePOS != null && dgWorkConferencePOS.size() != 0) {
            Integer[] userIds = new Integer[dgWorkConferencePOS.size() * 5];
            // 获取所有的userId
            for (int i = 0; i < userIds.length; i += 5) {
                for (int j = i / 5; j < i / 5 + 1; j++) {
                    userIds[i] = dgWorkConferencePOS.get(j).getAUserId();
                    userIds[i + 1] = dgWorkConferencePOS.get(j).getBUserId();
                    userIds[i + 2] = dgWorkConferencePOS.get(j).getIUserId();
                    userIds[i + 3] = dgWorkConferencePOS.get(j).getAssUserId();
                    userIds[i + 4] = dgWorkConferencePOS.get(j).getMUserId();
                }
            }
            // 根据userId获取对应的员工姓名
            ResponseResult<List<EmployeeVO>> responseResult = hrmsFeignService.listEmpNameByUserId(userIds);
            if (responseResult.getData() != null) {
                // 将对应的userId转换为name
                listResponse = this.transitionUserName(dgWorkConferencePOS, responseResult.getData());
            } else {
                for (DgWorkConferencePO workConferencePO : dgWorkConferencePOS) {
                    DgWorkConferenceResponse workConference = new DgWorkConferenceResponse();
                    BeanUtils.copyProperties(workConferencePO, workConference);
                    listResponse.add(workConference);
                }
            }
        }
        // 分页属性复制并返回
        IPage<DgWorkConferenceResponse> iPage = new Page<>();
        BeanUtils.copyProperties(list, iPage);
        iPage.setRecords(listResponse);
        return new PageUtils(iPage);
    }

    /**
     * 审议
     */
    @Transactional
    @Override
    public Boolean check(Integer id, String opinion, Integer status, List<String> empNames) {
        if (id == null || status == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        String userId = this.getUserId();
        // String userId = "1";
        // 同意
        if (status == 1) {
            if (null == empNames || empNames.size() == 0) {
                ExceptionCast.cast(GuaranteeCode.WORK_JUDGE_NOT_NULL);
            }
            // 拼接评委名字
            StringBuilder names = new StringBuilder();
            for (int i = 0; i < empNames.size(); i++) {
                if (i == empNames.size() - 1) {
                    names.append(empNames.get(i));
                } else {
                    names.append(empNames.get(i)).append(",");
                }
            }
            Boolean result = this.getBaseMapper().check(id, Integer.parseInt(userId), opinion,
                    StatusCode.SP_REVIEWED, StatusCode.CZ_ON_HAND, new Date(), names.toString());
            if (result) {
                boolean b=false;
                try {
                    b=dgCopyForService.copySendUser(this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(),6,id);
                } catch (IOException e) {
                    e.printStackTrace();
                }

                // 在业务流程表中插入一条审议通过的数据
                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(userId));
                return AuditProcessUtil.save(Integer.parseInt(userId),this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                        departmentId, opinion, PromptSuccess.GZU_PROCESS_ID,PromptSuccess.GZU_ID,"","","", StatusCode.SP_REVIEWED);
            }
        }
        // 拒绝
        if (status == 3) {
            if (opinion == null || opinion.length() == 0) {
                ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
            }
            // 绑定该条数据的userId,表示该用户操作了该条数据
            Boolean result3 = this.getBaseMapper().updateWorkConferenceUserIdById(id, Integer.parseInt(userId));
            if (result3) {
                // 先改变所有的审核状态为已拒绝
                Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(
                        this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(), StatusCode.SP_REFUSE);
                if (result) {
                    // 再改变自己当前的操作状态为已处理
                    Boolean result1 = this.getBaseMapper().updateOperationStatusById(id, StatusCode.CZ_PROCESSED, opinion);
                    if (result1) {
                        // 最后改变该条贷款信息的业务状态为已否决
                        Boolean result2 = dgComplianceInvestigationMapper.updateBusinessStatusByBusinessId(
                                this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(), StatusCode.YW_REJECTED);
                        if (result2) {
                            boolean b=false;
                            try {
                                b=dgCopyForService.copySendUser(this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(),6,id);
                            } catch (IOException e) {
                                e.printStackTrace();
                            }

                            // 在业务流程表中插入一条拒绝数据(记得带上拒绝意见)
                            String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(userId));
                            return AuditProcessUtil.save(Integer.parseInt(userId),this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                    departmentId, opinion, PromptSuccess.GZU_PROCESS_ID,PromptSuccess.GZU_ID,"","","", StatusCode.SP_REFUSE);
                        }
                    }
                }
            }
        }
        // 驳回
        if (status == 2) {
            if (opinion == null || opinion.length() == 0) {
                ExceptionCast.cast(GuaranteeCode.COMPLIANCE_INVESTIGATION_REMARK_NOT_NULL);
            }
            // 绑定该条数据的userId,表示该用户操作了该条数据
            Boolean result4 = this.getBaseMapper().updateWorkConferenceUserIdById(id, Integer.parseInt(userId));
            if (result4) {
                // 先修改工作会的审核状态为驳回
                Boolean result = this.getBaseMapper().updateAuditStatusByBusinessId(this.getBaseMapper()
                        .getWorkConferenceConsider(id).getBusinessId(), StatusCode.SP_REJECT);
                if (result) {
                    // 再修改自己的状态为已处理
                    Boolean result1 = this.getBaseMapper().updateOperationStatusById(id, StatusCode.CZ_PROCESSED, opinion);
                    if (result1) {
                        // 修改合规调查的审核状态为驳回
                        Boolean result2 = dgComplianceInvestigationMapper.updateAuditStatusByBusinessId(this.getBaseMapper()
                                .getWorkConferenceConsider(id).getBusinessId(), StatusCode.SP_REJECT);
                        if (result2) {
                            // 修改法规领导的操作状态为待处理
                            Boolean result3 = dgComplianceInvestigationMapper.updateToAudit(this.getBaseMapper()
                                    .getWorkConferenceConsider(id).getBusinessId(), StatusCode.CZ_ON_HAND, PromptSuccess.FG_LD_ID);
                            if (result3) {
                                // 在业务流程表中插入一条工作会驳回的数据
                                String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(userId));
                                return AuditProcessUtil.save(Integer.parseInt(userId),this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                        departmentId, opinion, PromptSuccess.GZU_PROCESS_ID,PromptSuccess.GZU_ID,"","","", StatusCode.SP_REJECT);
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 评委抽取
     */
    @Transactional
    @Override
    public Boolean theJudgesDrawn(Integer id, List<String> empNames) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        if (empNames == null || empNames.size() == 0) {
            ExceptionCast.cast(GuaranteeCode.WORK_JUDGE_NOT_NULL);
        }
        // 拼接评委名字
        StringBuilder names = new StringBuilder();
        for (int i = 0; i < empNames.size(); i++) {
            if (i == empNames.size() - 1) {
                names.append(empNames.get(i));
            } else {
                names.append(empNames.get(i)).append(",");
            }
        }
        DgWorkConferenceConsider workConferenceConsider = this.getBaseMapper().getWorkConferenceConsider(id);
        // 新增一条贷审会数据
        if (workConferenceConsider.getBusinessId() != null) {
            DgLoanCommitteeConsider loanCommitteeConsider = new DgLoanCommitteeConsider();
            loanCommitteeConsider.setBusinessId(workConferenceConsider.getBusinessId());
            loanCommitteeConsider.setCompanyId(workConferenceConsider.getCompanyId());
            loanCommitteeConsider.setRoleId(PromptSuccess.DSH_ID);
            loanCommitteeConsider.setStatus(StatusCode.SP_IN_REVIEW);
            loanCommitteeConsider.setOperatingStatus(StatusCode.CZ_ON_HAND);
            loanCommitteeConsider.setCreateTime(new Date());
            loanCommitteeConsider.setUpdateTime(new Date());
            Boolean result = this.getBaseMapper().insertLoanCommitteeConsider(loanCommitteeConsider);
            // 保存抽取的评委名单
            if (result) {
                Boolean result1 = dgLoanCommitteeConsiderMapper.TheJudgesDrawn(workConferenceConsider.getBusinessId(), names.toString());
                if (result1) {
                    // 改变自己的操作状态为已处理
                    Boolean result2 = this.getBaseMapper().updateOperationStatus(id, StatusCode.CZ_PROCESSED);
                    if (result2) {
                        // 在业务流程表中插入一条评委抽取成功的数据
                        String departmentId = OrdinaryUtil.getDepartmentId(Integer.parseInt(this.getUserId()));
                        return AuditProcessUtil.save(Integer.parseInt(this.getUserId()),this.getBaseMapper().getWorkConferenceConsider(id).getBusinessId(), DgApplyAmountInfoController.getAccount(),
                                departmentId, null, PromptSuccess.GZU_PROCESS_ID,PromptSuccess.GZU_ID,"","","", StatusCode.SP_REVIEWED);
                    }
                }
            }
        }
        return false;
    }

    /**
     * 查询业务申请详情、AB角、资产部调查、信息部调查、法规部调查的审核意见、审核人姓名及时间
     */
    @Override
    public AuditOpinionResponse getAuditOpinion(Integer businessId) {
        if (businessId == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        AuditOpinionPO auditOpinion = dgLoanCommitteeConsiderMapper.getAuditOpinion(businessId);
        AuditOpinionResponse auditOpinionResponse = new AuditOpinionResponse();
        // 将对应的用户id转换成姓名返回回去
        System.out.println(auditOpinion);
        Integer[] userIds = new Integer[5];
        userIds[0] = auditOpinion.getAEmpId();
        userIds[1] = auditOpinion.getBEmpId();
        userIds[2] = auditOpinion.getAssetEmpId();
        userIds[3] = auditOpinion.getMessageEmpId();
        userIds[4] = auditOpinion.getRegulationEmpId();
        ResponseResult<List<EmployeeVO>> responseResult = hrmsFeignService.listEmpNameByUserId(userIds);
        List<EmployeeVO> employees = responseResult.getData();
        if (employees != null) {
            for (EmployeeVO employee : employees) {
                if (Objects.equals(employee.getId(), auditOpinion.getAEmpId().longValue())) {
                    auditOpinionResponse.setAName(employee.getName());
                }
                if (Objects.equals(employee.getId(), auditOpinion.getBEmpId().longValue())) {
                    auditOpinionResponse.setBName(employee.getName());
                }
                if (Objects.equals(employee.getId(), auditOpinion.getAssetEmpId().longValue())) {
                    auditOpinionResponse.setAssetName(employee.getName());
                }
                if (Objects.equals(employee.getId(), auditOpinion.getMessageEmpId().longValue())) {
                    auditOpinionResponse.setMessageName(employee.getName());
                }
                if (Objects.equals(employee.getId(), auditOpinion.getRegulationEmpId().longValue())) {
                    auditOpinionResponse.setRegulationName(employee.getName());
                }
            }
        }
        BeanUtils.copyProperties(auditOpinion, auditOpinionResponse);
        // 获取担保信息
        Integer guaranteeId = dgComplianceInvestigationMapper.getGuaranteeId(businessId, PromptSuccess.DB_LD_ID);
        if (guaranteeId != null) {
            GuaranteeDetailResponse guaranteeDetailResponse = guaranteeAssignUserService.guaranteeDetail(guaranteeId);
            auditOpinionResponse.setDgApplyAmountInfo(guaranteeDetailResponse.getDgApplyAmountInfo());
            //auditOpinionResponse.setLinkedHashMap(guaranteeDetailResponse.getLinkedHashMap());
            auditOpinionResponse.setDgBusinessCompany(guaranteeDetailResponse.getDgBusinessCompany());
        }
        return auditOpinionResponse;
    }

    /**
     * 查看
     */
    @Override
    public DgWorkConferenceConsiderQueryResponse query(Integer id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        DgWorkConferenceConsiderQueryResponse queryResponse = new DgWorkConferenceConsiderQueryResponse();
        DgWorkConferenceConsider dgWorkConferenceConsider = this.getBaseMapper().query(id);
        if (dgWorkConferenceConsider != null) {
            if (dgWorkConferenceConsider.getJoinJudge() != null && dgWorkConferenceConsider.getJoinJudge().length() != 0) {
                queryResponse.setJoinJudges(dgWorkConferenceConsider.getJoinJudge().split(","));
            }
            if (dgWorkConferenceConsider.getRemark() != null && dgWorkConferenceConsider.getRemark().length() != 0) {
                queryResponse.setRemark(dgWorkConferenceConsider.getRemark());
            }
        }
        return queryResponse;
    }

    /**
     * 导出
     */
    @Transactional
    @Override
    public Boolean excelExport(HttpServletResponse response, List<String> ids,Integer userId) {
/*        if (ids == null || ids.size() == 0) {
            ExceptionCast.cast(GuaranteeCode.EXCEL_ID_NOT_NULL);
        }*/
        List<Integer> idList = new ArrayList<>();
        for (String id : ids) {
            idList.add(Integer.parseInt(id));
        }
        List<DgWorkConferencePO> list = this.getBaseMapper().excelList(idList,userId);
        System.out.println(list);
        // 获取所有的userId
        Integer[] userIds = new Integer[list.size() * 5];
        for (int i = 0; i < userIds.length; i += 5) {
            for (int j = i / 5; j < i / 5 + 1; j++) {
                userIds[i] = list.get(j).getAUserId();
                userIds[i + 1] = list.get(j).getBUserId();
                userIds[i + 2] = list.get(j).getIUserId();
                userIds[i + 3] = list.get(j).getAssUserId();
                userIds[i + 4] = list.get(j).getMUserId();
            }
        }
        // 根据userId获取对应的员工姓名
        ResponseResult<List<EmployeeVO>> responseResult = hrmsFeignService.listEmpNameByUserId(userIds);
        System.out.println(responseResult.getData());
        List<DgWorkConferenceResponse> excelList = new ArrayList<>();
        if (responseResult.getData() != null) {
            excelList = this.transitionUserName(list, responseResult.getData());
        } else {
            for (DgWorkConferencePO dgWorkConferencePO : list) {
                DgWorkConferenceResponse workConferenceResponse = new DgWorkConferenceResponse();
                BeanUtils.copyProperties(dgWorkConferencePO, workConferenceResponse);
                excelList.add(workConferenceResponse);
            }
        }
        try {
            EasyExcelUtil.download(response, DgWorkConferenceResponse.class,excelList,"工作会数据一览表","第一页");

/*            ExcelUtil.writeExcelWithSheets(response, excelList, "工作会数据一览表", "NO.1", new DgWorkConferenceResponse())
                    .finish();*/
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
        }
        return false;
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId == null || userId.length() == 0) {
            ExceptionCast.cast(CommonCode.GET_LOGIN_USER_FAIL);
        }
        return userId;
    }

    /**
     * 将userId转换为对应的名字返回给前端
     */
    private List<DgWorkConferenceResponse> transitionUserName(List<DgWorkConferencePO> workConferences, List<EmployeeVO> employees) {
        List<DgWorkConferenceResponse> workConferenceResponses = new ArrayList<>();
        DgWorkConferenceResponse workConferenceResponse;
        for (DgWorkConferencePO workConference : workConferences) {
            workConferenceResponse = new DgWorkConferenceResponse();
            for (EmployeeVO employee : employees) {
                if (Objects.equals(workConference.getAUserId().longValue(), employee.getId())) {
                    workConferenceResponse.setAroleName(employee.getName());
                }
                if (Objects.equals(workConference.getBUserId().longValue(), employee.getId())) {
                    workConferenceResponse.setBroleName(employee.getName());
                }
                if (Objects.equals(workConference.getIUserId().longValue(), employee.getId())) {
                    workConferenceResponse.setRegulationName(employee.getName());
                }
                if (Objects.equals(workConference.getAssUserId().longValue(), employee.getId())) {
                    workConferenceResponse.setAssetName(employee.getName());
                }
                if (Objects.equals(workConference.getMUserId().longValue(), employee.getId())) {
                    workConferenceResponse.setMessageName(employee.getName());
                }
            }
            BeanUtils.copyProperties(workConference, workConferenceResponse);
            workConferenceResponses.add(workConferenceResponse);
        }
        return workConferenceResponses;
    }
}
