package com.daqing.financial.guarantee.util;

import cn.hutool.core.date.DateUtil;
import com.daqing.financial.guarantee.model.request.ExcelImpInsuranceVO;
import com.deepoove.poi.util.RegexUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author chen
 * @DATE 2021/9/2 11:17
 * @Version 1.0
 */
public class ExcelImportHelper {

    private static final Calendar fromCal = Calendar.getInstance();

    /**
     * 按日期加天数得出全新日期
     *
     * @param date 需要加天数的日期
     * @param day  需要增加的天数
     * @return 新的日期
     */
    public static Date addDate(Date date, int day) {
        try {
            fromCal.setTime(date);
            fromCal.add(Calendar.DATE, day);

            return fromCal.getTime();
        } catch (Exception e) {
            return null;
        }
    }

    private static Workbook getWorkbook(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        Workbook workbook = null;

        if (fileName.endsWith("xlsx")) {
            try {
                workbook = new XSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else if (fileName.endsWith("xls")) {
            try {
                workbook = new HSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return workbook;
    }

    //判断row是否为空
    public static boolean isRowEmpty(Row row) {
        if (null == row) {
            return true;
        }
        int firstCellNum = row.getFirstCellNum();   //第一个列位置
        int lastCellNum = row.getLastCellNum();     //最后一列位置
        int nullCellNum = 0;    //空列数量
        for (int c = firstCellNum; c < lastCellNum; c++) {
            Cell cell = row.getCell(c);
            if (null == cell || CellType.BLANK == cell.getCellType()) {
                nullCellNum++;
                continue;
            }
            cell.setCellType(CellType.STRING);
            String cellValue = cell.getStringCellValue().trim();
            if (StringUtils.isEmpty(cellValue)) {
                nullCellNum++;
            }
        }
        //所有列都为空
        return nullCellNum == (lastCellNum - firstCellNum);
    }

    /**
     * 读取文件数据
     *
     * @param file
     * @return
     */
    public static List<ExcelImpInsuranceVO> readInsuranceManagement(MultipartFile file) throws Exception {
        List<ExcelImpInsuranceVO> list = new ArrayList<ExcelImpInsuranceVO>();

        Workbook workbook = getWorkbook(file);
        ExcelImpInsuranceVO impInsuranceVO = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        throw new Exception("导入失败，导入数据为空！");
                    }
                    continue;
                }

                Cell customerName;//客户名称
                Cell bank;//贷款银行
                Cell guaranteeAmount;//担保额度
                Cell loanStartTime;//贷款开始日期
                Cell loanEndTime;//贷款结束日期
                Cell guaranteeTime;//担保期限
                Cell businessType;//业务类别
                Cell amountWide;//贷款用途
                Cell compensationAmount;//代偿金额
                Cell compensatoryBalance;//代偿余额
                Cell compensatoryTime;//代偿时间
                Cell personLiable;//清收负责人
                Cell collectionGroup;//所属清收组
                Cell area;//所在区域
                Cell operation;//经营情况
                Cell litigation;//诉讼情况
                Cell mortgage;//抵押情况

                if (row != null) {

                    impInsuranceVO = new ExcelImpInsuranceVO();

                    if (row.getCell(16) != null) {
                        row.getCell(16).setCellType(CellType.STRING);
                        mortgage = row.getCell(16);
                    } else {
                        mortgage = row.createCell(16);
                    }

                    if (row.getCell(15) != null) {
                        row.getCell(15).setCellType(CellType.STRING);
                        litigation = row.getCell(15);
                    } else {
                        litigation = row.createCell(15);
                    }

                    if (row.getCell(14) != null) {
                        row.getCell(14).setCellType(CellType.STRING);
                        operation = row.getCell(14);
                    } else {
                        operation = row.createCell(14);
                    }

                    if (row.getCell(13) != null) {
                        row.getCell(13).setCellType(CellType.STRING);
                        area = row.getCell(13);
                    } else {
                        area = row.createCell(13);
                    }

                    if (row.getCell(12) != null) {
                        row.getCell(12).setCellType(CellType.STRING);
                        collectionGroup = row.getCell(12);
                    } else {
                        collectionGroup = row.createCell(12);
                    }

                    if (row.getCell(11) != null) {
                        row.getCell(11).setCellType(CellType.STRING);
                        personLiable = row.getCell(11);
                    } else {
                        personLiable = row.createCell(11);
                    }

                    if (row.getCell(10) != null) {
                        row.getCell(10).setCellType(CellType.STRING);
                        compensatoryTime = row.getCell(10);
                    } else {
                        compensatoryTime = row.createCell(10);
                    }

                    if (row.getCell(9) != null) {
                        row.getCell(9).setCellType(CellType.STRING);
                        compensatoryBalance = row.getCell(9);
                    } else {
                        compensatoryBalance = row.createCell(9);
                    }
                    if (row.getCell(8) != null) {
                        row.getCell(8).setCellType(CellType.STRING);
                        compensationAmount = row.getCell(8);
                    } else {
                        compensationAmount = row.createCell(8);
                    }
                    if (row.getCell(7) != null) {
                        row.getCell(7).setCellType(CellType.STRING);
                        amountWide = row.getCell(7);
                    } else {
                        amountWide = row.createCell(7);
                    }

                    if (row.getCell(6) != null) {
                        row.getCell(6).setCellType(CellType.STRING);
                        businessType = row.getCell(6);
                    } else {
                        businessType = row.createCell(6);
                    }

                    if (row.getCell(5) != null) {
                        row.getCell(5).setCellType(CellType.STRING);
                        guaranteeTime = row.getCell(5);
                    } else {
                        guaranteeTime = row.createCell(5);
                    }

                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        loanEndTime = row.getCell(4);
                    } else {
                        loanEndTime = row.createCell(4);
                    }

                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        loanStartTime = row.getCell(3);
                    } else {
                        loanStartTime = row.createCell(3);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        guaranteeAmount = row.getCell(2);
                    } else {
                        guaranteeAmount = row.createCell(3);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        bank = row.getCell(1);
                    } else {
                        bank = row.createCell(1);
                    }

                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        customerName = row.getCell(0);
                    } else {
                        customerName = row.createCell(0);
                    }

                    impInsuranceVO.setCustomerName(customerName.getStringCellValue());
                    impInsuranceVO.setBank(bank.getStringCellValue());
                    impInsuranceVO.setGuaranteeAmount(guaranteeAmount.getStringCellValue());

                    Calendar calendar = new GregorianCalendar(1900, 0, -1);
                    Date d = calendar.getTime();

                    boolean numeric = StringUtils.isNumeric(loanStartTime.getStringCellValue());
                    if (numeric && !"".equals(loanStartTime.getStringCellValue())) {
                        Date d1 = ExcelImportHelper.addDate(d, Integer.parseInt(loanStartTime.getStringCellValue()));
                        String start = DateUtil.formatDateTime(d1);
                        impInsuranceVO.setLoanStartTime(start);
                    }

                    boolean numeric1 = StringUtils.isNumeric(loanEndTime.getStringCellValue());
                    if (numeric1 && !"".equals(loanEndTime.getStringCellValue())) {
                        Date d2 = ExcelImportHelper.addDate(d, Integer.parseInt(loanEndTime.getStringCellValue()));
                        String end = DateUtil.formatDateTime(d2);
                        impInsuranceVO.setLoanEndTime(end);
                    }

                    impInsuranceVO.setGuaranteeTime(guaranteeTime.getStringCellValue());
                    impInsuranceVO.setBusinessType(businessType.getStringCellValue());
                    impInsuranceVO.setAmountWide(amountWide.getStringCellValue());
                    impInsuranceVO.setCompensationAmount(compensationAmount.getStringCellValue());
                    impInsuranceVO.setCompensatoryBalance(compensatoryBalance.getStringCellValue());

                    String compensatory = "";
                    boolean numeric2 = StringUtils.isNumeric(compensatoryTime.getStringCellValue());
                    if (numeric2 && !"".equals(compensatoryTime.getStringCellValue())) {
                        Date d3 = ExcelImportHelper.addDate(d, Integer.parseInt(compensatoryTime.getStringCellValue()));
                        compensatory = DateUtil.formatDateTime(d3);
                    }

                    impInsuranceVO.setCompensatoryTime(compensatory);
                    impInsuranceVO.setPersonLiable(personLiable.getStringCellValue());
                    impInsuranceVO.setCollectionGroup(collectionGroup.getStringCellValue());
                    impInsuranceVO.setArea(area.getStringCellValue());
                    impInsuranceVO.setOperation(operation.getStringCellValue());
                    impInsuranceVO.setLitigation(litigation.getStringCellValue());
                    impInsuranceVO.setMortgage(mortgage.getStringCellValue());

                    list.add(impInsuranceVO);
                }
            }

        }

        return list;
    }

}
