package com.daqing.financial.hrauth.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.security.web.DefaultRedirectStrategy;
import org.springframework.security.web.RedirectStrategy;
import org.springframework.security.web.session.SessionInformationExpiredEvent;
import org.springframework.security.web.session.SessionInformationExpiredStrategy;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
@Component
public class CustomExpiredSessionStrategy implements SessionInformationExpiredStrategy {

    //前后端分离返回JSON的方式
    //jackson的JSON处理对象
    private final ObjectMapper objectMapper = new ObjectMapper();


    @Override
    public void onExpiredSessionDetected(SessionInformationExpiredEvent event)
            throws IOException, ServletException {

        Map<String, Object> map = new HashMap<>();
        map.put("code", 403);
        map.put("msg", "您的登陆已经超时或者已经在另一台机器登陆，您被迫下线！"
                + event.getSessionInformation().getLastRequest());

        String json = objectMapper.writeValueAsString(map);

        event.getResponse().setContentType("application/json;charset=utf-8");
        event.getResponse().getWriter().write(json);
    }

    //页面跳转的方式
    //页面跳转的处理逻辑
//    private RedirectStrategy redirectStrategy = new DefaultRedirectStrategy();
//
//
//    @Override
//    public void onExpiredSessionDetected(SessionInformationExpiredEvent event)
//            throws IOException, ServletException {
//        redirectStrategy.sendRedirect(event.getRequest(), event.getResponse(), "踢下线后跳转的路径");
//    }

}
