package com.daqing.financial.hrauth.controller;

import com.daqing.financial.hrauth.service.RolePermissionService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.hrms.PermissionEntity;
import com.daqing.framework.domain.hrms.RoleEntity;
import com.daqing.framework.domain.hrms.RolePermissionEntity;
import com.daqing.framework.domain.hrms.request.RolePermissionRequest;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.domain.hrms.response.RolePermissionResponse;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @auther River
 * @date 2020/9/22 15:27
 */
@RestController
@RequestMapping("/hrms/auth/permission")
@Api(tags = {"角色权限管理"})
public class RolePermissionController {

    @Autowired
    private RolePermissionService permissionService;

    /**
     * 权限菜单树形列表展示
     * @return
     */
    @Log(detail = "权限菜单树形列表展示",level = 3,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.SELECT)
    @GetMapping("/tree")
    public ResponseResult tree() {
        List<PermissionEntity> list = permissionService.listWithTree();
        return new ResponseResult<List<PermissionEntity>>().SUCCESS(list);
    }

    /**
     * 角色权限列表展示
     * @param page
     * @param size
     * @return
     */
    @Log(detail = "角色权限列表展示",level = 3,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.SELECT)
    @GetMapping("/rolePermissionList")
    public ResponseResult rolePermissionList(@RequestParam("page") Integer page,
                               @RequestParam("size") Integer size) {
        PageUtils data = permissionService.queryPage(page, size);
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 查看权限
     * @param roleId
     * @return
     */
    @Log(detail = "查看角色权限",level = 4,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.SELECT)
    @GetMapping("/queryPermissionByRoleId")
    public ResponseResult queryPermissionByRoleId(@RequestParam("roleId") Long roleId) {
        List<RolePermissionResponse> list = permissionService.listWithPermissionTree(roleId);
        return new ResponseResult<List<RolePermissionResponse>>().SUCCESS(list);
    }

    /**
     * 查看权限数组
     * @param roleId
     * @return
     */
    @Log(detail = "查看权限数组",level = 4,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.SELECT)
    @GetMapping("/queryPermissionArrById")
    public ResponseResult queryPermissionArrById(@RequestParam("roleId") Long roleId) {
        List<RolePermissionEntity> list = permissionService.queryPermissionArrById(roleId);
        ArrayList<Integer> sb = new ArrayList<Integer>();
        for (RolePermissionEntity rolePermission:list) {
            sb.add(rolePermission.getPermissionId().intValue());
        }
        Integer[] ids = sb.toArray(new Integer[sb.size()]);
        return new ResponseResult<List<RolePermissionResponse>>().SUCCESS(ids);
    }

    /**
     * 新增角色权限
     */
    @Log(detail = "新增角色权限",level = 4,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.INSERT)
    @Transactional
    @PostMapping("/saveRolePermission")
    public ResponseResult saveRolePermission(@Valid @RequestBody RolePermissionRequest rolePermission) {
        Date date = new Date();
        rolePermission.setCreateTime(date);
        rolePermission.setMotifyTime(date);
        //此处要判重，新增的角色名称要唯一
        int count = permissionService.uniqueRoleName(rolePermission.getName());
        if(count >0){
            ExceptionCast.cast(HrmsCode.ROLE_NAME_EXIST);
        }
        //新增角色到角色表
        int request = permissionService.addHrmsRole(rolePermission);
        //根据选中的permissionId批量新增数据到hrms_role_permission
        List<Long> asList = Arrays.asList(rolePermission.getPermissionIds());
        List<RolePermissionRequest> rolePermissionList = getRolePermissionList(asList, rolePermission.getRoleId());
        boolean result = permissionService.addHrmsRolePermission(rolePermissionList);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    private List<RolePermissionRequest> getRolePermissionList(List<Long> asList, Long roleId) {
        return asList.stream().map(permissionId -> {
            RolePermissionRequest request = new RolePermissionRequest();
            request.setRoleId(roleId);
            request.setPermissionId(permissionId);
            return request;
        }).collect(Collectors.toList());
    }

    /**
     * 修改角色权限
     */
    @Log(detail = "修改角色权限",level = 4,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.UPDATE)
    @Transactional
    @PostMapping("/updateRolePermission")
    public ResponseResult updateRolePermission(@Valid @RequestBody RolePermissionRequest rolePermission) {
        Date date = new Date();
        rolePermission.setMotifyTime(date);
        //根据角色Id修改角色内容
        int request = permissionService.updateHrmsRole(rolePermission);
        //删除该角色Id对应的所有权限
        List<Long> asList = Arrays.asList(rolePermission.getPermissionIds());
        permissionService.deleteHrmsRolePermission(rolePermission.getRoleId());
        //根据选中的permissionId批量新增数据到hrms_role_permission
        List<RolePermissionRequest> rolePermissionList = getRolePermissionList(asList, rolePermission.getRoleId());
        boolean result = permissionService.addHrmsRolePermission(rolePermissionList);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }


    /**
     * 删除用户角色权限
     */
    @Log(detail = "删除用户角色权限",level = 4,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.DELETE)
    @Transactional
    @PostMapping("/delRolePermission")
    public ResponseResult delRolePermission(@RequestBody RolePermissionRequest rolePermission) {
        //根据角色Id查询该角色是否有用户关联
        int count = permissionService.queryCountByRoleId(rolePermission.getRoleId());
        if(count >0){
            ExceptionCast.cast(HrmsCode.ROLE_EMP_EXIST);
        }
        //如果角色id对应了业务管理里面的角色id限制不让删除
        if(rolePermission.getRoleId().equals(PromptSuccess.DB_YWY_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.DB_JL_ID.longValue())||
        rolePermission.getRoleId().equals(PromptSuccess.DB_A_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.DB_B_ID.longValue())||
        rolePermission.getRoleId().equals(PromptSuccess.DB_LD_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.ZC_JL_ID.longValue())||
        rolePermission.getRoleId().equals(PromptSuccess.ZC_ZY_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.ZC_LD_ID.longValue())||
        rolePermission.getRoleId().equals(PromptSuccess.XX_JL_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.XX_ZY_ID.longValue())||
        rolePermission.getRoleId().equals(PromptSuccess.XX_LD_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.FG_JL_ID.longValue())||
        rolePermission.getRoleId().equals(PromptSuccess.FG_C_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.FG_LD_ID.longValue())||
        rolePermission.getRoleId().equals(PromptSuccess.GZU_ID.longValue()) || rolePermission.getRoleId().equals(PromptSuccess.DSH_ID.longValue())||
                rolePermission.getRoleId().equals(PromptSuccess.CWB_ID.longValue())){
            ExceptionCast.cast(HrmsCode.ROLE_EMP_LINK);
        }
        //没有关联则根据roleId删除hrms_role表对应记录
        permissionService.deleteHrmsRoleByRoleId(rolePermission.getRoleId());
        //根据角色id删除对应所有权限
        boolean result = permissionService.deleteHrmsRolePermission(rolePermission.getRoleId());

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 根据用户Id查询角色信息
     * @param userId
     * @return
     */
    @Log(detail = "查询用户角色信息",level = 4,operationUnit = OperationUnit.ROLEPERMISSION,operationType = OperationType.SELECT)
    @Transactional
    @GetMapping("/getUserRoles")
    public ResponseResult getUserRoles(@RequestParam("userId")Integer userId) {
        List<RoleEntity> roleList = permissionService.getUserRoles(userId);

        return new ResponseResult<List<RoleEntity>>().SUCCESS(roleList);
    }

    @Transactional
    @GetMapping("/queryPermissionList")
    public ResponseResult queryPermissionList(@RequestParam("jiacan")String jiacan) {
        List<PermissionEntity> permissionList = permissionService.getList(jiacan);

        return new ResponseResult<List<PermissionEntity>>().SUCCESS(permissionList);
    }


}
