package com.daqing.financial.hrauth.controller;

import com.daqing.financial.hrauth.UserLoginControllerApi;
import com.daqing.financial.hrauth.config.WeChatConfig;
import com.daqing.financial.hrauth.feign.HrmsFeignService;
import com.daqing.financial.hrauth.model.XccLoginReq;
import com.daqing.financial.hrauth.service.UserLoginService;
import com.daqing.financial.hrauth.service.UserService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.hrms.UserEntity;
import com.daqing.framework.domain.hrms.request.*;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.SendUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;


/**
 * @auther River
 * @date 2020/9/22 15:27
 */
@RestController
@RequestMapping("/hrms/auth/userlogin")
@Api(tags = {"登录"})
@Slf4j
public class UserLoginController implements UserLoginControllerApi {

    @Autowired
    private UserLoginService userLoginService;

    @Autowired
    private WeChatConfig weChatConfig;

    @Autowired
    private UserService userService;

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Log(detail = "验证手机号及验证码",level = 2,operationUnit = OperationUnit.LOGIN,operationType = OperationType.UPDATE)
    @PostMapping("/verifyMobile")
    @ApiOperation(value = "验证手机号及验证码")
    public ResponseResult verifyMobile(@RequestBody @Valid UserLoginRequest user){
        UserLoginRequest result = userLoginService.verifyMobile(user);
        return ResponseResult.SUCCESS(result);
    }

    //@Log(detail = "重置密码",level = 2,operationUnit = OperationUnit.LOGIN,operationType = OperationType.UPDATE)
    @PostMapping("/getBackPwd")
    @ApiOperation(value = "重置密码")
    public ResponseResult getBackPwd(@RequestBody @Valid ResetPwdRequest user){
        boolean result = userLoginService.getBackPwd(user);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

//    @Log(detail = "用户登录",level = 1,operationUnit = OperationUnit.LOGIN,operationType = OperationType.INSERT)
    @PostMapping(value = "/login")
    @ApiOperation(value = "用户登录")
    /*@SysLog(MODULE = "登录日志", REMARK = "登录日志")*/
    public ResponseResult login(@RequestBody LoginRequest loginRequest){
        System.out.println("wechatId================"+loginRequest.getWechatId());
        return userLoginService.login(loginRequest);
    }

    @Log(detail = "修改平台超管员密码",level = 2,operationUnit = OperationUnit.LOGIN,operationType = OperationType.UPDATE)
    @PostMapping("/updatePwd")
    @ApiOperation(value = "修改平台超管员密码")
    public ResponseResult updatePwd(@RequestBody @Valid UpdatePwdRequest user){
        boolean result = userLoginService.updatePwd(user);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 拼装微信扫一扫登录url
     * accessPage是指用户当前所在的页面，就是用户扫码时当前所在页面
     * @return
     */
    @GetMapping("wxLoginUrl")
    @ResponseBody
    public ResponseResult loginUrl(@RequestParam(value = "accessPage",required = true)String accessPage) throws UnsupportedEncodingException {

        String redirectUrl = weChatConfig.getOpenRedirectUrl(); //获取开放平台重定向地址，就是用户扫完码微信回调我们的地址

        String callbackUrl = URLEncoder.encode(redirectUrl,"GBK"); //进行编码

        String qrcodeUrl = String.format(weChatConfig.getOPEN_QRCODE_URL(),weChatConfig.getOpenAppid(),callbackUrl,accessPage);

        return ResponseResult.SUCCESS(qrcodeUrl);
    }

    /**
     * 微信扫码登录，用户确认授权后，微信回调我们服务器的接口
     * 这个接口里的逻辑应该是拿到微信回传的授权code码调用微信的服务获取access_token
     * 获取的access_token的返回值里有openId，用openId再次调微信服务去拿用户的信息
     * @param code
     * @param state 用户扫码时当前所在页面
     * @param response
     * @throws IOException
     */
//    @Log(detail = "微信扫码登录",level = 1,operationUnit = OperationUnit.LOGIN,operationType = OperationType.INSERT)
    @GetMapping("/user/callback")
    public void wechatUserCallback(@RequestParam(value = "code",required = true) String code,
                                   String state, HttpServletResponse response) throws IOException {
        log.info("来了微信回调url=============================");
        if("STATE".equals(state)){
            UserEntity user = userService.saveWeChatUser(code,response);
            if(user != null){
/*                long times = 86400;
                //生成jwt
                String token = JwtUtils.createJWT(user.getId()+"","token", times);
                RedisUtil.setEx("dq:token:"+token,String.valueOf(user.getId()),times);*/

                String token = user.getToken();
                // state 当前用户的页面地址，需要拼接 http://  这样才不会站内跳转
                //String name = URLEncoder.encode(user.getAccount(),"UTF-8");
                //String account = "";
/*                try {
                    account = URLDecoder.decode(name,"UTF-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }*/
                String username = user.getAccount();
                String imgUrl = user.getHeadPortaritUrl();
                String encoder = "UTF-8";
                String account = URLEncoder.encode(username,encoder);
                Long userId = user.getId();
                String headerImg=null;
                if(imgUrl !=null)
                  headerImg = URLDecoder.decode(imgUrl,encoder);
                //account = new String(username.getBytes("GBK"),"iso-8859-1");
                System.out.println("account==========="+account+"headerImg============="+headerImg+"userId===="+userId);
                response.sendRedirect("https://test.feifanhitech.com/dq/index.html#/login?token="+token+"&account="+account+"&headerImg="+headerImg+"&id="+userId);
//                response.sendRedirect("https://www.feifanhitech.com/index/#/login?token="+token+"&account="+account+"&headerImg="+headerImg+"&id="+userId);
            }
        }else {
            userService.weChatBinding(code,response,state);
        }
    }

    /**
     * 绑定手机号和openId
     * @param bindOpenIdRequest
     * @return
     */
    @Log(detail = "绑定手机号和openId",level = 1,operationUnit = OperationUnit.LOGIN,operationType = OperationType.UPDATE)
    @PostMapping("/bindPhoneAndOpenId")
    public ResponseResult bindPhoneAndOpenId(@RequestBody @Valid BindOpenIdRequest bindOpenIdRequest){
        boolean result = userLoginService.bindPhoneAndOpenId(bindOpenIdRequest);
        LoginRequest loginRequest = new LoginRequest();
        if(result){
            //绑定手机号成功调登录接口需要的参数
            loginRequest.setWechatId(bindOpenIdRequest.getWechatId());
            loginRequest.setTenDayEffective(2);
            loginRequest.setType(2);
            loginRequest.setPhone(bindOpenIdRequest.getPhoneAccount());
        }
        return result ? ResponseResult.SUCCESS(loginRequest) : ResponseResult.FAIL();
    }

    /**
     * 获取验证码
     * @param mobile
     * @return
     */
    @Log(detail = "获取验证码",level = 1,operationUnit = OperationUnit.LOGIN,operationType = OperationType.UPDATE)
    @GetMapping("/sendCode")
    public ResponseResult sendCode(@RequestParam("mobile") String mobile){
        boolean result = SendUtil.sendMs(mobile);
        return result ? ResponseResult.SUCCESS("短信发送成功") : ResponseResult.FAIL(55001,"短信发送失败");
    }

    /**
     * 判断手机号是否存在
     * @param mobile
     * @return
     */
    @GetMapping("/isPhoneExist")
    public ResponseResult isPhoneExist(@RequestParam("mobile") String mobile){
        int result = userLoginService.isPhoneExist(mobile);
        return result>0 ? ResponseResult.SUCCESS() : ResponseResult.FAIL(55002,"该手机号不存在");
    }

    /**
     * 微信小程序获取openid和session_key授权登录
     * @param
     * @return
     */
//    @Log(detail = "微信小程序授权登录",level = 1,operationUnit = OperationUnit.LOGIN,operationType = OperationType.INSERT)
    @PostMapping(value = "/wxAuthLogin")
    public ResponseResult wxAuthLogin(@RequestBody XccLoginReq xccLoginReq){

        String encryptedData = xccLoginReq.getEncryptedData();
        String code = xccLoginReq.getCode();
        String iv = xccLoginReq.getIv();
        Integer identify = xccLoginReq.getIdentify();
        return userLoginService.wxAuthLogin(encryptedData,iv,code,identify);
    }

    /**
     * 是否开启通知提醒
     * @param isRemind
     * @return
     */
    @GetMapping("/isRemind")
    public ResponseResult isRemind(@RequestParam("isRemind") Integer isRemind){
        int result = userLoginService.isRemind(isRemind);
        return result>0 ? ResponseResult.SUCCESS() : ResponseResult.FAIL(55003,"开启通知提醒失败！");
    }

    /**
     * 读取是否已开启通知提醒
     * @return
     */
    @GetMapping("/readIsRemind")
    public ResponseResult readIsRemind(){
        String result = userLoginService.readIsRemind();
        return ResponseResult.SUCCESS(result);
    }
}
