package com.daqing.financial.hrauth.handle;

import com.daqing.financial.hrauth.model.ApiResult;
import com.daqing.financial.hrauth.service.impl.UserDetailsServiceImpl;
import com.daqing.financial.hrauth.util.ResponseUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.web.access.AccessDeniedHandler;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;

/**
 * Security自定义异常处理类
 * @Author: jialing xu
 * @Description: xvjialing@outlook.com
 * @Date: 17:24 2018/8/7
 */
@Slf4j
@Component
public class CustomAccessDeniedHandler implements AccessDeniedHandler {

    @Autowired
    private ObjectMapper objectMapper;

    private final UserDetailsServiceImpl userDetailsService;

    protected CustomAccessDeniedHandler(UserDetailsServiceImpl userDetailsService) {
        this.userDetailsService = userDetailsService;
    }

    @Override
    public void handle(HttpServletRequest request, HttpServletResponse response, AccessDeniedException e) throws IOException, ServletException {
/*        log.error("Token异常处理来了~~~~"+accessDeniedException.getMessage());
        response.setContentType("application/json;charset=UTF-8");
        Map map = new HashMap();
        map.put("code", "403");
        map.put("msg", accessDeniedException.getMessage());
        map.put("data","");
        response.setContentType("application/json");
        response.setStatus(HttpServletResponse.SC_OK);
        response.getWriter().write(objectMapper.writeValueAsString(map));*/

//        ResponseUtils.out(response, ApiResult.fail(401, e.getMessage()));
        String token = request.getHeader("token");
        if (StringUtils.isNotBlank(token)) {
            // 检查token
            SecurityUser securityUser = userDetailsService.getUserByToken(token);
            if (securityUser == null || securityUser.getCurrentUserInfo() == null) {
                //ExceptionCast.cast(HrmsCode.TOKEN_EXCEPTION);
                throw new AccessDeniedException("TOKEN已过期，请重新登录~~~");
            }
        }
    }
}
