package com.daqing.financial.hrauth.handle;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.daqing.financial.hrauth.dao.AuthPermissionDao;
import com.daqing.financial.hrauth.dao.AuthRoleDao;
import com.daqing.financial.hrauth.dao.RolePermissionMapper;
import com.daqing.financial.hrauth.enums.Constants;
import com.daqing.financial.hrauth.feign.HrauthFeignService;
import com.daqing.financial.hrauth.feign.HrmsFeignService;
import com.daqing.framework.domain.hrms.PermissionEntity;
import com.daqing.framework.domain.hrms.RoleEntity;
import com.daqing.framework.domain.hrms.RolePermissionEntity;
import com.daqing.framework.model.response.ResponseResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.access.SecurityConfig;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.access.intercept.FilterInvocationSecurityMetadataSource;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 * <p> 获取访问该url所需要的用户角色权限信息 </p>
 *
 * @author : zhengqing
 * @description : 执行完之后到 `UrlAccessDecisionManager` 中认证权限
 * @date : 2019/10/15 14:36
 */
@Slf4j
@Component
public class UrlFilterInvocationSecurityMetadataSource implements FilterInvocationSecurityMetadataSource {

    @Resource
    AuthPermissionDao permissionMapper;
    @Resource
    RolePermissionMapper rolePermissionMapper;
    @Resource
    AuthRoleDao roleMapper;

    @Resource
    HrmsFeignService hrmsFeignService;


    /***
     * 返回该url所需要的用户权限信息
     *
     * @param object: 储存请求url信息
     * @return: null：标识不需要任何权限都可以访问
     */
    @Override
    public Collection<ConfigAttribute> getAttributes(Object object) throws IllegalArgumentException {
        log.info("来了返回url所需要的用户权限信息~~~~~~~~~~~~~~~~~~~~");
        // 获取当前请求url
        String beforeRequestUrl = ((FilterInvocation) object).getRequestUrl();
        String requestUrl = null;
        boolean status = beforeRequestUrl.contains("?");
        if(status){
            requestUrl = beforeRequestUrl.substring(0, beforeRequestUrl.indexOf("?"));
        }else{
            requestUrl = beforeRequestUrl;
        }

        // TODO 忽略url请放在此处进行过滤放行
        if ("/login".equals(requestUrl) || requestUrl.contains("logout") || "/error".equals(requestUrl)) {
            return null;
        }

        // 数据库中所有url
        //List<PermissionEntity> permissionList = permissionMapper.selectList(null);
        //String jiacan=null;
        ResponseResult responseResult = hrmsFeignService.queryPermissionList();
        List<PermissionEntity> permissionList = (List<PermissionEntity>) responseResult.getData();
        for (PermissionEntity permission : permissionList) {
            // 获取该url所对应的权限
            if (requestUrl.equals(permission.getUrl())) {
                //List<RolePermissionEntity> permissionEntityList = rolePermissionMapper.selectList(new QueryWrapper<RolePermissionEntity>().eq("permission_id",permission.getId()));
                ResponseResult responseResult2 = hrmsFeignService.queryRolePermissionList(permission.getId());
                List<RolePermissionEntity> permissionEntityList = (List<RolePermissionEntity>) responseResult2.getData();

                SecurityUser userRole = (SecurityUser) SecurityContextHolder.getContext().getAuthentication().getPrincipal();//获取spring security封装的当前用户信息对象
                List<RoleEntity> list = userRole.getRoleList();

                List<String> roles = new LinkedList<>();
                int isPermission = 0;//标识是否有权限，有的话跳出循环
                for(int i = 0; i < permissionEntityList.size(); i ++){
                    if(isPermission > 0){
                        break;
                    }
                    for(int j = 0; j < list.size(); j ++){
                        if(permissionEntityList.get(i).getRoleId() == list.get(j).getId()){
                            isPermission ++;
                            break;
                        }
                    }
                }
                if(isPermission > 0){
                    //保存该url对应角色权限信息
                    return SecurityConfig.createList(roles.toArray(new String[roles.size()]));
                }else {
                    return SecurityConfig.createList(Constants.ROLE_LOGIN);
                }
            }
        }
        // 如果数据中没有找到相应url资源则为非法访问，要求用户登录再进行操作
        return SecurityConfig.createList(Constants.ROLE_LOGIN);
    }

    @Override
    public Collection<ConfigAttribute> getAllConfigAttributes() {
        return null;
    }

    @Override
    public boolean supports(Class<?> aClass) {
        return FilterInvocation.class.isAssignableFrom(aClass);
    }
}
