package com.daqing.financial.hrauth.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrauth.dao.*;
import com.daqing.financial.hrauth.service.RolePermissionService;
import com.daqing.framework.domain.hrms.EmployeeRoleEntity;
import com.daqing.framework.domain.hrms.PermissionEntity;
import com.daqing.framework.domain.hrms.RoleEntity;
import com.daqing.framework.domain.hrms.RolePermissionEntity;
import com.daqing.framework.domain.hrms.request.RolePermissionRequest;
import com.daqing.framework.domain.hrms.response.RolePermissionResponse;
import com.daqing.framework.utils.PageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;


@Service("rolePermissionService")
public class RolePermissionServiceImpl extends ServiceImpl<RolePermissionMapper, RolePermissionEntity> implements RolePermissionService {

    @Resource
    private AuthRoleDao roleMapper;
    @Resource
    private AuthEmployeeRoleDao userRoleMapper;

    @Resource
    private AuthPermissionDao permissionMapper;

    @Override
    public List<PermissionEntity> listWithTree() {
        List<PermissionEntity>list = this.getBaseMapper().queryPermissionTree();
        return getPermissionTreeList(list,0L);
        /*System.out.println("this.list()=================="+this.list());
        return getPermissionTreeList(this.list(), 0L);*/
    }

    @Override
    public PageUtils queryPage(Integer page, Integer size) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<RolePermissionResponse> systemLogVo = this.getBaseMapper().pageByCondition(new Page(page, size));

        return new PageUtils(systemLogVo);
    }

    private List<PermissionEntity> getPermissionTreeList(List<PermissionEntity> list, Long parentId) {
        return list.stream()
                .filter(item -> parentId.equals(item.getParentId()))
                .peek(item -> item.setChildren(getPermissionTreeList(list, item.getId())))
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort())))
                .collect(Collectors.toList());
    }

    @Override
    public List<RolePermissionResponse> listWithPermissionTree(Long roleId) {
        //根据角色id查询单个角色对应的所有权限
        List<RolePermissionResponse>list = this.getBaseMapper().queryPermissionByRoleId(roleId);
        return getRolePermissionTreeList(list, 0L);
    }

    private List<RolePermissionResponse> getRolePermissionTreeList(List<RolePermissionResponse> list, Long parentId) {
        return list.stream()
                .filter(item -> parentId.equals(item.getParentId()))
                .peek(item -> item.setChildren(getRolePermissionTreeList(list, item.getPermissionId())))
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort())))
                .collect(Collectors.toList());
    }

    @Override
    public int addHrmsRole(RolePermissionRequest rolePermission) {
        int request = this.getBaseMapper().addHrmsRole(rolePermission);
        return request;
    }

    @Override
    public boolean addHrmsRolePermission(List<RolePermissionRequest> rolePermissionList) {
        return this.getBaseMapper().addHrmsRolePermission(rolePermissionList);
    }

    @Override
    public int updateHrmsRole(RolePermissionRequest rolePermission) {
        return this.getBaseMapper().updateHrmsRole(rolePermission);
    }

    @Override
    public boolean deleteHrmsRolePermission(Long roleId) {
        return this.getBaseMapper().deleteHrmsRolePermission(roleId);
    }

    @Override
    public int queryCountByRoleId(Long roleId) {
        int count = this.getBaseMapper().queryCountByRoleId(roleId);
        return count;
    }

    @Override
    public boolean deleteHrmsRoleByRoleId(Long roleId) {
       return this.getBaseMapper().deleteHrmsRoleByRoleId(roleId);
    }

    @Override
    public int uniqueRoleName(String name) {
        return this.getBaseMapper().uniqueRoleName(name);
    }

    @Override
    public List<RolePermissionEntity> queryPermissionArrById(Long roleId) {
        return this.getBaseMapper().queryPermissionArrById(roleId);
    }

    @Override
    public List<RoleEntity> getUserRoles(Integer userId) {
        List<EmployeeRoleEntity> userRoles = userRoleMapper.selectList(new QueryWrapper<EmployeeRoleEntity>().eq("user_id", userId));
        List<RoleEntity> roleList = new LinkedList<>();
        for (EmployeeRoleEntity userRole : userRoles) {
            RoleEntity role = roleMapper.selectById(userRole.getRoleId());
            roleList.add(role);
        }
        return roleList;
    }

    @Override
    public List<PermissionEntity> getList(String jiacan) {
        List<PermissionEntity> permissionList = permissionMapper.selectList(null);
        return permissionList;
    }

}