package com.daqing.financial.hrauth.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.daqing.financial.hrauth.dao.AuthEmployeeRoleDao;
import com.daqing.financial.hrauth.dao.AuthRoleDao;
import com.daqing.financial.hrauth.dao.UserLoginDao;
import com.daqing.financial.hrauth.feign.HrauthFeignService;
import com.daqing.financial.hrauth.handle.SecurityUser;
import com.daqing.financial.hrauth.service.RolePermissionService;
import com.daqing.framework.domain.hrms.EmployeeRoleEntity;
import com.daqing.framework.domain.hrms.RoleEntity;
import com.daqing.framework.domain.hrms.UserEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeTO;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.LinkedList;
import java.util.List;

/**
 * <p> 自定义userDetailsService - 认证用户详情 </p>
 *
 * @author : zhengqing
 * @description :
 * @date : 2019/10/14 17:46
 */
@Slf4j
@Service("userDetailsService")
public class UserDetailsServiceImpl implements UserDetailsService {

    @Resource
    private UserLoginDao userMapper;
    @Resource
    private AuthRoleDao roleMapper;
    @Resource
    private AuthEmployeeRoleDao userRoleMapper;
    @Autowired
    private HrauthFeignService hrauthFeignService;
    @Autowired
    private UserLoginDao userLoginDao;
    @Autowired
    private RolePermissionService permissionService;

    /***
     * 根据账号获取用户信息
     * @param username:
     * @return: org.springframework.security.core.userdetails.UserDetails
     */
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        // 从数据库中取出用户信息
        List<UserEntity> userList = userMapper.selectList(new QueryWrapper<UserEntity>().eq("phone_account", username));
        List<UserEntity> userList2 = userMapper.selectList(new QueryWrapper<UserEntity>().eq("account", username));
        UserEntity user;
        String name = null;
        Long userId;
        // 判断用户是否存在
        if (!CollectionUtils.isEmpty(userList)) {
            user = userList.get(0);
            name = userLoginDao.getNameByUserId(userList.get(0).getId());
            userId = userList.get(0).getId();
        } else if(!CollectionUtils.isEmpty(userList2)){
            user = userList2.get(0);
            name = userLoginDao.getNameByUserId(userList2.get(0).getId());
            userId = userList2.get(0).getId();
        } else {
            throw new UsernameNotFoundException("用户名不存在或已禁用！");
        }
        user.setAccount(name);
        user.setId(userId);
        // 返回UserDetails实现类
        return new SecurityUser(user, getUserRoles(user.getId().intValue()));
    }

    /***
     * 根据token获取用户权限与基本信息
     *
     * @param token:
     * @return: com.zhengqing.config.security.dto.SecurityUser
     */
    public SecurityUser getUserByToken(String token) {
        UserEntity user = null;

        String userId = RedisUtil.get("dq:token:"+token);
        String userEntityStr = RedisUtil.get("dq:userId:"+userId);
        UserEntity systemUser = JSON.parseObject(userEntityStr,UserEntity.class);
        if(systemUser != null){
            user = systemUser;
        }

/*        List<UserEntity> loginList = userMapper.selectList(new QueryWrapper<UserEntity>().eq("token", token));
        if (!CollectionUtils.isEmpty(loginList)) {
            user = loginList.get(0);
        }*/
        return user != null ? new SecurityUser(user, getUserRoles(user.getId().intValue())) : null;
    }

    /**
     * 根据用户id获取角色权限信息
     *
     * @param userId
     * @return
     */
    private List<RoleEntity> getUserRoles(Integer userId) {
//        List<EmployeeRoleEntity> userRoles = userRoleMapper.selectList(new QueryWrapper<EmployeeRoleEntity>().eq("user_id", userId));
//        List<RoleEntity> roleList = new LinkedList<>();
//        for (EmployeeRoleEntity userRole : userRoles) {
//            RoleEntity role = roleMapper.selectById(userRole.getRoleId());
//            roleList.add(role);
//        }
        log.info("来了调用远程接口~~~~~~~~~~~~~~~~~~~~~~~~~~");
        ResponseResult responseResult = hrauthFeignService.getUserRoles(userId);
        List<RoleEntity> roleList = (List<RoleEntity>) responseResult.getData();
        //List<RoleEntity> roleList = permissionService.getUserRoles(userId);
        return roleList;
    }

}
