package com.daqing.financial.hrauth.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrauth.config.WeChatConfig;
import com.daqing.financial.hrauth.dao.AuthEmployeeRoleDao;
import com.daqing.financial.hrauth.dao.HrmsAccessRecordsLogMapper;
import com.daqing.financial.hrauth.dao.UserLoginDao;
import com.daqing.financial.hrauth.feign.HrmsFeignService;
import com.daqing.financial.hrauth.model.HrmsAccessRecordsLog;
import com.daqing.financial.hrauth.service.LoginLogService;
import com.daqing.financial.hrauth.service.TokenService;
import com.daqing.financial.hrauth.service.UserLoginService;
import com.daqing.financial.hrauth.util.*;
import com.daqing.framework.domain.hrms.*;
import com.daqing.framework.domain.hrms.request.*;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.domain.hrms.response.LoginResponse;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.JwtUtils;
import com.daqing.framework.util.Md5Util;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.util.SendUtil;
import lombok.extern.java.Log;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @auther River
 * @date 2020/9/22 15:01
 */
@Service("userLoginService")
public class UserLoginServiceImpl extends ServiceImpl<UserLoginDao, UserEntity> implements UserLoginService{

    @Resource
    private UserLoginDao userLoginDao;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private LoginLogService loginLogService;

    @Resource
    private WeChatConfig weChatConfig;

    @Autowired
    private HrmsFeignService hrmsFeignService;

    @Resource
    private AuthEmployeeRoleDao authEmployeeRoleDao;

    @Resource
    private HrmsAccessRecordsLogMapper hrmsAccessRecordsLogMapper;

/*    @Autowired
    private OdcProperties properties;*/

/*    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        UserEntity user = userLoginDao.getUser(username);
        List<SimpleGrantedAuthority> authorities = new ArrayList<>();
        authorities.add(new SimpleGrantedAuthority("ROLE_ADMIN"));
        System.out.println(user);
        UserDetails userDetails = new User(user.getAccount(),new BCryptPasswordEncoder().encode(user.getPassword()),authorities);

        return userDetails;
    }*/

    /**
     * 验证手机号及验证码
     */
    @Override
    public UserLoginRequest verifyMobile(UserLoginRequest user){
        //校验手机账号是否存在
        int count = userLoginDao.selectPhoneCount(user.getPhoneAccount());
        if (count == 0) {
            ExceptionCast.cast(HrmsCode.PHNOE_ACCOUNT_NOT_EXIST);
        }
        //校验验证码是否正确
        Boolean isCodeTrue = SendUtil.validCode(user.getPhoneAccount(),user.getVerifyCode());
        if(!isCodeTrue){
            ExceptionCast.cast(HrmsCode.VERIFY_CODE_ILLEGAL);
        }
        return user;
    }


    /**
     * 重置密码
     */
    @Override
    public Boolean getBackPwd(ResetPwdRequest user){
        if(!user.getConfirmPwd().equals(user.getNewPwd())){
            ExceptionCast.cast(HrmsCode.TWICE_PWD_DIFFERENT);
        }

        //根据手机号查询相关信息
        UserEntity userEntity = baseMapper.selectByPhoneAccount(user.getPhoneAccount());

        //新密码不能与账号相同
        if(user.getNewPwd().equals(userEntity.getAccount())){
            ExceptionCast.cast(HrmsCode.NEW_PASSWORD_DIFFERENT);
        }

        String newMD5 = Md5Util.md5(user.getNewPwd()+userEntity.getId());
        //根据手机号码修改密码
        int i = userLoginDao.updatePasswordByPhoneAccount(user.getPhoneAccount(),newMD5);
        return i > 0;
    }

    @Transactional
    public ResponseResult login(LoginRequest loginRequest) {
        //判断用户登录方式
        UserEntity userEntity = null;
        System.out.println("type===================="+loginRequest.getType());
        if(loginRequest.getType()==3){//小程序登录
            userEntity = baseMapper.xcxLogin(loginRequest);
        }else{
            userEntity = baseMapper.login(loginRequest);
        }

        String name = null;
        if(userEntity != null){
            //获取员工name
            name = baseMapper.getNameByUserId(userEntity.getId());
            //删除redis里面的userEntity
            JwtUtils.removeUserEntityByUserId(userEntity.getId());
        }
        if(loginRequest.getType() == 1){//手机号码登录

            if(userEntity == null){
                throw new UsernameNotFoundException("用户不存在或已禁用！");
                //return ResponseResult.FAIL(CommonCode.USER_IS_NOT_EXIST.code(), CommonCode.USER_IS_NOT_EXIST.message());
            }
            String newPassword = Md5Util.md5(loginRequest.getPassword()+userEntity.getId());
            if(!newPassword.equals(userEntity.getPassword())){
                throw new UsernameNotFoundException("密码错误！");
                //return ResponseResult.FAIL(CommonCode.PASSWORD_IS_ERROR.code(), CommonCode.PASSWORD_IS_ERROR.message());
            }

        }else {//微信登录
            if(userEntity == null){
                throw new UsernameNotFoundException("用户不存在或已禁用！");
                //return ResponseResult.FAIL(CommonCode.USER_IS_NOT_EXIST.code(), CommonCode.USER_IS_NOT_EXIST.message());
            }
        }
        //根据userId获取roleId
        List<Map<String, Object>> roleList = authEmployeeRoleDao.selectMaps(new QueryWrapper<EmployeeRoleEntity>().eq("user_id", userEntity.getId()));
        List<String> list = new ArrayList<>();
        for (Map map:roleList) {
            list.add(map.get("role_id").toString());
        }
        String roleIds = StringUtils.join(list,",");
        if(userEntity != null){
            //删除redis里面的roleId
            JwtUtils.removeRoleIdByUserId(userEntity.getId());
        }

        //登录成功，token生成
        long times = 86400;//24h
        //long times =60;//1min
        if(loginRequest.getTenDayEffective() == 1){
            times = 864000;
        }
        String token = JwtUtils.createJWT(userEntity.getId()+"",name,"token", times);
        Token userToken = tokenService.getOne(new QueryWrapper<Token>().eq("user_id", userEntity.getId()));
        if(userToken != null){
            //删除redis里面的token
            //JwtUtils.removeTokenByToken(userToken.getToken());
            //数据库重置token
            userToken.setToken(token);
            userToken.setCreateTime(new Date());
            tokenService.updateById(userToken);
        }else {
            userToken = new Token();
            userToken.setUserId(userEntity.getId());
            userToken.setToken(token);
            userToken.setCreateTime(new Date());
            tokenService.save(userToken);
        }
        //设置最新的token到redis
        JwtUtils.putTokenToRedis(userEntity.getId(), token, times);
        //设置userEntity到redis
        JwtUtils.putUserEntityToRedis(userEntity.getId(),userEntity,times);
        //设置最新的roleId到redis
        JwtUtils.putRoleIdToRedis(userEntity.getId(), roleIds, times);

        //新增登录日志
        LoginLog loginLog = loginLogService.getOne(new QueryWrapper<LoginLog>().eq("user_id", userEntity.getId()));
        if(loginLog == null){
            loginLog = new LoginLog();
            loginLog.setUserId(userEntity.getId());
            loginLog.setLoginNum(1);
            loginLog.setCreateTime(new Date());
            loginLog.setNewestTime(new Date());
            if(loginRequest.getType()==3){//客户小程序登录就添加客户小程序标识
                loginLog.setType(1);//标识为客户
            }else{
                loginLog.setType(0);//标识为员工
            }
            loginLogService.save(loginLog);
        }else {
            loginLog.setLoginNum(loginLog.getLoginNum() + 1);
            loginLog.setNewestTime(new Date());
            loginLogService.updateById(loginLog);
        }

        // 更新用户登录信息
        this.getBaseMapper().updateUserLogin(userEntity.getId(),new Date());

        //根据userId查询用户头像地址
        ResponseResult responseResult = hrmsFeignService.getEmployeeByUserId(userEntity.getId().intValue());
        EmployeeEntity employeeEntity = (EmployeeEntity) responseResult.getData();
        String headImg = null;
        if(employeeEntity != null)
            headImg = employeeEntity.getHeadPortaritUrl();
        System.out.println("headImg================"+headImg);

        //根据用户id查询部门名称
        List<Integer> arr = new ArrayList<>();
            arr.add(userEntity.getId().intValue());
        //根据提单人id查询其部门名称
        ResponseResult res = hrmsFeignService.getAccountAndDeptNameById(arr);
        List<LinkedHashMap> employeeMessage = new ArrayList();
        if(res.getData() != null){
            employeeMessage = (List<LinkedHashMap>) res.getData();
        }
        //新增一条PC端的访问记录
        HrmsAccessRecordsLog recordsLog = new HrmsAccessRecordsLog();
        for(LinkedHashMap res2 : employeeMessage) {
            //recordsLog.setDeptName(res2.getDeptName());
            recordsLog.setDeptName(JSONObject.toJSONString(res2.get("deptName")).replace("\"", ""));
        }
        recordsLog.setAccount(userEntity.getAccount());
        recordsLog.setName(name);
        if(loginRequest.getType()==3 || loginRequest.getType()==4){//小程序登录
            recordsLog.setType(1);//用户端类型：0->PC；1->小程序；
        }else{
            recordsLog.setType(0);
        }
        recordsLog.setLoginTime(new Date());
        recordsLog.setUserId(userEntity.getId().intValue());
        hrmsAccessRecordsLogMapper.insert(recordsLog);

        //返回用户信息
        LoginResponse loginResponse = new LoginResponse();
        loginResponse.setId(userEntity.getId());

        if(name == null || name == ""){//名称如果为空，则代表是客户的名称，小程序端登录，名称默认为微信名称
            name = userEntity.getAccount();
        }
        if(headImg != null || headImg != ""){//头像如果为空，则默认为微信头像
            if(userEntity.getHeadPortaritUrl() != null){
                headImg = userEntity.getHeadPortaritUrl();
            }
        }
        loginResponse.setAccount(name);
        loginResponse.setToken(token);
        if(headImg != null)
            loginResponse.setHeadPortaritUrl(headImg);

        return ResponseResult.SUCCESS(loginResponse);
    }

    /**
     * 修改平台超管员密码
     */
    @Override
    public Boolean updatePwd(UpdatePwdRequest user){
        if(!user.getConfirmPwd().equals(user.getNewPwd())){
            ExceptionCast.cast(HrmsCode.TWICE_PWD_DIFFERENT);
        }

        //根据手机号查询相关信息
        //UserEntity userEntity = baseMapper.selectByPhoneAccount(user.getPhoneAccount());
        UserEntity userEntity=baseMapper.selectByManagerId();
        if(userEntity == null){
            ExceptionCast.cast(HrmsCode.PHNOE_ACCOUNT_NOT_EXIST);
        }

        //新密码不能与账号相同
        if(user.getNewPwd().equals(userEntity.getAccount())){
            ExceptionCast.cast(HrmsCode.NEW_PASSWORD_DIFFERENT);
        }

        //检测当前密码是否输入正确
        if(!Md5Util.md5(user.getPassword()+userEntity.getId()).equals(userEntity.getPassword())){
            ExceptionCast.cast(HrmsCode.PASSWORD_ERROR);
        }

        //新密码不能与当前密码保持一致
        if(Md5Util.md5(user.getNewPwd()+userEntity.getId()).equals(userEntity.getPassword())){
            ExceptionCast.cast(HrmsCode.NEWPWD_PASSWORD_SAME);
        }

        String newMD5 = Md5Util.md5(user.getNewPwd()+userEntity.getId());
        //修改平台超管密码
        int i = userLoginDao.updateManagePassword(newMD5);
        return i > 0;
    }

    @Override
    public boolean bindPhoneAndOpenId(BindOpenIdRequest bindOpenIdRequest) {
        //校验验证码是否正确
        Boolean isCodeTrue = SendUtil.validCode(bindOpenIdRequest.getPhoneAccount(),bindOpenIdRequest.getCode());
        if(!isCodeTrue){
            ExceptionCast.cast(HrmsCode.VERIFY_CODE_ILLEGAL);
        }
        //根据手机账号查询是否存在该用户
        int count = userLoginDao.selectPhoneCount(bindOpenIdRequest.getPhoneAccount());
        if(count > 0){
            //存在，直接绑定openId
            userLoginDao.updateOpenIdByPhoneAccount(bindOpenIdRequest.getPhoneAccount(),bindOpenIdRequest.getWechatId());
        }else{
            //不存在，提示该手机号不存在
            ExceptionCast.cast(HrmsCode.PHNOE_ACCOUNT_NOT_EXIST);
        }
        return count>0;
    }

    @Override
    public int isPhoneExist(String mobile) {
        return userLoginDao.selectPhoneCount(mobile);
    }

    @Override
    public ResponseResult wxAuthLogin(String encryptedData, String iv,String code,Integer identify) {
        System.out.println("code****"+code+"encryptedData**************"+encryptedData+"iv**********"+iv);
        System.out.println("来了小程序登录------------------------------------");
        //微信那边的接口，grant_type=authorization_code是固定的
        String url = String.format(weChatConfig.getWX_AUTH_URL(),weChatConfig.getXcxAppid(),weChatConfig.getXcxAppsecret(),code);
        System.out.println("url==============="+url);

        //向微信服务器 使用登录凭证 code 获取 session_key 和 openid
        // 请求参数
        String params = "appid=" + weChatConfig.getXcxAppid() + "&secret=" + weChatConfig.getXcxAppsecret() + "&js_code=" + code + "&grant_type="
                + "authorization_code";
        // 发送请求
        String sr = HttpRequest.sendGet("https://api.weixin.qq.com/sns/jscode2session", params);

        System.out.println("sr==================="+sr);
        // 解析相应内容（转换成json对象）
        cn.hutool.json.JSONObject json = new cn.hutool.json.JSONObject(sr);
        // 获取会话密钥（session_key）
        String session_key = json.get("session_key").toString();
        // 用户的唯一标识（openid）
        String openid = (String) json.get("openid");

        String unionid = json.get("unionid").toString();

        System.out.println("session_key======"+session_key+"openid===="+openid+"unionid===="+unionid);

        Map map = new HashMap();

        //对encryptedData加密数据进行AES解密
        try {
            String result = WXBizDataCrypt.getInstance().decrypt(encryptedData, session_key, iv, "utf-8");
            System.out.println("AES解密值==="+result);
            if (null != result && result.length() > 0) {
                map.put("status", 1);
                map.put("msg", "解密成功");

                JSONObject userInfoJSON = JSONObject.parseObject(result);
                System.out.println("userInfoJSON===="+userInfoJSON);
                Map userInfo = new HashMap();
                // 解密unionId & openId;
                userInfo.put("openId", userInfoJSON.get("openId"));
                userInfo.put("nickName", userInfoJSON.get("nickName"));
                userInfo.put("gender", userInfoJSON.get("gender"));
                userInfo.put("city", userInfoJSON.get("city"));
                userInfo.put("province", userInfoJSON.get("province"));
                userInfo.put("country", userInfoJSON.get("country"));
                userInfo.put("avatarUrl", userInfoJSON.get("avatarUrl"));
                //userInfo.put("unionId", userInfoJSON.get("unionId"));
                userInfo.put("unionId", unionid);

                System.out.println("unionId1============"+userInfoJSON.get("unionId"));
                System.out.println("unionId2============"+userInfoJSON.get("unionId"));
                map.put("userInfo", userInfo);
            } else {
                map.put("status", 0);
                map.put("msg", "解密失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        Map userMap = (Map<String,Object>)map.get("userInfo");

        String unionId = userMap.get("unionId").toString();
        String md5UnionId = Md5Util.md5(unionId);

        System.out.println("字符串格式userMap里面的unionId===="+userMap.get("unionId").toString());
        //根据unionId查询用户是否注册过，如果已经是老用户，则直接调登录接口，如果为新用户，则先保存用户信息，再调登录
        int count = userLoginDao.selectCount(new QueryWrapper<UserEntity>().eq("wechat_id",md5UnionId));

        //根据标识判断是员工还是客户登录，如果为员工登录，则直接登录，无需保存记录，如果为客户登录，则先查询是否已注册
        LoginRequest loginRequest = new LoginRequest();
        loginRequest.setWechatId(md5UnionId);

        loginRequest.setTenDayEffective(2);
        //查询员工数据
        int count2 = userLoginDao.selectUserEmpCount(md5UnionId);
        if(identify==0){//员工小程序登录
            loginRequest.setType(4);
            if(count<=0 || count2<=0){//如果微信绑定的不是员工
                return ResponseResult.FAIL(60001,"您的账号没有权限，无法登录！");
            }
        }else{//客户小程序登录
            //如果员工使用客户小程序登录的情况，关联用户表和员工表查询，如果有数据则代表是员工的数据
            loginRequest.setType(3);
            //如果同时为员工和客户的身份，则在客户小程序默认使用客户的身份
            //或者如果仅为客户身份,则在客户小程序默认使用客户的身份
            if((count==2 && count2>0) || (count==1 && count2<1)){
                loginRequest.setPhone(userMap.get("nickName").toString());//账号默认为微信名称
            }else if((count==1 && count2>0) || (count<=0&&count2<=0)){//如果仅为员工身份，或者不是任何身份，则新增一条客户数据到用户表
                UserEntity userEntity = new UserEntity();
                userEntity.setAccount(userMap.get("nickName").toString());//账号默认为微信名称
                userEntity.setWechatId(md5UnionId);//加密后unionId
                userEntity.setHeadPortaritUrl(userMap.get("avatarUrl").toString());//头像路径
                userEntity.setCreateTime(new Date());//创建时间
                userEntity.setType(loginRequest.getType());//登录类型(1:手机号 2:微信扫码登录 3:客户小程序登录 4：企业小程序登录)
                userLoginDao.insert(userEntity);
                loginRequest.setPhone(userMap.get("nickName").toString());//账号默认为微信名称
            }

        }

        ResponseResult result = login(loginRequest);
        return result;
    }

    @Override
    public int isRemind(Integer isRemind) {
        String userId = this.getUserId();
        UserEntity entity = new UserEntity();
        entity.setIsRemind(isRemind);
        entity.setId(Long.valueOf(userId));

        return userLoginDao.updateById(entity);
    }

    @Override
    public String readIsRemind() {
        String userId = this.getUserId();
        UserEntity entity = new UserEntity();
        entity.setId(Long.valueOf(userId));

        return userLoginDao.readIsRemind(entity.getId());
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:" + token);
    }

    }
