package com.daqing.financial.hrauth.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrauth.config.WeChatConfig;
import com.daqing.financial.hrauth.dao.UserLoginDao;
import com.daqing.financial.hrauth.handle.MyAuthenticationToken;
import com.daqing.financial.hrauth.handle.SecurityUser;
import com.daqing.financial.hrauth.service.UserLoginService;
import com.daqing.financial.hrauth.service.UserService;
import com.daqing.financial.hrauth.util.HttpUtils;
import com.daqing.framework.domain.hrms.UserEntity;
import com.daqing.framework.domain.hrms.request.LoginRequest;
import com.daqing.framework.domain.hrms.response.LoginResponse;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.Md5Util;
import com.daqing.framework.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.Map;

@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserLoginDao, UserEntity> implements UserService {

    @Autowired
    private WeChatConfig weChatConfig;

    @Resource
    private UserLoginDao userMapper;

    @Resource
    private UserLoginService userLoginService;

    @Resource
    private UserDetailsService userDetailsService;

    @Override
    public UserEntity saveWeChatUser(String code,HttpServletResponse response) {

        String accessTokenUrl = String.format(weChatConfig.getOPEN_ACCESS_TOKEN_URL(),weChatConfig.getOpenAppid(),weChatConfig.getOpenAppsecret(),code);
        System.out.println("code===="+code);
        System.out.println("accessTokenUrl===="+accessTokenUrl);

        //获取access_token
        Map<String ,Object> baseMap =  HttpUtils.doGet(accessTokenUrl);

        if(baseMap == null || baseMap.isEmpty()){ return  null; }
        String accessToken = (String)baseMap.get("access_token");
        String openId  = (String) baseMap.get("openid");
        String unionId  = (String) baseMap.get("unionid");
        System.out.println("微信扫码加密前openId============="+openId+"unionId=============="+unionId+"baseMap============"+baseMap);
        //对code(回调过来的openid)加密
        String md5OpenId = Md5Util.md5(openId);
        String md5UnionId = Md5Util.md5(unionId);
        log.info("accessToken========> "+accessToken+"openId========> "+openId+"code=======> "+code+"md5OpenId============>"+md5OpenId);

        UserEntity dbUser = userMapper.findByOpenid(md5UnionId);

        if(dbUser!=null && dbUser.getPhoneAccount()!=null) { //dbUser不为空，存在直接让它登录
            log.info("来了微信扫码登录~~~~~~~~~~~~~~~~~~~~~~~~~~~");
            LoginRequest loginRequest = new LoginRequest();
            loginRequest.setTenDayEffective(1);
            loginRequest.setType(2);
            loginRequest.setWechatId(md5UnionId);
            ResponseResult login = userLoginService.login(loginRequest);
            LoginResponse data = (LoginResponse) login.getData();

            log.info("token========================"+data.getToken()+"   account=============="+data.getAccount());
            dbUser.setToken(data.getToken());
            dbUser.setAccount(data.getAccount());
            dbUser.setHeadPortaritUrl(data.getHeadPortaritUrl());
            dbUser.setId(data.getId());


/*            SecurityUser userInfo = (SecurityUser) userDetailsService.loadUserByUsername(dbUser.getPhoneAccount());

            userInfo.getCurrentUserInfo().setToken(data.getToken());
            new UsernamePasswordAuthenticationToken(userInfo, null, userInfo.getAuthorities());*/
            return dbUser;
        }

        if(dbUser == null){//openId不存在，返回绑定手机号页面,须另写绑定手机号接口
            log.info("dbUser为空，openId不存在，请先绑定手机号哦~~~");
            try {
//                response.sendRedirect("https://www.feifanhitech.com/index/#/bind-phone?matched="+md5UnionId);
                response.sendRedirect("https://test.feifanhitech.com/dq/index.html#/bind-phone?matched="+md5UnionId);//跳转绑定手机号页面
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;

        //获取用户基本信息
/*        String userInfoUrl = String.format(weChatConfig.getOPEN_USER_INFO_URL(),accessToken,openId);
        Map<String ,Object> baseUserMap =  HttpUtils.doGet(userInfoUrl);

        if(baseUserMap == null || baseUserMap.isEmpty()){
            return  null;
        }

        UserEntity user = new UserEntity();
        user.setWechatId(openId);
        user.setCreateTime(new Date());
        userMapper.insert(user);
        return user;*/

/*        String nickname = (String)baseUserMap.get("nickname");

        Double sexTemp  = (Double) baseUserMap.get("sex");
        int sex = sexTemp.intValue();
        String province = (String)baseUserMap.get("province");
        String city = (String)baseUserMap.get("city");
        String country = (String)baseUserMap.get("country");
        String headimgurl = (String)baseUserMap.get("headimgurl");
        StringBuilder sb = new StringBuilder(country).append("||").append(province).append("||").append(city);
        String finalAddress = sb.toString();
        try {
            //解决乱码
            nickname = new String(nickname.getBytes("ISO-8859-1"), "UTF-8");
            finalAddress = new String(finalAddress.getBytes("ISO-8859-1"), "UTF-8");

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }*/
    }

    /**
     * 绑定微信的回调
     */
    @Override
    public Boolean weChatBinding(String code, HttpServletResponse response,String state) {

        String accessTokenUrl = String.format(weChatConfig.getOPEN_ACCESS_TOKEN_URL(),weChatConfig.getOpenAppid(),weChatConfig.getOpenAppsecret(),code);
        //获取access_token
        Map<String,Object> baseMap =  HttpUtils.doGet(accessTokenUrl);
        if(baseMap == null || baseMap.isEmpty()){
            return false;
        }
        String openId  = (String) baseMap.get("openid");
        String unionId  = (String) baseMap.get("unionid");
        log.info("openId========="+openId+"unionId===="+unionId);
        String md5OpenId = Md5Util.md5(openId);
        String md5UnionId = Md5Util.md5(unionId);
        // 判断要绑定的微信是否已经存在
        // Integer countWeChatId = this.getBaseMapper().countWeChatId(md5UnionId);
        int countWeChatId = this.getBaseMapper().selectUserEmpCount(md5UnionId);
        String token = this.getBaseMapper().getTokenByUserId(Long.parseLong(state));
        if (countWeChatId > 0){
                try {
                    log.info("微信重复了,我走到了这里.............................");
                    response.sendRedirect("https://test.feifanhitech.com/dq/index.html#/workbench-manpower?token="+token+"&message=1");
//                    response.sendRedirect("https://www.feifanhitech.com/index/#/workbench-manpower?token="+token+"&message=1");
                } catch (IOException e) {
                    e.printStackTrace();
                }
        }else {
            Boolean result = this.getBaseMapper().updateWeChat(Long.parseLong(state), md5UnionId);
            log.info("绑定成功---------------------------------------");
            if (result){
                try {
                    log.info("转发成功---------------------------------------");
                    response.sendRedirect("https://test.feifanhitech.com/dq/index.html#/workbench-manpower?token="+token);
//                    response.sendRedirect("https://www.feifanhitech.com/index/#/workbench-manpower?token="+token);
                }catch (IOException e){
                    e.printStackTrace();
                }
            }
        }
        return false;
    }
}
