package com.daqing.financial.hrms.controller;

import com.daqing.financial.hrms.DeptControllerApi;
import com.daqing.financial.hrms.service.DeptService;
import com.daqing.framework.domain.hrms.DepartmentVO;
import com.daqing.framework.domain.hrms.DeptEntity;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.domain.hrms.request.DeptAddRequest;
import com.daqing.framework.model.response.ResponseResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * 记录部门信息
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@RestController
@RequestMapping("hrms/dept")
public class DeptController implements DeptControllerApi {

    @Autowired
    private DeptService deptService;

    /**
     * 保存一条部门信息
     *
     * @param deptAddRequest 新增部门请求体对象
     * @return 统一响应对象
     */
    @Override
    @PostMapping("/save")
    //@RequiresPermissions("hrms:dept:save")
    public ResponseResult save(@RequestBody DeptAddRequest deptAddRequest) {
        boolean result = deptService.saveDept(deptAddRequest);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 树形结构展示所有部门信息
     *
     * @return 统一响应对象
     */
    @Override
    @GetMapping("/tree")
    //@RequiresPermissions("hrms:dept:tree")
    public ResponseResult tree() {
        List<DeptEntity> list = deptService.listWithTree();
        return new ResponseResult<List<DeptEntity>>().SUCCESS(list);
    }

    /**
     * 拖拉拽重新排序，批量修改部门信息
     *
     * @param depts 部门实体类数组
     * @return 统一响应对象
     */
    @Override
    @PutMapping("/update/sort")
    //@RequiresPermissions("hrms:dept:update_sort")
    public ResponseResult updateSort(@RequestBody List<DeptEntity> depts) {
        return deptService.updateBatchById(depts) ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 信息
     */
    @GetMapping("/info/{id}")
    //@RequiresPermissions("hrms:dept:info")
    public ResponseResult info(@PathVariable("id") Long id){
		DeptEntity dept = deptService.getById(id);
        return new ResponseResult<DeptEntity>().SUCCESS(dept);
    }

    /**
     * 修改
     */
    @PutMapping("/update")
    //@RequiresPermissions("hrms:dept:update")
    public ResponseResult update(@RequestBody DeptEntity dept){
        boolean result = deptService.updateDept(dept);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 删除
     */
    @DeleteMapping("/delete")
    //@RequiresPermissions("hrms:dept:delete")
    public ResponseResult delete(@RequestBody Long[] ids){
        boolean result = deptService.deleteByIds(ids);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     *所有部门及员工
     */
    @GetMapping("/trees")
    public ResponseResult trees(@RequestParam(value = "account", required = false) String account) {
        return new ResponseResult<List<DepartmentVO>>().SUCCESS(deptService.listDepartmentAndEmployeeTree(account));
    }

    /**
     *所有部门及员工
     */
    @GetMapping("/empTrees")
    public ResponseResult empTrees(@RequestParam(value = "account", required = false) String account) {
        return new ResponseResult<List<DepartmentVO>>().SUCCESS(deptService.listEmployeeTrees(account));
    }


    /**
     * 该部门下所有子部门及部门下员工
     */
    @Override
    @GetMapping("/children")
    public ResponseResult children(Long id){
        return new ResponseResult<DepartmentVO>().SUCCESS(deptService.getDepartmentAndEmployee(id));
    }

    /**
     * 根据部门id获取该部门下所有的员工
     */
    @GetMapping("/listEmployeeByDeptId")
    @Override
    public ResponseResult listEmployeeByDeptId(@RequestParam("deptId") Long deptId) {

        return ResponseResult.SUCCESS(deptService.listEmployeeByDeptId(deptId));
    }

    /**
     * 获取所有的部门id和名称(用户团队效率的部门筛选)
     */
    @GetMapping("/listDepartment")
    public ResponseResult listDepartment(){
        return ResponseResult.SUCCESS(deptService.listDepartment());
    }

    /**
     * 根据员工id获取对应部门下所有的客户经理
     */
    @GetMapping("/listEmployeeById")
    public ResponseResult listEmployeeById(@RequestParam("id") Long id){

        return ResponseResult.SUCCESS(deptService.listEmployeeById(id));
    }


}
