package com.daqing.financial.hrms.controller;


import com.daqing.financial.hrms.EmployeeControllerApi;
import com.daqing.financial.hrms.service.EmployeeService;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.domain.crms.request.EmployeeDisableRequest;
import com.daqing.framework.domain.guarantee.po.DgGuaranteePO;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.domain.hrms.UserEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeInfoVO;
import com.daqing.framework.domain.hrms.ext.EmployeeTO;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.domain.hrms.request.EmployeeRequest;
import com.daqing.framework.domain.hrms.response.RolePermissionResponse;
import com.daqing.framework.enums.OperationType;
import com.daqing.framework.enums.OperationUnit;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;


/**
 * 记录员工的基本信息，如姓名、电话、部门等
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@RestController
@RequestMapping("hrms/employee")
public class EmployeeController implements EmployeeControllerApi {

    @Autowired
    private EmployeeService employeeService;

    @GetMapping("/list")
    @Log(detail = "员工列表",level = 3,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.SELECT)
    //@RequiresPermissions("hrms:employee:info")
    public ResponseResult list(@RequestParam("page") Integer page,
                               @RequestParam("size") Integer size,
                               EmployeeRequest employeeRequest) {
        PageUtils data = employeeService.queryPage(page, size, employeeRequest);
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    /**
     * 详情
     */
    @Log(detail = "员工详情",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.SELECT)
    @GetMapping("/info")
    //@RequiresPermissions("hrms:employee:info")
    public ResponseResult info(@RequestParam("id") Long id) {

        EmployeeInfoVO data = employeeService.info(id);

        return new ResponseResult<EmployeeInfoVO>().SUCCESS(data);
    }

    /**
     * 权限树
     */
    @Log(detail = "员工权限树",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.SELECT)
    @Override
    @PostMapping("/tree")
    public ResponseResult tree(@RequestBody List<Long> ids) {
        List<RolePermissionResponse> result = employeeService.permissionTreeList(ids);
        return ResponseResult.SUCCESS(result);
    }

    /**
     * 修改
     */
    @Override
    @PostMapping("/update")
    @Log(detail = "修改员工信息",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.UPDATE)
    public ResponseResult update(@RequestBody @Valid EmployeeInfoVO employeeInfoVO) {

        Boolean result = employeeService.update(employeeInfoVO);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }

    /**
     * 保存
     */
    @PostMapping("/save")
    @Log(detail = "保存员工信息",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.INSERT)
    //@RequiresPermissions("hrms:employee:save")
    public ResponseResult save(@RequestBody @Valid EmployeeInfoVO employeeInfoVO) {

        Boolean result = employeeService.save(employeeInfoVO);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.SAVE_FAIL);
    }

    /**
     * 导出
     */
    @Log(detail = "导出员工信息",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.SELECT)
    @GetMapping("/excelExport")
    @Override
    public ResponseResult excelExport(HttpServletResponse response) {

        Boolean result = employeeService.excelExport(response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 导出模板
     */
    @Log(detail = "导出模板",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.SELECT)
    @GetMapping("/excelTemplate")
    @Override
    public ResponseResult excelTemplate(HttpServletResponse response) {

        Boolean result = employeeService.excelTemplate(response);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 导入
     */
    @Log(detail = "导入员工信息",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.INSERT)
    @PostMapping("/excelImport")
    @Override
    public ResponseResult excelImport(MultipartFile file) {

        Boolean result = employeeService.excelImport(file);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 删除
     */
    @Log(detail = "删除员工信息",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.DELETE)
    @DeleteMapping("/delete")
    //@RequiresPermissions("hrms:employee:delete")
    public ResponseResult delete(@RequestBody List<Long> ids) {
        Boolean result = employeeService.delete(ids);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.DELETE_FAIL);
    }

    /**
     * 启用
     */
    @Log(detail = "启用",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.UPDATE)
    @Override
    @GetMapping("/start")
    public ResponseResult start(@RequestParam("id") Long id) {

        Boolean result = employeeService.start(id);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 禁用
     */
    @Log(detail = "禁用",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.UPDATE)
    @Override
    @PostMapping("/stop")
    public ResponseResult stop(@RequestBody @Valid EmployeeDisableRequest employeeDisableRequest) {

        Boolean result = employeeService.stop(employeeDisableRequest);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.DISABLE_FAIL);
    }

    /**
     * 禁用详情
     */
    @Log(detail = "禁用详情",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.SELECT)
    @GetMapping("/forbidInfo")
    @Override
    public ResponseResult forbidInfo(@RequestParam("id") Long id) {

        UserEntity user = employeeService.forbidInfo(id);

        return new ResponseResult<UserEntity>().SUCCESS(user);
    }

    /**
     * 重置密码
     */
    @Log(detail = "重置密码",level = 4,operationUnit = OperationUnit.EMPLOYEE,operationType = OperationType.SELECT)
    @Override
    @GetMapping("/reset")
    public ResponseResult reset(@RequestParam("id") Long id) {
        Boolean result = employeeService.reset(id);
        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.RESET_FAIL);
    }

    /**
     * 根据id查询员工的基本信息和部门信息
     *
     * @param ids
     * @return
     */
    //@Override
    @GetMapping("/getEmployeeAndDeptById")
    public ResponseResult getEmployeeAndDeptById(@RequestParam("ids") Long[] ids) {
        List<EmployeeTO> employeeTO = employeeService.getEmployeeAndDeptById(Arrays.asList(ids));
        return new ResponseResult<List<EmployeeTO>>().SUCCESS(employeeTO);
    }

    @GetMapping("/getAccountAndDeptNameByIds")
    public ResponseResult getAccountAndDeptNameByIds(@RequestParam("ids") Long[] ids) {
        List<EmployeeTO> employeeTO = employeeService.getAccountAndDeptNameByIds(Arrays.asList(ids));
        return new ResponseResult<List<EmployeeTO>>().SUCCESS(employeeTO);
    }

    /**
     * 根据id查询员工姓名
     *
     * @param id
     * @return
     */
    @Override
    @GetMapping("/getEmployeeById")
    public ResponseResult getEmployeeById(@RequestParam("id") Long id) {
        return new ResponseResult<EmployeeEntity>().SUCCESS(employeeService.getEmployeeById(id));
    }

    /**
     * 查询所有员工姓名和账户id
     */
    @GetMapping("/listEmployeeName")
    public ResponseResult listEmployeeName() {
        return new ResponseResult<List<EmployeeVO>>().SUCCESS(employeeService.listEmployeeName());
    }

    /**
     * 根据userId查询员工头像
     *
     * @param userId
     * @return
     */
    @GetMapping("/getEmployeeByUserId")
    public ResponseResult getEmployeeByUserId(@RequestParam("userId") Long userId) {
        return new ResponseResult<EmployeeEntity>().SUCCESS(employeeService.getEmployeeByUserId(userId));
    }

    /**
     * 根据userId获取用户名以及部门名称
     *
     * @param ids
     * @return
     */
    @PostMapping("/getAccountAndDeptNameById")
    public ResponseResult getAccountAndDeptNameById(@RequestBody List<Integer> ids) {
        return new ResponseResult<>().SUCCESS(employeeService.getAccountAndDeptNameById(ids));
    }

    /**
     * 根据userId获取对应的员工姓名
     */
    @GetMapping("/listEmpNameByUserId")
    public ResponseResult listEmpNameByUserId(@RequestParam("userIds") Integer[] userIds) {

        return new ResponseResult<List<EmployeeVO>>().SUCCESS(employeeService.listEmpNameByUserId(Arrays.asList(userIds)));
    }

    /**
     * 新增用户角色
     *
     * @param userId
     * @return
     */
    @GetMapping("/insertUserRole")
    public ResponseResult insertUserRole(@RequestParam("userId") Long userId, @RequestParam("roleId") Long roleId) {
        return new ResponseResult<>().SUCCESS(employeeService.insertUserRole(userId, roleId));
    }

    @PostMapping("/insertUsrRole")
    public ResponseResult insertUsrRole(@RequestBody List<DgGuaranteePO> list) {
        return new ResponseResult<>().SUCCESS(employeeService.insertUsrRole(list));
    }

    @GetMapping("/saveUserRole")
    public ResponseResult saveUserRole(@RequestParam("userId") Long userId, @RequestParam("roleId") Long roleId) {
        return ResponseResult.SUCCESS(employeeService.saveUserRole(userId, roleId));
    }

    @GetMapping("/selectUserRole")
    public ResponseResult selectUserRole(@RequestParam("userId") Long userId, @RequestParam("roleId") Long roleId) {
        int i = employeeService.selectUserRole(userId, roleId);
        return ResponseResult.SUCCESS(i);
    }


    /**
     * 根据用户id获取所在部门ids
     */
    @ApiOperation(value = "根据用户id获取部门")
    @GetMapping("/getDepartmentIds")
    public String getDepartmentIds(Integer userId) {
        return employeeService.getDeptIds(userId);
    }

    /**
     * 根据id查询员工姓名和工号
     *
     * @param id
     * @return
     */
    @GetMapping("/getEmpmsgById")
    public EmployeeEntity getEmpmsgById(@RequestParam("id") Long id) {
        return employeeService.getEmpmsgById(id);
    }

    /**
     * 查询超级管理员下面的userId
     *
     * @param
     * @return
     */
    @GetMapping("/getUserIdByRoleId")
    public ResponseResult getUserIdByRoleId() {
        return ResponseResult.SUCCESS(employeeService.getUserIdByRoleId());
    }
}
