package com.daqing.financial.hrms.controller;

import com.daqing.financial.hrms.PermissionControllerApi;
import com.daqing.financial.hrms.service.PermissionService;
import com.daqing.framework.domain.hrms.PermissionEntity;
import com.daqing.framework.domain.hrms.RolePermissionEntity;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.RedisUtil;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;


/**
 * 记录菜单权限
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@RestController
@RequestMapping("hrms/permission")
public class PermissionController implements PermissionControllerApi {
    @Autowired
    private PermissionService permissionService;

    @Override
    public ResponseResult tree() {
        List<PermissionEntity> list = permissionService.listWithTree();
        return new ResponseResult<List<PermissionEntity>>().SUCCESS(list);
    }

    /**
     * 信息
     */
    @Override
    @GetMapping("/info")
    //@RequiresPermissions("hrms:permission:info")
    public ResponseResult info(@RequestParam("id") Long id) {
        PermissionEntity permission = permissionService.getById(id);
        return new ResponseResult<PermissionEntity>().SUCCESS(permission);
    }

    /**
     * 保存
     */
    @Override
    @PostMapping("/save")
    //@RequiresPermissions("hrms:permission:save")
    public ResponseResult save(@RequestBody PermissionEntity permission) {
        permissionService.save(permission);
        return ResponseResult.SUCCESS();
    }


    @Override
    public ResponseResult updateSort(PermissionEntity[] permissions) {
        permissionService.updateBatchById(Arrays.asList(permissions));
        return ResponseResult.SUCCESS();
    }

    /**
     * 修改
     */
    @Override
    @PutMapping("/update")
    //@RequiresPermissions("hrms:permission:update")
    public ResponseResult update(@RequestBody PermissionEntity permission) {
        permissionService.updateById(permission);
        return ResponseResult.SUCCESS();
    }

    /**
     * 删除
     */
    @Override
    @DeleteMapping("/delete")
    //@RequiresPermissions("hrms:permission:delete")
    public ResponseResult delete(@RequestBody Long[] ids) {
        permissionService.removeByIds(Arrays.asList(ids));
        return ResponseResult.SUCCESS();
    }

    @Transactional
    @GetMapping("/queryPermissionList")
    public ResponseResult queryPermissionList() {
        List<PermissionEntity> permissionList = permissionService.queryPermissionList();

        return new ResponseResult<List<PermissionEntity>>().SUCCESS(permissionList);
    }

    @Transactional
    @GetMapping("/queryRolePermissionList")
    public ResponseResult queryRolePermissionList(@RequestParam("id") Long id) {
        List<RolePermissionEntity> permissionEntityList = permissionService.queryRolePermissionList(id);

        return new ResponseResult<List<RolePermissionEntity>>().SUCCESS(permissionEntityList);
    }

/*    @Transactional
    @GetMapping("/queryPermissionsByRoleIds")
    public ResponseResult queryPermissionsByRoleIds(@RequestParam("ids") Long[] ids) {
        List<RolePermissionEntity> permissionEntityList = permissionService.queryPermissionsByRoleIds(Arrays.asList(ids));

        return new ResponseResult<>().SUCCESS(permissionEntityList);
    }

    @Transactional
    @GetMapping("/queryPermissionsByIds")
    public ResponseResult queryPermissionsByIds(@RequestParam("ids") Integer[] ids) {
        List<PermissionEntity> permissionEntityList = permissionService.queryPermissionsByIds(Arrays.asList(ids));

        return new ResponseResult<List<RolePermissionEntity>>().SUCCESS(permissionEntityList);
    }*/

    @Transactional
    @GetMapping("/getPermission")
    @ApiOperation(value = "获取个人权限列表", notes = "获取个人权限列表")
    public ResponseResult getPermission() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        //获取当前用户拥有的角色ids
        String roleIds= RedisUtil.get("dq:userRole:" + userId);
        if(roleIds.equals(null) || roleIds.equals("")){
            return ResponseResult.FAIL(40010,"无登录权限，请联系管理员处理！");
        }
        List list = permissionService.getPermission(roleIds);
        return new ResponseResult<List<RolePermissionEntity>>().SUCCESS(list);
    }

}
