package com.daqing.financial.hrms.controller;

import com.daqing.financial.hrms.PositionControllerApi;
import com.daqing.financial.hrms.service.PositionService;
import com.daqing.framework.domain.hrms.PositionEntity;
import com.daqing.framework.domain.hrms.request.PositionRequest;
import com.daqing.framework.domain.hrms.response.PositionResponse;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;


/**
 * 记录职位信息
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@RestController
@RequestMapping("hrms/position")
public class PositionController implements PositionControllerApi {
    @Autowired
    private PositionService positionService;

    @Override
    @PostMapping("/save")
    public ResponseResult save(@RequestBody PositionRequest positionRequest) {
        boolean result = positionService.savePosition(positionRequest);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    @Override
    @GetMapping("/list")
    public ResponseResult list(@RequestParam("page") Integer page,
                               @RequestParam("size") Integer size,
                               @RequestParam(value = "queryMsg", required = false) String queryMsg) {
        PageUtils data = positionService.queryPage(page, size, queryMsg);
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

    @Override
    @PutMapping("/update")
    public ResponseResult update(@RequestBody PositionRequest positionRequest) {
        boolean result = positionService.updatePosition(positionRequest);
        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    @GetMapping("/info")
    //@RequiresPermissions("hrms:position:info")
    public ResponseResult info(@RequestParam("id") Long id) {
        PositionResponse position = positionService.getPositionById(id);
        return new ResponseResult<PositionResponse>().SUCCESS(position);
    }


    /**
     * 删除
     */
    @DeleteMapping("/delete")
    //@RequiresPermissions("hrms:position:delete")
    public ResponseResult delete(@RequestBody Long[] ids) {
        positionService.removePositionByIds(Arrays.asList(ids));
        return ResponseResult.SUCCESS();
    }

    /**
     * 所有的职位id和名称
     */
    @PostMapping("/listPositionIdAndName")
    @Override
    public ResponseResult listPositionIdAndName(@RequestBody List<Long> deptIds) {

        return ResponseResult.SUCCESS(positionService.listPositionIdAndName(deptIds));
    }

    /**
     * 所有的角色id和名称
     */
    @GetMapping("/listRoleIdAndName")
    @Override
    public ResponseResult listRoleIdAndName() {
        return new ResponseResult<PositionResponse>().SUCCESS(positionService.listRoleIdAndName());
    }
}
