package com.daqing.financial.hrms.controller;

import com.baomidou.mybatisplus.extension.api.R;
import com.daqing.financial.hrms.entity.vo.EmployeeVO;
import com.daqing.financial.hrms.entity.vo.UserRoleVO;
import com.daqing.financial.hrms.service.RoleService;
import com.daqing.framework.domain.hrms.RoleEntity;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * 员工角色表
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@RestController
@RequestMapping("hrms/role")
public class RoleController {
    @Autowired
    private RoleService roleService;

    /**
     * 列表
     */
    @GetMapping("/list")
    //@RequiresPermissions("hrms:role:list")
    public ResponseResult list(@RequestParam Map<String, Object> params) {
        PageUtils page = roleService.queryPage(params);

        return ResponseResult.SUCCESS();
    }

    //查询角色名称
    @GetMapping("/roleList")
    public ResponseResult roleList() {
        List<RoleEntity> list = roleService.list();
        return ResponseResult.SUCCESS(list);
    }


    /**
     * 信息
     */
    @GetMapping("/info/{id}")
    //@RequiresPermissions("hrms:role:info")
    public ResponseResult info(@PathVariable("id") Long id) {
        RoleEntity role = roleService.getById(id);

        return ResponseResult.SUCCESS();
    }

    /**
     * 保存
     */
    @PostMapping("/save")
    //@RequiresPermissions("hrms:role:save")
    public ResponseResult save(@RequestBody RoleEntity role) {
        roleService.save(role);

        return ResponseResult.SUCCESS();
    }

    /**
     * 修改
     */
    @PutMapping("/update")
    //@RequiresPermissions("hrms:role:update")
    public ResponseResult update(@RequestBody RoleEntity role) {
        roleService.updateById(role);

        return ResponseResult.SUCCESS();
    }

    /**
     * 删除
     */
    @DeleteMapping("/delete")
    //@RequiresPermissions("hrms:role:delete")
    public ResponseResult delete(@RequestBody Long[] ids) {
        roleService.removeByIds(Arrays.asList(ids));

        return ResponseResult.SUCCESS();
    }

    @ApiOperation(value = "根据角色查询员工")
    @GetMapping("getEmployeesByRoleId")
    public ResponseResult getEmployeesByRoleId(
            @ApiParam(name = "roleId", value = "角色id", required = true)
            @RequestParam("roleId") String roleId){
        List<EmployeeVO> employees = roleService.getEmployeesByRoleId(roleId);
        return ResponseResult.SUCCESS(employees);
    }

    @ApiOperation(value = "更新用户角色")
    @PostMapping("updateUserRole")
    public ResponseResult updateUserRole(@RequestBody UserRoleVO userRoleVO){
        List<String> userIds = userRoleVO.getUserIds();
        if (userIds.size()==0){
            return ResponseResult.FAIL(20000,"请至少选择一个用户！");
        }
        boolean save = roleService.saveUserRole(userRoleVO);
        return ResponseResult.SUCCESS(save);
    }

}
