package com.daqing.financial.hrms.controller;

import com.daqing.financial.hrms.UserControllerApi;
import com.daqing.financial.hrms.service.UserService;
import com.daqing.framework.domain.hrms.UserEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeAndUserVO;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.utils.PageUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Arrays;
import java.util.Map;


/**
 * 人资管理系统，员工账号信息表，用于存储员工账号密码等登录相关数据
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@Slf4j
@RestController
@RequestMapping("hrms/user")
public class UserController implements UserControllerApi {

    @Autowired
    private UserService userService;

    /**
     * 信息
     */
    @GetMapping("/info")
    //@RequiresPermissions("hrms:user:info")
    public ResponseResult info() {

        EmployeeAndUserVO info = userService.info();

        return new ResponseResult<EmployeeAndUserVO>().SUCCESS(info);
    }

    /**
     * 修改
     */
    @PostMapping("/update")
    @Override
    public ResponseResult update(@RequestBody @Valid EmployeeAndUserVO employeeAndUserVO) {

        Boolean result = userService.update(employeeAndUserVO);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }

    /**
     * 图像文件上传
     */
    @PostMapping("/uploadFile")
    public ResponseResult uploadFile(@RequestParam("file") MultipartFile file){

        String result = userService.uploadFile(file);

        return ResponseResult.SUCCESS(result);
    }

    /**
     * 获取验证码
     */
    @GetMapping("/code")
    @Override
    public ResponseResult code(@RequestParam("phone") String phone) {

        Boolean result = userService.getVerificationCode(phone);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 修改密码
     */
    @GetMapping("/updatePassword")
    @Override
    public ResponseResult updatePassword(@RequestParam("primaryPassword") String primaryPassword,
                                         @RequestParam("newPassword") String newPassword,
                                         @RequestParam("affirmNewPassword") String affirmNewPassword) {
        Boolean result = userService.updatePassword(primaryPassword, newPassword, affirmNewPassword);

        return result ? ResponseResult.SUCCESS() : new ResponseResult(CommonCode.UPDATE_FAIL);
    }

    /**
     * 绑定手机号
     */
    @GetMapping("/binding")
    @Override
    public ResponseResult binding(String phone, String code) {

        Boolean result = userService.bindingPhoneAccount(phone, code);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 取消微信绑定
     */
    @GetMapping("/cancel")
    @Override
    public ResponseResult cancel() {

        Boolean result = userService.cancel();

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 更新员工图像(小程序端)
     * @param headPortraitUrl 图像路径
     */
    @GetMapping("/update/url")
    public ResponseResult updateHeadPortraitUrl(@RequestParam("headPortraitUrl") String headPortraitUrl){

        Boolean result = userService.updateHeadPortraitUrl(headPortraitUrl);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 只有当访问用户具有 ROLE_ADMIN 权限时才能访问，否则返回 401 未授权。
     * 通过 Authentication 参数或者 SecurityContextHolder.getContext().getAuthentication() 可以拿到授权信息进行查看。
     * @param authentication
     * @return
     */
/*    @GetMapping(value = "get")
    // 访问路径`/admin` 具有`ADMIN`角色权限   【这种是写死方式】
//    @PreAuthorize("hasPermission('/admin','ADMIN')")
    @PreAuthorize("hasAnyRole('ROLE_ADMIN')")
    public Object get(Authentication authentication){
        //Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        authentication.getCredentials();
        OAuth2AuthenticationDetails details = (OAuth2AuthenticationDetails)authentication.getDetails();
        String token = details.getTokenValue();
        return token;
    }*/
}
