package com.daqing.financial.hrms.dao;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.daqing.framework.domain.guarantee.po.DgGuaranteePO;
import com.daqing.framework.domain.guarantee.response.EmployeeMessageResponse;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.daqing.framework.domain.hrms.UserEntity;
import com.daqing.framework.domain.hrms.ext.*;
import com.daqing.framework.domain.hrms.request.EmployeeRequest;
import com.daqing.framework.model.response.ResponseResult;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * 记录员工的基本信息，如姓名、电话、部门等
 *
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@Mapper
public interface EmployeeDao extends BaseMapper<EmployeeEntity> {

    List<EmployeeTO> getEmployeeAndDeptById(@Param("ids") List<Long> ids);

    EmployeeEntity getEmployeeById(Long id);

    List<EmployeeVO> listEmployeeByDeptId(Long id);

    List<UserDeptVO> employeeListByDeptId(@Param("id")Long id, @Param("account")String account);

    List<EmployeeVO> listEmployeeByDeptIdAndEmpId(@Param("id") Long deptId, @Param("name") String name, @Param("empIds") List<Long> empIds);

    List<EmployeeVO> listEmployeeName();

    IPage<EmployeeVO> pageByCondition(Page page, @Param("employee") EmployeeEntity employee);

    IPage<EmployeeListVO> pageList(Page page, @Param("employee") EmployeeRequest employeeRequest);

    List<EmployeeListVO> excelList();

    IPage<EmployeeListVO> pageListByDept(Page page, @Param("employee") EmployeeRequest employeeRequest);

    List<Long> queryIds(@Param("ids") List<Long> ids);

    Boolean deleteByIds(@Param("ids") List<Long> ids);

    Integer getStatus(Long id);

    Boolean startOrStop(UserEntity user);

    UserEntity forbidInfo(Long id);

    List<UserEntity> listDisableTime();

    Long getUserId(Long id);

    Boolean updatePassword(@Param("id") Long id, @Param("password") String password);

    EmployeeInfoVO getInfo(Long id);

    Boolean updateEmployee(EmployeeEntity employeeEntity);

    Boolean updateUser(@Param("id") Long id, @Param("account") String account, @Param("modifyTime") Date modifyTime);

    Boolean deleteDepartment(Long id);

    Boolean deletePosition(Long id);

    Boolean deleteRole(Long id);

    Boolean insertDepartment(@Param("id") Long id, @Param("deptIds") List<Long> ids);

    Boolean insertPosition(@Param("id") Long id, @Param("positionIds") List<Long> ids);

    Boolean insertRole(@Param("id") Long id, @Param("roleIds") List<Long> roleIds);

    Boolean insertUser(UserEntity user);

    Boolean insertEmployee(EmployeeEntity employeeEntity);

    EmployeeAndUserVO getEmployeeByUserId(Long id);

    Boolean updateEmployeeAndUser(EmployeeEntity employeeEntity);

    EmployeeEntity getEmployeeEntityByUserId(Long userId);

    Integer employeeNameCount(String name);

    Integer userAccountCount(String account);

    Integer employeeJobNumberCount(String jobNumber);

    EmployeeRepetition getEmployeeRepetitionByEmpId(Long empId);

    List<EmployeeMessageResponse> getAccountAndDeptNameById(@Param("ids")List<Integer> ids);

    boolean insertUserRole(Long userId, Long roleId);

    boolean insertUsrRole(@Param("list") List<DgGuaranteePO> list);

    Boolean saveUserRole(@Param("userId") Long userId, @Param("roleId") Long roleId);

    List<EmployeeVO> listEmpNameByUserId(@Param("userIds") List<Long> userIds);

    List<UserDeptVO> getListEmployeeTrees(@Param("id")Long id, @Param("account")String account);

    Integer countUserIdAndRoleId(@Param("userId") Long userId, @Param("roleId") Long roleId);

    int selectUserRole(@Param("userId") Long userId, @Param("roleId") Long roleId);

    Boolean deleteRoleByUserId(@Param("ids") List<Long> ids);

    EmployeeEntity getEmpmsgById(Long id);

    List<EmployeeTO> getAccountAndDeptNameByIds(@Param("ids")List<Long> ids);

    List<EmployeeVO> getUserIdByRoleId();

    Long selectUserIdByEmpId(Long id);
}
