package com.daqing.financial.hrms.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrms.dao.*;
import com.daqing.financial.hrms.feign.GuaranteeFeignService;
import com.daqing.financial.hrms.service.EmployeeService;
import com.daqing.framework.domain.crms.request.EmployeeDisableRequest;
import com.daqing.framework.domain.crms.response.CrmsCode;
import com.daqing.framework.domain.guarantee.po.DgGuaranteePO;
import com.daqing.framework.domain.guarantee.response.EmployeeMessageResponse;
import com.daqing.framework.domain.hrms.*;
import com.daqing.framework.domain.hrms.ext.*;
import com.daqing.framework.domain.hrms.request.EmployeeRequest;
import com.daqing.framework.domain.hrms.response.EmployeeExcelResponse;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.domain.hrms.response.RolePermissionResponse;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.Md5Util;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.excel.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;


@Service("employeeService")
@Slf4j
public class EmployeeServiceImpl extends ServiceImpl<EmployeeDao, EmployeeEntity> implements EmployeeService {

    @Autowired
    private PermissionDao permissionDao;

    @Autowired
    private DeptDao deptDao;

    @Autowired
    private EmployeeDeptDao employeeDeptDao;

    @Autowired
    private PositionDao positionDao;

    @Autowired
    private RoleDao roleDao;

    @Autowired
    private GuaranteeFeignService guaranteeFeignService;

    /**
     * 列表
     */
    @Override
    public PageUtils queryPage(Integer page, Integer size, EmployeeRequest employeeRequest) {
        //分页参数参数校验,后续优化为自定义选择每页条数和跳转到第几页
        if (page == null || size == null) {
            page = 1;
            size = 10;
        }
        IPage<EmployeeListVO> iPage;
        if (employeeRequest.getId() == null) {
            iPage = this.getBaseMapper().pageList(new Page(page, size), employeeRequest);
        } else {
            iPage = this.getBaseMapper().pageListByDept(new Page(page, size), employeeRequest);
        }
        return new PageUtils(iPage);
    }

    /**
     * 删除
     */
    @Transactional
    @Override
    public Boolean delete(List<Long> ids) {
        if (ids == null || ids.size() == 0) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        List<Long> user_ids = this.getBaseMapper().queryIds(ids);
        if (user_ids == null || user_ids.size() == 0) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_IS_NULL);
        }
        this.getBaseMapper().deleteRoleByUserId(user_ids);

        List<Integer> userIds = JSONArray.parseArray(user_ids.toString(),Integer.class);
        //删除员工信息的同时要删除审批流程可见信息
        guaranteeFeignService.delBatchUserVisual(userIds);
        return this.getBaseMapper().deleteByIds(user_ids);
    }

    /**
     * 启用
     */
    @Transactional
    @Override
    public Boolean start(Long id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        Long userId = this.getBaseMapper().getUserId(id);
        UserEntity user = new UserEntity();
        user.setId(userId);
        user.setStatus(PromptSuccess.START_STATUS);
        return this.getBaseMapper().startOrStop(user);
    }

    /**
     * 禁用
     */
    @Transactional
    @Override
    public Boolean stop(EmployeeDisableRequest employeeDisableRequest) {
        UserEntity user = new UserEntity();
        BeanUtils.copyProperties(employeeDisableRequest, user);
        if (employeeDisableRequest.getEmployeeId() == null || user.getDisableStartTime() == null
                || user.getDisableEndTime() == null || user.getDisableCause() == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        if (user.getDisableStartTime().getTime() >= user.getDisableEndTime().getTime()) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_DISABLE_TIME_ERROR);
        }
        Long userId = this.getBaseMapper().getUserId(employeeDisableRequest.getEmployeeId());
        if (user.getDisableStartTime().getTime() <= (new Date()).getTime()
                && (new Date()).getTime() <= user.getDisableEndTime().getTime()) {
            user.setId(userId);
            user.setStatus(PromptSuccess.STOP_STATUS);
            return this.getBaseMapper().startOrStop(user);
        } else if (user.getDisableStartTime().getTime() > (new Date()).getTime()) {
            user.setId(userId);
            user.setStatus(PromptSuccess.UPCOMING_STOP_STATUS);
            return this.getBaseMapper().startOrStop(user);
        } else {
            return false;
        }
    }

    /**
     * 禁用详情
     */
    @Override
    public UserEntity forbidInfo(Long id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        Long userId = this.getBaseMapper().getUserId(id);
        return this.getBaseMapper().forbidInfo(userId);
    }

    /**
     * 定时检测是否禁用
     */
    @Transactional
    @Override
    public void disableDetection() {
        List<UserEntity> users = this.getBaseMapper().listDisableTime();
        if (users.size() != 0) {
            for (UserEntity user : users) {
                if (user.getDisableStartTime().getTime() <= (new Date()).getTime()
                        && (new Date()).getTime() <= user.getDisableEndTime().getTime()) {
                    user.setStatus(PromptSuccess.STOP_STATUS);
                    this.getBaseMapper().startOrStop(user);
                } else if ((new Date()).getTime() < user.getDisableStartTime().getTime()) {
                    user.setStatus(PromptSuccess.UPCOMING_STOP_STATUS);
                    this.getBaseMapper().startOrStop(user);
                } else if ((new Date()).getTime() > user.getDisableEndTime().getTime()) {
                    user.setStatus(PromptSuccess.START_STATUS);
                    user.setDisableStartTime(null);
                    user.setDisableEndTime(null);
                    user.setDisableCause(null);
                    this.getBaseMapper().startOrStop(user);
                }
            }
        }
    }

    /**
     * 重置密码
     */
    @Override
    public Boolean reset(Long id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        Long userId = this.getBaseMapper().getUserId(id);
        if (userId == null) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_IS_NULL);
        }
        String password = Md5Util.md5(PromptSuccess.RESET_PASSWORD + userId);
        return this.getBaseMapper().updatePassword(userId, password);
    }

    /**
     * 详细信息
     */
    @Override
    public EmployeeInfoVO info(Long id) {
        if (id == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        EmployeeInfoVO info = this.getBaseMapper().getInfo(id);
        if (info == null) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_IS_NULL);
        }
        if (info.getRoles().size() != 0 && info.getRoles() != null) {
            List<Long> roleIds = new ArrayList<>();
            for (RoleEntity role : info.getRoles()) {
                roleIds.add(role.getId());
            }
            List<RolePermissionResponse> list = permissionDao.listPermissionByRoleIds(roleIds);
            if (list.size() != 0) {
                // 去重
                Map<Long, RolePermissionResponse> map = new HashMap<>();
                for (RolePermissionResponse rolePermissionResponse : list) {
                    map.put(rolePermissionResponse.getPermissionId(), rolePermissionResponse);
                }
                list.clear();
                for (Long key : map.keySet()) {
                    list.add(map.get(key));
                }
                List<RolePermissionResponse> permissionTreeList = this.getRolePermissionTreeList(list, 0L);
                if (permissionTreeList != null && permissionTreeList.size() != 0) {
                    info.setPermissions(permissionTreeList);
                }
            }
        }
        return info;
    }

    /**
     * 选择角色后对应的权限展示
     */
    @Override
    public List<RolePermissionResponse> permissionTreeList(List<Long> ids) {
        if (ids == null || ids.size() == 0) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        List<RolePermissionResponse> list = permissionDao.listPermissionByRoleIds(ids);
        if (list.size() != 0) {
            // 去重
            Map<Long, RolePermissionResponse> map = new HashMap<>();
            for (RolePermissionResponse rolePermissionResponse : list) {
                map.put(rolePermissionResponse.getPermissionId(), rolePermissionResponse);
            }
            list.clear();
            for (Long key : map.keySet()) {
                list.add(map.get(key));
            }
            return this.getRolePermissionTreeList(list, 0L);
        } else {
            return null;
        }
    }

    /**
     * 更新
     */
    @Transactional
    @Override
    public Boolean update(EmployeeInfoVO employeeInfoVO) {
        if (employeeInfoVO.getId() == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        if (employeeInfoVO.getDepartmentIds() == null || employeeInfoVO.getDepartmentIds().size() == 0) {
            ExceptionCast.cast(HrmsCode.DEPT_NOT_NULL);
        }
        if (employeeInfoVO.getPositionIds() == null || employeeInfoVO.getPositionIds().size() == 0) {
            ExceptionCast.cast(HrmsCode.POSITION_NOT_NULL);
        }
        if (employeeInfoVO.getRoleIds() == null || employeeInfoVO.getRoleIds().size() == 0) {
            ExceptionCast.cast(HrmsCode.ROLE_NOT_NULL);
        }
        Long userId = this.getBaseMapper().getUserId(employeeInfoVO.getId());
        if (userId == null) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_USER_IS_NULL);
        }
        if (employeeInfoVO.getName().length() == 0
                || employeeInfoVO.getAccount().length() == 0
                || employeeInfoVO.getCompanyMail().length() == 0
                || employeeInfoVO.getJobNumber().length() == 0
                || employeeInfoVO.getPhone().length() == 0) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_INFO_NOT_NULL);
        }
        // 工号判重,修改判重先要判断是否与原来相同
        EmployeeRepetition employeeRepetition = this.getBaseMapper().getEmployeeRepetitionByEmpId(employeeInfoVO.getId());
        if (employeeRepetition != null && !Objects.equals(employeeInfoVO.getJobNumber(), employeeRepetition.getJobNumber())) {
            Integer numberCount = this.getBaseMapper().employeeJobNumberCount(employeeInfoVO.getJobNumber());
            if (numberCount > 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_JOB_NUMBER_EXIST);
            }
        }
        // 账号判重
        if (employeeRepetition != null && !Objects.equals(employeeInfoVO.getAccount(), employeeRepetition.getAccount())) {
            Integer accountCount = this.getBaseMapper().userAccountCount(employeeInfoVO.getAccount());
            if (accountCount > 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_USER_ACCOUNT_EXIST);
            }
        }
        // 姓名判重,如果重复,自动在后面加字母"A、B、C、D、E"
        if (employeeRepetition != null && !Objects.equals(employeeInfoVO.getName(), employeeRepetition.getName())) {
            String name = employeeInfoVO.getName();
            for (int i = 0; i < PromptSuccess.LETTERS.length; i++) {
                Integer nameCount = this.getBaseMapper().employeeNameCount(name);
                if (nameCount > 0) {
                    name = employeeInfoVO.getName() + PromptSuccess.LETTERS[i];
                } else {
                    employeeInfoVO.setName(name);
                    break;
                }
                if (i == PromptSuccess.LETTERS.length - 1) {
                    ExceptionCast.cast(HrmsCode.EMPLOYEE_REPETITION_EXCEED);
                }
            }
        }
        String account = employeeInfoVO.getAccount();
        EmployeeEntity employee = new EmployeeEntity();
        BeanUtils.copyProperties(employeeInfoVO, employee);
        Boolean result1 = this.getBaseMapper().updateEmployee(employee);
        Boolean result2 = this.getBaseMapper().updateUser(userId, account, new Date());
        this.getBaseMapper().deleteDepartment(employeeInfoVO.getId());
        this.getBaseMapper().deletePosition(employeeInfoVO.getId());
        this.getBaseMapper().deleteRole(userId);
        Boolean result3 = this.getBaseMapper().insertDepartment(employeeInfoVO.getId(), employeeInfoVO.getDepartmentIds());
        Boolean result4 = this.getBaseMapper().insertPosition(employeeInfoVO.getId(), employeeInfoVO.getPositionIds());
        Boolean result5 = this.getBaseMapper().insertRole(userId, employeeInfoVO.getRoleIds());

        //修改，则先要将该用户对应得审批模板都删除
        Long id = this.getBaseMapper().selectUserIdByEmpId(employeeInfoVO.getId());
        guaranteeFeignService.delUserVisual(id.intValue());
        //如果新增的角色包含超管，则要设置该用户可以看到审批模板
        if(employeeInfoVO.getRoleIds().contains(1L)){
            ResponseResult result = guaranteeFeignService.updateManagerVisual(id.intValue());
        }

        return result1 && result2 && result3 && result4 && result5;
    }

    /**
     * 新增
     */
    @Transactional
    @Override
    public Boolean save(EmployeeInfoVO employeeInfoVO) {
        if (employeeInfoVO.getDepartmentIds() == null || employeeInfoVO.getDepartmentIds().size() == 0) {
            ExceptionCast.cast(HrmsCode.DEPT_NOT_NULL);
        }
        if (employeeInfoVO.getPositionIds() == null || employeeInfoVO.getPositionIds().size() == 0) {
            ExceptionCast.cast(HrmsCode.POSITION_NOT_NULL);
        }
        if (employeeInfoVO.getRoleIds() == null || employeeInfoVO.getRoleIds().size() == 0) {
            ExceptionCast.cast(HrmsCode.ROLE_NOT_NULL);
        }
        if (employeeInfoVO.getName().length() == 0
                || employeeInfoVO.getAccount().length() == 0
                || employeeInfoVO.getCompanyMail().length() == 0
                || employeeInfoVO.getJobNumber().length() == 0
                || employeeInfoVO.getPhone().length() == 0) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_INFO_NOT_NULL);
        }
        // 判重
        Integer numberCount = this.getBaseMapper().employeeJobNumberCount(employeeInfoVO.getJobNumber());
        if (numberCount > 0) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_JOB_NUMBER_EXIST);
        }
        Integer accountCount = this.getBaseMapper().userAccountCount(employeeInfoVO.getAccount());
        if (accountCount > 0) {
            ExceptionCast.cast(HrmsCode.EMPLOYEE_USER_ACCOUNT_EXIST);
        }
        String name = employeeInfoVO.getName();
        for (int i = 0; i < PromptSuccess.LETTERS.length; i++) {
            Integer nameCount = this.getBaseMapper().employeeNameCount(name);
            if (nameCount > 0) {
                name = employeeInfoVO.getName() + PromptSuccess.LETTERS[i];
            } else {
                employeeInfoVO.setName(name);
                break;
            }
            if (i == PromptSuccess.LETTERS.length - 1) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_REPETITION_EXCEED);
            }
        }
        // 员工对应的用户信息
        UserEntity user = new UserEntity();
        user.setAccount(employeeInfoVO.getAccount());
        user.setCreateTime(new Date());
        user.setMotifyTime(new Date());
        Boolean result1 = this.getBaseMapper().insertUser(user);
        EmployeeEntity employee = new EmployeeEntity();

        //如果新增的角色包含超管，则要设置该用户可以看到审批模板
        if(employeeInfoVO.getRoleIds().contains(1L)){
            ResponseResult result = guaranteeFeignService.updateManagerVisual(user.getId().intValue());
        }

        if (result1) {
            // 根据新增的user主键id新增初始密码
            Boolean result6 = this.getBaseMapper().updatePassword(user.getId(), Md5Util.md5(PromptSuccess.RESET_PASSWORD + user.getId()));
            employee.setUserId(user.getId());
            BeanUtils.copyProperties(employeeInfoVO, employee);
            Boolean result2 = this.getBaseMapper().insertEmployee(employee);
            if (result2 && result6) {
                Boolean result3 = this.getBaseMapper().insertDepartment(employee.getId(), employeeInfoVO.getDepartmentIds());
                Boolean result4 = this.getBaseMapper().insertPosition(employee.getId(), employeeInfoVO.getPositionIds());
                Boolean result5 = this.getBaseMapper().insertRole(user.getId(), employeeInfoVO.getRoleIds());
                return result3 && result4 && result5;
            }
        }
        return false;
    }

    /**
     * 导出
     */
    @Transactional
    @Override
    public Boolean excelExport(HttpServletResponse response) {
        try {
            List<EmployeeListVO> list = this.getBaseMapper().excelList();
            List<EmployeeExcelResponse> excelResponse = new ArrayList<>();
            for (EmployeeListVO employee : list) {
                EmployeeExcelResponse employeeExcelResponse = new EmployeeExcelResponse();
                if (employee.getDepartmentNameList() != null && employee.getDepartmentNameList().size() != 0) {
                    StringBuilder stringBuilder = new StringBuilder();
                    for (int i = 0; i < employee.getDepartmentNameList().size(); i++) {
                        if (i == employee.getDepartmentNameList().size() - 1) {
                            stringBuilder.append(employee.getDepartmentNameList().get(i));
                        } else {
                            stringBuilder.append(employee.getDepartmentNameList().get(i)).append(",");
                        }
                    }
                    employeeExcelResponse.setDepartmentNameList(stringBuilder.toString());
                }
                if (employee.getPositionNameList() != null && employee.getPositionNameList().size() != 0) {
                    StringBuilder stringBuilder = new StringBuilder();
                    for (int i = 0; i < employee.getPositionNameList().size(); i++) {
                        if (i == employee.getPositionNameList().size() - 1) {
                            stringBuilder.append(employee.getPositionNameList().get(i));
                        } else {
                            stringBuilder.append(employee.getPositionNameList().get(i)).append(",");
                        }
                    }
                    employeeExcelResponse.setPositionNameList(stringBuilder.toString());
                }
                if (employee.getRoleNameList() != null && employee.getRoleNameList().size() != 0) {
                    StringBuilder stringBuilder = new StringBuilder();
                    for (int i = 0; i < employee.getRoleNameList().size(); i++) {
                        if (i == employee.getRoleNameList().size() - 1) {
                            stringBuilder.append(employee.getRoleNameList().get(i));
                        } else {
                            stringBuilder.append(employee.getRoleNameList().get(i)).append(",");
                        }
                    }
                    employeeExcelResponse.setRoleNameList(stringBuilder.toString());
                }
                BeanUtils.copyProperties(employee, employeeExcelResponse);
                excelResponse.add(employeeExcelResponse);
            }
            ExcelUtil.writeExcelWithSheets(response, excelResponse, "员工信息一览表", "employee", new EmployeeExcelResponse())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    /**
     * 导出模板
     */
    @Transactional
    @Override
    public Boolean excelTemplate(HttpServletResponse response) {
        try {
            ExcelUtil.writeExcelWithSheets(response, new ArrayList<EmployeeTemplate>(), "员工信息导入模板", "employee", new EmployeeTemplate())
                    .finish();
            return true;
        } catch (Exception e) {
            ExceptionCast.cast(CrmsCode.CUSTOMER_EXPORT_EXSIT);
            return false;
        }
    }

    /**
     * 导入
     */
    @Transactional
    @Override
    public Boolean excelImport(MultipartFile file) {
        if (file == null || file.isEmpty()) {
            ExceptionCast.cast(CrmsCode.NOT_NULL);
        }
        EmployeeExcelImport employeeExcelImport = new EmployeeExcelImport();
        UserEntity user = new UserEntity();
        EmployeeEntity employee = new EmployeeEntity();
        // 获取所有的部门、职位、角色,excel文件中的数据对比
        List<DepartmentVO> listDepartmentIdAndName = deptDao.listDepartmentIdAndName();
        List<PositionVO> listPositionIdAndName = positionDao.listPositionIdAndName();
        List<RoleVO> listRoleIdAndName = roleDao.listRoleIdAndName();
        Set<Long> deptIds = new HashSet<>();
        Set<Long> positionIds = new HashSet<>();
        Set<Long> roleIds = new HashSet<>();
        List<Object> employees = ExcelUtil.readExcel(file, employeeExcelImport, 1);
        if (employees.size() == 0) {
            ExceptionCast.cast(HrmsCode.EXCEL_DATA_IS_NULL);
        }
        for (Object e : employees) {
            // 判空
            BeanUtils.copyProperties(e, employeeExcelImport);
            if (employeeExcelImport.getName() == null || employeeExcelImport.getName().length() == 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_NAME_IS_NULL);
            }
            if (employeeExcelImport.getAccount() == null || employeeExcelImport.getAccount().length() == 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_ACCOUNT_IS_NULL);
            }
            if (employeeExcelImport.getJobNumber() == null || employeeExcelImport.getJobNumber().length() == 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_JOB_NUMBER_IS_NULL);
            }
            // 判重
            Integer jobNumberCount = this.getBaseMapper().employeeJobNumberCount(employeeExcelImport.getJobNumber());
            if (jobNumberCount > 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_JOB_NUMBER_EXIST);
            }
            Integer accountCount = this.getBaseMapper().userAccountCount(employeeExcelImport.getAccount());
            if (accountCount > 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_USER_ACCOUNT_EXIST);
            }
            String name = employeeExcelImport.getName();
            for (int i = 0; i < PromptSuccess.LETTERS.length; i++) {
                Integer nameCount = this.getBaseMapper().employeeNameCount(name);
                if (nameCount > 0) {
                    name = employeeExcelImport.getName() + PromptSuccess.LETTERS[i];
                } else {
                    employeeExcelImport.setName(name);
                    break;
                }
                if (i == PromptSuccess.LETTERS.length - 1) {
                    ExceptionCast.cast(HrmsCode.EMPLOYEE_REPETITION_EXCEED);
                }
            }
            user.setAccount(employeeExcelImport.getAccount());
            user.setCreateTime(new Date());
            user.setMotifyTime(new Date());
            this.getBaseMapper().insertUser(user);
            // 根据新增的user主键id新增初始密码
            this.getBaseMapper().updatePassword(user.getId(), Md5Util.md5(PromptSuccess.RESET_PASSWORD + user.getId()));
            BeanUtils.copyProperties(e, employee);
            employee.setUserId(user.getId());
            // 此处员工姓名可能重复,所以要如下设置
            employee.setName(employeeExcelImport.getName());
            this.getBaseMapper().insertEmployee(employee);
            // 部门
            String[] departmentNames = employeeExcelImport.getDepartmentNameList().split("&");
            if (departmentNames.length == 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_DEPARTMENT_IS_NULL);
            }
            for (DepartmentVO department : listDepartmentIdAndName) {
                for (String s : departmentNames) {
                    if (Objects.equals(department.getName(), s)) {
                        deptIds.add(department.getId());
                    }
                }
            }
            if (deptIds.size() != departmentNames.length) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_DEPARTMENT_NAME_IS_NOT);
            }
            this.getBaseMapper().insertDepartment(employee.getId(), new ArrayList<>(deptIds));
            // 职位
            String[] positionNames = employeeExcelImport.getPositionNameList().split("&");
            if (positionNames.length == 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_POSITION_IS_NULL);
            }
            for (PositionVO position : listPositionIdAndName) {
                for (String s : positionNames) {
                    if (Objects.equals(position.getName(), s)) {
                        positionIds.add(position.getId());
                    }
                }
            }
            if (positionIds.size() != positionNames.length) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_POSITION_NAME_IS_NOT);
            }
            this.getBaseMapper().insertPosition(employee.getId(), new ArrayList<>(positionIds));
            // 角色
            String[] roleNames = employeeExcelImport.getRoleNameList().split("&");
            if (roleNames.length == 0) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_ROLE_IS_NULL);
            }
            for (RoleVO role : listRoleIdAndName) {
                for (String s : roleNames) {
                    if (Objects.equals(role.getName(), s)) {
                        roleIds.add(role.getId());
                    }
                }
            }
            if (roleIds.size() != roleNames.length) {
                ExceptionCast.cast(HrmsCode.EMPLOYEE_ROLE_NAME_IS_NOT);
            }
            this.getBaseMapper().insertRole(user.getId(), new ArrayList<>(roleIds));
        }
        return true;
    }

    /**
     * 封装员工的权限树
     */
    private List<RolePermissionResponse> getRolePermissionTreeList(List<RolePermissionResponse> list, Long parentId) {
        return list.stream()
                .filter(item -> parentId.equals(item.getParentId()))
                .peek(item -> item.setChildren(getRolePermissionTreeList(list, item.getPermissionId())))
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort())))
                .collect(Collectors.toList());
    }

    /**
     * 根据id查询员工的姓名
     */
    @Override
    public EmployeeEntity getEmployeeById(Long id) {
        return this.getBaseMapper().getEmployeeById(id);
    }

    /**
     * 查询所有的员工id和姓名
     */
    @Override
    public List<EmployeeVO> listEmployeeName() {

        return this.getBaseMapper().listEmployeeName();
    }

    /**
     * 根据员工id查询员工的姓名和部门信息
     *
     * @param ids 员工id集合
     * @return
     */
    @Override
    public List<EmployeeTO> getEmployeeAndDeptById(List<Long> ids) {

        return this.getBaseMapper().getEmployeeAndDeptById(ids);
    }

    @Override
    public List<EmployeeTO> getAccountAndDeptNameByIds(List<Long> ids) {

        return this.getBaseMapper().getAccountAndDeptNameByIds(ids);
    }


    /**
     * 根据userId查询员工的头像
     *
     * @return
     */
    @Override
    public EmployeeEntity getEmployeeByUserId(Long userId) {
        return this.getBaseMapper().getEmployeeEntityByUserId(userId);
    }

    @Override
    public List<EmployeeMessageResponse> getAccountAndDeptNameById(List<Integer> ids) {
        return this.getBaseMapper().getAccountAndDeptNameById(ids);
    }

    @Override
    public boolean insertUserRole(Long userId, Long roleId) {
        return this.getBaseMapper().insertUserRole(userId, roleId);
    }


    @Override
    public boolean insertUsrRole(List<DgGuaranteePO> list) {
        return this.getBaseMapper().insertUsrRole(list);
    }

    /**
     * 给用户赋予一个角色
     */
    @Override
    public Boolean saveUserRole(Long userId, Long roleId) {
        if (userId == null || roleId == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        // 先判断该条数据是否存在
        Integer count = this.getBaseMapper().countUserIdAndRoleId(userId, roleId);
        if (count < 1) {
            return this.getBaseMapper().saveUserRole(userId, roleId);
        }
        return true;
    }


    /**
     * 根据用户id获取对应的员工姓名
     */
    @Override
    public List<EmployeeVO> listEmpNameByUserId(List<Integer> userIds) {
        List<Long> userIdList = new ArrayList<>();
        for (Integer userId : userIds) {
            userIdList.add(userId.longValue());
        }
        return this.getBaseMapper().listEmpNameByUserId(userIdList);
    }

    @Override
    public int selectUserRole(Long userId, Long roleId) {
        return this.getBaseMapper().selectUserRole(userId, roleId);
    }

    @Override
    public String getDeptIds(Integer userId) {
        QueryWrapper<EmployeeEntity> wrapper = new QueryWrapper<>();
        wrapper.eq("user_id",userId).select("id");
        EmployeeEntity employeeEntity = baseMapper.selectOne(wrapper);
        //获取员工id
        Long employeeId = employeeEntity.getId();
        //根据员工id获取对应部门ids
        QueryWrapper<EmployeeDeptEntity> employeeDeptEntityQueryWrapper = new QueryWrapper<>();
        employeeDeptEntityQueryWrapper.eq("employee_id",employeeId);
        List<EmployeeDeptEntity> deptList = employeeDeptDao.selectList(employeeDeptEntityQueryWrapper);
        StringBuilder stringBuilder = new StringBuilder();
        if (deptList!=null && deptList.size()!=0){
            //拼接部门id
            deptList.forEach(employeeDeptEntity -> {
                Long deptId = employeeDeptEntity.getDeptId();
                stringBuilder.append(deptId).append(",");
            });
        }
        String s = stringBuilder.toString();
        String ids = "";
        if (s.length()!=0){
            ids = s.substring(0, s.length()-1);
        }
        return ids;
    }

    /**
     * 根据id查询员工的姓名
     */
    @Override
    public EmployeeEntity getEmpmsgById(Long id) {
        return this.getBaseMapper().getEmpmsgById(id);
    }

    @Override
    public List<EmployeeVO> getUserIdByRoleId() {
        return this.getBaseMapper().getUserIdByRoleId();
    }
}