package com.daqing.financial.hrms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrms.dao.PermissionDao;
import com.daqing.financial.hrms.dao.RolePermissionDao;
import com.daqing.financial.hrms.service.PermissionService;
import com.daqing.framework.domain.hrms.PermissionEntity;
import com.daqing.framework.domain.hrms.RolePermissionEntity;
import com.daqing.framework.model.response.ResponseResult;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.ConvertUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;


@Service("permissionService")
public class PermissionServiceImpl extends ServiceImpl<PermissionDao, PermissionEntity> implements PermissionService {

    @Resource
    PermissionDao permissionMapper;
    @Resource
    RolePermissionDao rolePermissionDao;
    @Resource
    PermissionService permissionService;

    @Override
    public List<PermissionEntity> listWithTree() {
        return getPermissionTreeList(this.list(), 0L);
    }

    private List<PermissionEntity> getPermissionTreeList(List<PermissionEntity> list, Long parentId) {
        return list.stream()
                .filter(item -> parentId.equals(item.getParentId()))
                .peek(item -> item.setChildren(getPermissionTreeList(list, item.getId())))
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort())))
                .collect(Collectors.toList());
    }

    @Override
    public List<PermissionEntity> queryPermissionList() {
        List<PermissionEntity> permissionList = permissionMapper.selectList(null);
        return permissionList;
    }

    @Override
    public List<RolePermissionEntity> queryRolePermissionList(Long id) {
        List<RolePermissionEntity> permissionEntityList = rolePermissionDao.selectList(new QueryWrapper<RolePermissionEntity>().eq("permission_id",id));
        return permissionEntityList;
    }

/*    @Override
    public List<RolePermissionEntity> queryPermissionsByRoleIds(List<Long> ids) {
        List<RolePermissionEntity> permissionEntityList = rolePermissionDao.queryPermissionsByRoleIds(ids);
        return permissionEntityList;
    }

    @Override
    public List<PermissionEntity> queryPermissionsByIds(List<Integer> ids) {
        List<PermissionEntity> permissionEntityList = permissionMapper.queryPermissionsByIds(ids);
        return permissionEntityList;
    }*/

    @Override
    public List getPermission(String roleIds) {
        //截取字符串
        String[] strArr = roleIds.split(",");
        //转换long类型的数组
        Long[] strArrNum = (Long[]) ConvertUtils.convert(strArr,Long.class);

        //根据角色id查询对应的权限id
        List<RolePermissionEntity> rolePermissionList = rolePermissionDao.queryPermissionsByRoleIds(Arrays.asList(strArrNum));
        List<Long> permissionList = new ArrayList();
        for(RolePermissionEntity res : rolePermissionList){
            permissionList.add(res.getPermissionId());
        }

        Long[] permissionIds = permissionList.toArray(new Long[permissionList.size()]);
        //根据权限ids查询对应权限
        List<PermissionEntity> permissionEntityList = permissionMapper.queryPermissionsByIds(Arrays.asList(permissionIds));
        return getListDepartmentAndEmployeeTree(permissionEntityList, 0L);
    }

    private List<PermissionEntity> getListDepartmentAndEmployeeTree(List<PermissionEntity> list, Long parentId) {
        return list.stream()
                .filter(dept -> parentId.equals(dept.getParentId())) // 过滤出部门id为所选id的部门信息(也为递归的结果条件，找不到所对应的部门id)
                .peek(dept -> {
                    dept.setChildren(getListDepartmentAndEmployeeTree(list, dept.getId())); // 设置该部门的子部门:递归调用

                    PermissionEntity permission = permissionService.getById(dept.getId());
                    /*dept.setUserDept(employeeDao.employeeListByDeptId(dept.getId(),account));*/
                })
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort()))) // 根据所选排序信息进行排序
                .collect(Collectors.toList());
    }

}