package com.daqing.financial.hrms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrms.dao.*;
import com.daqing.financial.hrms.service.PositionService;
import com.daqing.framework.domain.hrms.DeptEntity;
import com.daqing.framework.domain.hrms.DeptPositionEntity;
import com.daqing.framework.domain.hrms.EmployeePositionEntity;
import com.daqing.framework.domain.hrms.PositionEntity;
import com.daqing.framework.domain.hrms.ext.PositionVO;
import com.daqing.framework.domain.hrms.ext.RoleVO;
import com.daqing.framework.domain.hrms.request.PositionRequest;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.domain.hrms.response.PositionResponse;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.CommonCode;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.utils.PageUtils;
import com.daqing.framework.utils.SnowflakeIdUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;


@Service("positionService")
public class PositionServiceImpl extends ServiceImpl<PositionDao, PositionEntity> implements PositionService {

    @Autowired
    DeptPositionDao deptPositionDao;

    @Autowired
    DeptDao deptDao;

    @Autowired
    EmployeePositionDao employeePositionDao;

    @Autowired
    private RoleDao roleDao;

    @Override
    public PageUtils queryPage(Integer page, Integer size, String queryMsg) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<PositionVO> positionVO = this.getBaseMapper().pageByCondition(new Page(page, size), queryMsg);

        return new PageUtils(positionVO);
    }

    @Override
    @Transactional
    public boolean savePosition(PositionRequest position) {
        //TODO 用户id从当前登录token中获取，不由前端传递
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:" + token);
        if (userId != null){
            position.setUserId(Long.parseLong(userId));
        }
        if (position == null
                || position.getUserId() == null) {
            ExceptionCast.cast(CommonCode.USER_LOGIN_NULL);
        }
        if (position.getDeptIds() == null
                || position.getDeptIds().length == 0
                || position.getName() == null
                || position.getName().length() == 0){
            ExceptionCast.cast(CommonCode.POSITION_DEPT_NOT_NULL);
        }
        //校验该部门下面有没有重复的职位
        List<Long> deptIdList = Arrays.asList(position.getDeptIds());
        deptIdList.forEach(deptId -> {
            int count = this.getBaseMapper().getCountByNameAndDeptId(deptId, position.getName());
            if (count > 0) {
                ExceptionCast.cast(HrmsCode.DEPT_POSITION_EXSIT);
            }
        });

        //添加职位信息
        PositionEntity positionEntity = new PositionEntity();
        BeanUtils.copyProperties(position, positionEntity);
        positionEntity.setCreateUser(position.getUserId());
        positionEntity.setCreateTime(new Date());
        positionEntity.setMotifyTime(new Date());
        boolean savePosition = this.save(positionEntity);

        //添加部门职位关系表信息
        List<DeptPositionEntity> deptPositionList = getDeptPositionList(deptIdList, positionEntity.getId());

        int insertDeptPosition = deptPositionDao.saveBatch(deptPositionList);

        return savePosition && insertDeptPosition > 0;
    }

    @Override
    @Transactional
    public boolean updatePosition(PositionRequest positionRequest) {
        if (positionRequest == null
                || positionRequest.getId() == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        if (positionRequest.getDeptIds() == null
                || positionRequest.getDeptIds().length == 0
                || positionRequest.getName() == null
                || positionRequest.getName().length() == 0){
            ExceptionCast.cast(CommonCode.POSITION_DEPT_NOT_NULL);
        }
        //校验该部门下面有没有重复的职位
        List<Long> deptIdList = Arrays.asList(positionRequest.getDeptIds());
        deptIdList.forEach(deptId -> {
            int count = this.getBaseMapper().getCountByNameAndDeptIdAndId(deptId, positionRequest.getName(), positionRequest.getId());
            if (count > 0) {
                ExceptionCast.cast(HrmsCode.DEPT_POSITION_EXSIT);
            }
        });
        PositionEntity positionEntity = new PositionEntity();
        BeanUtils.copyProperties(positionRequest, positionEntity);
        positionEntity.setMotifyTime(new Date());

        boolean updateById = this.updateById(positionEntity);

        //删除原关系
        deptPositionDao.deleteByPositionId(positionRequest.getId());

        List<DeptPositionEntity> deptPositionList = getDeptPositionList(deptIdList, positionEntity.getId());

        //批量保存中间表
        int insertDeptPosition = deptPositionDao.saveBatch(deptPositionList);

        return updateById && insertDeptPosition > 0;

    }

    @Override
    public PositionResponse getPositionById(Long id) {
        List<DeptEntity> dept = deptDao.getByPositionId(id);
        PositionResponse positionResponse = new PositionResponse();
        BeanUtils.copyProperties(this.getById(id), positionResponse);
        positionResponse.setDeptList(dept);
        return positionResponse;
    }

    @Override
    @Transactional
    public void removePositionByIds(List<Long> idList) {
        idList.forEach(id -> {
            Integer count = employeePositionDao.selectCount(new QueryWrapper<EmployeePositionEntity>().eq("position_id", id));
            if (count > 0) {
               ExceptionCast.cast(HrmsCode.EMP_POSITION_EXSIT);
            }
        });
        this.removeByIds(idList);
        idList.forEach(id -> deptPositionDao.deleteByPositionId(id));
    }

    /**
     * 所有的职位id和名称
     */
    @Override
    public List<PositionVO> listPositionIdAndName(List<Long> deptIds) {
        if (deptIds == null || deptIds.size() == 0){
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        List<PositionVO> positionList = this.getBaseMapper().listPositionIdAndNameByDeptId(deptIds);
        if (positionList == null || positionList.size() == 0){
            return null;
        }
        // 去重
        HashMap<Long,PositionVO> map = new HashMap<>();
        for (PositionVO position : positionList) {
            map.put(position.getId(),position);
        }
        positionList.clear();
        for (Long key : map.keySet()) {
            positionList.add(map.get(key));
        }
        return positionList;
    }

    /**
     * 所有的角色id和名称
     */
    @Override
    public List<RoleVO> listRoleIdAndName() {
        return roleDao.listRoleIdAndName();
    }

    private List<DeptPositionEntity> getDeptPositionList(List<Long> deptIdList, Long id) {
        return deptIdList.stream().map(deptId -> {
            DeptPositionEntity deptPositionEntity = new DeptPositionEntity();
            deptPositionEntity.setDeptId(deptId);
            deptPositionEntity.setPositionId(id);
            return deptPositionEntity;
        }).collect(Collectors.toList());
    }

}