package com.daqing.financial.hrms.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrms.dao.EmployeeDao;
import com.daqing.financial.hrms.dao.UserDao;
import com.daqing.financial.hrms.service.UserService;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.domain.hrms.UserEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeAndUserVO;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.model.response.PromptSuccess;
import com.daqing.framework.util.Md5Util;
import com.daqing.framework.util.RedisUtil;
import com.daqing.framework.util.SendSMS;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.List;
import java.util.Objects;
import java.util.UUID;


@Slf4j
@Service("userService")
public class UserServiceImpl extends ServiceImpl<UserDao, UserEntity> implements UserService {

    @Autowired
    private EmployeeDao employeeDao;

    /**
     * 当前登录的用户信息
     */
    @Override
    public EmployeeAndUserVO info() {
        String userId = this.getUserId();
        EmployeeAndUserVO employeeAndUser = employeeDao.getEmployeeByUserId(Long.parseLong(userId));
        if (employeeAndUser == null) {
            return employeeAndUser;
        }
        if (employeeAndUser.getWechatId() == null || employeeAndUser.getWechatId().length() == 0) {
            employeeAndUser.setWechatId(PromptSuccess.QR_CODE_FAIL);
        } else {
            employeeAndUser.setWechatId(PromptSuccess.QR_CODE_BINDING_SUCCESS);
        }
        return employeeAndUser;
    }

    /**
     * 更新
     */
    @Transactional
    @Override
    public Boolean update(EmployeeAndUserVO employeeAndUserVO) {
        EmployeeEntity employee = new EmployeeEntity();
        String userId = this.getUserId();
        Long employeeId = this.getBaseMapper().getEmployeeIdByUserId(Long.parseLong(userId));
        employeeAndUserVO.setId(employeeId);
        employeeAndUserVO.setUserId(Long.parseLong(userId));
        Boolean result = true;
        BeanUtils.copyProperties(employeeAndUserVO, employee);
        String phoneAccount = this.getBaseMapper().getPhoneAccount(employee.getUserId());
        if ((phoneAccount == null || phoneAccount.length() == 0)
                && employeeAndUserVO.getPrimaryPhone() != null
                && employeeAndUserVO.getPrimaryPhone().length() != 0
                && employeeAndUserVO.getPrimaryCode() != null
                && employeeAndUserVO.getPrimaryCode().length() != 0) {
            // 没有绑定手机号,现在绑定
            List<String> phoneAccounts = this.getBaseMapper().listPhoneAccount();
            for (String pAccount : phoneAccounts) {
                // 判断该手机号是否已经绑定过了
                if (employeeAndUserVO.getPrimaryPhone().equals(pAccount)) {
                    ExceptionCast.cast(HrmsCode.USER_PHONE_ACCOUNT_REPETITION);
                }
            }
            String code = RedisUtil.get(employeeAndUserVO.getPrimaryPhone());
            if (code == null || code.length() == 0 || !Objects.equals(employeeAndUserVO.getPrimaryCode(), code)) {
                ExceptionCast.cast(HrmsCode.USER_PHONE_CODE_MISTAKE);
            }
            RedisUtil.del(employeeAndUserVO.getPrimaryPhone());
            result = this.getBaseMapper().updatePhoneAccount(employee.getUserId(), employeeAndUserVO.getPrimaryPhone());
        }
        if (employeeAndUserVO.getPrimaryPhone() != null
                && employeeAndUserVO.getPrimaryPhone().length() != 0
                && employeeAndUserVO.getPrimaryCode() != null
                && employeeAndUserVO.getPrimaryCode().length() != 0
                && employeeAndUserVO.getNewPhone() != null
                && employeeAndUserVO.getNewPhone().length() != 0
                && employeeAndUserVO.getNewCode() != null
                && employeeAndUserVO.getNewCode().length() != 0
                && phoneAccount != null && phoneAccount.length() != 0) {
            // 更换绑定的手机号
            if (!Objects.equals(phoneAccount, employeeAndUserVO.getPrimaryPhone())) {
                ExceptionCast.cast(HrmsCode.USER_PRIMARY_PHONE_MISTAKE);
            }
            String primaryCode = RedisUtil.get(employeeAndUserVO.getPrimaryPhone());
            if (primaryCode == null || primaryCode.length() == 0
                    || !Objects.equals(primaryCode, employeeAndUserVO.getPrimaryCode())) {
                ExceptionCast.cast(HrmsCode.USER_PRIMARY_CODE_MISTAKE);
            }
            RedisUtil.del(employeeAndUserVO.getPrimaryPhone());
            // 判断更换的手机号是否已绑定过用户
            List<String> phoneAccounts = this.getBaseMapper().listPhoneAccount();
            for (String pAccount : phoneAccounts) {
                if (employeeAndUserVO.getNewPhone().equals(pAccount)) {
                    ExceptionCast.cast(HrmsCode.USER_PHONE_ACCOUNT_REPETITION);
                }
            }
            String newCode = RedisUtil.get(employeeAndUserVO.getNewPhone());
            if (newCode == null || newCode.length() == 0
                    || !Objects.equals(newCode, employeeAndUserVO.getNewCode())) {
                ExceptionCast.cast(HrmsCode.USER_NEW_CODE_MISTAKE);
            }
            RedisUtil.del(employeeAndUserVO.getNewPhone());
            result = this.getBaseMapper().updatePhoneAccount(employee.getUserId(), employeeAndUserVO.getNewPhone());
        }
        return result && employeeDao.updateEmployeeAndUser(employee);
    }

    /**
     * 图像文件上传
     */
    @Override
    public String uploadFile(MultipartFile file) {
        if (file == null || file.isEmpty()) {
            ExceptionCast.cast(HrmsCode.USER_FILE_IS_NULL);
        }
        try {
            // 通过获取图片文件的宽度和高度来校验是否为图片文件
            BufferedImage image = ImageIO.read(file.getInputStream());
            if (image == null || image.getHeight() <= 0 || image.getWidth() <= 0) {
                ExceptionCast.cast(HrmsCode.USER_FILE_TYPE_MISTAKE);
            }
        } catch (IOException e) {
            ExceptionCast.cast(HrmsCode.USER_FILE_UPLOAD_FAIL);
        }
        String filename = file.getOriginalFilename();
        if (filename == null) {
            ExceptionCast.cast(HrmsCode.USER_FILE_TYPE_MISTAKE);
        }
        String substring = filename.substring(filename.lastIndexOf("."));
        String filePrefix = UUID.randomUUID().toString();
        // 图片上传绝对路径
        String filePath = PromptSuccess.UPLOAD_FILE_PATH + filePrefix + substring;
        try {
            file.transferTo(new File(filePath));
        } catch (IOException e) {
            ExceptionCast.cast(HrmsCode.USER_FILE_UPLOAD_FAIL);
        }
        // 图片访问路径
        return PromptSuccess.IMAGE_URL_PATH + filePrefix + substring;
    }

    /**
     * 修改密码
     */
    @Transactional
    @Override
    public Boolean updatePassword(String primaryPassword, String newPassword, String affirmNewPassword) {
        if (primaryPassword == null || primaryPassword.length() == 0
                || newPassword == null || newPassword.length() == 0
                || affirmNewPassword == null || affirmNewPassword.length() == 0) {
            ExceptionCast.cast(HrmsCode.USER_PASSWORD_NOT_NULL);
        }
        // 两次输入是否相同
        if (!Objects.equals(affirmNewPassword, newPassword)) {
            ExceptionCast.cast(HrmsCode.USER_PASSWORD_NO_CONSISTENT);
        }
        // 旧密码是否正确
        String userId = this.getUserId();
        if (!Objects.equals(Md5Util.md5(primaryPassword + Long.parseLong(userId)), this.getBaseMapper().getPasswordById(Long.parseLong(userId)))) {
            ExceptionCast.cast(HrmsCode.USER_PASSWORD_MISTAKE);
        }
        // 正则校验
        if (!newPassword.matches(PromptSuccess.MATCHES_STRING)) {
            ExceptionCast.cast(HrmsCode.USER_PASSWORD_MATCHES);
        }
        // 是否与账号相同
        if (Objects.equals(newPassword, this.getBaseMapper().getAccountById(Long.parseLong(userId)))) {
            ExceptionCast.cast(HrmsCode.USER_PASSWORD_IS_NO_ACCOUNT);
        }
        return this.getBaseMapper().updatePassword(Long.parseLong(userId), Md5Util.md5(newPassword + Long.parseLong(userId)));
    }

    /**
     * 获取验证码
     */
    @Override
    public Boolean getVerificationCode(String phone) {
        if (phone == null || phone.length() == 0) {
            ExceptionCast.cast(HrmsCode.USER_PHONE_NOT_NULL);
        }
        String code = (int) ((Math.random() * 9 + 1) * 100000) + "";
        // 发送短信
        Boolean result = SendSMS.sendCode(phone, "SMS_204155294", code);
        if (result) {
            RedisUtil.setEx(phone, code, PromptSuccess.OVERDUE_TIME);
            return true;
        }
        return false;
    }

    /**
     * 绑定手机号
     */
    @Transactional
    @Override
    public Boolean bindingPhoneAccount(String phone, String code) {
        if (phone == null || phone.length() == 0 || code == null || code.length() == 0) {
            ExceptionCast.cast(HrmsCode.USER_PHONE_OR_CODE_NOT_NULL);
        }
        // 判断当前手机号是否已绑定用户
        List<String> phoneAccounts = this.getBaseMapper().listPhoneAccount();
        for (String phoneAccount : phoneAccounts) {
            if (phone.equals(phoneAccount)) {
                ExceptionCast.cast(HrmsCode.USER_PHONE_ACCOUNT_REPETITION);
            }
        }
        if (RedisUtil.get(phone) == null || RedisUtil.get(phone).length() == 0
                || !Objects.equals(code, RedisUtil.get(phone))) {
            ExceptionCast.cast(HrmsCode.USER_BINDING_FAIL);
        }
        RedisUtil.del(phone);
        String userId = this.getUserId();
        return this.getBaseMapper().updatePhoneAccount(Long.parseLong(userId), phone);
    }

    /**
     * 取消微信绑定
     */
    @Override
    public Boolean cancel() {
        String userId = this.getUserId();
        return this.getBaseMapper().updateWeChat(Long.parseLong(userId), null);
    }

    /**
     * 更新图像(小程序端)
     * @param headPortraitUrl 图像路径
     */
    @Override
    public Boolean updateHeadPortraitUrl(String headPortraitUrl) {
        if (StringUtils.isEmpty(headPortraitUrl) || headPortraitUrl.trim().length() == 0) {
            ExceptionCast.cast(HrmsCode.USER_HEAD_PORTRAIT_URL_IS_NULL);
        }
        String userId = this.getUserId();
        return this.getBaseMapper().updateHeadPortraitUrl(this.getBaseMapper().getEmployeeIdByUserId(Long.parseLong(userId)), headPortraitUrl);
    }

    /**
     * 获取当前登录用户信息
     */
    private String getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        return RedisUtil.get("dq:token:" + token);
    }
}