package com.daqing.framework.aspect;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.daqing.framework.annotation.Log;
import com.daqing.framework.feign.HrauthFeignService;
import com.daqing.framework.model.EmployeeEntity;
import com.daqing.framework.model.SystemLog;
import com.daqing.framework.model.UserEntity;
import com.daqing.framework.model.response.ResponseResult;
import com.daqing.framework.util.IpUtils;
import com.daqing.framework.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @ClassName Operation
 * @Description 操作日志类
 * @Date 2020/9/29 10:06
 * @Version 1.0
 */
@Slf4j
@Component
@EnableDiscoveryClient
@EnableAspectJAutoProxy(proxyTargetClass = true)
public class Operation {
    @Autowired
    private HrauthFeignService hrauthFeignService;

    @Async
    public void addOperationLog(JoinPoint joinPoint, Object res, long time, UserEntity systemUser, EmployeeEntity employeeEntity) {
//        synchronized (SysLogAspect.class) {//获得登录用户信息
//        User systemUser = (User) SecurityUtils.getSubject().getPrincipal();
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        SystemLog operationLog = new SystemLog();
        operationLog.setJobNumber(employeeEntity.getJobNumber());
        //获取内网地址IpUtils.intranetIp()
        //获取外网地址IpUtils.internetIp()
//        operationLog.setIpAddressLan("");
//        operationLog.setIpAddressWan("");
        operationLog.setIpAddressLan(IpUtils.intranetIp());
        //获取不到外网IP设置内网IP
        if (StringUtils.isBlank(IpUtils.internetIp())) {
            operationLog.setIpAddressWan(IpUtils.intranetIp());
        } else {
            operationLog.setIpAddressWan(IpUtils.internetIp());
        }
//        operationLog.setRunTime(time);
        //operationLog.setRunTime(0L);
        operationLog.setReturnValue(JSONObject.toJSONString(res));
        operationLog.setId(UUID.randomUUID().toString());
        operationLog.setArgs(JSONObject.toJSONString(joinPoint.getArgs()));
        operationLog.setCreateTime(new Date());
        operationLog.setMethod(signature.getDeclaringTypeName() + "." + signature.getName());
        operationLog.setUserId(systemUser.getId() + "");
        operationLog.setUserName(employeeEntity.getName());
        Log annotation = signature.getMethod().getAnnotation(Log.class);
        if (annotation != null) {
            operationLog.setLogLevel(annotation.level());
            operationLog.setLogDescribe(getDetail(((MethodSignature) joinPoint.getSignature()).getParameterNames(), joinPoint.getArgs(), annotation));
            operationLog.setOperationType(annotation.operationType().getValue());
            operationLog.setOperationUnit(annotation.operationUnit().getValue());
        }

        //这里保存日志
//        log.info("######记录日志:{}######", operationLog.toString());
        //int b = systemLogMapper.insert(operationLog);
        ResponseResult insert = hrauthFeignService.insert(operationLog);
        log.info("######记录日志:{}######", operationLog.toString());
        if (insert.getCode()== 40007) {
            log.error("#####新增###记录日志失败:{}####", operationLog);
        }
//        }
    }

    /**
     * 对当前登录用户和占位符处理
     *
     * @param argNames   方法参数名称数组
     * @param args       方法参数数组
     * @param annotation 注解信息
     * @return 返回处理后的描述
     */
    private String getDetail(String[] argNames, Object[] args, Log annotation) {
        //获得登录用户信息
        //User systemUser = (User) SecurityUtils.getSubject().getPrincipal();
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String token = request.getHeader("token");
        String userId = RedisUtil.get("dq:token:"+token);
        String userEntityStr = RedisUtil.get("dq:userId:"+userId);
        UserEntity systemUser = JSON.parseObject(userEntityStr,UserEntity.class);
        //Token userToken = tokenService.getOne(new QueryWrapper<Token>().eq("token", token));
        //UserEntity systemUser = userLoginService.getOne(new QueryWrapper<UserEntity>().eq("id",userToken.getUserId()));

        Map<Object, Object> map = new HashMap<>(4);
        for (int i = 0; i < argNames.length; i++) {
            map.put(argNames[i], args[i]);
        }

        String detail = annotation.detail();
        try {
            detail = "'" + systemUser.getAccount() + "'=》" + annotation.detail();
            for (Map.Entry<Object, Object> entry : map.entrySet()) {
                Object k = entry.getKey();
                Object v = entry.getValue();
                detail = detail.replace("{{" + k + "}}", JSONObject.toJSONString(v));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return detail;
    }
}
