package com.daqing.framework.model;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;

/**
 * 人资管理系统，员工账号信息表，用于存储员工账号密码等登录相关数据
 * 
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-07 16:26:04
 */
@Data
@TableName("hrms_user")
public class UserEntity implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * 主键
	 */
	@TableId(value = "id", type = IdType.INPUT)
	private Long id;
	/**
	 * 用于登录的账号，必填
	 */
	private String account;
	/**
	 * 用户可以绑定手机账号，用于登录
	 */
	private String phoneAccount;
	/**
	 * 登录密码
	 */
	private String password;
	/**
	 * 登录次数
	 */
	private Integer loginNum;
	/**
	 * 上次的登录时间
	 */
	private Date lasttime;
	/**
	 * 微信的唯一标识
	 */
	private String wechatId;
	/**
	 * 状态：0、启用，1、禁用
	 */
	private Integer status;
	/**
	 * 禁用开始时间
	 */
	@JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
	@ApiModelProperty("禁用开始时间")
	private Date disableStartTime;
	/**
	 * 禁用结束时间
	 */
	@JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
	@ApiModelProperty("禁用结束时间")
	private Date disableEndTime;
	/**
	 * 禁用原因
	 */
	@ApiModelProperty("禁用原因")
	private String disableCause;
	/**
	 * 是否删除：0、未删除，1、删除
	 */
	private Integer delOrNot;
	/**
	 * 创建时间
	 */
	private Date createTime;
	/**
	 * 更新时间
	 */
	private Date motifyTime;

	/**
	 * Token
	 */
	private String token;

	/**
	 * 盐值
	 */
	//private String salt;
	/**
	 * 头像地址
	 */
	private String headPortaritUrl;

	/**
	 * 是否通知提醒：0->默认开启；1->关闭；
	 */
	private Integer isRemind;

}
