package com.daqing.framework.util;


import com.alibaba.fastjson.JSONObject;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.apache.commons.codec.binary.Base64;

import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.security.SignatureException;
import java.util.Date;

public class JwtUtils {
    /**
     * 签发JWT
     *
     * @param id
     * @param subject   可以是JSON数据 尽可能少
     * @param ttlMillis
     * @return String
     *
     */
    public static String createJWT(String id, String account,String subject, long ttlMillis) {
        ttlMillis = ttlMillis * 1000;
        SignatureAlgorithm signatureAlgorithm = SignatureAlgorithm.HS256;
        long nowMillis = System.currentTimeMillis();
        Date now = new Date(nowMillis);
        SecretKey secretKey = generalKey();
        JwtBuilder builder = Jwts.builder().setId(String.valueOf(id)).setSubject(subject) // 主题
                .setIssuer(account) // 签发者 "user"
                .setIssuedAt(now) // 签发时间
                .signWith(signatureAlgorithm, secretKey); // 签名算法以及密匙
        if (ttlMillis >= 0) {
            long expMillis = nowMillis + ttlMillis;
            Date expDate = new Date(expMillis);
            builder.setExpiration(expDate); // 过期时间
        }
        return builder.compact();
    }

    public static void main(String[] args) throws Exception {
        //System.out.printf(createJWT("5","七七","111", 10000000));
        boolean isTrue = validateJWT("eyJhbGciOiJIUzI1NiJ9.eyJqdGkiOiI1Iiwic3ViIjoiMTExIiwiaXNzIjoi5LiD5LiDIiwiaWF0IjoxNjA2NDYxMDI1LCJleHAiOjE2MTY0NjEwMjV9.Pywtr4On2QAjhkckvXO2eED_K0304dpUmuGwnYoKITM");
        System.out.println(isTrue);
        System.out.println(parseJWT("eyJhbGciOiJIUzI1NiJ9.eyJqdGkiOiI1Iiwic3ViIjoiMTExIiwiaXNzIjoi5LiD5LiDIiwiaWF0IjoxNjA2NDYxMDI1LCJleHAiOjE2MTY0NjEwMjV9.Pywtr4On2QAjhkckvXO2eED_K0304dpUmuGwnYoKITM"));
    }

    /**
     * 验证JWT
     *
     * @param jwtStr
     * @return
     */
    public static Boolean validateJWT(String jwtStr) {
        //boolean isValidate = false;
        Claims claims = null;
        try {
            claims = parseJWT(jwtStr);
            return true;
        } catch (ExpiredJwtException e) {
            return false;
        } catch (SignatureException e) {
            return false;
        } catch (Exception e) {
            return false;
        }
        //return checkResult;
    }

    public static SecretKey generalKey() {
        byte[] encodedKey = Base64.decodeBase64("DQJWT");
        SecretKey key = new SecretKeySpec(encodedKey, 0, encodedKey.length, "AES");
        return key;
    }

    /**
     *
     * 解析JWT字符串
     *
     * @param jwt
     * @return
     * @throws Exception
     */
    public static Claims parseJWT(String jwt) throws Exception {
        SecretKey secretKey = generalKey();
        return Jwts.parser().setSigningKey(secretKey).parseClaimsJws(jwt).getBody();
    }

    public static void putTokenToRedis(Long userId, String token, long times) {
        RedisUtil.setEx("dq:token:"+token, String.valueOf(userId), times);
    }

    public static <UserEntity> void putUserEntityToRedis(Long userId, UserEntity userEntity, long times) {
        RedisUtil.setEx("dq:userId:"+userId, JSONObject.toJSONString(userEntity), times);
    }

    public static void putRoleIdToRedis(Long userId, String roleId, long times) {
        RedisUtil.setEx("dq:userRole:"+userId, roleId, times);
    }

    public static void removeUserEntityByUserId(Long userId) {
        RedisUtil.del("dq:userId:"+userId);
    }

    public static void removeRoleIdByUserId(Long userId) {
        RedisUtil.del("dq:userRole:"+userId);
    }

    public static void removeTokenByToken(String token) {
        RedisUtil.del("dq:token:"+token);
    }
}