package com.daqing.framework.domain.crms;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;

import java.io.Serializable;
import java.util.Date;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

/**
 * 个人类型客户信息表
 * 
 * @author gongsj
 * @email gongsj@gmail.com
 * @date 2020-09-08 09:57:32
 */
@Data
@TableName("crms_personal_customer")
@ToString
public class PersonalCustomerEntity implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * 主键
	 */
	@ApiModelProperty(value = "id")
	@TableId(value = "id", type = IdType.INPUT)
	private Long id;
	/**
	 * 客户基本信息表id
	 */
	@ApiModelProperty(value = "客户基本表id")
	private Long customerId;
	/**
	 * 身份证号
	 */
	@NotNull(message = "身份证号不能为空")
	@ApiModelProperty(value = "身份证号")
	private String idCard;
	/**
	 * 年龄
	 */
	@ApiModelProperty(value = "年龄")
	private Integer age;
	/**
	 * 性别：1、男，0、女
	 */
	@ApiModelProperty(value = "性别")
	private Integer gender;
	/**
	 * 婚姻状况,0:未婚,1:已婚,2:离异,3:再婚
	 */
	@ApiModelProperty(value = "婚姻状况")
	private Integer maritalStatus;
	/**
	 * 学历,0:本科,1:大专,2:高职,3:中专,4:其他
	 */
	@ApiModelProperty(value = "学历")
	private Integer education;
	/**
	 * 工作单位
	 */
	@ApiModelProperty(value = "工作单位")
	private String employer;
	/**
	 * 职务
	 */
	@ApiModelProperty(value = "职务")
	private String position;
	/**
	 * 工作年限
	 */
	@Pattern(regexp = "^\\d+$",message = "工作年限只能为数字")
	@ApiModelProperty(value = "工作年限")
	private Integer workingYears;
	/**
	 * 社保账号
	 */
	@ApiModelProperty(value = "社保账号")
	private String socialSecurityNum;
	/**
	 * 居住情况
	 */
	@ApiModelProperty(value = "居住情况")
	private String livingSituation;
	/**
	 * 户籍地址
	 */
	@ApiModelProperty(value = "户籍地址")
	private String residenceAddr;
	/**
	 * 业务来源
	 */
	@ApiModelProperty(value = "业务来源")
	private String businessSource;
	/**
	 * 紧急联系人
	 */
	@ApiModelProperty(value = "紧急联系人")
	private String emergencyLinkman;
	/**
	 * 紧急联系人关系
	 */
	@ApiModelProperty(value = "紧急联系人关系")
	private String emergencyLinkmanRelationship;
	/**
	 * 紧急联系人电话
	 */
	@ApiModelProperty(value = "紧急联系人电话")
	private String emergencyLinkmanPhone;

}
