package com.daqing.framework.domain.crms.ext;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;

import javax.validation.constraints.*;
import java.io.Serializable;

/**
 * 企业类型客户信息新增
 * @auther River
 * @date 2020/10/9 10:32
 */
@Data
@ToString
public class CustomerCompanyTOI implements Serializable {
    /**
     * 客户类型：1、企业类型，0：个人类型
     */
    @Min(value = 0,message = "客户类型格式有误")
    @Max(value = 1,message = "客户类型格式有误")
    @NotNull(message = "客户类型不能为空")
    @ApiModelProperty(value = "客户类型")
    private Integer type;
    /**
     * 客户的经理人id
     */
    @NotNull(message = "客户经理人不能为空")
    @ApiModelProperty(value = "客户经理人id")
    private Long manager;
    /**
     * 客户名称
     */
    @NotNull(message = "客户名称不能为空")
    @ApiModelProperty(value = "客户名称")
    private String name;
    /**
     * 联系地址
     */
    @NotNull(message = "联系地址不能为空")
    @ApiModelProperty(value = "联系地址")
    private String addr;
    /**
     * 联系电话
     */
    @NotNull(message = "联系电话不能为空")
    @Pattern(regexp = "^[0-9]*$",message = "联系电话格式错误")
    @ApiModelProperty(value = "联系电话")
    private String phone;
    /**
     * 注册资金
     */
    @Pattern(regexp = "^[0-9]+\\.{0,1}[0-9]{0,2}$",message = "注册资金格式有误")
    @NotNull(message = "注册资金不能为空")
    @ApiModelProperty(value = "注册资金")
    private String registeredCapital;
    /**
     * 所属行业
     */
    @NotNull(message = "所属行业不能为空")
    @ApiModelProperty(value = "所属行业")
    private String industry;
    /**
     * 成立年限
     */
    @NotNull(message = "成立年限不能为空")
    @ApiModelProperty(value = "成立年限")
    private Integer years;
    /**
     * 所在区域
     */
    @NotNull(message = "所在区域不能为空")
    @ApiModelProperty(value = "所在区域")
    private String region;
    /**
     * 股东名称
     */
    @NotNull(message = "股东名称不能为空")
    @ApiModelProperty(value = "股东名称")
    private String shareholder;
    /**
     * 关联企业
     */
    @NotNull(message = "关联企业不能为空")
    @ApiModelProperty(value = "关联企业")
    private String affiliatedCompany;
    /**
     * 员工个数
     */
    @Pattern(regexp = "^[0-9]*$",message = "员工个数格式错误")
    @NotNull(message = "员工个数不能为空")
    @ApiModelProperty(value = "员工个数")
    private String empNum;
    /**
     * 联系人
     */
    @NotNull(message = "联系人不能为空")
    @ApiModelProperty(value = "联系人")
    private String linkman;
    /**
     * 业务来源
     */
    @NotNull(message = "业务来源不能为空")
    @ApiModelProperty(value = "业务来源")
    private String businessSource;
}
