package com.daqing.framework.domain.crms.ext;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;

/**
 * 个人类型客户信息新增
 * @auther River
 * @date 2020/10/9 10:32
 */
@Data
@ToString
public class CustomerPersonalTOI implements Serializable {

    /**
     * 客户类型：1、企业类型，0：个人类型
     */
    @ApiModelProperty(value = "客户类型")
    private Integer type;
    /**
     * 客户的经理人id
     */
    @ApiModelProperty(value = "客户经理人id")
    private Long manager;
    /**
     * 客户名称
     */
    @NotNull(message = "客户名称不能为空")
    @ApiModelProperty(value = "客户名称")
    private String name;
    /**
     * 联系地址
     */
    @NotNull(message = "联系地址不能为空")
    @ApiModelProperty(value = "联系地址")
    private String addr;
    /**
     * 联系电话
     */
    @Pattern(regexp = "^[0-9]*$",message = "联系电话格式错误")
    @NotNull(message = "联系电话不能为空")
    @ApiModelProperty(value = "联系电话")
    private String phone;
    /**
     * 身份证号
     */
    @NotNull(message = "身份证号不能为空")
    @ApiModelProperty(value = "身份证号")
    private String idCard;
    /**
     * 年龄
     */
    @NotNull(message = "年龄不能为空")
    @ApiModelProperty(value = "年龄")
    private Integer age;
    /**
     * 性别：1、男，0、女
     */
    @Min(value = 0,message = "性别格式有误")
    @Max(value = 1,message = "性别格式有误")
    @NotNull(message = "性别不能为空")
    @ApiModelProperty(value = "性别")
    private Integer gender;
    /**
     * 婚姻状况,0:未婚,1:已婚,2:离异,3:再婚
     */
    @Min(value = 0,message = "婚姻状况格式有误")
    @Max(value = 3,message = "婚姻状况格式有误")
    @NotNull(message = "婚姻状况不能为空")
    @ApiModelProperty(value = "婚姻状况")
    private Integer maritalStatus;
    /**
     * 学历,0:本科,1:大专,2:高职,3:中专,4:其他
     */
    @Min(value = 0,message = "学历格式有误")
    @Max(value = 4,message = "学历格式有误")
    @NotNull(message = "学历不能为空")
    @ApiModelProperty(value = "学历")
    private Integer education;
    /**
     * 工作单位
     */
    @NotNull(message = "工作单位不能为空")
    @ApiModelProperty(value = "工作单位")
    private String employer;
    /**
     * 职务
     */
    @NotNull(message = "职务不能为空")
    @ApiModelProperty(value = "职务")
    private String position;
    /**
     * 工作年限
     */
    @NotNull(message = "工作年限不能为空")
    @ApiModelProperty(value = "工作年限")
    private Integer workingYears;
    /**
     * 社保账号
     */
    @NotNull(message = "社保账号不能为空")
    @ApiModelProperty(value = "社保账号")
    private String socialSecurityNum;
    /**
     * 居住情况
     */
    @NotNull(message = "居住情况不能为空")
    @ApiModelProperty(value = "居住情况")
    private String livingSituation;
    /**
     * 户籍地址
     */
    @NotNull(message = "户籍地址不能为空")
    @ApiModelProperty(value = "户籍地址")
    private String residenceAddr;
    /**
     * 业务来源
     */
    @NotNull(message = "业务来源不能为空")
    @ApiModelProperty(value = "业务来源")
    private String businessSource;
    /**
     * 紧急联系人
     */
    @NotNull(message = "紧急联系人不能为空")
    @ApiModelProperty(value = "紧急联系人")
    private String emergencyLinkman;
    /**
     * 紧急联系人关系
     */
    @NotNull(message = "紧急联系人关系不能为空")
    @ApiModelProperty(value = "紧急联系人关系")
    private String emergencyLinkmanRelationship;
    /**
     * 紧急联系人电话
     */
    @Pattern(regexp = "^[0-9]*$",message = "紧急联系人电话格式错误")
    @NotNull(message = "紧急联系人电话不能为空")
    @ApiModelProperty(value = "紧急联系人电话")
    private String emergencyLinkmanPhone;
}
