package com.daqing.framework.domain.guarantee;

import java.math.BigDecimal;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.metadata.BaseRowModel;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import java.io.Serializable;

import com.daqing.framework.utils.excel.InsuranceStatusConverter;
import com.daqing.framework.utils.excel.SourceStatusConverter;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * <p>
 * 保后管理列表
 * </p>
 *
 * @author lr
 * @since 2021-08-12
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value="AlInsuranceList对象", description="保后管理列表")
public class AlInsuranceList implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键id")
    @TableId(value = "id", type = IdType.AUTO)
    @ExcelIgnore
    private Integer id;

    @ApiModelProperty(value = "业务编号")
    @ExcelProperty(value = "业务编号",index = 0)
    private String businessCode;

    @ApiModelProperty(value = "客户名称")
    @ExcelProperty(value = "客户名称",index = 1)
    private String customerName;

    @ApiModelProperty(value = "法人-姓名")
    @ExcelIgnore
    private String legalName;

    @ApiModelProperty(value = "联系电话")
    @ExcelProperty(value = "联系电话",index = 2)
    private String phone;

    @ApiModelProperty(value = "业务类别")
    @ExcelProperty(value = "业务类型",index = 3)
    private String businessType;

    @ApiModelProperty(value = "担保额度（元）")
    @ExcelProperty(value = "担保额度（元）",index = 4)
    private BigDecimal guaranteeAmount;

    @ApiModelProperty(value = "担保期限")
    @ExcelProperty(value = "担保期限",index = 5)
    private String guaranteeTime;

    @ApiModelProperty(value = "贷款开始日期")
    @ExcelIgnore
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date loanStartTime;

    @ApiModelProperty(value = "贷款结束日期")
    @ExcelIgnore
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date loanEndTime;

    @ApiModelProperty(value = "申请日期")
    @ExcelProperty(value = "申请日期",index = 6)
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @TableField(fill = FieldFill.INSERT)
    private Date applyTime;

    @ApiModelProperty(value = "贷款银行（用中划线隔开）")
    @ExcelIgnore
    private String bank;

    @ApiModelProperty(value = "贷款用途")
    @ExcelIgnore
    private String amountWide;

    @ApiModelProperty(value = "还款额度（元）")
    @ExcelProperty(value = "还款额度（元）",index = 7)
    private BigDecimal repaymentAmount;

    @ApiModelProperty(value = "剩余额度（元）")
    @ExcelProperty(value = "剩余额度（元）",index = 8)
    private BigDecimal remainAmount;

    @ApiModelProperty(value = "还款期数")
    @ExcelProperty(value = "还款期数",index = 9)
    private Integer repaymentTime;

    // @ApiModelProperty(value = "所属部门")
    // @ExcelProperty(value = "所属部门",index = 10)
    // private String department;

    @ApiModelProperty(value = "还款状态：1->还款中；2->已逾期；3->已还清；4->已结项；")
    @ExcelProperty(value = "还款状态",index = 10,converter = InsuranceStatusConverter.class)
    private Integer paymentStatus;

    @ExcelProperty(value = "来源",index = 11,converter = SourceStatusConverter.class)
    @ApiModelProperty(value = "来源，默认为0系统流转，1为手动添加")
    private int source;

    @ExcelIgnore
    @ApiModelProperty(value = "是否逾期过：0->是；1->否；")
    private Integer isOverdue;

    @ExcelIgnore
    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ExcelIgnore
    @ApiModelProperty(value = "修改时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

}

