package com.daqing.framework.domain.guarantee;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 还款记录表
 * </p>
 *
 * @author Qyq
 * @since 2021-03-17
 */
@Data
@TableName("al_repayment_entry")
public class AlRepaymentEntry implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键id")
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty(value = "保后id")
    private Integer insuranceId;

    @NotNull(message = "应还款日不能为空")
    @ApiModelProperty(value = "应还款日")
    private Date repaymentDate;

    @ApiModelProperty(value = "实际还款日")
    @TableField(value = "actual_repayment_date",strategy = FieldStrategy.IGNORED)
    private String actualRepaymentDate;

    @ApiModelProperty(value = "逾期天数")
    private Integer overdueDays;

    @ApiModelProperty(value = "还款总额（元）")
    private BigDecimal totalRepayment;

    @NotNull(message = "本次还款不能为空")
    @ApiModelProperty(value = "本次还款（元）")
    private BigDecimal currentRepayment;

    @NotNull(message = "利息不能为空")
    @ApiModelProperty(value = "利息（元）")
    private BigDecimal interest;

    @ApiModelProperty(value = "其他费用（元）")
    private BigDecimal otherExpenses;

    @ApiModelProperty(value = "减免金额（元）")
    private BigDecimal deductionAmount;

    @ApiModelProperty(value = "还款备注")
    @TableField(strategy = FieldStrategy.IGNORED)
    private String repaymentNotes;

    @ApiModelProperty(value = "还款状态：1->待还款；2->已还款；3->已逾期；4->未到期；")
    private Integer repaymentStatus;

    @ApiModelProperty(value = "提交人id")
    private Integer submitterId;

    @ApiModelProperty(value = "提交人名称")
    private String submitterName;

    @ApiModelProperty(value = "清偿现金")
    @TableField(strategy = FieldStrategy.IGNORED)
    private BigDecimal repaymentCash;

    @ApiModelProperty(value = "清偿资产名称")
    @TableField(strategy = FieldStrategy.IGNORED)
    private String repaymentAssetsName;

    @ApiModelProperty(value = "清偿资产金额")
    @TableField(strategy = FieldStrategy.IGNORED)
    private BigDecimal repaymentAssetsAmount;

    @ApiModelProperty(value = "清偿其他名称")
    @TableField(strategy = FieldStrategy.IGNORED)
    private String repaymentOtherName;

    @ApiModelProperty(value = "清偿其他金额")
    @TableField(strategy = FieldStrategy.IGNORED)
    private BigDecimal repaymentOtherAmount;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill= FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "修改时间")
    @TableField(fill= FieldFill.INSERT_UPDATE)
    private Date updateTime;
}
