package com.daqing.framework.domain.hrms.ext;

import com.daqing.framework.domain.hrms.DeptEntity;
import com.daqing.framework.domain.hrms.PositionEntity;
import com.daqing.framework.domain.hrms.RoleEntity;
import com.daqing.framework.domain.hrms.response.RolePermissionResponse;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;

import javax.validation.constraints.Email;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 员工详细信息展示类
 * @auther River
 * @date 2020/10/12 16:23
 */
@Data
@ToString
public class EmployeeInfoVO implements Serializable {

    @ApiModelProperty("id")
    private Long id;

    @NotNull(message = "姓名不能为空")
    @ApiModelProperty("姓名")
    private String name;

    @NotNull(message = "工号不能为空")
    @ApiModelProperty("工号")
    private String jobNumber;

    @NotNull(message = "性别不能为空")
    @ApiModelProperty("性别")
    private Integer gender;

    @NotNull(message = "生日不能为空")
    @ApiModelProperty("生日")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date birthday;

    @Pattern(regexp = "^1[3|4|5|7|8][0-9]\\d{4,8}$",message = "手机号码格式错误")
    @NotNull(message = "手机号码不能为空")
    @ApiModelProperty("手机号码")
    private String phone;

    @Email(message = "邮箱格式错误")
    @NotNull(message = "公司邮箱不能为空")
    @ApiModelProperty("公司邮箱")
    private String companyMail;

    @NotNull(message = "账号不能为空")
    @ApiModelProperty("账号")
    private String account;

    @ApiModelProperty("状态")
    private Integer status;

    @ApiModelProperty("部门")
    private List<DeptEntity> departments;

    @ApiModelProperty("职位")
    private List<PositionEntity> positions;

    @ApiModelProperty("角色")
    private List<RoleEntity> roles;

    @ApiModelProperty("权限")
    private List<RolePermissionResponse> permissions;

    /**
     * 接收部门、职位、角色
     */
    @ApiModelProperty("部门id集合")
    private List<Long> departmentIds;

    @ApiModelProperty("职位id集合")
    private List<Long> positionIds;

    @ApiModelProperty("角色id集合")
    private List<Long> roleIds;
}
