package com.daqing.financial.gateway.config;

import com.alibaba.fastjson.JSONObject;
import com.daqing.framework.util.JwtUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.reactive.CorsWebFilter;
import org.springframework.web.cors.reactive.UrlBasedCorsConfigurationSource;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.List;
@Slf4j
@Component
@PropertySource(value = "classpath:jwt.properties")
public class ApiGlobalFilter implements GlobalFilter, Ordered {

    /**
     * 不进行token校验的请求地址
     */
    @Value("#{'${jwt.ignoreUrlList}'.split(',')}")
    public List<String> ignoreUrl;

    /**
     * 拦截所有的请求头
     * @param exchange
     * @param chain
     * @return
     */
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        String requestUrl = exchange.getRequest().getPath().toString();
        log.info("requestUrl========================="+requestUrl);

        boolean status = CollectionUtils.contains(ignoreUrl.iterator(), requestUrl);
        log.info("status========================"+status);
        if (!status){
            String token = exchange.getRequest().getHeaders().getFirst("token");
            log.info("token1======================="+token);
            ServerHttpResponse response = exchange.getResponse();
            //没有数据
            if (StringUtils.isBlank(token)) {
                JSONObject message = new JSONObject();
                message.put("code", "401");
                message.put("message", "请先登录");
                byte[] bits = message.toString().getBytes(StandardCharsets.UTF_8);
                DataBuffer buffer = response.bufferFactory().wrap(bits);
                response.setStatusCode(HttpStatus.UNAUTHORIZED);
                response.getHeaders().add("Content-Type", "text/json;charset=UTF-8");
                return response.writeWith(Mono.just(buffer));
                //有数据
            }//else {
//                //校验token
//                boolean isVerify = verifyJWT(token);
//                //String userId = RedisUtil.get("dq:token:"+token);
//                if (! isVerify){
//                    JSONObject message = new JSONObject();
//                    message.put("message", "登录已失效,请重新登录");
//                    message.put("code", "401");
//                    byte[] bits = message.toString().getBytes(StandardCharsets.UTF_8);
//                    DataBuffer buffer = response.bufferFactory().wrap(bits);
//                    response.setStatusCode(HttpStatus.UNAUTHORIZED);
//                    response.getHeaders().add("Content-Type", "text/json;charset=UTF-8");
//                    return response.writeWith(Mono.just(buffer));
//                }
//                //将现在的request，添加当前身份
//                ServerHttpRequest mutableReq = exchange.getRequest().mutate().header("Authorization", token).build();
//                ServerWebExchange mutableExchange = exchange.mutate().request(mutableReq).build();
//                return chain.filter(mutableExchange);
//            }
            ServerHttpRequest mutableReq = exchange.getRequest().mutate().header("Authorization", token).build();
            ServerWebExchange mutableExchange = exchange.mutate().request(mutableReq).build();
            return chain.filter(mutableExchange);
        }
        return chain.filter(exchange);
    }

    /**
     * JWT验证
     * @param token
     * @return userPhone
     */
    private Boolean verifyJWT(String token){
        /*String id = RedisUtil.get("dq:token:"+token);
        if(id == null || "".equals(id)){
            return false;
        }*/
        return JwtUtils.validateJWT(token);
        //return Long.parseLong(id);
    }

    @Override
    public int getOrder() {
        return -200;
    }
}

